@@ -165,14 +165,14 @@ define out. */
 //! Define _IRR_COMPILE_WITH_SOFTWARE_ to compile the Irrlicht engine with software driver
 /** If you do not need the software driver, or want to use Burning's Video instead,
 comment this define out */
-#define _IRR_COMPILE_WITH_SOFTWARE_
+//#define _IRR_COMPILE_WITH_SOFTWARE_
 #ifdef NO_IRR_COMPILE_WITH_SOFTWARE_
 #undef _IRR_COMPILE_WITH_SOFTWARE_
 #endif
 
 //! Define _IRR_COMPILE_WITH_BURNINGSVIDEO_ to compile the Irrlicht engine with Burning's video driver
 /** If you do not need this software driver, you can comment this define out. */
-#define _IRR_COMPILE_WITH_BURNINGSVIDEO_
+//#define _IRR_COMPILE_WITH_BURNINGSVIDEO_
 #ifdef NO_IRR_COMPILE_WITH_BURNINGSVIDEO_
 #undef _IRR_COMPILE_WITH_BURNINGSVIDEO_
 #endif
@@ -456,27 +456,27 @@ B3D, MS3D or X meshes */
 #endif
 
 //! Define _IRR_COMPILE_WITH_IRR_WRITER_ if you want to write static .irrMesh files
-#define _IRR_COMPILE_WITH_IRR_WRITER_
+//#define _IRR_COMPILE_WITH_IRR_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_IRR_WRITER_
 #undef _IRR_COMPILE_WITH_IRR_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_COLLADA_WRITER_ if you want to write Collada files
-#define _IRR_COMPILE_WITH_COLLADA_WRITER_
+//#define _IRR_COMPILE_WITH_COLLADA_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_COLLADA_WRITER_
 #undef _IRR_COMPILE_WITH_COLLADA_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_STL_WRITER_ if you want to write .stl files
-#define _IRR_COMPILE_WITH_STL_WRITER_
+//#define _IRR_COMPILE_WITH_STL_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_STL_WRITER_
 #undef _IRR_COMPILE_WITH_STL_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_OBJ_WRITER_ if you want to write .obj files
-#define _IRR_COMPILE_WITH_OBJ_WRITER_
+//#define _IRR_COMPILE_WITH_OBJ_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_OBJ_WRITER_
 #undef _IRR_COMPILE_WITH_OBJ_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_PLY_WRITER_ if you want to write .ply files
-#define _IRR_COMPILE_WITH_PLY_WRITER_
+//#define _IRR_COMPILE_WITH_PLY_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_PLY_WRITER_
 #undef _IRR_COMPILE_WITH_PLY_WRITER_
 #endif
@@ -493,7 +493,7 @@ B3D, MS3D or X meshes */
 #undef _IRR_COMPILE_WITH_JPG_LOADER_
 #endif
 //! Define _IRR_COMPILE_WITH_PCX_LOADER_ if you want to load .pcx files
-#define _IRR_COMPILE_WITH_PCX_LOADER_
+//#define _IRR_COMPILE_WITH_PCX_LOADER_
 #ifdef NO_IRR_COMPILE_WITH_PCX_LOADER_
 #undef _IRR_COMPILE_WITH_PCX_LOADER_
 #endif
@@ -503,12 +503,12 @@ B3D, MS3D or X meshes */
 #undef _IRR_COMPILE_WITH_PNG_LOADER_
 #endif
 //! Define _IRR_COMPILE_WITH_PPM_LOADER_ if you want to load .ppm/.pgm/.pbm files
-#define _IRR_COMPILE_WITH_PPM_LOADER_
+//#define _IRR_COMPILE_WITH_PPM_LOADER_
 #ifdef NO_IRR_COMPILE_WITH_PPM_LOADER_
 #undef _IRR_COMPILE_WITH_PPM_LOADER_
 #endif
 //! Define _IRR_COMPILE_WITH_PSD_LOADER_ if you want to load .psd files
-#define _IRR_COMPILE_WITH_PSD_LOADER_
+//#define _IRR_COMPILE_WITH_PSD_LOADER_
 #ifdef NO_IRR_COMPILE_WITH_PSD_LOADER_
 #undef _IRR_COMPILE_WITH_PSD_LOADER_
 #endif
@@ -544,17 +544,17 @@ B3D, MS3D or X meshes */
 #endif
 
 //! Define _IRR_COMPILE_WITH_BMP_WRITER_ if you want to write .bmp files
-#define _IRR_COMPILE_WITH_BMP_WRITER_
+//#define _IRR_COMPILE_WITH_BMP_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_BMP_WRITER_
 #undef _IRR_COMPILE_WITH_BMP_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_JPG_WRITER_ if you want to write .jpg files
-#define _IRR_COMPILE_WITH_JPG_WRITER_
+//#define _IRR_COMPILE_WITH_JPG_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_JPG_WRITER_
 #undef _IRR_COMPILE_WITH_JPG_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_PCX_WRITER_ if you want to write .pcx files
-#define _IRR_COMPILE_WITH_PCX_WRITER_
+//#define _IRR_COMPILE_WITH_PCX_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_PCX_WRITER_
 #undef _IRR_COMPILE_WITH_PCX_WRITER_
 #endif
@@ -564,17 +564,17 @@ B3D, MS3D or X meshes */
 #undef _IRR_COMPILE_WITH_PNG_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_PPM_WRITER_ if you want to write .ppm files
-#define _IRR_COMPILE_WITH_PPM_WRITER_
+//#define _IRR_COMPILE_WITH_PPM_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_PPM_WRITER_
 #undef _IRR_COMPILE_WITH_PPM_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_PSD_WRITER_ if you want to write .psd files
-#define _IRR_COMPILE_WITH_PSD_WRITER_
+//#define _IRR_COMPILE_WITH_PSD_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_PSD_WRITER_
 #undef _IRR_COMPILE_WITH_PSD_WRITER_
 #endif
 //! Define _IRR_COMPILE_WITH_TGA_WRITER_ if you want to write .tga files
-#define _IRR_COMPILE_WITH_TGA_WRITER_
+//#define _IRR_COMPILE_WITH_TGA_WRITER_
 #ifdef NO_IRR_COMPILE_WITH_TGA_WRITER_
 #undef _IRR_COMPILE_WITH_TGA_WRITER_
 #endif
@@ -603,7 +603,7 @@ defined. */
 #undef _IRR_USE_NON_SYSTEM_ZLIB_
 #endif
 //! Define _IRR_COMPILE_WITH_ZIP_ENCRYPTION_ if you want to read AES-encrypted ZIP archives
-#define _IRR_COMPILE_WITH_ZIP_ENCRYPTION_
+//#define _IRR_COMPILE_WITH_ZIP_ENCRYPTION_
 #ifdef NO_IRR_COMPILE_WITH_ZIP_ENCRYPTION_
 #undef _IRR_COMPILE_WITH_ZIP_ENCRYPTION_
 #endif
@@ -611,7 +611,7 @@ defined. */
 /** bzip2 is superior to the original zip file compression modes, but requires
 a certain amount of memory for decompression and adds several files to the
 library. */
-#define _IRR_COMPILE_WITH_BZIP2_
+//#define _IRR_COMPILE_WITH_BZIP2_
 #ifdef NO_IRR_COMPILE_WITH_BZIP2_
 #undef _IRR_COMPILE_WITH_BZIP2_
 #endif
@@ -626,7 +626,7 @@ defined. */
 //! Define _IRR_COMPILE_WITH_LZMA_ if you want to use LZMA compressed zip files.
 /** LZMA is a very efficient compression code, known from 7zip. Irrlicht
 currently only supports zip archives, though. */
-#define _IRR_COMPILE_WITH_LZMA_
+//#define _IRR_COMPILE_WITH_LZMA_
 #ifdef NO_IRR_COMPILE_WITH_LZMA_
 #undef _IRR_COMPILE_WITH_LZMA_
 #endif
@@ -638,12 +638,12 @@ currently only supports zip archives, though. */
 #undef __IRR_COMPILE_WITH_MOUNT_ARCHIVE_LOADER_
 #endif
 //! Define __IRR_COMPILE_WITH_PAK_ARCHIVE_LOADER_ if you want to open ID software PAK archives
-#define __IRR_COMPILE_WITH_PAK_ARCHIVE_LOADER_
+//#define __IRR_COMPILE_WITH_PAK_ARCHIVE_LOADER_
 #ifdef NO__IRR_COMPILE_WITH_PAK_ARCHIVE_LOADER_
 #undef __IRR_COMPILE_WITH_PAK_ARCHIVE_LOADER_
 #endif
 //! Define __IRR_COMPILE_WITH_NPK_ARCHIVE_LOADER_ if you want to open Nebula Device NPK archives
-#define __IRR_COMPILE_WITH_NPK_ARCHIVE_LOADER_
+//#define __IRR_COMPILE_WITH_NPK_ARCHIVE_LOADER_
 #ifdef NO__IRR_COMPILE_WITH_NPK_ARCHIVE_LOADER_
 #undef __IRR_COMPILE_WITH_NPK_ARCHIVE_LOADER_
 #endif
@@ -653,7 +653,7 @@ currently only supports zip archives, though. */
 #undef __IRR_COMPILE_WITH_TAR_ARCHIVE_LOADER_
 #endif
 //! Define __IRR_COMPILE_WITH_WAD_ARCHIVE_LOADER_ if you want to open WAD archives
-#define __IRR_COMPILE_WITH_WAD_ARCHIVE_LOADER_
+//#define __IRR_COMPILE_WITH_WAD_ARCHIVE_LOADER_
 #ifdef NO__IRR_COMPILE_WITH_WAD_ARCHIVE_LOADER_
 #undef __IRR_COMPILE_WITH_WAD_ARCHIVE_LOADER_
 #endif
