@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2008 Joerg Henrichs
+//  Copyright (C) 2008-2013 Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -59,11 +59,11 @@ SFXManager::SFXManager()
     m_master_gain = UserConfigParams::m_sfx_volume;
     // Init position, since it can be used before positionListener is called.
     m_position    = Vec3(0,0,0);
-    
+
     loadSfx();
     if (!sfxAllowed()) return;
     setMasterSFXVolume( UserConfigParams::m_sfx_volume );
-    
+
 }  // SoundManager
 
 //-----------------------------------------------------------------------------
@@ -87,7 +87,7 @@ SFXManager::~SFXManager()
         }
     }
     m_quick_sounds.clear();
-    
+
     // ---- clear m_all_sfx_types
     {
         std::map<std::string, SFXBuffer*>::iterator i = m_all_sfx_types.begin();
@@ -100,7 +100,7 @@ SFXManager::~SFXManager()
         m_all_sfx_types.clear();
     }
     m_all_sfx_types.clear();
-    
+
     sfx_manager = NULL;
 }   // ~SFXManager
 
@@ -117,9 +117,9 @@ void SFXManager::soundToggled(const bool on)
             SFXBuffer* buffer = (*i).second;
             buffer->load();
         }
-        
+
         resumeAll();
-        
+
         const int sfx_amount = m_all_sfx.size();
         for (int n=0; n<sfx_amount; n++)
         {
@@ -136,7 +136,7 @@ void SFXManager::soundToggled(const bool on)
 
 bool SFXManager::sfxAllowed()
 {
-    if(!UserConfigParams::m_sfx || !m_initialized) 
+    if(!UserConfigParams::m_sfx || !m_initialized)
         return false;
     else
         return true;
@@ -153,15 +153,17 @@ void SFXManager::loadSfx()
     {
         std::cerr << "Could not read sounf effects XML file " << sfx_config_name.c_str() << std::endl;
     }
-    
+
+    int i;
+
     const int amount = root->getNumNodes();
-    for (int i=0; i<amount; i++)
+    for (i=0; i<amount; i++)
     {
         const XMLNode* node = root->getNode(i);
-        
+
         if (node->getName() == "sfx")
         {
-            loadSingleSfx(node);
+            loadSingleSfx(node, "", false);
         }
         else
         {
@@ -169,9 +171,29 @@ void SFXManager::loadSfx()
             throw std::runtime_error("Unknown node in sfx XML file");
         }
     }// nend for
-    
+
     delete root;
-   }   // loadSfx
+
+    // Now load them in parallel
+    const int max = m_all_sfx_types.size();
+    SFXBuffer **array = new SFXBuffer *[max];
+    i = 0;
+
+    for (std::map<std::string, SFXBuffer*>::iterator it = m_all_sfx_types.begin();
+         it != m_all_sfx_types.end(); it++)
+    {
+        SFXBuffer* const buffer = (*it).second;
+        array[i++] = buffer;
+    }
+
+    #pragma omp parallel for private(i)
+    for (i = 0; i < max; i++)
+    {
+        array[i]->load();
+    }
+
+    delete [] array;
+}   // loadSfx
 
 // -----------------------------------------------------------------------------
 /** Introduces a mechanism by which one can load sound effects beyond the basic
@@ -188,25 +210,26 @@ SFXBuffer* SFXManager::addSingleSfx(const std::string &sfx_name,
                                     bool               positional,
                                     float              rolloff,
                                     float              max_width,
-                                    float              gain)
+                                    float              gain,
+                                    const bool         load)
 {
 
     SFXBuffer* buffer = new SFXBuffer(sfx_file, positional, rolloff, max_width, gain);
-    
+
     m_all_sfx_types[sfx_name] = buffer;
-    
-    if (!m_initialized) 
+
+    if (!m_initialized)
     {
         // Keep the buffer even if SFX is disabled, in case
         // SFX is enabled back later
         return NULL;
     }
 
-    if (UserConfigParams::logMisc()) 
-        printf("Loading SFX %s\n", sfx_file.c_str());
-    
-    if (buffer->load()) return buffer;
-    
+    if (UserConfigParams::logMisc())
+        Log::debug("SFXManager", "Loading SFX %s\n", sfx_file.c_str());
+
+    if (load && buffer->load()) return buffer;
+
     return NULL;
 } // addSingleSFX
 
@@ -215,29 +238,30 @@ SFXBuffer* SFXManager::addSingleSfx(const std::string &sfx_name,
  *  \param node The XML node with the data for this sfx.
  */
 SFXBuffer* SFXManager::loadSingleSfx(const XMLNode* node,
-                                     const std::string &path)
+                                     const std::string &path,
+                                     const bool load)
 {
     std::string filename;
 
     if (node->get("filename", &filename) == 0)
     {
-        fprintf(stderr, 
+        Log::error("SFXManager",
                 "/!\\ The 'filename' attribute is mandatory in the SFX XML file!\n");
         return NULL;
     }
-    
+
     std::string sfx_name = StringUtils::removeExtension(filename);
     /*
     if (node->get("name", &sfx_name) == 0)
     {
-        fprintf(stderr, 
+        fprintf(stderr,
                 "/!\\ The 'name' attribute is mandatory in the SFX XML file!\n");
         return;
     }
      */
     if(m_all_sfx_types.find(sfx_name)!=m_all_sfx_types.end())
     {
-        fprintf(stderr, 
+        Log::error("SFXManager",
                 "There is already a sfx named '%s' installed - new one is ignored.\n",
                 sfx_name.c_str());
         return NULL;
@@ -247,15 +271,16 @@ SFXBuffer* SFXManager::loadSingleSfx(const XMLNode* node,
     // to load terrain specific sfx.
     const std::string full_path = (path == "") ? file_manager->getSFXFile(filename)
                                                : path;
-    
+
     SFXBuffer tmpbuffer(full_path, node);
 
     return addSingleSfx(sfx_name, full_path,
                         tmpbuffer.isPositional(),
                         tmpbuffer.getRolloff(),
                         tmpbuffer.getMaxDist(),
-                        tmpbuffer.getGain());
-    
+                        tmpbuffer.getGain(),
+                        load);
+
 }   // loadSingleSfx
 
 //----------------------------------------------------------------------------
@@ -265,8 +290,9 @@ SFXBuffer* SFXManager::loadSingleSfx(const XMLNode* node,
  *  call deleteSFX().
  *  \param id Identifier of the sound effect to create.
  */
-SFXBase* SFXManager::createSoundSource(SFXBuffer* buffer, 
-                                       const bool add_to_SFX_list)
+SFXBase* SFXManager::createSoundSource(SFXBuffer* buffer,
+                                       const bool add_to_SFX_list,
+                                       const bool owns_buffer)
 {
     bool positional = false;
 
@@ -279,18 +305,18 @@ SFXBase* SFXManager::createSoundSource(SFXBuffer* buffer,
     //       buffer->getFileName().c_str(), (unsigned int)buffer,
     //       positional,
     //       race_manager->getNumLocalPlayers(), buffer->isPositional());
-    
+
 #if HAVE_OGGVORBIS
     assert( alIsBuffer(buffer->getBufferID()) );
-    SFXBase* sfx = new SFXOpenAL(buffer, positional, buffer->getGain());
+    SFXBase* sfx = new SFXOpenAL(buffer, positional, buffer->getGain(), owns_buffer);
 #else
-    SFXBase* sfx = new DummySFX(buffer, positional, buffer->getGain());
+    SFXBase* sfx = new DummySFX(buffer, positional, buffer->getGain(), owns_buffer);
 #endif
-    
-    sfx->volume(m_master_gain);
-    
+
+    sfx->masterVolume(m_master_gain);
+
     if (add_to_SFX_list) m_all_sfx.push_back(sfx);
-        
+
     return sfx;
 }   // createSoundSource
 
@@ -300,21 +326,21 @@ void SFXManager::dump()
 {
     for(int n=0; n<(int)m_all_sfx.size(); n++)
     {
-        printf("Sound %i : %s \n", n, m_all_sfx[n]->getBuffer()->getFileName().c_str());
+        Log::debug("SFXManager", "Sound %i : %s \n", n, m_all_sfx[n]->getBuffer()->getFileName().c_str());
     }
 }
 
 //----------------------------------------------------------------------------
-SFXBase* SFXManager::createSoundSource(const std::string &name, 
+SFXBase* SFXManager::createSoundSource(const std::string &name,
                                        const bool addToSFXList)
 {
     std::map<std::string, SFXBuffer*>::iterator i = m_all_sfx_types.find(name);
     if ( i == m_all_sfx_types.end() )
     {
-        fprintf( stderr, "SFXManager::createSoundSource could not find the requested sound effect : '%s'\n", name.c_str());
+        Log::error("SFXManager", "SFXManager::createSoundSource could not find the requested sound effect : '%s'\n", name.c_str());
         return NULL;
     }
-    
+
     return createSoundSource( i->second, addToSFXList );
 }  // createSoundSource
 
@@ -339,11 +365,11 @@ void SFXManager::deleteSFXMapping(const std::string &name)
 
     if (i == m_all_sfx_types.end())
     {
-        fprintf(stderr, "SFXManager::deleteSFXMapping : Warning: sfx not found in list.\n");
+        Log::warn("SFXManager", "SFXManager::deleteSFXMapping : Warning: sfx not found in list.\n");
         return;
     }
     (*i).second->unload();
-    
+
     m_all_sfx_types.erase(i);
 
 }   // deleteSFXMapping
@@ -362,10 +388,10 @@ void SFXManager::deleteSFX(SFXBase *sfx)
 
     if(i==m_all_sfx.end())
     {
-        fprintf(stderr, "SFXManager::deleteSFX : Warning: sfx not found in list.\n");
+        Log::warn("SFXManager", "SFXManager::deleteSFX : Warning: sfx not found in list.\n");
         return;
     }
-    
+
     delete sfx;
 
     m_all_sfx.erase(i);
@@ -394,18 +420,18 @@ void SFXManager::resumeAll()
 {
     // ignore unpausing if sound is disabled
     if (!sfxAllowed()) return;
-    
+
     for (std::vector<SFXBase*>::iterator i=m_all_sfx.begin();
         i!=m_all_sfx.end(); i++)
     {
         SFXStatus status = (*i)->getStatus();
-        // Initial happens when 
+        // Initial happens when
         if (status==SFX_PAUSED) (*i)->resume();
     }   // for i in m_all_sfx
 }   // resumeAll
 
 //-----------------------------------------------------------------------------
-/** Returns whether or not an openal error has occurred. If so, an error 
+/** Returns whether or not an openal error has occurred. If so, an error
  *  message is printed containing the given context.
  *  \param context Context to specify in the error message.
  */
@@ -417,7 +443,7 @@ bool SFXManager::checkError(const std::string &context)
 
     if(error != AL_NO_ERROR)
     {
-        fprintf(stdout, "SFXOpenAL OpenAL error while %s: %s\n",
+        Log::error("SFXManager", "SFXOpenAL OpenAL error while %s: %s\n",
             context.c_str(), SFXManager::getErrorString(error).c_str());
         return false;
     }
@@ -441,19 +467,19 @@ void SFXManager::setMasterSFXVolume(float gain)
         for (std::vector<SFXBase*>::iterator i=m_all_sfx.begin();
             i!=m_all_sfx.end(); i++)
         {
-            (*i)->volume(m_master_gain);
+            (*i)->masterVolume(m_master_gain);
         }   // for i in m_all_sfx
     }
-    
+
     // quick SFX
     {
         std::map<std::string, SFXBase*>::iterator i = m_quick_sounds.begin();
         for (; i != m_quick_sounds.end(); i++)
         {
-            (*i).second->volume(m_master_gain);
+            (*i).second->masterVolume(m_master_gain);
         }
     }
-    
+
 }   // setMasterSFXVolume
 
 //-----------------------------------------------------------------------------
@@ -481,19 +507,19 @@ void SFXManager::positionListener(const Vec3 &position, const Vec3 &front)
 {
 #if HAVE_OGGVORBIS
     if (!UserConfigParams::m_sfx || !m_initialized) return;
-    
+
     m_position = position;
-    
+
     //forward vector
-    m_listenerVec[0] = front.getX(); 
+    m_listenerVec[0] = front.getX();
     m_listenerVec[1] = front.getY();
-    m_listenerVec[2] = front.getZ(); 
-    
+    m_listenerVec[2] = front.getZ();
+
     //up vector
-    m_listenerVec[3] = 0; 
+    m_listenerVec[3] = 0;
     m_listenerVec[4] = 0;
     m_listenerVec[5] = 1;
-    
+
     alListener3f(AL_POSITION, position.getX(), position.getY(), position.getZ());
     alListenerfv(AL_ORIENTATION, m_listenerVec);
 #endif
@@ -510,7 +536,7 @@ SFXBase* SFXManager::quickSound(const std::string &sound_type)
 {
     if (!sfxAllowed()) return NULL;
     std::map<std::string, SFXBase*>::iterator sound = m_quick_sounds.find(sound_type);
-    
+
     if (sound == m_quick_sounds.end())
     {
         // sound not yet in our local list of quick sounds
