@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //  Modelled after Supertux's configfile.cpp
 //
 //  This program is free software; you can redistribute it and/or
@@ -35,6 +35,7 @@ static PtrVector<UserConfigParam, REF> all_params;
 #define PARAM_DEFAULT(X) = X
 #include "config/user_config.hpp"
 
+#include "config/saved_grand_prix.hpp"
 #include "config/player.hpp"
 #include "config/stk_config.hpp"
 #include "guiengine/engine.hpp"
@@ -45,12 +46,27 @@ static PtrVector<UserConfigParam, REF> all_params;
 #include "utils/string_utils.hpp"
 #include "utils/translation.hpp"
 
+const int UserConfig::m_current_config_version = 8;
+
+
 // ----------------------------------------------------------------------------
 UserConfigParam::~UserConfigParam()
 {
     all_params.remove(this);
 }   // ~UserConfigParam
 
+// ----------------------------------------------------------------------------
+/** Writes an inner node.
+ *  \param stream the xml writer.
+ *  \param level determines indentation level.
+ */
+void UserConfigParam::writeInner(XMLWriter& stream, int level) const
+{
+    std::string tab(level * 4,' ');
+    stream << L"    " << tab.c_str() << m_param_name.c_str() << L"=\""
+           << toString() << L"\"\n";
+}   // writeInner
+
 // ============================================================================
 GroupUserConfigParam::GroupUserConfigParam(const char* group_name,
                                            const char* comment)
@@ -60,46 +76,78 @@ GroupUserConfigParam::GroupUserConfigParam(const char* group_name,
     if(comment != NULL) m_comment = comment;
 }   // GroupUserConfigParam
 
+// ============================================================================
+GroupUserConfigParam::GroupUserConfigParam(const char* group_name,
+                                           GroupUserConfigParam* group,
+                                           const char* comment)
+{
+    m_param_name = group_name;
+    group->addChild(this);
+    if(comment != NULL) m_comment = comment;
+}   // GroupUserConfigParam
+
 // ----------------------------------------------------------------------------
 void GroupUserConfigParam::write(XMLWriter& stream) const
 {
-    const int children_amount = m_children.size();
+    const int attr_amount = m_attributes.size();
 
     // comments
     if(m_comment.size() > 0) stream << "    <!-- " << m_comment.c_str();
-    for(int n=0; n<children_amount; n++)
+    for(int n=0; n<attr_amount; n++)
     {
-        if(m_children[n]->m_comment.size() > 0)
-            stream << L"\n             " << m_children[n]->m_param_name.c_str()
-                   << L" : " << m_children[n]->m_comment.c_str();
+        if(m_attributes[n]->m_comment.size() > 0)
+            stream << L"\n             " << m_attributes[n]->m_param_name.c_str()
+                   << L" : " << m_attributes[n]->m_comment.c_str();
     }
 
     stream << L" -->\n    <" << m_param_name.c_str() << "\n";
 
     // actual values
+    for (int n=0; n<attr_amount; n++)
+    {
+        m_attributes[n]->writeInner(stream, 1);
+    }
+    stream << L"    >\n";
+    const int children_amount = m_children.size();
     for (int n=0; n<children_amount; n++)
     {
-        stream << L"        " << m_children[n]->m_param_name.c_str() << L"=\"" 
-               << m_children[n]->toString() << L"\"\n";
+        m_children[n]->writeInner(stream, 1);
     }
-    stream << L"        />\n\n";
+    stream << L"    </" << m_param_name.c_str() << ">\n\n";
 }   // write
 
+// ----------------------------------------------------------------------------
+void GroupUserConfigParam::writeInner(XMLWriter& stream, int level) const
+{
+    std::string tab(level * 4,' ');
+    for(int i = 0; i < level; i++) tab =+ "    ";
+    const int children_amount = m_attributes.size();
+
+    stream << L"    " << tab.c_str() << "<" << m_param_name.c_str() << "\n";
+
+    // actual values
+    for (int n=0; n<children_amount; n++)
+    {
+        m_attributes[n]->writeInner(stream, level+1);
+    }
+    stream << L"    " << tab.c_str() << "/>\n";
+}   // writeInner
+
 // ----------------------------------------------------------------------------
 void GroupUserConfigParam::findYourDataInAChildOf(const XMLNode* node)
 {
     const XMLNode* child = node->getNode( m_param_name );
     if (child == NULL)
     {
-        //std::cerr << "/!\\ User Config : Couldn't find parameter group " 
+        //std::cerr << "/!\\ User Config : Couldn't find parameter group "
         //          << paramName << std::endl;
         return;
     }
 
-    const int children_amount = m_children.size();
-    for (int n=0; n<children_amount; n++)
+    const int attributes_amount = m_attributes.size();
+    for (int n=0; n<attributes_amount; n++)
     {
-        m_children[n]->findYourDataInAnAttributeOf(child);
+        m_attributes[n]->findYourDataInAnAttributeOf(child);
     }
 
 }   // findYourDataInAChildOf
@@ -116,13 +164,26 @@ irr::core::stringw GroupUserConfigParam::toString() const
 }   // toString
 
 // ----------------------------------------------------------------------------
-void GroupUserConfigParam::addChild(UserConfigParam* child)
+void GroupUserConfigParam::clearChildren()
+{
+    m_children.clear();
+}   // clearChildren
+
+// ----------------------------------------------------------------------------
+void GroupUserConfigParam::addChild(GroupUserConfigParam* child)
 {
     m_children.push_back(child);
 }   // addChild
 
+// ----------------------------------------------------------------------------
+void GroupUserConfigParam::addChild(UserConfigParam* child)
+{
+    m_attributes.push_back(child);
+}   // addChild
+
+
 // ============================================================================
-IntUserConfigParam::IntUserConfigParam(int default_value, 
+IntUserConfigParam::IntUserConfigParam(int default_value,
                                        const char* param_name,
                                        const char* comment)
 {
@@ -134,9 +195,9 @@ IntUserConfigParam::IntUserConfigParam(int default_value,
 }   // IntUserConfigParam
 
 // ----------------------------------------------------------------------------
-IntUserConfigParam::IntUserConfigParam(int default_value, 
+IntUserConfigParam::IntUserConfigParam(int default_value,
                                        const char* param_name,
-                                       GroupUserConfigParam* group, 
+                                       GroupUserConfigParam* group,
                                        const char* comment)
 {
     m_value         = default_value;
@@ -149,9 +210,9 @@ IntUserConfigParam::IntUserConfigParam(int default_value,
 // ----------------------------------------------------------------------------
 void IntUserConfigParam::write(XMLWriter& stream) const
 {
-    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str() 
+    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str()
                                     << L" -->\n";
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value 
+    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value
            << L"\" />\n\n";
 }   // write
 
@@ -184,8 +245,8 @@ void IntUserConfigParam::findYourDataInAnAttributeOf(const XMLNode* node)
 }   // findYourDataInAnAttributeOf
 
 // ============================================================================
-TimeUserConfigParam::TimeUserConfigParam(Time::TimeType default_value, 
-                                         const char* param_name, 
+TimeUserConfigParam::TimeUserConfigParam(StkTime::TimeType default_value,
+                                         const char* param_name,
                                          const char* comment)
 {
     m_value         = default_value;
@@ -196,9 +257,9 @@ TimeUserConfigParam::TimeUserConfigParam(Time::TimeType default_value,
 }   // TimeUserConfigParam
 
 // ----------------------------------------------------------------------------
-TimeUserConfigParam::TimeUserConfigParam(Time::TimeType default_value, 
+TimeUserConfigParam::TimeUserConfigParam(StkTime::TimeType default_value,
                                          const char* param_name,
-                                         GroupUserConfigParam* group, 
+                                         GroupUserConfigParam* group,
                                          const char* comment)
 {
     m_value         = default_value;
@@ -215,7 +276,7 @@ void TimeUserConfigParam::write(XMLWriter& stream) const
                                     << L" -->\n";
     std::ostringstream o;
     o<<m_value;
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" 
+    stream << L"    <" << m_param_name.c_str() << L" value=\""
            << core::stringw(o.str().c_str()) << L"\" />\n\n";
 }   // write
 
@@ -226,7 +287,6 @@ irr::core::stringw TimeUserConfigParam::toString() const
     // we can't use an irrlicht's stringw directly. Since it's only a
     // number, we can use std::string, and convert to stringw
 
-    std::string tmp;
     std::ostringstream o;
     o<<m_value;
     return core::stringw(o.str().c_str());
@@ -241,7 +301,7 @@ void TimeUserConfigParam::findYourDataInAChildOf(const XMLNode* node)
         //std::cout << "Couldn't find int parameter " << paramName <<std::endl;
         return;
     }
-    
+
     int64_t tmp;
     child->get( "value", &tmp );
     m_value = tmp;
@@ -257,7 +317,7 @@ void TimeUserConfigParam::findYourDataInAnAttributeOf(const XMLNode* node)
 
 // ============================================================================
 StringUserConfigParam::StringUserConfigParam(const char* default_value,
-                                             const char* param_name, 
+                                             const char* param_name,
                                              const char* comment)
 {
     m_value         = default_value;
@@ -268,9 +328,9 @@ StringUserConfigParam::StringUserConfigParam(const char* default_value,
 }   // StringUserConfigParam
 
 // ----------------------------------------------------------------------------
-StringUserConfigParam::StringUserConfigParam(const char* default_value, 
+StringUserConfigParam::StringUserConfigParam(const char* default_value,
                                              const char* param_name,
-                                             GroupUserConfigParam* group, 
+                                             GroupUserConfigParam* group,
                                              const char* comment)
 {
     m_value         = default_value;
@@ -283,9 +343,9 @@ StringUserConfigParam::StringUserConfigParam(const char* default_value,
 // ----------------------------------------------------------------------------
 void StringUserConfigParam::write(XMLWriter& stream) const
 {
-    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str() 
+    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str()
                                     << L" -->\n";
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" 
+    stream << L"    <" << m_param_name.c_str() << L" value=\""
            << m_value.c_str() << L"\" />\n\n";
 }   // write
 
@@ -305,14 +365,14 @@ void StringUserConfigParam::findYourDataInAnAttributeOf(const XMLNode* node)
 }   // findYourDataInAnAttributeOf
 
 // ============================================================================
-WStringUserConfigParam::WStringUserConfigParam(const core::stringw& default_value, 
-                                               const char* param_name, 
+WStringUserConfigParam::WStringUserConfigParam(const core::stringw& default_value,
+                                               const char* param_name,
                                                const char* comment)
 {
-    
+
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     all_params.push_back(this);
     if(comment != NULL) m_comment = comment;
@@ -321,12 +381,12 @@ WStringUserConfigParam::WStringUserConfigParam(const core::stringw& default_valu
 // ----------------------------------------------------------------------------
 WStringUserConfigParam::WStringUserConfigParam(const core::stringw& default_value,
                                                const char* param_name,
-                                               GroupUserConfigParam* group, 
+                                               GroupUserConfigParam* group,
                                                const char* comment)
 {
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     group->addChild(this);
     if(comment != NULL) m_comment = comment;
@@ -337,7 +397,7 @@ void WStringUserConfigParam::write(XMLWriter& stream) const
 {
     if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str()
                                     << L" -->\n";
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value 
+    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value
            << L"\" />\n\n";
 }   // write
 // ----------------------------------------------------------------------------
@@ -345,7 +405,7 @@ void WStringUserConfigParam::findYourDataInAChildOf(const XMLNode* node)
 {
     const XMLNode* child = node->getNode( m_param_name );
     if(child == NULL) return;
-    
+
     child->get( "value", &m_value );
 }   // findYourDataInAChildOf
 
@@ -356,13 +416,13 @@ void WStringUserConfigParam::findYourDataInAnAttributeOf(const XMLNode* node)
 }   // findYourDataInAnAttributeOf
 
 // ============================================================================
-BoolUserConfigParam::BoolUserConfigParam(bool default_value, 
-                                         const char* param_name, 
+BoolUserConfigParam::BoolUserConfigParam(bool default_value,
+                                         const char* param_name,
                                          const char* comment)
 {
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     all_params.push_back(this);
     if(comment != NULL) m_comment = comment;
@@ -371,12 +431,12 @@ BoolUserConfigParam::BoolUserConfigParam(bool default_value,
 // ----------------------------------------------------------------------------
 BoolUserConfigParam::BoolUserConfigParam(bool default_value,
                                          const char* param_name,
-                                         GroupUserConfigParam* group, 
+                                         GroupUserConfigParam* group,
                                          const char* comment)
 {
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     group->addChild(this);
     if(comment != NULL) m_comment = comment;
@@ -388,7 +448,7 @@ void BoolUserConfigParam::write(XMLWriter& stream) const
 {
     if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str()
                                     << L" -->\n";
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" 
+    stream << L"    <" << m_param_name.c_str() << L" value=\""
            << (m_value ? L"true" : L"false" ) << L"\" />\n\n";
 }   // write
 
@@ -411,7 +471,7 @@ void BoolUserConfigParam::findYourDataInAChildOf(const XMLNode* node)
     }
     else
     {
-        std::cerr << "Unknown value for " << m_param_name 
+        std::cerr << "Unknown value for " << m_param_name
                   << "; expected true or false\n";
     }
 }   // findYourDataInAChildOf
@@ -432,7 +492,7 @@ void BoolUserConfigParam::findYourDataInAnAttributeOf(const XMLNode* node)
     }
     else
     {
-        std::cerr << "Unknown value for " << m_param_name 
+        std::cerr << "Unknown value for " << m_param_name
                   << "; expected true or false\n";
     }
 }   // findYourDataInAnAttributeOf
@@ -444,27 +504,27 @@ irr::core::stringw BoolUserConfigParam::toString() const
 }   // toString
 
 // ============================================================================
-FloatUserConfigParam::FloatUserConfigParam(float default_value, 
-                                           const char* param_name, 
+FloatUserConfigParam::FloatUserConfigParam(float default_value,
+                                           const char* param_name,
                                            const char* comment)
 {
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     all_params.push_back(this);
     if(comment != NULL) m_comment = comment;
 }   // FloatUserConfigParam
 
 // ----------------------------------------------------------------------------
-FloatUserConfigParam::FloatUserConfigParam(float default_value, 
+FloatUserConfigParam::FloatUserConfigParam(float default_value,
                                            const char* param_name,
-                                           GroupUserConfigParam* group, 
+                                           GroupUserConfigParam* group,
                                            const char* comment)
 {
     m_value         = default_value;
     m_default_value = default_value;
-    
+
     m_param_name = param_name;
     group->addChild(this);
     if(comment != NULL) m_comment = comment;
@@ -473,9 +533,9 @@ FloatUserConfigParam::FloatUserConfigParam(float default_value,
 // ----------------------------------------------------------------------------
 void FloatUserConfigParam::write(XMLWriter& stream) const
 {
-    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str() 
+    if(m_comment.size() > 0) stream << L"    <!-- " << m_comment.c_str()
                                     << L" -->\n";
-    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value 
+    stream << L"    <" << m_param_name.c_str() << L" value=\"" << m_value
            << L"\" />\n\n";
 }   // write
 
@@ -524,6 +584,7 @@ UserConfig::UserConfig()
 UserConfig::~UserConfig()
 {
     UserConfigParams::m_all_players.clearAndDeleteAll();
+    UserConfigParams::m_saved_grand_prix_list.clearAndDeleteAll();
 }   // ~UserConfig
 
 // -----------------------------------------------------------------------------
@@ -539,7 +600,7 @@ void UserConfig::addDefaultPlayer()
     else if(getenv("LOGNAME")!=NULL)    // Linux, Macs
         username = getenv("LOGNAME");
 
-    
+
     class GuestPlayerProfile : public PlayerProfile
     {
     public:
@@ -548,19 +609,20 @@ void UserConfig::addDefaultPlayer()
             m_is_guest_account = true;
         }
     };
-    
+
     // add default guest player
     UserConfigParams::m_all_players.push_back( new GuestPlayerProfile() );
 
     // Set the name as the default name for all players.
-    UserConfigParams::m_all_players.push_back( new PlayerProfile(username.c_str()) );
+    UserConfigParams::m_all_players.push_back(
+                                         new PlayerProfile(username.c_str()) );
 
 }   // addDefaultPlayer
 
 // -----------------------------------------------------------------------------
 
 /** Comparison used to sort players. Most frequent players should be
- *  listed first, so a<b actually means that 
+ *  listed first, so a<b actually means that
  *  a.m_use_frequency > b.m_use_frequency
  *  This way we get a reversed sorted list.
  */
@@ -569,6 +631,13 @@ bool operator<(const PlayerProfile &a, const PlayerProfile &b)
     return a.getUseFrequency() > b.getUseFrequency();
 }   // operator<
 
+// -----------------------------------------------------------------------------
+/** \brief Needed for toggling sort order **/
+bool operator>(const PlayerProfile &a, const PlayerProfile &b)
+{
+    return a.getUseFrequency() < b.getUseFrequency();
+}   // operator>
+
 // -----------------------------------------------------------------------------
 /** Load configuration values from file. */
 bool UserConfig::loadConfig()
@@ -577,25 +646,26 @@ bool UserConfig::loadConfig()
     XMLNode* root = file_manager->createXMLTree(filename);
     if(!root || root->getName() != "stkconfig")
     {
-        std::cerr << "Could not read user config file file " << filename.c_str() << std::endl;
+        std::cerr << "Could not read user config file file " << filename << std::endl;
         if(root) delete root;
         return false;
     }
 
     // ---- Read config file version
-    int configFileVersion = CURRENT_CONFIG_VERSION;
-    if(root->get("version", &configFileVersion) < 1)
+    int config_file_version = m_current_config_version;
+    if(root->get("version", &config_file_version) < 1)
     {
         GUIEngine::showMessage( _("Your config file was malformed, so it was deleted and a new one will be created."), 10.0f);
-        std::cerr << "Warning, malformed user config file! Contains no version\n";
+        Log::error("UserConfig", 
+                   "Warning, malformed user config file! Contains no version");
     }
-    if (configFileVersion < CURRENT_CONFIG_VERSION)
+    if (config_file_version < m_current_config_version)
     {
         // current version (8) is 100% incompatible with other versions (which were lisp)
         // so we just delete the old config. in the future, for smaller updates, we can
         // add back the code previously there that upgraded the config file to the new
         // format instead of overwriting it.
-        
+
         GUIEngine::showMessage( _("Your config file was too old, so it was deleted and a new one will be created."), 10.0f);
         printf("Your config file was too old, so it was deleted and a new one will be created.");
         delete root;
@@ -610,13 +680,13 @@ bool UserConfig::loadConfig()
     {
         all_params[i].findYourDataInAChildOf(root);
     }
-    
-    
+
+
     // ---- Read players
     // we create those AFTER other values are being read simply because we have many Player
     // nodes that all bear the same name, so the generic loading code won't work here
     UserConfigParams::m_all_players.clearAndDeleteAll();
-    
+
     std::vector<XMLNode*> players;
     root->getNodes("Player", players);
     const int amount = players.size();
@@ -624,17 +694,39 @@ bool UserConfig::loadConfig()
     {
         //std::string name;
         //players[i]->get("name", &name);
-        UserConfigParams::m_all_players.push_back( new PlayerProfile(players[i]) );
+        UserConfigParams::m_all_players.push_back(
+                                               new PlayerProfile(players[i]) );
     }
-    
+
     // sort players by frequency of use
     UserConfigParams::m_all_players.insertionSort();
-    
+
+
+    // ---- Read Saved GP's
+    UserConfigParams::m_saved_grand_prix_list.clearAndDeleteAll();
+    std::vector<XMLNode*> saved_gps;
+    root->getNodes("SavedGP", saved_gps);
+    const int gp_amount = saved_gps.size();
+    for (int i=0; i<gp_amount; i++)
+    {
+        UserConfigParams::m_saved_grand_prix_list.push_back(
+                                           new SavedGrandPrix( saved_gps[i]) );
+    }
     delete root;
 
     return true;
 }   // loadConfig
 
+// ----------------------------------------------------------------------------
+
+void UserConfig::postLoadInit()
+{
+    for (int i = 0; i < UserConfigParams::m_all_players.size(); i++)
+    {
+        PlayerProfile* player = UserConfigParams::m_all_players.get(i);
+        if (player->isGuestAccount()) player->setName(_LTR("Guest"));
+    }
+}
 
 // ----------------------------------------------------------------------------
 /** Write settings to config file. */
@@ -654,7 +746,8 @@ void UserConfig::saveConfig()
         XMLWriter configfile(filename.c_str());
 
         configfile << L"<?xml version=\"1.0\"?>\n";
-        configfile << L"<stkconfig version=\"" << CURRENT_CONFIG_VERSION << L"\" >\n\n";
+        configfile << L"<stkconfig version=\"" << m_current_config_version
+                   << L"\" >\n\n";
 
         const int paramAmount = all_params.size();
         for(int i=0; i<paramAmount; i++)
@@ -662,7 +755,7 @@ void UserConfig::saveConfig()
             //std::cout << "saving parameter " << i << " to file\n";
             all_params[i].write(configfile);
         }
-            
+
         configfile << L"</stkconfig>\n";
         configfile.close();
     }
@@ -679,7 +772,7 @@ bool UserConfigParams::logMemory()
      { return (m_verbosity&LOG_MEMORY) == LOG_MEMORY;}
 
 // ----------------------------------------------------------------------------
-bool UserConfigParams::logGUI () 
+bool UserConfigParams::logGUI ()
      { return (m_verbosity&LOG_GUI) == LOG_GUI;   }
 
 // ----------------------------------------------------------------------------
@@ -687,7 +780,7 @@ bool UserConfigParams::logAddons()
      { return (m_verbosity&LOG_ADDONS) == LOG_ADDONS;}
 
 // ----------------------------------------------------------------------------
-bool UserConfigParams::logFlyable() 
+bool UserConfigParams::logFlyable()
      { return (m_verbosity&LOG_FLYABLE) == LOG_FLYABLE;  }
 
 // ----------------------------------------------------------------------------
