@@ -1,7 +1,7 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
-//  Copyright (C) 2006 SuperTuxKart-Team, Steve Baker
+//  Copyright (C) 2004-2013 Steve Baker <sjbaker1@airmail.net>
+//  Copyright (C) 2006-2013 SuperTuxKart-Team, Steve Baker
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -28,42 +28,48 @@
 #include "audio/music_manager.hpp"
 #include "config/user_config.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/rain.hpp"
 #include "io/xml_node.hpp"
 #include "karts/abstract_kart.hpp"
 #include "karts/explosion_animation.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/skidding.hpp"
 #include "modes/world.hpp"
+#include "physics/btKart.hpp"
 #include "race/race_manager.hpp"
 #include "tracks/track.hpp"
 #include "utils/aligned_array.hpp"
 #include "utils/constants.hpp"
 
-#include <ICameraSceneNode.h>
-#include <ISceneManager.h>
+#include "ICameraSceneNode.h"
+#include "ISceneManager.h"
 
 AlignedArray<Camera::EndCameraInformation> Camera::m_end_cameras;
+std::vector<Camera*>                       Camera::m_all_cameras;
 
-Camera::Camera(int camera_index, AbstractKart* kart)
+// ============================================================================
+Camera::Camera(int camera_index, AbstractKart* kart) : m_kart(NULL)
 {
-    m_mode     = CM_NORMAL;
-    m_index    = camera_index;
-    m_camera   = irr_driver->addCameraSceneNode();
-    
+    m_mode          = CM_NORMAL;
+    m_index         = camera_index;
+    m_rain          = NULL;
+    m_original_kart = kart;
+    m_camera        = irr_driver->addCameraSceneNode();
+
 #ifdef DEBUG
     m_camera->setName(core::stringc("Camera for ") + kart->getKartProperties()->getName());
 #endif
-    
+
     setupCamera();
     m_distance = kart->getKartProperties()->getCameraDistance();
-    m_kart     = kart;
+    setKart(kart);
     m_ambient_light = World::getWorld()->getTrack()->getDefaultAmbientColor();
 
     // TODO: Put these values into a config file
     //       Global or per split screen zone?
-    //       Either global or per user (for instance, some users may not like 
-    //       the extra camera rotation so they could set m_rotation_range to 
-    //       zero to disable it for themselves). 
+    //       Either global or per user (for instance, some users may not like
+    //       the extra camera rotation so they could set m_rotation_range to
+    //       zero to disable it for themselves).
     m_position_speed = 8.0f;
     m_target_speed   = 10.0f;
     m_rotation_range = 0.4f;
@@ -79,10 +85,11 @@ Camera::Camera(int camera_index, AbstractKart* kart)
 }   // Camera
 
 // ----------------------------------------------------------------------------
-/** Removes the camera scene node from the scene. 
+/** Removes the camera scene node from the scene.
  */
 Camera::~Camera()
 {
+    if(m_rain) delete m_rain;
     irr_driver->removeCameraSceneNode(m_camera);
 }   // ~Camera
 
@@ -90,12 +97,18 @@ Camera::~Camera()
 /** Changes the owner of this camera to the new kart.
  *  \param new_kart The new kart to use this camera.
  */
-void Camera::changeOwner(AbstractKart *new_kart)
+void Camera::setKart(AbstractKart *new_kart)
 {
-    m_kart->setCamera(NULL);
     m_kart = new_kart;
-    new_kart->setCamera(this);
-}   // changeOwner
+#ifdef DEBUG
+    if(new_kart)
+    {
+        std::string name = new_kart->getIdent()+"'s camera";
+        getCameraSceneNode()->setName(name.c_str() );
+    }
+#endif
+
+}   // setKart
 
 //-----------------------------------------------------------------------------
 /** This function clears all end camera data structure. This is necessary
@@ -119,7 +132,7 @@ void Camera::readEndCamera(const XMLNode &root)
     for(unsigned int i=0; i<root.getNumNodes(); i++)
     {
         unsigned int index = i;
-        // In reverse mode, reverse the order in which the 
+        // In reverse mode, reverse the order in which the
         // end cameras are read.
         if(QuadGraph::get()->isReverse())
             index = root.getNumNodes() - 1 - i;
@@ -139,16 +152,16 @@ void Camera::setupCamera()
     m_aspect = (float)(UserConfigParams::m_width)/UserConfigParams::m_height;
     switch(race_manager->getNumLocalPlayers())
     {
-    case 1: m_viewport = core::recti(0, 0, 
-                                     UserConfigParams::m_width, 
+    case 1: m_viewport = core::recti(0, 0,
+                                     UserConfigParams::m_width,
                                      UserConfigParams::m_height);
             m_scaling  = core::vector2df(1.0f, 1.0f);
             m_fov      = DEGREE_TO_RAD*75.0f;
             break;
-    case 2: m_viewport = core::recti(0, 
-                                     m_index==0 ? 0 
+    case 2: m_viewport = core::recti(0,
+                                     m_index==0 ? 0
                                                 : UserConfigParams::m_height>>1,
-                                     UserConfigParams::m_width, 
+                                     UserConfigParams::m_width,
                                      m_index==0 ? UserConfigParams::m_height>>1
                                                 : UserConfigParams::m_height);
             m_scaling  = core::vector2df(1.0f, 0.5f);
@@ -159,19 +172,19 @@ void Camera::setupCamera()
             /*
             if(m_index<2)
             {
-                m_viewport = core::recti(m_index==0 ? 0 
+                m_viewport = core::recti(m_index==0 ? 0
                                                     : UserConfigParams::m_width>>1,
-                                         0, 
-                                         m_index==0 ? UserConfigParams::m_width>>1 
-                                                    : UserConfigParams::m_width, 
+                                         0,
+                                         m_index==0 ? UserConfigParams::m_width>>1
+                                                    : UserConfigParams::m_width,
                                          UserConfigParams::m_height>>1);
                 m_scaling  = core::vector2df(0.5f, 0.5f);
                 m_fov      = DEGREE_TO_RAD*50.0f;
             }
             else
             {
-                m_viewport = core::recti(0, UserConfigParams::m_height>>1, 
-                                         UserConfigParams::m_width, 
+                m_viewport = core::recti(0, UserConfigParams::m_height>>1,
+                                         UserConfigParams::m_width,
                                          UserConfigParams::m_height);
                 m_scaling  = core::vector2df(1.0f, 0.5f);
                 m_fov      = DEGREE_TO_RAD*65.0f;
@@ -193,8 +206,8 @@ void Camera::setupCamera()
             if(UserConfigParams::logMisc())
                 fprintf(stderr, "Incorrect number of players: '%d' - assuming 1.\n",
                         race_manager->getNumLocalPlayers());
-            m_viewport = core::recti(0, 0, 
-                                     UserConfigParams::m_width, 
+            m_viewport = core::recti(0, 0,
+                                     UserConfigParams::m_width,
                                      UserConfigParams::m_height);
             m_scaling  = core::vector2df(1.0f, 1.0f);
             m_fov      = DEGREE_TO_RAD*75.0f;
@@ -203,7 +216,14 @@ void Camera::setupCamera()
     m_camera->setFOV(m_fov);
     m_camera->setAspectRatio(m_aspect);
     m_camera->setFarValue(World::getWorld()->getTrack()->getCameraFar());
-    }   // setupCamera
+
+    if (UserConfigParams::m_weather_effects &&
+        World::getWorld()->getTrack()->getWeatherType() == WEATHER_RAIN)
+    {
+        m_rain = new Rain(this, NULL);
+    }
+
+}   // setupCamera
 
 // ----------------------------------------------------------------------------
 /** Sets the mode of the camera.
@@ -211,8 +231,8 @@ void Camera::setupCamera()
  */
 void Camera::setMode(Mode mode)
 {
-    // If we switch from reverse view, move the camera immediately to the 
-    // correct position. 
+    // If we switch from reverse view, move the camera immediately to the
+    // correct position.
     if(m_mode==CM_REVERSE && mode==CM_NORMAL)
     {
         Vec3 wanted_position, wanted_target;
@@ -252,6 +272,7 @@ Camera::Mode Camera::getMode()
 */
 void Camera::reset()
 {
+    m_kart = m_original_kart;
     setMode(CM_NORMAL);
     setInitialTransform();
 }   // reset
@@ -301,7 +322,7 @@ void Camera::smoothMoveCamera(float dt, const Vec3 &wanted_position,
     assert(!isnan(m_camera->getPosition().X));
     assert(!isnan(m_camera->getPosition().Y));
     assert(!isnan(m_camera->getPosition().Z));
-    
+
     if (race_manager->getNumLocalPlayers() < 2)
     {
         sfx_manager->positionListener(current_position,  current_target - current_position);
@@ -321,15 +342,15 @@ void Camera::computeNormalCameraPosition(Vec3 *wanted_position,
 {
     *wanted_target = m_kart->getXYZ();
     wanted_target->setY(wanted_target->getY()+ 0.75f);
-    
-    // This first line moves the camera around behind the kart, pointing it 
+
+    // This first line moves the camera around behind the kart, pointing it
     // towards where the kart is turning (and turning even more while skidding).
     // The skidding effect is dampened.
-    float steering = m_kart->getSteerPercent() 
+    float steering = m_kart->getSteerPercent()
                    * (1.0f + (m_kart->getSkidding()->getSkidFactor() - 1.0f)
                              /2.3f );
     // quadratically to dampen small variations (but keep sign)
-    float dampened_steer =  fabsf(steering) * steering; 
+    float dampened_steer =  fabsf(steering) * steering;
 
     float tan_up = tan(m_kart->getKartProperties()->getCameraForwardUpAngle());
     Vec3 relative_position(-m_distance*m_rotation_range*dampened_steer*0.5f,
@@ -346,7 +367,7 @@ void Camera::computeNormalCameraPosition(Vec3 *wanted_position,
  *  \param sideway Sideway movement of the camera.
  *  \param distance Distance from kart.
  */
-void Camera::getCameraSettings(float *above_kart, float *cam_angle, 
+void Camera::getCameraSettings(float *above_kart, float *cam_angle,
                                float *sideway, float *distance,
                                bool *smoothing)
 {
@@ -367,13 +388,21 @@ void Camera::getCameraSettings(float *above_kart, float *cam_angle,
         // Fall through to falling mode.
     case CM_FALLING:
         {
-            *above_kart    = 0.75f;
-            float steering = m_kart->getSteerPercent() 
+            if(UserConfigParams::m_camera_debug==2)
+            {
+                *above_kart = 0;
+                *cam_angle  = 0;
+            }
+            else
+            {
+                *above_kart    = 0.75f;
+                *cam_angle     = kp->getCameraForwardUpAngle();
+            }
+            float steering = m_kart->getSteerPercent()
                            * (1.0f + (m_kart->getSkidding()->getSkidFactor()
                                       - 1.0f)/2.3f );
             // quadratically to dampen small variations (but keep sign)
-            float dampened_steer = fabsf(steering) * steering; 
-            *cam_angle           = kp->getCameraForwardUpAngle();
+            float dampened_steer = fabsf(steering) * steering;
             *sideway             = -m_rotation_range*dampened_steer*0.5f;
             *distance            = -m_distance;
             *smoothing           = true;
@@ -392,7 +421,7 @@ void Camera::getCameraSettings(float *above_kart, float *cam_angle,
         {
             *above_kart = 0.75f;
             *cam_angle  = 20.0f*DEGREE_TO_RAD;
-            *sideway    = m_rotation_range 
+            *sideway    = m_rotation_range
                         * m_kart->getSteerPercent()
                         * m_kart->getSkidding()->getSkidFactor();
             *distance   = -0.5f*m_distance;
@@ -422,36 +451,33 @@ void Camera::getCameraSettings(float *above_kart, float *cam_angle,
  */
 void Camera::update(float dt)
 {
+    float above_kart, cam_angle, side_way, distance;
+    bool  smoothing;
+
     // The following settings give a debug camera which shows the track from
     // high above the kart straight down.
-    if(UserConfigParams::m_camera_debug)
+    if (UserConfigParams::m_camera_debug==1)
     {
         core::vector3df xyz = m_kart->getXYZ().toIrrVector();
         m_camera->setTarget(xyz);
         xyz.Y = xyz.Y+55;
         xyz.Z -= 5.0f;
         m_camera->setPosition(xyz);
-        // To view inside tunnels (FIXME 27>15 why??? makes no sense 
+        // To view inside tunnels (FIXME 27>15 why??? makes no sense
         // - the kart should not be visible, but it works)
         m_camera->setNearValue(27.0);
-        return;
     }
 
-    if(m_mode==CM_FINAL)
+    else if (m_mode==CM_FINAL)
     {
         handleEndCamera(dt);
-        return;
     }
 
-    float above_kart, cam_angle, side_way, distance;
-    bool  smoothing;
-    getCameraSettings(&above_kart, &cam_angle, &side_way, &distance,
-                      &smoothing);
-
     // If an explosion is happening, stop moving the camera,
     // but keep it target on the kart.
-    if(dynamic_cast<ExplosionAnimation*>(m_kart->getKartAnimation()))
+    else if (dynamic_cast<ExplosionAnimation*>(m_kart->getKartAnimation()))
     {
+        getCameraSettings(&above_kart, &cam_angle, &side_way, &distance, &smoothing);
         // The camera target needs to be 'smooth moved', otherwise
         // there will be a noticable jump in the first frame
 
@@ -465,10 +491,19 @@ void Camera::update(float dt)
         current_target += ((wanted_target-current_target)*m_target_speed)*dt;
 
         m_camera->setTarget(current_target);
-        return;
     }
 
-    positionCamera(dt, above_kart, cam_angle, side_way, distance, smoothing);
+    else
+    {
+        getCameraSettings(&above_kart, &cam_angle, &side_way, &distance, &smoothing);
+        positionCamera(dt, above_kart, cam_angle, side_way, distance, smoothing);
+    }
+
+    if (UserConfigParams::m_graphical_effects && m_rain)
+    {
+        m_rain->setPosition( getCameraSceneNode()->getPosition() );
+        m_rain->update(dt);
+    }  // UserConfigParams::m_graphical_effects
 }   // update
 
 // ----------------------------------------------------------------------------
@@ -478,20 +513,21 @@ void Camera::update(float dt)
  *  \param sideway Sideway movement of the camera.
  *  \param distance Distance from kart.
 */
-void Camera::positionCamera(float dt, float above_kart, float cam_angle, 
+void Camera::positionCamera(float dt, float above_kart, float cam_angle,
                            float side_way, float distance, float smoothing)
 {
     Vec3 wanted_position;
     Vec3 wanted_target = m_kart->getXYZ();
-
-    wanted_target.setY(wanted_target.getY()+above_kart);
-
+    if(UserConfigParams::m_camera_debug==2)
+        wanted_target.setY(m_kart->getVehicle()->getWheelInfo(2).m_raycastInfo.m_contactPointWS.getY());
+    else
+        wanted_target.setY(wanted_target.getY()+above_kart);
     float tan_up = tan(cam_angle);
     Vec3 relative_position(side_way,
                            fabsf(distance)*tan_up+above_kart,
                            distance);
     btTransform t=m_kart->getTrans();
-    if(stk_config->m_camera_follow_skid && 
+    if(stk_config->m_camera_follow_skid &&
         m_kart->getSkidding()->getVisualSkidRotation()!=0)
     {
         // If the camera should follow the graphical skid, add the
@@ -510,10 +546,10 @@ void Camera::positionCamera(float dt, float above_kart, float cam_angle,
         if (m_mode!=CM_FALLING)
             m_camera->setPosition(wanted_position.toIrrVector());
         m_camera->setTarget(wanted_target.toIrrVector());
-        
+
         if (race_manager->getNumLocalPlayers() < 2)
         {
-            sfx_manager->positionListener(m_camera->getPosition(),  
+            sfx_manager->positionListener(m_camera->getPosition(),
                                       wanted_target - m_camera->getPosition());
         }
     }
@@ -522,7 +558,7 @@ void Camera::positionCamera(float dt, float above_kart, float cam_angle,
 
 // ----------------------------------------------------------------------------
 /** This function handles the end camera. It adjusts the camera position
- *  according to the current camera type, and checks if a switch to the 
+ *  according to the current camera type, and checks if a switch to the
  *  next camera should be made.
  *  \param dt Time step size.
 */
@@ -530,7 +566,7 @@ void Camera::handleEndCamera(float dt)
 {
     // First test if the kart is close enough to the next end camera, and
     // if so activate it.
-    if( m_end_cameras.size()>0 && 
+    if( m_end_cameras.size()>0 &&
         m_end_cameras[m_next_end_camera].isReached(m_kart->getXYZ()))
     {
         m_current_end_camera = m_next_end_camera;
@@ -543,11 +579,11 @@ void Camera::handleEndCamera(float dt)
         }
         m_camera->setFOV(m_fov);
         m_next_end_camera++;
-        if(m_next_end_camera>=(unsigned)m_end_cameras.size()) 
+        if(m_next_end_camera>=(unsigned)m_end_cameras.size())
             m_next_end_camera = 0;
     }
 
-    EndCameraInformation::EndCameraType info 
+    EndCameraInformation::EndCameraType info
         = m_end_cameras.size()==0 ? EndCameraInformation::EC_AHEAD_OF_KART
                                   : m_end_cameras[m_current_end_camera].m_type;
 
@@ -577,8 +613,8 @@ void Camera::handleEndCamera(float dt)
             const KartProperties *kp=m_kart->getKartProperties();
             float cam_angle  = kp->getCameraBackwardUpAngle();
 
-            positionCamera(dt, /*above_kart*/0.75f, 
-                           cam_angle, /*side_way*/0, 
+            positionCamera(dt, /*above_kart*/0.75f,
+                           cam_angle, /*side_way*/0,
                            2.0f*m_distance, /*smoothing*/false);
             break;
         }
