@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2011  Joerg Henrichs, Marianne Gagnon
+//  Copyright (C) 2011-2013  Joerg Henrichs, Marianne Gagnon
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -32,15 +32,16 @@
 #include <IParticleSystemSceneNode.h>
 #include <IParticleBoxEmitter.h>
 #include <ISceneManager.h>
+#include <algorithm>
 
 class FadeAwayAffector : public scene::IParticleAffector
 {
     /** (Squared) distance from camera at which a particle started being faded out */
     float m_start_fading;
-    
+
     /** (Squared) distance from camera at which a particle is completely faded out */
     float m_end_fading;
-    
+
 public:
     FadeAwayAffector(float start, float end)
     {
@@ -48,17 +49,17 @@ public:
         m_end_fading = end;
         assert(m_end_fading >= m_start_fading);
     }   // FadeAwayAffector
-    
+
     // ------------------------------------------------------------------------
-    
+
     virtual void affect(u32 now, scene::SParticle* particlearray, u32 count)
     {
-        scene::ICameraSceneNode* curr_cam = 
+        scene::ICameraSceneNode* curr_cam =
             irr_driver->getSceneManager()->getActiveCamera();
         const core::vector3df& cam_pos = curr_cam->getPosition();
-        
+
         // printf("Affect called with now=%u, camera=%s\n", now, curr_cam->getName());
-        
+
         for (unsigned int n=0; n<count; n++)
         {
             scene::SParticle& curr = particlearray[n];
@@ -67,7 +68,7 @@ public:
             const float y = diff.Y;
             const float z = diff.Z;
             const float distance_squared = x*x + y*y + z*z;
-            
+
             if (distance_squared < m_start_fading)
             {
                 curr.color.setAlpha(255);
@@ -78,20 +79,20 @@ public:
             }
             else
             {
-                curr.color.setAlpha((int)((distance_squared - m_start_fading) 
+                curr.color.setAlpha((int)((distance_squared - m_start_fading)
                                         / (m_end_fading - m_start_fading)));
             }
         }   // for n<count
     }   // affect
 
     // ------------------------------------------------------------------------
-    
+
     virtual scene::E_PARTICLE_AFFECTOR_TYPE getType() const
     {
         // FIXME: this method seems to make sense only for built-in affectors
         return scene::EPAT_FADE_OUT;
     }
-    
+
 };   // FadeAwayAffector
 
 
@@ -102,14 +103,14 @@ class HeightMapCollisionAffector : public scene::IParticleAffector
     std::vector< std::vector<float> > m_height_map;
     Track* m_track;
     bool m_first_time;
-    
+
 public:
     HeightMapCollisionAffector(Track* t) : m_height_map(t->buildHeightMap())
     {
         m_track = t;
         m_first_time = true;
     }
-    
+
     virtual void affect(u32 now, scene::SParticle* particlearray, u32 count)
     {
         const Vec3* aabb_min;
@@ -119,7 +120,7 @@ public:
         float track_z = aabb_min->getZ();
         const float track_x_len = aabb_max->getX() - aabb_min->getX();
         const float track_z_len = aabb_max->getZ() - aabb_min->getZ();
-        
+
         for (unsigned int n=0; n<count; n++)
         {
             scene::SParticle& curr = particlearray[n];
@@ -135,7 +136,7 @@ public:
             core::vector3df lp = curr.pos;
             core::vector3df lp2 = curr.pos;
             lp2.Y = m_height_map[i][j] + 0.02f;
-            
+
             irr_driver->getVideoDriver()->draw3DLine(lp, lp2, video::SColor(255,255,0,0));
             core::vector3df lp3 = lp2;
             lp3.X += 0.1f;
@@ -146,10 +147,10 @@ public:
             lp2.Z -= 0.1f;
             irr_driver->getVideoDriver()->draw3DBox(core::aabbox3d< f32 >(lp2, lp3), video::SColor(255,255,0,0));
             */
-            
+
             if (m_first_time)
             {
-                curr.pos.Y = m_height_map[i][j] 
+                curr.pos.Y = m_height_map[i][j]
                            + (curr.pos.Y - m_height_map[i][j])
                                 *((rand()%500)/500.0f);
             }
@@ -162,10 +163,10 @@ public:
                 }
             }
         }
-        
+
         if (m_first_time) m_first_time = false;
     }
-    
+
     virtual scene::E_PARTICLE_AFFECTOR_TYPE getType() const
     {
         // FIXME: this method seems to make sense only for built-in affectors
@@ -176,9 +177,9 @@ public:
 
 // ============================================================================
 
-ParticleEmitter::ParticleEmitter(const ParticleKind* type, 
+ParticleEmitter::ParticleEmitter(const ParticleKind* type,
                                  const Vec3 &position,
-                                 scene::ISceneNode* parent) 
+                                 scene::ISceneNode* parent)
                : m_position(position)
 {
     assert(type != NULL);
@@ -188,10 +189,10 @@ ParticleEmitter::ParticleEmitter(const ParticleKind* type,
     m_particle_type       = NULL;
     m_parent              = parent;
     m_emission_decay_rate = 0;
-    
+
     setParticleType(type);
     assert(m_node != NULL);
-    
+
 }   // KartParticleSystem
 
 //-----------------------------------------------------------------------------
@@ -203,7 +204,7 @@ ParticleEmitter::~ParticleEmitter()
     if (m_node != NULL)
         irr_driver->removeNode(m_node);
     m_emitter->drop();
-    
+
     m_magic_number = 0xDEADBEEF;
 }   // ~ParticleEmitter
 
@@ -212,26 +213,26 @@ ParticleEmitter::~ParticleEmitter()
 void ParticleEmitter::update(float dt)
 {
     assert(m_magic_number == 0x58781325);
-    
+
     // No particles to emit, nothing to do
     if (m_emitter->getMinParticlesPerSecond() == 0) return;
-    
+
     // the emission direction does not automatically follow the orientation of
     // the node so fix that manually...
-    core::matrix4 	transform = m_node->getAbsoluteTransformation();
+    core::matrix4   transform = m_node->getAbsoluteTransformation();
     core::vector3df velocity(m_particle_type->getVelocityX(),
                              m_particle_type->getVelocityY(),
                              m_particle_type->getVelocityZ());
-    
+
     transform.rotateVect(velocity);
     m_emitter->setDirection(velocity);
-    
+
     if (m_emission_decay_rate > 0)
     {
         m_max_rate = m_min_rate = std::max(0.0f, (m_min_rate - m_emission_decay_rate*dt));
         setCreationRateAbsolute(m_min_rate);
     }
-    
+
     // There seems to be no way to randomise the velocity for particles,
     // so we have to do this manually, by changing the default velocity.
     // Irrlicht expects velocity (called 'direction') in m/ms!!
@@ -247,7 +248,7 @@ void ParticleEmitter::update(float dt)
 }   // update
 
 //-----------------------------------------------------------------------------
-/** Sets the creation rate as a relative fraction between minimum (f=0) and 
+/** Sets the creation rate as a relative fraction between minimum (f=0) and
  *  maximum (f=1) of the creation rates defined in the particle kind.
  *  \param fraction Fraction to use.
  */
@@ -268,10 +269,10 @@ void ParticleEmitter::setCreationRateAbsolute(float f)
 {
     m_emitter->setMinParticlesPerSecond(int(f));
     m_emitter->setMaxParticlesPerSecond(int(f));
-    
+
     m_min_rate = f;
     m_max_rate = f;
- 
+
     // FIXME: to work around irrlicht bug, when an emitter is paused by setting the rate
     //        to 0 results in a massive emission when enabling it back. In irrlicht 1.8
     //        the node has a method called "clearParticles" that should be cleaner than this
@@ -312,7 +313,7 @@ void ParticleEmitter::clearParticles()
 //-----------------------------------------------------------------------------
 
 void ParticleEmitter::setParticleType(const ParticleKind* type)
-{   
+{
     assert(m_magic_number == 0x58781325);
     bool is_new_type = (m_particle_type != type);
     if (is_new_type)
@@ -327,26 +328,26 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
         {
             m_node = irr_driver->addParticleNode();
         }
-        
+
         if (m_parent != NULL)
         {
             m_node->setParent(m_parent);
         }
-        
+
         m_particle_type = type;
     }
-    
+
     m_emission_decay_rate = type->getEmissionDecayRate();
-    
+
     Material* material    = type->getMaterial();
     const float minSize   = type->getMinSize();
     const float maxSize   = type->getMaxSize();
     const int lifeTimeMin = type->getMinLifetime();
     const int lifeTimeMax = type->getMaxLifetime();
-        
+
     assert(maxSize >= minSize);
     assert(lifeTimeMax >= lifeTimeMin);
-    
+
 #ifdef DEBUG
     if (material != NULL)
     {
@@ -354,41 +355,43 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
         assert(tex != NULL);
         const io::SNamedPath& name = tex->getName();
         const io::path& tpath = name.getPath();
-        
+
         std::string debug_name = std::string("particles(") + tpath.c_str() + ")";
         m_node->setName(debug_name.c_str());
     }
 #endif
     m_min_rate = (float)type->getMinRate();
     m_max_rate = (float)type->getMaxRate();
-    
+
     if (is_new_type)
     {
         video::SMaterial& mat0 = m_node->getMaterial(0);
-        
+
         m_node->setPosition(m_position.toIrrVector());
-        
+
         if (material != NULL)
         {
             assert(material->getTexture() != NULL);
             material->setMaterialProperties(&mat0, NULL);
             m_node->setMaterialTexture(0, material->getTexture());
-        
+
             mat0.ZWriteEnable = !material->isTransparent(); // disable z-buffer writes if material is transparent
         }
         else
         {
-            m_node->setMaterialTexture(0, irr_driver->getTexture((file_manager->getDataDir() + "/gui/main_help.png").c_str()));
+            m_node->setMaterialTexture(0, irr_driver->getTexture((file_manager->getDataDir() + "gui/main_help.png").c_str()));
         }
-        
+
+        // velocity in m/ms
+        core::vector3df velocity(m_particle_type->getVelocityX(),
+                                 m_particle_type->getVelocityY(),
+                                 m_particle_type->getVelocityZ());
 
         switch (type->getShape())
         {
             case EMITTER_POINT:
             {
-                m_emitter = m_node->createPointEmitter(core::vector3df(m_particle_type->getVelocityX(),
-                                                                       m_particle_type->getVelocityY(),
-                                                                       m_particle_type->getVelocityZ()),   // velocity in m/ms
+                m_emitter = m_node->createPointEmitter(velocity,
                                                        type->getMinRate(),  type->getMaxRate(),
                                                        type->getMinColor(), type->getMaxColor(),
                                                        lifeTimeMin, lifeTimeMax,
@@ -396,23 +399,21 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
                                                        );
                 break;
             }
-            
+
             case EMITTER_BOX:
             {
                 const float box_size_x = type->getBoxSizeX()/2.0f;
                 const float box_size_y = type->getBoxSizeY()/2.0f;
-                
+
                 m_emitter = m_node->createBoxEmitter(core::aabbox3df(-box_size_x, -box_size_y, -0.6f,
                                                                      box_size_x,  box_size_y,  -0.6f - type->getBoxSizeZ()),
-                                                     core::vector3df(m_particle_type->getVelocityX(),
-                                                                     m_particle_type->getVelocityY(),
-                                                                     m_particle_type->getVelocityZ()),   // velocity in m/ms
+                                                     velocity,
                                                      type->getMinRate(),  type->getMaxRate(),
                                                      type->getMinColor(), type->getMaxColor(),
                                                      lifeTimeMin, lifeTimeMax,
-                                                     m_particle_type->getAngleSpread() /* angle */
+                                                     m_particle_type->getAngleSpread()
                                                      );
-                
+
     #if VISUALIZE_BOX_EMITTER
                 if (m_parent != NULL)
                 {
@@ -435,6 +436,18 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
     #endif
                 break;
             }
+            case EMITTER_SPHERE:
+            {
+                m_emitter = m_node->createSphereEmitter(core::vector3df(0.0f,0.0f,0.0f) /* center */,
+                                                        m_particle_type->getSphereRadius(),
+                                                        velocity,
+                                                        type->getMinRate(),  type->getMaxRate(),
+                                                        type->getMinColor(), type->getMaxColor(),
+                                                        lifeTimeMin, lifeTimeMax,
+                                                        m_particle_type->getAngleSpread()
+                                                 );
+                break;
+            }
             default:
             {
                 fprintf(stderr, "[ParticleEmitter] Unknown shape\n");
@@ -447,19 +460,19 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
         m_emitter->setMinParticlesPerSecond(int(m_min_rate));
         m_emitter->setMaxParticlesPerSecond(int(m_max_rate));
     }
-    
+
     m_emitter->setMinStartSize(core::dimension2df(minSize, minSize));
     m_emitter->setMaxStartSize(core::dimension2df(maxSize, maxSize));
-    
+
     if (is_new_type)
     {
         m_node->setEmitter(m_emitter); // this grabs the emitter
-        
+
         scene::IParticleFadeOutAffector *af = m_node->createFadeOutParticleAffector(video::SColor(0, 255, 255, 255),
                                                                                     type->getFadeoutTime());
         m_node->addAffector(af);
         af->drop();
-        
+
         if (type->getGravityStrength() != 0)
         {
             scene::IParticleGravityAffector *gaf = m_node->createGravityAffector(core::vector3df(00.0f, type->getGravityStrength(), 0.0f),
@@ -467,7 +480,7 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
             m_node->addAffector(gaf);
             gaf->drop();
         }
-        
+
         const float fas = type->getFadeAwayStart();
         const float fae = type->getFadeAwayEnd();
         if (fas > 0.0f && fae > 0.0f)
@@ -476,6 +489,16 @@ void ParticleEmitter::setParticleType(const ParticleKind* type)
             m_node->addAffector(faa);
             faa->drop();
         }
+        
+        if (type->hasScaleAffector())
+        {
+            core::dimension2df factor = core::dimension2df(type->getScaleAffectorFactorX(),
+                                                           type->getScaleAffectorFactorY());
+            scene::IParticleAffector* scale_affector =
+                m_node->createScaleParticleAffector(factor);
+            m_node->addAffector(scale_affector);
+            scale_affector->drop();
+        }
     }
 }   // setParticleType
 
@@ -493,14 +516,14 @@ void ParticleEmitter::addHeightMapAffector(Track* t)
 void ParticleEmitter::resizeBox(float size)
 {
     scene::IParticleBoxEmitter* emitter = (scene::IParticleBoxEmitter*)m_emitter;
-    
+
     const float box_size_x = m_particle_type->getBoxSizeX()/2.0f;
     const float box_size_y = m_particle_type->getBoxSizeY()/2.0f;
-    
-    
+
+
     emitter->setBox( core::aabbox3df(-box_size_x, -box_size_y, -0.6f,
                                      box_size_x,  box_size_y,  -0.6f - size) );
-    
+
 #if VISUALIZE_BOX_EMITTER
     if (m_parent != NULL)
     {
