@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2010  Joerg Henrichs
+//  Copyright (C) 2010-2013  Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -50,18 +50,18 @@ SlipStream::SlipStream(AbstractKart* kart) : MovingTexture(0, 0), m_kart(kart)
     m.setFlag(video::EMF_COLOR_MATERIAL, true);
 
     m.ColorMaterial = video::ECM_DIFFUSE_AND_AMBIENT;
-    
+
     m.MaterialType = video::EMT_TRANSPARENT_ADD_COLOR;
 
     createMesh(m);
     m_node = irr_driver->addMesh(m_mesh);
     m_mesh->drop();
-    
+
 #ifdef DEBUG
     std::string debug_name = m_kart->getIdent()+" (slip-stream)";
     m_node->setName(debug_name.c_str());
 #endif
-    m_node->setPosition(core::vector3df(0, 
+    m_node->setPosition(core::vector3df(0,
                                         0*0.25f+2.5,
                                         m_kart->getKartLength()) );
     m_node->setVisible(false);
@@ -90,7 +90,7 @@ SlipStream::SlipStream(AbstractKart* kart) : MovingTexture(0, 0), m_kart(kart)
         m_debug_mesh = irr_driver->createQuadMesh(&material, true);
         scene::IMeshBuffer *buffer = m_debug_mesh->getMeshBuffer(0);
         assert(buffer->getVertexType()==video::EVT_STANDARD);
-        irr::video::S3DVertex* vertices 
+        irr::video::S3DVertex* vertices
             = (video::S3DVertex*)buffer->getVertices();
         video::SColor red(128, 255, 0, 0);
         for(unsigned int i=0; i<4; i++)
@@ -152,7 +152,7 @@ void SlipStream::createMesh(const video::SMaterial &material)
     float radius[] = {1.5f, 1.0f, 0.5f, 0.0f};
 
     // The distance of each of the circle from the kart. The number of
-    // entries in this array must be the same as the number of non-zero 
+    // entries in this array must be the same as the number of non-zero
     // entries in the radius[] array above. No 'end of list' entry required.
     // Note also that in order to avoid a 'bent' in the texture the
     // difference between the distances must be linearly correlated to the
@@ -172,10 +172,12 @@ void SlipStream::createMesh(const video::SMaterial &material)
     // The alpha values for the rings, no 'end of list' entry required.
     int alphas[]     = {0, 255, 0};
 
-    // Loop through all given radius to determine the number 
+    // Loop through all given radius to determine the number
     // of segments to create.
     unsigned int num_circles=0;
-    while(radius[num_circles]) num_circles++;
+    while(radius[num_circles]>0.0f) num_circles++;
+
+    assert(num_circles > 0);
 
     // Length is distance of last circle to distance of first circle:
     m_length = distance[num_circles-1] - distance[0];
@@ -232,7 +234,7 @@ void SlipStream::createMesh(const video::SMaterial &material)
     buffer->drop();
     m_mesh = mesh;
 }   // createMesh
- 
+
 //-----------------------------------------------------------------------------
 /** Sets the animation intensity (or speed).
  *  \param f Intensity: 0 = no slip stream,
@@ -252,7 +254,7 @@ void SlipStream::setIntensity(float f, const AbstractKart *kart)
     core::vector3df my_pos = m_kart->getNode()->getPosition();
     my_pos.Y = m_kart->getHoT()+above_terrain;
     m_node->setPosition(my_pos);
-    
+
     core::vector3df other_pos = kart->getNode()->getPosition();
     other_pos.Y = kart->getHoT()+above_terrain;
     core::vector3df diff =   other_pos - my_pos;
@@ -264,11 +266,11 @@ void SlipStream::setIntensity(float f, const AbstractKart *kart)
     // For real testing in game: this needs some tuning!
     m_node->setVisible(f!=0);
     MovingTexture::setSpeed(f, 0);
-    
+
 
     int c = (int)(f*255);
     if (c > 255) c = 255;
-        
+
     const unsigned int bcount = m_node->getMesh()->getMeshBufferCount();
     for (unsigned int b=0; b<bcount; b++)
     {
@@ -300,7 +302,7 @@ bool SlipStream::isSlipstreamReady() const
 }   // isSlipstreamReady
 
 //-----------------------------------------------------------------------------
-/** Returns the additional force being applied to the kart because of 
+/** Returns the additional force being applied to the kart because of
  *  slipstreaming.
  */
 void SlipStream::updateSlipstreamPower()
@@ -331,7 +333,7 @@ void SlipStream::setDebugColor(const video::SColor &color)
 {
     if(!UserConfigParams::m_slipstream_debug) return;
     scene::IMeshBuffer *buffer = m_debug_mesh->getMeshBuffer(0);
-    irr::video::S3DVertex* vertices = 
+    irr::video::S3DVertex* vertices =
         (video::S3DVertex*)buffer->getVertices();
     for(unsigned int i=0; i<4; i++)
         vertices[i].Color=color;
@@ -350,9 +352,9 @@ void SlipStream::update(float dt)
     MovingTexture::update(dt);
 
     // Update this karts slipstream quad (even for low level AI which don't
-    // use slipstream, since even then player karts can get slipstream, 
+    // use slipstream, since even then player karts can get slipstream,
     // and so have to compare with the modified slipstream quad.
-    m_slipstream_original_quad->transform(m_kart->getTrans(), 
+    m_slipstream_original_quad->transform(m_kart->getTrans(),
                                           m_slipstream_quad);
 
     if(m_slipstream_mode==SS_USE)
@@ -394,24 +396,24 @@ void SlipStream::update(float dt)
         m_target_kart= world->getKart(i);
         // Don't test for slipstream with itself, a kart that is being
         // rescued or exploding, or an eliminated kart
-        if(m_target_kart==m_kart               || 
+        if(m_target_kart==m_kart               ||
             m_target_kart->getKartAnimation()  ||
             m_target_kart->isEliminated()        ) continue;
 
-        float diff = fabsf(m_target_kart->getXYZ().getY() 
+        float diff = fabsf(m_target_kart->getXYZ().getY()
                            - m_kart->getXYZ().getY()      );
         // If the kart is 'on top' of this kart (e.g. up on a bridge),
         // don't consider it for slipstreaming.
 
         if(diff>6.0f) continue;
         // If the kart we are testing against is too slow, no need to test
-        // slipstreaming. Note: We compare the speed of the other kart 
-        // against the minimum slipstream speed kart of this kart - not 
-        // entirely sure if this makes sense, but it makes it easier to 
+        // slipstreaming. Note: We compare the speed of the other kart
+        // against the minimum slipstream speed kart of this kart - not
+        // entirely sure if this makes sense, but it makes it easier to
         // give karts different slipstream properties.
 #ifndef DISPLAY_SLIPSTREAM_WITH_0_SPEED_FOR_DEBUGGING
         if(m_target_kart->getSpeed() <
-            m_kart->getKartProperties()->getSlipstreamMinSpeed()) 
+            m_kart->getKartProperties()->getSlipstreamMinSpeed())
         {
             if(UserConfigParams::m_slipstream_debug &&
                 m_kart->getController()->isPlayerController())
@@ -425,10 +427,10 @@ void SlipStream::update(float dt)
         // slipstream length+0.5*kart_length()+0.5*target_kart_length
         // away from the other kart
         Vec3 delta = m_kart->getXYZ() - m_target_kart->getXYZ();
-        float l    = m_target_kart->getKartProperties()->getSlipstreamLength() 
+        float l    = m_target_kart->getKartProperties()->getSlipstreamLength()
                    + 0.5f*( m_target_kart->getKartLength()
                            +m_kart->getKartLength()        );
-        if(delta.length2_2d() > l*l) 
+        if(delta.length2_2d() > l*l)
         {
             if(UserConfigParams::m_slipstream_debug &&
                 m_kart->getController()->isPlayerController())
@@ -456,16 +458,16 @@ void SlipStream::update(float dt)
             m_target_kart->getSlipstream()
                          ->setDebugColor(video::SColor(255, 255, 0, 0));
 
-        if(isSlipstreamReady()) 
+        if(isSlipstreamReady())
         {
             // The first time slipstream is ready after collecting
             // and you are leaving the slipstream area, you get a
-            // zipper bonus. 
+            // zipper bonus.
             if(m_slipstream_mode==SS_COLLECT)
             {
                 m_slipstream_mode = SS_USE;
                 m_kart->handleZipper();
-                m_slipstream_time = 
+                m_slipstream_time =
                     m_kart->getKartProperties()->getSlipstreamCollectTime();
                 return;
             }
@@ -480,7 +482,7 @@ void SlipStream::update(float dt)
         m_kart->getController()->isPlayerController())
         m_target_kart->getSlipstream()->setDebugColor(video::SColor(255, 0, 255, 0));
     // Accumulate slipstream credits now
-    m_slipstream_time = m_slipstream_mode==SS_NONE ? dt 
+    m_slipstream_time = m_slipstream_mode==SS_NONE ? dt
                                                    : m_slipstream_time+dt;
     if(isSlipstreamReady())
         m_kart->setSlipstreamEffect(9.0f);
