@@ -1,5 +1,5 @@
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2010 Marianne Gagnon
+//  Copyright (C) 2010-2013 Marianne Gagnon
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -58,14 +58,10 @@ int atoi_p(const char* val)
 // ----------------------------------------------------------------------------
 
 bool LayoutManager::convertToCoord(std::string& x, int* absolute /* out */, int* percentage /* out */)
-{    
-    bool is_number;
-    int i;
-    std::istringstream myStream(x);
-    is_number = (myStream >> i)!=0;
-    
-    if(!is_number) return false;
-    
+{
+    int i = 0;
+    if (!StringUtils::fromString<int>(x, i /* out */)) return false;
+
     if( x[x.size()-1] == '%' ) // percentage
     {
         *percentage = i;
@@ -82,13 +78,13 @@ bool LayoutManager::convertToCoord(std::string& x, int* absolute /* out */, int*
 // ----------------------------------------------------------------------------
 
 void LayoutManager::readCoords(Widget* self)
-{    
+{
     // determine widget position and size if not already done by sizers
     std::string x      = self->m_properties[PROP_X];
     std::string y      = self->m_properties[PROP_Y];
     std::string width  = self->m_properties[PROP_WIDTH];
     std::string height = self->m_properties[PROP_HEIGHT];
-    
+
     /*
     // retrieve parent size (or screen size if none). Will be useful for layout
     // and especially for percentages.
@@ -111,7 +107,7 @@ void LayoutManager::readCoords(Widget* self)
         parent_y = parent->m_y;
     }
     */
-    
+
     // ---- try converting to number
     // x coord
     {
@@ -132,7 +128,7 @@ void LayoutManager::readCoords(Widget* self)
             }
         }
     }
-    
+
     // y coord
     {
         int abs_y = 0x7FFFFFFF, percent_y = 0x7FFFFFFF;
@@ -152,22 +148,22 @@ void LayoutManager::readCoords(Widget* self)
             }
         }
     }
-    
+
     // ---- if this widget has an icon, get icon size. this can helpful determine its optimal size
     int texture_w = -1, texture_h = -1;
-    
+
     if (self->m_properties[PROP_ICON].size() > 0)
     {
         ITexture* texture = irr_driver->getTexture((file_manager->getDataDir() + "/" +
                                                     self->m_properties[PROP_ICON]).c_str());
-        
+
         if (texture != NULL)
         {
             texture_w = texture->getSize().Width;
             texture_h = texture->getSize().Height;
         }
     }
-    
+
     // ---- if this widget has a label, get text size. this can helpful determine its optimal size
     int label_w = -1, label_h = -1;
     if (self->m_text.size() > 0)
@@ -184,10 +180,10 @@ void LayoutManager::readCoords(Widget* self)
         // User text height to guess checkbox size
         IGUIFont* font = (self->m_title_font ? GUIEngine::getTitleFont() : GUIEngine::getFont());
         core::dimension2d< u32 > dim = font->getDimension( L"X" );
-        label_h = dim.Height + self->getHeightNeededAroundLabel();   
+        label_h = dim.Height + self->getHeightNeededAroundLabel();
         label_w = label_h; // a checkbox is square
     }
-    
+
     if (label_h == -1)
     {
         if (self->getType() == WTYPE_TEXTBOX ||
@@ -202,7 +198,7 @@ void LayoutManager::readCoords(Widget* self)
             label_h = dim.Height + self->getHeightNeededAroundLabel();
         }
     }
-    
+
     // ---- if this widget has children, get their size size. this can helpful determine its optimal size
     if (self->m_properties[PROP_WIDTH] == "fit" || self->m_properties[PROP_HEIGHT] == "fit")
     {
@@ -211,7 +207,7 @@ void LayoutManager::readCoords(Widget* self)
 
         int child_max_width = -1, child_max_height = -1;
         int total_width = 0, total_height = 0;
-        
+
         for (int child=0; child<self->m_children.size(); child++)
         {
             if (self->m_children[child].m_absolute_w > -1)
@@ -228,7 +224,7 @@ void LayoutManager::readCoords(Widget* self)
                     }
                 }
             }
-            
+
             if (self->m_children[child].m_absolute_h > -1)
             {
                 if (is_vertical_row)
@@ -244,7 +240,7 @@ void LayoutManager::readCoords(Widget* self)
                 }
             }
         }
-        
+
         if (self->m_properties[PROP_WIDTH] == "fit")
         {
             self->m_absolute_w = (is_horizontal_row ? total_width : child_max_width);
@@ -254,7 +250,7 @@ void LayoutManager::readCoords(Widget* self)
             self->m_absolute_h = (is_vertical_row ? total_height : child_max_height);
         }
     }
-    
+
     // ---- read dimension
     // width
     {
@@ -268,7 +264,7 @@ void LayoutManager::readCoords(Widget* self)
         else if(texture_w > -1) self->m_absolute_w = texture_w;
         else if(label_w > -1)   self->m_absolute_w = label_w;
     }
-    
+
     // height
     {
         int abs_h = -1, percent_h = -1;
@@ -309,25 +305,37 @@ void LayoutManager::applyCoords(Widget* self, AbstractTopLevelContainer* topLeve
         parent_y = parent->m_y;
     }
     
+    if (parent != NULL && parent->getType() == WTYPE_DIV && parent->m_show_bounding_box)
+    {
+        int padding = 15;
+        if (parent->m_properties[PROP_DIV_PADDING].length() > 0)
+            padding = atoi(parent->m_properties[PROP_DIV_PADDING].c_str());
+            
+        parent_x += padding;
+        parent_y += padding;
+        parent_w -= padding*2;
+        parent_h -= padding*2;
+    }
+    
     if      (self->m_absolute_x > -1)         self->m_x = parent_x + self->m_absolute_x;
     else if (self->m_absolute_reverse_x > -1) self->m_x = parent_x + (parent_w - self->m_absolute_reverse_x);
     else if (self->m_relative_x > -1)         self->m_x = (int)(parent_x + parent_w*self->m_relative_x/100);
-    
+
     if      (self->m_absolute_y > -1)         self->m_y = parent_y + self->m_absolute_y;
     else if (self->m_absolute_reverse_y > -1) self->m_y = parent_y + (parent_h - self->m_absolute_reverse_y);
     else if (self->m_relative_y > -1)         self->m_y = (int)(parent_y + parent_h*self->m_relative_y/100);
-    
+
     if (self->m_absolute_w > -1)      self->m_w = self->m_absolute_w;
     else if (self->m_relative_w > -1) self->m_w = (int)round(parent_w*self->m_relative_w/100.0);
-    
+
     if (self->m_absolute_h > -1)      self->m_h = self->m_absolute_h;
     else if (self->m_relative_h > -1) self->m_h = (int)round(parent_h*self->m_relative_h/100.0);
-    
+
     // ---- can't make widget bigger than parent
     if (self->m_h > (int)parent_h)
     {
         float ratio = (float)parent_h / self->m_h;
-        
+
         self->m_w = (int)(self->m_w*ratio);
         self->m_h = (int)(self->m_h*ratio);
     }
@@ -335,23 +343,23 @@ void LayoutManager::applyCoords(Widget* self, AbstractTopLevelContainer* topLeve
     {
         // scale down while keeping aspect ratio (don't do this for text widgets though...)
         float ratio = (float)parent_w / self->m_w;
-        
+
         self->m_w = (int)(self->m_w * ratio);
-        
+
         // FIXME: ugly to hardcode widgets types here
         if (self->m_type != WTYPE_LABEL && self->m_type != WTYPE_BUBBLE)
         {
             self->m_h = (int)(self->m_h * ratio);
         }
     }
-    
+
     // ------ check for given max size
     if (self->m_properties[PROP_MAX_WIDTH].size() > 0)
     {
         const int max_width = atoi_p( self->m_properties[PROP_MAX_WIDTH].c_str() );
         if (self->m_w > max_width) self->m_w = max_width;
     }
-    
+
     if (self->m_properties[PROP_MAX_HEIGHT].size() > 0)
     {
         const int max_height = atoi_p( self->m_properties[PROP_MAX_HEIGHT].c_str() );
@@ -364,20 +372,20 @@ void LayoutManager::applyCoords(Widget* self, AbstractTopLevelContainer* topLeve
 void LayoutManager::recursivelyReadCoords(PtrVector<Widget>& widgets)
 {
     const unsigned short widgets_amount = widgets.size();
-    
+
     // ----- deal with containers' children
     for (int n=0; n<widgets_amount; n++)
     {
         if (widgets[n].m_type == WTYPE_DIV) recursivelyReadCoords(widgets[n].m_children);
     }
-    
+
     // ----- read x/y/size parameters
     for (unsigned short n=0; n<widgets_amount; n++)
     {
         readCoords(widgets.get(n));
-    }//next widget     
+    }//next widget
 }
-    
+
 // ----------------------------------------------------------------------------
 
 void LayoutManager::calculateLayout(PtrVector<Widget>& widgets, AbstractTopLevelContainer* topLevelContainer)
@@ -392,22 +400,22 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                       Widget* parent)
 {
     const unsigned short widgets_amount = widgets.size();
- 
+
     for (int n=0; n<widgets_amount; n++)
     {
         applyCoords(widgets.get(n), topLevelContainer, parent);
     }
-    
+
     // ----- manage 'layout's if relevant
     do // i'm using 'while false' here just to be able to 'break' ...
     {
         if(parent == NULL) break;
-        
+
         std::string layout_name = parent->m_properties[PROP_LAYOUT];
         if(layout_name.size() < 1) break;
-        
+
         bool horizontal = false;
-        
+
         if (!strcmp("horizontal-row", layout_name.c_str()))
             horizontal = true;
         else if(!strcmp("vertical-row", layout_name.c_str()))
@@ -417,9 +425,24 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
             std::cerr << "Unknown layout name : " << layout_name.c_str() << std::endl;
             break;
         }
-        
-        const int w = parent->m_w, h = parent->m_h;
-        
+
+        int x = parent->m_x;
+        int y = parent->m_y;
+        int w = parent->m_w;
+        int h = parent->m_h;
+
+        if (parent != NULL && parent->getType() == WTYPE_DIV && parent->m_show_bounding_box)
+        {
+            int padding = 15;
+            if (parent->m_properties[PROP_DIV_PADDING].length() > 0)
+                padding = atoi(parent->m_properties[PROP_DIV_PADDING].c_str());
+            
+            x += padding;
+            y += padding;
+            w -= padding*2;
+            h -= padding*2;
+        }
+    
         // find space left after placing all absolutely-sized widgets in a row
         // (the space left will be divided between remaining widgets later)
         int left_space = (horizontal ? w : h);
@@ -433,20 +456,18 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                 total_proportion += atoi_p( prop.c_str() );
                 continue;
             }
-            
+
             // absolutely-sized widgets
             left_space -= (horizontal ? widgets[n].m_w : widgets[n].m_h);
         } // next widget
-        
+
         if (left_space < 0)
         {
             fprintf(stderr, "[LayoutManager] WARNING: statically sized widgets took all the place!!\n");
             left_space = 0;
         }
-        
+
         // ---- lay widgets in row
-        int x = parent->m_x;
-        int y = parent->m_y;
         for (int n=0; n<widgets_amount; n++)
         {
             std::string prop = widgets[n].m_properties[ PROP_PROPORTION ];
@@ -461,13 +482,13 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                               << "' is not a number for widget " << widgets[n].m_properties[PROP_ID].c_str()
                               << std::endl;
                 }
-                
+
                 const float fraction = (float)proportion/(float)total_proportion;
-                
+
                 if (horizontal)
                 {
                     widgets[n].m_x = x;
-                    
+
                     std::string align = widgets[n].m_properties[ PROP_ALIGN ];
                     if (align.size() < 1)
                     {
@@ -507,14 +528,14 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                    <<  "' is unknown (widget '" << widgets[n].m_properties[PROP_ID].c_str()
                                    << "', in a horiozntal-row layout)\n";
                     }
-                    
+
                     widgets[n].m_w = (int)(left_space*fraction);
                     if (widgets[n].m_properties[PROP_MAX_WIDTH].size() > 0)
                     {
                         const int max_width = atoi_p( widgets[n].m_properties[PROP_MAX_WIDTH].c_str() );
                         if (widgets[n].m_w > max_width) widgets[n].m_w = max_width;
                     }
-                    
+
                     if (widgets[n].m_w <= 0)
                     {
                         fprintf(stderr, "WARNING: widget '%s' has a width of %i (left_space = %i, "
@@ -522,19 +543,19 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                 widgets[n].m_w, left_space, fraction, widgets[n].m_properties[PROP_MAX_WIDTH].c_str());
                         widgets[n].m_w = 1;
                     }
-                    
+
                     x += widgets[n].m_w;
                 }
                 else
-                {    
+                {
                     widgets[n].m_h = (int)(left_space*fraction);
-                    
+
                     if (widgets[n].m_properties[PROP_MAX_HEIGHT].size() > 0)
                     {
                         const int max_height = atoi_p( widgets[n].m_properties[PROP_MAX_HEIGHT].c_str() );
                         if (widgets[n].m_h > max_height) widgets[n].m_h = max_height;
                     }
-                    
+
                     if (widgets[n].m_h <= 0)
                     {
                         fprintf(stderr, "WARNING: widget '%s' has a height of %i (left_space = %i, "
@@ -542,7 +563,7 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                 widgets[n].m_h, left_space, fraction, widgets[n].m_properties[PROP_MAX_WIDTH].c_str());
                         widgets[n].m_h = 1;
                     }
-                    
+
                     std::string align = widgets[n].m_properties[ PROP_ALIGN ];
                     if (align.size() < 1)
                     {
@@ -572,7 +593,7 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                     {
                         widgets[n].m_x = x + w/2 - widgets[n].m_w/2;
                     }
-                    else if (align == "right") 
+                    else if (align == "right")
                     {
                         widgets[n].m_x = x + w - widgets[n].m_w;
                     }
@@ -583,24 +604,24 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                   << "', in a vertical-row layout)\n";
                     }
                     widgets[n].m_y = y;
-                    
+
                     y += widgets[n].m_h;
                 }
             }
             else
             {
                 // absolute size
-                
+
                 if (horizontal)
                 {
                     widgets[n].m_x = x;
-                    
+
                     std::string align = widgets[n].m_properties[ PROP_ALIGN ];
-                    
+
                     if (align.size() < 1)
                     {
                         std::string prop_y = widgets[n].m_properties[ PROP_Y ];
-                        
+
                         if (prop_y.size() > 0)
                         {
                             if (prop_y[ prop_y.size()-1 ] == '%')
@@ -636,17 +657,17 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                   << "' is unknown in widget " << widgets[n].m_properties[PROP_ID].c_str()
                                   << std::endl;
                     }
-                    
+
                     x += widgets[n].m_w;
                 }
                 else
-                {                    
+                {
                     std::string align = widgets[n].m_properties[ PROP_ALIGN ];
-                    
+
                     if (align.size() < 1)
                     {
                         std::string prop_x = widgets[n].m_properties[ PROP_X ];
-                        
+
                         if (prop_x.size() > 0)
                         {
                             if (prop_x[ prop_x.size()-1 ] == '%')
@@ -682,19 +703,22 @@ void LayoutManager::doCalculateLayout(PtrVector<Widget>& widgets, AbstractTopLev
                                   << "' is unknown in widget " << widgets[n].m_properties[PROP_ID].c_str() << std::endl;
                     }
                     widgets[n].m_y = y;
-                    
+
                     y += widgets[n].m_h;
                 }
             } // end if property or absolute size
-            
+
         } // next widget
-        
+
     } while(false);
-    
+
     // ----- also deal with containers' children
     for (int n=0; n<widgets_amount; n++)
     {
-        if (widgets[n].m_type == WTYPE_DIV) doCalculateLayout(widgets[n].m_children, topLevelContainer, &widgets[n]);
+        if (widgets[n].m_type == WTYPE_DIV)
+        {
+            doCalculateLayout(widgets[n].m_children, topLevelContainer, &widgets[n]);
+        }
     }
 }   // calculateLayout
 
