@@ -1,5 +1,5 @@
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2009 Marianne Gagnon
+//  Copyright (C) 2009-2013 Marianne Gagnon
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@ using namespace irr::gui;
 #  define round(x)  (floor(x+0.5f))
 #endif
 
-const char* RibbonWidget::NO_ITEM_ID = "?";
+const char RibbonWidget::NO_ITEM_ID[] = "?";
 
 
 // ----------------------------------------------------------------------------
@@ -53,45 +53,45 @@ RibbonWidget::RibbonWidget(const RibbonType type) : Widget(WTYPE_RIBBON)
         m_selection[n] = -1;
     }
     m_selection[0] = 0; // only player 0 has a selection by default
-    
+
     m_ribbon_type = type;
     m_mouse_focus = NULL;
     m_listener    = NULL;
 
     m_check_inside_me = true;
     m_supports_multiplayer = (type == RIBBON_TOOLBAR);
-    
+
     updateSelection();
 }   // RibbonWidget
 
 // ----------------------------------------------------------------------------
 void RibbonWidget::add()
-{    
+{
     assert(m_magic_number == 0xCAFEC001);
     assert(m_x > -10.0f);
     assert(m_y > -10.0f);
     assert(m_w > 0.0f);
     assert(m_h > 0.0f);
-    
-    
+
+
     m_labels.clearWithoutDeleting();
-    
+
     rect<s32> widget_size = rect<s32>(m_x, m_y, m_x + m_w, m_y + m_h);
-    
+
     int id = (m_reserved_id == -1 ? getNewID() : m_reserved_id);
-    
-    IGUIButton * btn = GUIEngine::getGUIEnv()->addButton(widget_size, 
+
+    IGUIButton * btn = GUIEngine::getGUIEnv()->addButton(widget_size,
                                                          m_parent, id, L"");
     m_element = btn;
-    
+
     const int subbuttons_amount = m_children.size();
-    
+
     // For some ribbon types, we can have unequal sizes depending on whether
     // items have labels or not
     int with_label = 0;
     int without_label = 0;
-    
-    // ---- check how much space each child button will take and fit 
+
+    // ---- check how much space each child button will take and fit
     // them within available space
     int total_needed_space = 0;
     for (int i=0; i<subbuttons_amount; i++)
@@ -99,64 +99,64 @@ void RibbonWidget::add()
         // FIXME: why do I manually invoke the Layout Manager here?
         LayoutManager::readCoords(m_children.get(i));
         LayoutManager::applyCoords(m_children.get(i), NULL, this);
-        
-        if (m_children[i].m_type != WTYPE_ICON_BUTTON && 
+
+        if (m_children[i].m_type != WTYPE_ICON_BUTTON &&
             m_children[i].m_type != WTYPE_BUTTON)
         {
             fprintf(stderr, "/!\\ Warning /!\\ : ribbon widgets can only have "
                             "(icon)button widgets as children\n");
             continue;
         }
-        
-        // ribbon children must not be keyboard navigatable, the parent 
+
+        // ribbon children must not be keyboard navigatable, the parent
         // ribbon takes care of that
         if (m_children[i].m_type == WTYPE_ICON_BUTTON)
         {
             IconButtonWidget* icon = ((IconButtonWidget*)m_children.get(i));
             icon->m_tab_stop = false;
         }
-        
-        
+
+
         bool has_label_underneath = m_children[i].m_text.size() > 0;
         if (m_children[i].m_properties[PROP_LABELS_LOCATION].size() > 0)
         {
             has_label_underneath = false;
         }
-        
+
         if (has_label_underneath) with_label++;
         else                      without_label++;
-        
+
         total_needed_space += m_children[i].m_w;
     }
-    
+
     int free_w_space = m_w - total_needed_space;
-    
+
     //int biggest_y = 0;
     const int button_y = 10;
     float global_zoom = 1;
-    
+
     const int min_free_space = 50;
     global_zoom = (float)m_w / (float)( m_w - free_w_space + min_free_space );
     //free_w_space = (int)(m_w - total_needed_space*global_zoom);
-    
+
     const int one_button_space =
         (int)round((float)m_w / (float)subbuttons_amount);
-    
+
     int widget_x = -1;
-    
+
     // ---- add children
     for (int i=0; i<subbuttons_amount; i++)
-    {        
+    {
         // ---- tab ribbons
         if (getRibbonType() == RIBBON_TABS)
         {
             const int large_tab = (int)((with_label + without_label)
-                                        *one_button_space 
+                                        *one_button_space
                                         / (with_label + without_label/2.0f));
             const int small_tab = large_tab/2;
-            
+
             stringw& message = m_children[i].m_text;
-            
+
 
             if (message.size() == 0)
             {
@@ -168,31 +168,31 @@ void RibbonWidget::add()
                 if (widget_x == -1) widget_x = large_tab/2;
                 else widget_x += large_tab/2;
             }
-            
+
             IGUIButton * subbtn = NULL;
             rect<s32> subsize = rect<s32>(widget_x - large_tab/2+2,  0,
                                           widget_x + large_tab/2-2,  m_h);
-                        
+
             if (message.size() == 0)
             {
                 subsize = rect<s32>(widget_x - small_tab/2+2,  0,
                                     widget_x + small_tab/2-2,  m_h);
             }
-            
+
             if (m_children[i].m_type == WTYPE_BUTTON)
             {
                 subbtn = GUIEngine::getGUIEnv()
-                       ->addButton(subsize, btn, getNewNoFocusID(), 
+                       ->addButton(subsize, btn, getNewNoFocusID(),
                                    message.c_str(), L"");
                 subbtn->setTabStop(false);
                 subbtn->setTabGroup(false);
-                
+
                 if ((int)GUIEngine::getFont()->getDimension(message.c_str())
                                               .Width > subsize.getWidth()  &&
-                    message.findFirst(L' ') == -1                          && 
+                    message.findFirst(L' ') == -1                          &&
                     message.findFirst(L'\u00AD') == -1                        )
                 {
-                    // if message too long and contains no space and no soft 
+                    // if message too long and contains no space and no soft
                     // hyphen, make the font smaller
                     subbtn->setOverrideFont(GUIEngine::getSmallFont());
                 }
@@ -203,7 +203,7 @@ void RibbonWidget::add()
                                                 0,
                                                 subsize.getHeight()+15,
                                                 subsize.getHeight());
-                
+
                 if (message.size() == 0)
                 {
                     const int x = subsize.getWidth()/2 - subsize.getHeight()/2;
@@ -213,44 +213,44 @@ void RibbonWidget::add()
                                           x + subsize.getHeight(),
                                           subsize.getHeight());
                 }
-                
+
                 // label at the *right* of the icon (for tabs)
                 rect<s32> label_part = rect<s32>(subsize.getHeight()+15,
                                                  0,
                                                  subsize.getWidth()-15,
                                                  subsize.getHeight());
-                
-                // use the same ID for all subcomponents; since event handling 
-                // is done per-ID, no matter which one your hover, this 
+
+                // use the same ID for all subcomponents; since event handling
+                // is done per-ID, no matter which one your hover, this
                 // widget will get it
                 int same_id = getNewNoFocusID();
-                subbtn = GUIEngine::getGUIEnv()->addButton(subsize, btn, 
+                subbtn = GUIEngine::getGUIEnv()->addButton(subsize, btn,
                                                            same_id, L"", L"");
-                
-                IGUIButton* icon = 
-                    GUIEngine::getGUIEnv()->addButton(icon_part, subbtn, 
+
+                IGUIButton* icon =
+                    GUIEngine::getGUIEnv()->addButton(icon_part, subbtn,
                                                       same_id, L"");
                 icon->setScaleImage(true);
-                std::string filename = file_manager->getDataDir() + "/" 
+                std::string filename = file_manager->getDataDir()
                                      + m_children[i].m_properties[PROP_ICON];
                 icon->setImage( irr_driver->getTexture(filename.c_str()) );
                 icon->setUseAlphaChannel(true);
                 icon->setDrawBorder(false);
                 icon->setTabStop(false);
 
-                IGUIStaticText* label = 
-                    GUIEngine::getGUIEnv()->addStaticText(message.c_str(), 
+                IGUIStaticText* label =
+                    GUIEngine::getGUIEnv()->addStaticText(message.c_str(),
                                                           label_part,
                                                           false /* border */,
                                                           true /* word wrap */,
                                                           subbtn, same_id);
-                
+
                 if ((int)GUIEngine::getFont()->getDimension(message.c_str())
                                               .Width > label_part.getWidth()&&
-                    message.findFirst(L' ') == -1                           && 
+                    message.findFirst(L' ') == -1                           &&
                     message.findFirst(L'\u00AD') == -1                        )
                 {
-                    // if message too long and contains no space and no soft 
+                    // if message too long and contains no space and no soft
                     // hyphen, make the font smaller
                     label->setOverrideFont(GUIEngine::getSmallFont());
                 }
@@ -258,7 +258,7 @@ void RibbonWidget::add()
                 label->setTabStop(false);
                 label->setNotClipped(true);
                 m_labels.push_back(label);
-                
+
                 subbtn->setTabStop(false);
                 subbtn->setTabGroup(false);
             }
@@ -266,9 +266,9 @@ void RibbonWidget::add()
             {
                 fprintf(stderr, "Invalid tab bar contents\n");
             }
-            
+
             m_children[i].m_element = subbtn;
-            
+
             if (message.size() == 0) widget_x += small_tab/2;
             else                     widget_x += large_tab/2;
         }
@@ -276,68 +276,77 @@ void RibbonWidget::add()
         else if (m_children[i].m_type == WTYPE_ICON_BUTTON)
         {
             if (widget_x == -1) widget_x = one_button_space/2;
-            
+
             // find how much space to keep for the label under the button.
             // consider font size, whether the label is multiline, etc...
             bool has_label = m_children[i].m_text.size() > 0;
-            
+
             if (m_children[i].m_properties[PROP_LABELS_LOCATION].size() > 0)
             {
                 has_label = false;
             }
-            
-            const int needed_space_under_button = has_label 
-                                                ? GUIEngine::getFontHeight() 
+
+            const int needed_space_under_button = has_label
+                                                ? GUIEngine::getFontHeight()
                                                 : 10;
-            
-            float imageRatio = 
+
+            float imageRatio =
                 (float)m_children[i].m_w / (float)m_children[i].m_h;
-            
+
             // calculate the size of the image
-            std::string filename = file_manager->getDataDir()+"/"
+            std::string filename = file_manager->getDataDir()
                                  + m_children[i].m_properties[PROP_ICON];
-            video::ITexture* image = 
+            video::ITexture* image =
                 irr_driver->getTexture((filename).c_str());
+            if(!image)
+            {
+                std::string file = file_manager->getGUIDir() + "main_help.png";
+                image = irr_driver->getTexture(file);
+                if(!image)
+                    Log::fatal("RibbonWidget",
+                        "Can't find fallback texture 'gui/main_help.png, aborting.");
+            }
+
             float image_h = (float)image->getSize().Height;
             float image_w = image_h*imageRatio;
 
             // scale to fit (FIXME: calculate the right value directly...)
             float zoom = global_zoom;
-            
+
             if (button_y + image_h*zoom + needed_space_under_button > m_h)
             {
                 // scale down
-                while (button_y + image_h*zoom + 
+                while (button_y + image_h*zoom +
                        needed_space_under_button > m_h)  zoom -= 0.01f;
             }
             else
             {
                 // scale up
-                while (button_y + image_h*zoom + 
+                while (button_y + image_h*zoom +
                        needed_space_under_button < m_h)  zoom += 0.01f;
             }
-            
+
             // ---- add bitmap button part
-            // backup and restore position in case the same object is added 
+            // backup and restore position in case the same object is added
             // multiple times (FIXME: unclean)
             int old_x = m_children[i].m_x;
             int old_y = m_children[i].m_y;
             int old_w = m_children[i].m_w;
             int old_h = m_children[i].m_h;
-            
+
             m_children[i].m_x = widget_x - (int)(image_w*zoom/2.0f);
             m_children[i].m_y = button_y;
             m_children[i].m_w = (int)(image_w*zoom);
             m_children[i].m_h = (int)(image_h*zoom);
-            
+
             IconButtonWidget* icon = ((IconButtonWidget*)m_children.get(i));
-            
+
             if (icon->m_properties[PROP_EXTEND_LABEL].size() == 0)
             {
-                icon->m_properties[PROP_EXTEND_LABEL] = 
+                icon->m_properties[PROP_EXTEND_LABEL] =
                     StringUtils::toString(one_button_space - icon->m_w);
             }
-            
+
             m_children.get(i)->m_parent = btn;
             m_children.get(i)->add();
 
@@ -346,24 +355,24 @@ void RibbonWidget::add()
             m_children[i].m_y = old_y;
             m_children[i].m_w = old_w;
             m_children[i].m_h = old_h;
-            
-            // the label itself will be added by the icon widget. since it 
-            // adds the label outside of the widget area it is assigned to, 
+
+            // the label itself will be added by the icon widget. since it
+            // adds the label outside of the widget area it is assigned to,
             // the label will appear in the area we want at the bottom
-            
+
             widget_x += one_button_space;
         }
         else
         {
-            fprintf(stderr, 
+            fprintf(stderr,
                     "/!\\ Warning /!\\ : Invalid contents type in ribbon\n");
         }
-        
-        
+
+
         //m_children[i].id = subbtn->getID();
         m_children[i].m_event_handler = this;
     }// next sub-button
-    
+
     id = m_element->getID();
     m_element->setTabOrder(id);
     m_element->setTabGroup(false);
@@ -376,30 +385,30 @@ void RibbonWidget::addTextChild(const wchar_t* text, const std::string id)
 {
     // This method should only be called BEFORE a widget is added
     assert(m_element == NULL);
-    
+
     ButtonWidget* item = new ButtonWidget();
     item->m_text = text;
     item->m_properties[PROP_ID] = id;
-    
+
     m_children.push_back(item);
 }   // addTextChild
 
 // ----------------------------------------------------------------------------
 
 void RibbonWidget::addIconChild(const wchar_t* text, const std::string &id,
-                         const int w, const int h, 
+                         const int w, const int h,
                          const std::string &icon,
                          const IconButtonWidget::IconPathType icon_path_type)
 {
     // This method should only be called BEFORE a widget is added
     assert(m_element == NULL);
-    
+
     IconButtonWidget* ribbon_item = new IconButtonWidget();
-    
+
     ribbon_item->m_properties[PROP_ID] = id;
-    
+
     ribbon_item->setImage(icon.c_str(), icon_path_type);
-    
+
     ribbon_item->m_properties[PROP_WIDTH]  = StringUtils::toString(w);
     ribbon_item->m_properties[PROP_HEIGHT] = StringUtils::toString(h);
 
@@ -413,7 +422,7 @@ void RibbonWidget::clearAllChildren()
 {
     // This method should only be called BEFORE a widget is added
     assert(m_element == NULL);
-    
+
     m_children.clearAndDeleteAll();
 }   // clearAllChildren
 
@@ -423,7 +432,7 @@ void RibbonWidget::removeChildNamed(const char* name)
 {
     // This method should only be called BEFORE a widget is added
     assert(m_element == NULL);
-    
+
     Widget* child;
     for_in (child, m_children)
     {
@@ -440,7 +449,7 @@ void RibbonWidget::removeChildNamed(const char* name)
 void RibbonWidget::select(std::string item, const int mousePlayerID)
 {
     const int subbuttons_amount = m_children.size();
-    
+
     for (int i=0; i<subbuttons_amount; i++)
     {
         if (m_children[i].m_properties[PROP_ID] == item)
@@ -450,7 +459,7 @@ void RibbonWidget::select(std::string item, const int mousePlayerID)
             return;
         }
     }
-    
+
 }   // select
 
 // ----------------------------------------------------------------------------
@@ -461,7 +470,7 @@ EventPropagation RibbonWidget::rightPressed(const int playerID)
     if (m_children.size() < 2) return EVENT_LET;
 
     m_selection[playerID]++;
-    
+
     if (m_selection[playerID] >= m_children.size())
     {
         if (m_listener != NULL) m_listener->onRibbonWidgetScroll(1);
@@ -469,7 +478,7 @@ EventPropagation RibbonWidget::rightPressed(const int playerID)
         m_selection[playerID] = m_event_handler ? m_children.size()-1 : 0;
     }
     updateSelection();
-    
+
     if (m_ribbon_type == RIBBON_COMBO || m_ribbon_type == RIBBON_TABS)
     {
         const int mousePlayerID = input_manager->getPlayerKeyboardID();
@@ -478,16 +487,16 @@ EventPropagation RibbonWidget::rightPressed(const int playerID)
             m_mouse_focus = m_children.get(m_selection[playerID]);
         }
     }
-    
+
     // if we reached a filler item, move again (but don't wrap)
-    if (getSelectionIDString(playerID) == RibbonWidget::NO_ITEM_ID) 
+    if (getSelectionIDString(playerID) == RibbonWidget::NO_ITEM_ID)
     {
         if (m_selection[playerID] + 1 < m_children.size())
         {
             rightPressed(playerID);
         }
     }
-    
+
     return m_ribbon_type != RIBBON_TOOLBAR ? EVENT_LET : EVENT_BLOCK;
 }   // rightPressed
 
@@ -497,19 +506,19 @@ EventPropagation RibbonWidget::leftPressed(const int playerID)
     if (m_deactivated) return EVENT_LET;
     // empty ribbon, or only one item (can't move left)
     if (m_children.size() < 2) return EVENT_LET;
-    
+
     m_selection[playerID]--;
     if (m_selection[playerID] < 0)
     {
         if (m_listener != NULL) m_listener->onRibbonWidgetScroll(-1);
 
-        m_selection[playerID] = m_event_handler 
-                              ? 0 
+        m_selection[playerID] = m_event_handler
+                              ? 0
                               : m_children.size()-1;
     }
 
     updateSelection();
-    
+
     if (m_ribbon_type == RIBBON_COMBO)
     {
         const int mousePlayerID = input_manager->getPlayerKeyboardID();
@@ -518,13 +527,13 @@ EventPropagation RibbonWidget::leftPressed(const int playerID)
             m_mouse_focus = m_children.get(m_selection[playerID]);
         }
     }
-    
+
     // if we reached a filler item, move again (but don't wrap)
-    if (getSelectionIDString(playerID) == RibbonWidget::NO_ITEM_ID) 
+    if (getSelectionIDString(playerID) == RibbonWidget::NO_ITEM_ID)
     {
         if (m_selection[playerID] > 0) leftPressed(playerID);
     }
-    
+
     if (m_ribbon_type != RIBBON_TOOLBAR)
     {
         //GUIEngine::transmitEvent( this, m_properties[PROP_ID], playerID );
@@ -539,11 +548,11 @@ EventPropagation RibbonWidget::leftPressed(const int playerID)
 // ----------------------------------------------------------------------------
 
 EventPropagation RibbonWidget::focused(const int playerID)
-{    
+{
     Widget::focused(playerID);
-    
+
     if (m_children.size() < 1) return EVENT_LET; // empty ribbon
-    
+
     if (m_ribbon_type == RIBBON_COMBO || m_ribbon_type == RIBBON_TABS)
     {
         const int mousePlayerID = input_manager->getPlayerKeyboardID();
@@ -561,10 +570,10 @@ EventPropagation RibbonWidget::focused(const int playerID)
             m_children.get(m_selection[playerID])->focused(playerID);
         }
     }
-    
+
     if (m_listener != NULL) m_listener->onRibbonWidgetFocus( this, playerID );
 
-    
+
     return EVENT_LET;
 }   // focused
 
@@ -574,29 +583,29 @@ void RibbonWidget::unfocused(const int playerID, Widget* new_focus)
 {
     if (new_focus != NULL && new_focus != this && !m_children.contains(new_focus))
     {
-        if (m_selection[playerID] != -1)
+        if (m_selection[playerID] >= 0 && m_selection[playerID] < m_children.size())
         {
             m_children.get(m_selection[playerID])->unfocused(playerID, new_focus);
         }
     }
-    
+
     //if (m_selection[0] != -1) m_children.get(m_selection[0])->unfocused(0);
 }   // unfocused
 
 // ----------------------------------------------------------------------------
-EventPropagation RibbonWidget::mouseHovered(Widget* child, 
+EventPropagation RibbonWidget::mouseHovered(Widget* child,
                                             const int mousePlayerID)
 {
     if (m_deactivated) return EVENT_LET;
-    
+
     const int subbuttons_amount = m_children.size();
-    
+
     if (m_ribbon_type == RIBBON_COMBO || m_ribbon_type == RIBBON_TABS)
     {
         //std::cout << "SETTING m_mouse_focus\n";
         m_mouse_focus = child;
     }
-    
+
     // In toolbar ribbons, hovering selects
     if (m_ribbon_type == RIBBON_TOOLBAR)
     {
@@ -605,15 +614,15 @@ EventPropagation RibbonWidget::mouseHovered(Widget* child,
             if (m_children.get(i) == child)
             {
                 // Was already selected, don't send another event
-                if (m_selection[mousePlayerID] == i) return EVENT_BLOCK; 
-                
+                if (m_selection[mousePlayerID] == i) return EVENT_BLOCK;
+
                 // Don't change selection on hover for others
                 m_selection[mousePlayerID] = i;
                 break;
             }
         }
     }
-    
+
     updateSelection();
     return EVENT_BLOCK;
 }   // mouseHovered
@@ -638,9 +647,9 @@ const std::string& RibbonWidget::getSelectionIDString(const int playerID)
 void RibbonWidget::updateSelection()
 {
     const int subbuttons_amount = m_children.size();
-    
+
     // FIXME: m_selection, m_selected, m_mouse_focus... what a mess...
-    
+
     //std::cout << "----\n";
     // Update selection flags for mouse player
     for (int p=0; p<MAX_PLAYER_COUNT; p++)
@@ -656,22 +665,22 @@ void RibbonWidget::updateSelection()
             if (new_val) m_children[i].focused(PLAYER_ID_GAME_MASTER);
         }
     }
-        
+
     if (m_listener) m_listener->onSelectionChange();
 }   // updateSelection
 
 // ----------------------------------------------------------------------------
-EventPropagation RibbonWidget::transmitEvent(Widget* w, 
+EventPropagation RibbonWidget::transmitEvent(Widget* w,
                                              const std::string& originator,
                                              const int playerID)
-{    
+{
     assert(m_magic_number == 0xCAFEC001);
 
-    
+
     if (!m_deactivated)
     {
         const int subbuttons_amount = m_children.size();
-        
+
         for (int i=0; i<subbuttons_amount; i++)
         {
             if (m_children[i].m_properties[PROP_ID] == originator)
@@ -680,13 +689,13 @@ EventPropagation RibbonWidget::transmitEvent(Widget* w,
                 break;
             }
         }
-        
+
         updateSelection();
     }
-    
+
     // bring focus back to enclosing ribbon widget
     this->setFocusForPlayer( playerID );
-    
+
     if (m_selection[playerID] != -1)
     {
         if (m_children[m_selection[playerID]].m_deactivated)
@@ -697,7 +706,7 @@ EventPropagation RibbonWidget::transmitEvent(Widget* w,
             return EVENT_BLOCK;
         }
     }
-    
+
     return EVENT_LET;
 }   // transmitEvent
 
@@ -709,7 +718,7 @@ void RibbonWidget::setLabel(const int id, irr::core::stringw new_name)
 
     // ignore this call for ribbons without labels
     if (m_labels.size() == 0) return;
-    
+
     assert(id >= 0);
     assert(id < m_labels.size());
     m_labels[id].setText( new_name.c_str() );
@@ -718,14 +727,14 @@ void RibbonWidget::setLabel(const int id, irr::core::stringw new_name)
 
 // ----------------------------------------------------------------------------
 int RibbonWidget::findItemNamed(const char* internalName)
-{    
+{
     const int size = m_children.size();
-    
+
     //printf("Ribbon : Looking for %s among %i items\n", internalName, size);
-    
+
     for (int n=0; n<size; n++)
     {
-        //printf("     Ribbon : Looking for %s in item %i : %s\n", 
+        //printf("     Ribbon : Looking for %s in item %i : %s\n",
         // internalName, n, m_children[n].m_properties[PROP_ID].c_str());
         if (m_children[n].m_properties[PROP_ID] == internalName) return n;
     }
