@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2010 SuperTuxKart-Team
+//  Copyright (C) 2010-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -32,8 +32,8 @@
 
 #define INPUT_MODE_DEBUG 0
 
-const char* INPUT_FILE_NAME    = "input.xml";
-const int   INPUT_FILE_VERSION = 1;
+static const char  INPUT_FILE_NAME[]  = "input.xml";
+static const int   INPUT_FILE_VERSION = 1;
 
 DeviceManager::DeviceManager()
 {
@@ -49,7 +49,6 @@ bool DeviceManager::initialize()
     GamePadDevice *gamepadDevice = NULL;
     m_map_fire_to_select         = false;
     bool created                 = false;
-    int numGamepads;
 
 
     // Shutdown in case the device manager is being re-initialized
@@ -72,39 +71,42 @@ bool DeviceManager::initialize()
         m_keyboard_configs.push_back(new KeyboardConfig());
         created = true;
     }
-    
+
     const int keyboard_amount = m_keyboard_configs.size();
     for (int n=0; n<keyboard_amount; n++)
     {
         m_keyboards.push_back(new KeyboardDevice(m_keyboard_configs.get(n)));
     }
-    
-    if(UserConfigParams::logMisc()) 
+
+    if(UserConfigParams::logMisc())
             printf("Initializing gamepad support.\n");
 
     irr_driver->getDevice()->activateJoysticks(m_irrlicht_gamepads);
-    numGamepads = m_irrlicht_gamepads.size();    
+    int num_gamepads = m_irrlicht_gamepads.size();
     if(UserConfigParams::logMisc())
     {
-        printf("Irrlicht reports %d gamepads are attached to the system.\n", 
-               numGamepads);
+        printf("Irrlicht reports %d gamepads are attached to the system.\n",
+               num_gamepads);
     }
 
-    
-    
+
+
     // Create GamePadDevice for each physical gamepad and find a GamepadConfig to match
-    for (int id = 0; id < numGamepads; id++)
+    for (int id = 0; id < num_gamepads; id++)
     {
-        core::stringc name = m_irrlicht_gamepads[id].Name.c_str();
-        
+        core::stringc name = m_irrlicht_gamepads[id].Name;
+
+        // Some linux systems report a disk accelerometer as a gamepad, skip that
+        if (name.find("LIS3LV02DL") != -1) continue;
+
 #ifdef WIN32
-        // On Windows, unless we use DirectInput, all gamepads are given the 
+        // On Windows, unless we use DirectInput, all gamepads are given the
         // same name ('microsoft pc-joystick driver'). This makes configuration
-	// totally useless, so append an ID to the name. We can't test for the
-	// name, since the name is even translated.
-	name = name + " " + StringUtils::toString(id).c_str();
+        // totally useless, so append an ID to the name. We can't test for the
+        // name, since the name is even translated.
+        name = name + " " + StringUtils::toString(id).c_str();
 #endif
-        
+
         if (UserConfigParams::logMisc())
         {
             printf("#%d: %s detected...", id, name.c_str());
@@ -112,7 +114,7 @@ bool DeviceManager::initialize()
         // Returns true if new configuration was created
         if (getConfigForGamepad(id, name, &gamepadConfig) == true)
         {
-            if(UserConfigParams::logMisc()) 
+            if(UserConfigParams::logMisc())
                 printf("creating new configuration.\n");
             created = true;
         }
@@ -123,7 +125,7 @@ bool DeviceManager::initialize()
         }
 
         gamepadConfig->setPlugged();
-        gamepadDevice = new GamePadDevice(id, 
+        gamepadDevice = new GamePadDevice(id,
                                           name.c_str(),
                                           m_irrlicht_gamepads[id].Axes,
                                           m_irrlicht_gamepads[id].Buttons,
@@ -139,13 +141,13 @@ bool DeviceManager::initialize()
 void DeviceManager::setAssignMode(const PlayerAssignMode assignMode)
 {
     m_assign_mode = assignMode;
-    
+
 #if INPUT_MODE_DEBUG
     if (assignMode == NO_ASSIGN) std::cout << "====== DeviceManager::setAssignMode(NO_ASSIGN) ======\n";
     if (assignMode == ASSIGN) std::cout << "====== DeviceManager::setAssignMode(ASSIGN) ======\n";
     if (assignMode == DETECT_NEW) std::cout << "====== DeviceManager::setAssignMode(DETECT_NEW) ======\n";
 #endif
-    
+
     // when going back to no-assign mode, do some cleanup
     if (assignMode == NO_ASSIGN)
     {
@@ -177,14 +179,17 @@ GamePadDevice* DeviceManager::getGamePadFromIrrID(const int id)
 
 // -----------------------------------------------------------------------------
 /**
- * Check if we already have a config object for gamepad 'irr_id' as reported by irrLicht
- * If no, create one. Returns true if new configuration was created, otherwise false.
+ * Check if we already have a config object for gamepad 'irr_id' as reported by
+ * irrLicht, If no, create one. Returns true if new configuration was created,
+ *  otherwise false.
  */
-bool DeviceManager::getConfigForGamepad(const int irr_id, const core::stringc& name, GamepadConfig **config)
+bool DeviceManager::getConfigForGamepad(const int irr_id,
+                                        const core::stringc& name,
+                                        GamepadConfig **config)
 {
     bool found = false;
     bool configCreated = false;
-    
+
     // Find appropriate configuration
     for(int n=0; n < m_gamepad_configs.size(); n++)
     {
@@ -194,22 +199,21 @@ bool DeviceManager::getConfigForGamepad(const int irr_id, const core::stringc& n
             found = true;
         }
     }
-    
+
     // If we can't find an appropriate configuration then create one.
     if (!found)
     {
         if(irr_id < (int)(m_irrlicht_gamepads.size()))
         {
-            *config = new GamepadConfig( name.c_str(), 
-                                         m_irrlicht_gamepads[irr_id].Axes, 
+            *config = new GamepadConfig( name.c_str(),
+                                         m_irrlicht_gamepads[irr_id].Axes,
                                          m_irrlicht_gamepads[irr_id].Buttons );
         }
 #ifdef ENABLE_WIIUSE
         else    // Wiimotes have a higher ID and do not refer to m_irrlicht_gamepads
         {
-            *config = new GamepadConfig( name.c_str(), 
-                                         WIIMOTE_AXES, 
-                                         WIIMOTE_BUTTONS );
+            // The Wiimote manager will set number of buttons and axis
+            *config = new GamepadConfig(name.c_str());
         }
 #endif
 
@@ -261,17 +265,17 @@ bool DeviceManager::deleteConfig(DeviceConfig* config)
             n--;
         }
     }
-    
+
     if (m_keyboard_configs.erase(config))
     {
         return true;
     }
-    
+
     if (m_gamepad_configs.erase(config))
     {
         return true;
     }
-    
+
     return false;
 }   // deleteConfig
 
@@ -282,9 +286,9 @@ InputDevice* DeviceManager::mapKeyboardInput( int btnID, InputManager::InputDriv
                                               PlayerAction *action /* out */ )
 {
     const int keyboard_amount = m_keyboards.size();
-    
+
     //std::cout << "mapKeyboardInput " << btnID << " to " << keyboard_amount << " keyboards\n";
-    
+
     for (int n=0; n<keyboard_amount; n++)
     {
         KeyboardDevice *keyboard = m_keyboards.get(n);
@@ -308,7 +312,7 @@ InputDevice* DeviceManager::mapKeyboardInput( int btnID, InputManager::InputDriv
             return keyboard;
         }
     }
-    
+
     return NULL; // no appropriate binding found
 }   // mapKeyboardInput
 
@@ -318,21 +322,21 @@ InputDevice *DeviceManager::mapGamepadInput( Input::InputType type,
                                              int deviceID,
                                              int btnID,
                                              int axisDir,
-                                             int value,
+                                             int *value /* inout */,
                                              InputManager::InputDriverMode mode,
                                              StateManager::ActivePlayer **player /* out */,
                                              PlayerAction *action /* out */)
 {
     GamePadDevice  *gPad = getGamePadFromIrrID(deviceID);
 
-    if (gPad != NULL) 
+    if (gPad != NULL)
     {
         if (gPad->processAndMapInput(type, btnID, value, mode, gPad->getPlayer(), action))
         {
             if (m_single_player != NULL)
             {
                 *player = m_single_player;
-                
+
                 // in single-player mode, assign the gamepad as needed
                 if (gPad->getPlayer() != m_single_player) gPad->setPlayer(m_single_player);
             }
@@ -340,7 +344,7 @@ InputDevice *DeviceManager::mapGamepadInput( Input::InputType type,
             {
                 *player = NULL;
             }
-            else 
+            else
             {
                 *player = gPad->getPlayer();
             }
@@ -357,18 +361,18 @@ bool DeviceManager::translateInput( Input::InputType type,
                                     int deviceID,
                                     int btnID,
                                     int axisDir,
-                                    int value,
+                                    int* value /* inout */,
                                     InputManager::InputDriverMode mode,
                                     StateManager::ActivePlayer** player /* out */,
                                     PlayerAction* action /* out */ )
 {
     if (GUIEngine::getCurrentScreen() != NULL)
     {
-        GUIEngine::getCurrentScreen()->filterInput(type, deviceID, btnID, axisDir, value);
+        GUIEngine::getCurrentScreen()->filterInput(type, deviceID, btnID, axisDir, *value);
     }
-    
+
     InputDevice *device = NULL;
-    
+
     // If the input event matches a bind on an input device, get a pointer to the device
     switch (type)
     {
@@ -386,11 +390,11 @@ bool DeviceManager::translateInput( Input::InputType type,
             break;
         case Input::IT_STICKBUTTON:
         case Input::IT_STICKMOTION:
-            device = mapGamepadInput(type, deviceID, btnID, axisDir, value, 
+            device = mapGamepadInput(type, deviceID, btnID, axisDir, value,
                                      mode, player, action);
             if(!device && m_map_fire_to_select)
             {
-                device = mapGamepadInput(type, deviceID, btnID, axisDir, value, 
+                device = mapGamepadInput(type, deviceID, btnID, axisDir, value,
                                          InputManager::INGAME, player, action);
                 if(device && *action == PA_FIRE)
                     *action = PA_MENU_SELECT;
@@ -400,9 +404,9 @@ bool DeviceManager::translateInput( Input::InputType type,
             break;
     };
 
-    
+
     // Return true if input was successfully translated to an action and player
-    if (device != NULL && abs(value) > Input::MAX_VALUE/2)
+    if (device != NULL && abs(*value) > Input::MAX_VALUE/2)
     {
         m_latest_used_device = device;
     }
@@ -414,27 +418,27 @@ bool DeviceManager::translateInput( Input::InputType type,
 InputDevice* DeviceManager::getLatestUsedDevice()
 {
     // If none, probably the user clicked or used enter; give keyboard by default
-    
+
     if (m_latest_used_device == NULL)
-    {   
+    {
         //std::cout<< "========== No latest device, returning keyboard ==========\n";
         return m_keyboards.get(0); // FIXME: is this right?
     }
-    
+
     return m_latest_used_device;
 }   // getLatestUsedDevice
 
 // -----------------------------------------------------------------------------
 void DeviceManager::clearLatestUsedDevice()
 {
-	m_latest_used_device = NULL;
+    m_latest_used_device = NULL;
 }   // clearLatestUsedDevice
 
 // -----------------------------------------------------------------------------
 bool DeviceManager::deserialize()
 {
     static std::string filepath = file_manager->getConfigDir() + "/" + INPUT_FILE_NAME;
-    
+
     if(UserConfigParams::logMisc())
         printf("Deserializing input.xml...\n");
 
@@ -446,17 +450,17 @@ bool DeviceManager::deserialize()
     else
     {
         irr::io::IrrXMLReader* xml = irr::io::createIrrXMLReader( filepath.c_str() );
-    
+
         const int GAMEPAD = 1;
         const int KEYBOARD = 2;
         const int NOTHING = 3;
-    
+
         int reading_now = NOTHING;
-    
+
         KeyboardConfig* keyboard_config = NULL;
         GamepadConfig* gamepad_config = NULL;
-    
-    
+
+
         // parse XML file
         while(xml && xml->read())
         {
@@ -464,7 +468,7 @@ bool DeviceManager::deserialize()
             {
                 case irr::io::EXN_TEXT:
                     break;
-                    
+
                 case irr::io::EXN_ELEMENT:
                 {
                     if (strcmp("input", xml->getNodeName()) == 0)
@@ -474,7 +478,7 @@ bool DeviceManager::deserialize()
                         {
                             //I18N: shown when config file is too old
                             GUIEngine::showMessage( _("Please re-configure your key bindings.") );
-                            
+
                             GUIEngine::showMessage( _("Your input config file is not compatible with this version of STK.") );
                             return false;
                         }
@@ -491,13 +495,13 @@ bool DeviceManager::deserialize()
                     }
                     else if (strcmp("action", xml->getNodeName()) == 0)
                     {
-                        if(reading_now == KEYBOARD) 
+                        if(reading_now == KEYBOARD)
                         {
                             if(keyboard_config != NULL)
                                 if(!keyboard_config->deserializeAction(xml))
                                     std::cerr << "Ignoring an ill-formed keyboard action in input config.\n";
                         }
-                        else if(reading_now == GAMEPAD) 
+                        else if(reading_now == GAMEPAD)
                         {
                             if(gamepad_config != NULL)
                                 if(!gamepad_config->deserializeAction(xml))
@@ -522,18 +526,18 @@ bool DeviceManager::deserialize()
                     }
                 }
                     break;
-                    
+
                 default: break;
-                
+
             } // end switch
         } // end while
-        
+
         if(UserConfigParams::logMisc())
         {
-            printf("Found %d keyboard and %d gamepad configurations.\n", 
+            printf("Found %d keyboard and %d gamepad configurations.\n",
                    m_keyboard_configs.size(), m_gamepad_configs.size());
         }
-        
+
         // For Debugging....
         /*
         for (int n = 0; n < m_keyboard_configs.size(); n++)
@@ -542,7 +546,7 @@ bool DeviceManager::deserialize()
         for (int n = 0; n < m_gamepad_configs.size(); n++)
             printf("%s", m_gamepad_configs[n].toString().c_str());
         */
-        
+
         delete xml;
     }
 
@@ -552,24 +556,24 @@ bool DeviceManager::deserialize()
 // -----------------------------------------------------------------------------
 void DeviceManager::serialize()
 {
-    static std::string filepath = file_manager->getConfigDir() + "/" 
+    static std::string filepath = file_manager->getConfigDir() + "/"
                                 + INPUT_FILE_NAME;
     if(UserConfigParams::logMisc()) printf("Serializing input.xml...\n");
 
-    
+
     std::ofstream configfile;
     configfile.open (filepath.c_str());
-    
+
     if(!configfile.is_open())
     {
-        std::cerr << "Failed to open " << filepath.c_str() 
+        std::cerr << "Failed to open " << filepath.c_str()
                  << " for writing, controls won't be saved\n";
         return;
     }
-    
-    
+
+
     configfile << "<input version=\"" << INPUT_FILE_VERSION << "\">\n\n";
-    
+
     for(int n=0; n<m_keyboard_configs.size(); n++)
     {
         m_keyboard_configs[n].serialize(configfile);
@@ -578,12 +582,12 @@ void DeviceManager::serialize()
     {
         m_gamepad_configs[n].serialize(configfile);
     }
-    
+
     configfile << "</input>\n";
-    configfile.close(); 
+    configfile.close();
     if(UserConfigParams::logMisc()) printf("Serialization complete.\n\n");
 }   // serialize
-                    
+
 // -----------------------------------------------------------------------------
 
 KeyboardDevice* DeviceManager::getKeyboardFromBtnID(const int btnID)
@@ -591,10 +595,10 @@ KeyboardDevice* DeviceManager::getKeyboardFromBtnID(const int btnID)
     const int keyboard_amount = m_keyboards.size();
     for (int n=0; n<keyboard_amount; n++)
     {
-        if (m_keyboards[n].getConfiguration()->hasBindingFor(btnID)) 
+        if (m_keyboards[n].getConfiguration()->hasBindingFor(btnID))
             return m_keyboards.get(n);
     }
-    
+
     return NULL;
 }   // getKeyboardFromBtnID
 
