@@ -1,6 +1,7 @@
-//
 //  SuperTuxKart - a fun racing game with go-kart
 //
+//  Copyright (C) 2012-2013 SuperTuxKart-Team
+//
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
 //  as published by the Free Software Foundation; either version 3
@@ -21,6 +22,7 @@
 #include "guiengine/event_handler.hpp"
 #include "guiengine/modaldialog.hpp"
 #include "guiengine/screen.hpp"
+#include "graphics/irr_driver.hpp"
 #include "input/device_manager.hpp"
 #include "input/input.hpp"
 #include "karts/controller/controller.hpp"
@@ -37,6 +39,7 @@
 #include "states_screens/state_manager.hpp"
 #include "utils/string_utils.hpp"
 
+#include <ISceneManager.h>
 #include <ISceneNode.h>
 
 #include <map>
@@ -103,7 +106,7 @@ void InputManager::handleStaticAction(int key, int value)
     switch (key)
     {
 #ifdef DEBUG
-        // Special debug options for profile mode: switch the 
+        // Special debug options for profile mode: switch the
         // camera to show a different kart.
         case KEY_KEY_1:
         case KEY_KEY_2:
@@ -118,15 +121,7 @@ void InputManager::handleStaticAction(int key, int value)
             if(!ProfileWorld::isProfileMode() || !world) break;
             int kart_id = key - KEY_KEY_1;
             if(kart_id<0 || kart_id>=(int)world->getNumKarts()) break;
-            for(unsigned int i=0; i<world->getNumKarts(); i++)
-            {
-                if(world->getKart(i)->getCamera())
-                {
-                    world->getKart(i)->getCamera()
-                                     ->changeOwner(world->getKart(kart_id));
-
-                }
-            }
+            Camera::getCamera(0)->setKart(world->getKart(kart_id));
             break;
         }
 #endif
@@ -142,33 +137,41 @@ void InputManager::handleStaticAction(int key, int value)
         case KEY_KEY_I:
         {
             if (!world || !UserConfigParams::m_artist_debug_mode) break;
-            
+
             AbstractKart* kart = world->getLocalPlayerKart(0);
             if (kart == NULL) break;
-            
+
             kart->flyUp();
             break;
         }
         case KEY_KEY_K:
         {
             if (!world || !UserConfigParams::m_artist_debug_mode) break;
-            
+
             AbstractKart* kart = world->getLocalPlayerKart(0);
             if (kart == NULL) break;
-            
+
             kart->flyDown();
             break;
         }
+        case KEY_SNAPSHOT:
         case KEY_PRINT:
-            if (value != 0)
+            // on windows we don't get a press event, only release.  So
+            // save on release only (to avoid saving twice on other platforms)
+            if (value ==0 )
                 irr_driver->requestScreenshot();
             break;
-            
+
         case KEY_F1:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
-                kart->setPowerup(PowerupManager::POWERUP_BUBBLEGUM, 10000);
+                
+                if (control_is_pressed)
+                    kart->setPowerup(PowerupManager::POWERUP_SWATTER, 10000);
+                else
+                    kart->setPowerup(PowerupManager::POWERUP_RUBBERBALL, 10000);
+                    
 #ifdef FORCE_RESCUE_ON_FIRST_KART
                 // Can be useful for debugging places where the AI gets into
                 // a rescue loop: rescue, drive, crash, rescue to same place
@@ -180,6 +183,7 @@ void InputManager::handleStaticAction(int key, int value)
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
+                
                 kart->setPowerup(PowerupManager::POWERUP_PLUNGER, 10000);
             }
             break;
@@ -218,7 +222,7 @@ void InputManager::handleStaticAction(int key, int value)
                 kart->setPowerup(PowerupManager::POWERUP_ZIPPER, 10000);
             }
             break;
-        
+
         case KEY_F8:
             if (UserConfigParams::m_artist_debug_mode && value && world)
             {
@@ -226,12 +230,13 @@ void InputManager::handleStaticAction(int key, int value)
                 {
                     RaceGUIBase* gui = world->getRaceGUI();
                     if (gui != NULL) gui->m_enabled = !gui->m_enabled;
-                    
+
                     const int count = World::getWorld()->getNumKarts();
                     for (int n=0; n<count; n++)
                     {
-                        World::getWorld()->getKart(n)->getNode()
-                            ->setVisible(gui->m_enabled);
+                        if(World::getWorld()->getKart(n)->getController()->isPlayerController())
+                            World::getWorld()->getKart(n)->getNode()
+                                ->setVisible(gui->m_enabled);
                     }
                 }
                 else
@@ -241,7 +246,7 @@ void InputManager::handleStaticAction(int key, int value)
                 }
             }
             break;
-            
+
         case KEY_F9:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
@@ -254,7 +259,7 @@ void InputManager::handleStaticAction(int key, int value)
                     kart->setPowerup(PowerupManager::POWERUP_SWATTER, 10000);
             }
             break;
-            
+
         case KEY_F10:
             if(world && value)
             {
@@ -264,9 +269,9 @@ void InputManager::handleStaticAction(int key, int value)
                     history->Save();
             }
             break;
-            
+
         case KEY_F11:
-            if (UserConfigParams::m_artist_debug_mode && value && 
+            if (UserConfigParams::m_artist_debug_mode && value &&
                 control_is_pressed && world)
             {
                 world->getPhysics()->nextDebugMode();
@@ -275,16 +280,27 @@ void InputManager::handleStaticAction(int key, int value)
 
         case KEY_F12:
             if(value)
-                UserConfigParams::m_display_fps = 
+                UserConfigParams::m_display_fps =
                     !UserConfigParams::m_display_fps;
             break;
-            
+
         case KEY_KEY_P:
-            if (UserConfigParams::m_artist_debug_mode && value && 
+            if (UserConfigParams::m_artist_debug_mode && value &&
                 control_is_pressed)
-                UserConfigParams::m_profiler_enabled = 
+                UserConfigParams::m_profiler_enabled =
                                          !UserConfigParams::m_profiler_enabled;
             break;
+        case KEY_HOME:
+            if (value)
+            {
+                video::SOverrideMaterial &mat =
+                    irr_driver->getVideoDriver()->getOverrideMaterial();
+
+                mat.Material.Wireframe ^= 1;
+                mat.EnableFlags = video::EMF_WIREFRAME;
+                mat.EnablePasses = scene::ESNRP_SOLID | scene::ESNRP_TRANSPARENT;
+            }
+            break;
         default:
             break;
     } // switch
@@ -295,30 +311,30 @@ void InputManager::handleStaticAction(int key, int value)
 /**
   *  Handles input when an input sensing mode (when configuring input)
   */
-void InputManager::inputSensing(Input::InputType type, int deviceID, 
+void InputManager::inputSensing(Input::InputType type, int deviceID,
                                 int button, Input::AxisDirection axisDirection,
                                 int value)
 {
 #if INPUT_MODE_DEBUG
     std::cout << "INPUT SENSING... ";
 #endif
-        
-    // don't store if we're trying to do something like bindings keyboard 
+
+    // don't store if we're trying to do something like bindings keyboard
     // keys on a gamepad
     if (m_mode == INPUT_SENSE_KEYBOARD && type != Input::IT_KEYBOARD)
         return;
-    if (m_mode == INPUT_SENSE_GAMEPAD  && type != Input::IT_STICKMOTION && 
+    if (m_mode == INPUT_SENSE_GAMEPAD  && type != Input::IT_STICKMOTION &&
         type != Input::IT_STICKBUTTON)
         return;
 
 #if INPUT_MODE_DEBUG
     std::cout << (store_new ? "storing it" : "ignoring it") << "\n";
-#endif    
-        
+#endif
+
 
     switch(type)
     {
-    case Input::IT_KEYBOARD: 
+    case Input::IT_KEYBOARD:
         if (value > Input::MAX_VALUE/2)
         {
             m_sensed_input_high_kbd.insert(button);
@@ -353,7 +369,7 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
     case Input::IT_STICKMOTION:
         {
         std::cout << "%% storing new axis binding, value=" << value <<
-            " deviceID=" << deviceID << " button=" << button << 
+            " deviceID=" << deviceID << " button=" << button <<
             " axisDirection=" <<
             (axisDirection == Input::AD_NEGATIVE ? "-" : "+") << "\n";
         // We have to save the direction in which the axis was moved.
@@ -361,44 +377,71 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
         // be zero, we add one before changing the sign).
         int input_id = value>=0 ? 1+button : -(1+button);
 
-        bool id_was_high         = m_sensed_input_high_gamepad.find(input_id) 
+        bool id_was_high         = m_sensed_input_high_gamepad.find(input_id)
                                    != m_sensed_input_high_gamepad.end();
         bool inverse_id_was_high = m_sensed_input_high_gamepad.find(-input_id)
                                    != m_sensed_input_high_gamepad.end();
+        bool id_was_zero         = m_sensed_input_zero_gamepad.find(button)
+                                   != m_sensed_input_zero_gamepad.end();
 
         // A stick was pushed far enough (for the first time) to count as
         // 'triggered' - save the axis (coded with direction in the button
-        // value) for later, so that it can be registered when the stick is 
+        // value) for later, so that it can be registered when the stick is
         // released again.
         // This is mostly legacy behaviour, it is probably good enough
         // to register this as soon as the value is high enough.
-        if (!id_was_high && abs(value) > Input::MAX_VALUE*6.0f/7.0f) 
+        if (!id_was_high && abs(value) > Input::MAX_VALUE*6.0f/7.0f)
         {
-            m_sensed_input_high_gamepad.insert(input_id);
+            if(inverse_id_was_high && !id_was_zero) {
+                Input sensed_input;
+                sensed_input.m_type           = type;
+                sensed_input.m_device_id      = deviceID;
+                sensed_input.m_button_id      = button;
+                sensed_input.m_axis_direction = (value>=0) ? Input::AD_POSITIVE
+                                                           : Input::AD_NEGATIVE;
+                sensed_input.m_axis_range     = Input::AR_FULL;
+                sensed_input.m_character      = deviceID;
+                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+
+            }
+            else m_sensed_input_high_gamepad.insert(input_id);
         }
-        else if ( abs(value) < Input::MAX_VALUE/8.0f && id_was_high )
+        else if ( abs(value) < Input::MAX_VALUE/8.0f )
         {
-            Input sensed_input;
-            sensed_input.m_type           = type;
-            sensed_input.m_device_id      = deviceID;
-            sensed_input.m_button_id      = button;
-            sensed_input.m_axis_direction = value>=0 ? Input::AD_POSITIVE  
-                                                     : Input::AD_NEGATIVE;
-            sensed_input.m_character      = deviceID;
-            OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
-        }
-        else if ( abs(value) < Input::MAX_VALUE/8.0f && inverse_id_was_high )
-        {
-            Input sensed_input;
-            sensed_input.m_type           = type;
-            sensed_input.m_device_id      = deviceID;
-            sensed_input.m_button_id      = button;
-            // Since the inverse direction was high (i.e. stick went from
-            // +30000 to -100), we have to inverse the sign
-            sensed_input.m_axis_direction = value>=0 ? Input::AD_NEGATIVE
-                                                     : Input::AD_POSITIVE;
-            sensed_input.m_character      = deviceID;
-            OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+            if( id_was_high )
+            {
+                Input sensed_input;
+                sensed_input.m_type           = type;
+                sensed_input.m_device_id      = deviceID;
+                sensed_input.m_button_id      = button;
+                sensed_input.m_axis_direction = (value>=0) == id_was_zero
+                                                           ? Input::AD_POSITIVE
+                                                           : Input::AD_NEGATIVE;
+                sensed_input.m_axis_range     = id_was_zero ? Input::AR_HALF
+                                                            : Input::AR_FULL;
+                sensed_input.m_character      = deviceID;
+                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+            }
+            else if( inverse_id_was_high )
+            {
+                Input sensed_input;
+                sensed_input.m_type           = type;
+                sensed_input.m_device_id      = deviceID;
+                sensed_input.m_button_id      = button;
+                // Since the inverse direction was high (i.e. stick went from
+                // +30000 to -100), we have to inverse the sign
+                sensed_input.m_axis_direction = (value>=0) == id_was_zero
+                                                           ? Input::AD_NEGATIVE
+                                                           : Input::AD_POSITIVE;
+                sensed_input.m_axis_range     = id_was_zero ? Input::AR_HALF
+                                                            : Input::AR_FULL;
+                sensed_input.m_character      = deviceID;
+                OptionsScreenInput2::getInstance()->gotSensedInput(sensed_input);
+            }
+            else
+            {
+                m_sensed_input_zero_gamepad.insert(button);
+            }
         }
         break;
         }
@@ -415,11 +458,11 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
 //-----------------------------------------------------------------------------
 int InputManager::getPlayerKeyboardID() const
 {
-    // In no-assign mode, just return the GUI player ID (devices not 
+    // In no-assign mode, just return the GUI player ID (devices not
     // assigned yet)
-    if (m_device_manager->getAssignMode() == NO_ASSIGN) 
+    if (m_device_manager->getAssignMode() == NO_ASSIGN)
         return PLAYER_ID_GAME_MASTER;
-    
+
     // Otherwise, after devices are assigned, we can check in more depth
     // Return the first keyboard that is actually being used
     const int amount = m_device_manager->getKeyboardAmount();
@@ -431,7 +474,7 @@ int InputManager::getPlayerKeyboardID() const
             return m_device_manager->getKeyboard(k)->getPlayer()->getID();
         }
     }
-    
+
     return -1;
 }
 //-----------------------------------------------------------------------------
@@ -447,8 +490,8 @@ int InputManager::getPlayerKeyboardID() const
  * Note: It is the obligation of the called menu to switch of the sense mode.
  *
  */
-void InputManager::dispatchInput(Input::InputType type, int deviceID, 
-                                 int button, 
+void InputManager::dispatchInput(Input::InputType type, int deviceID,
+                                 int button,
                                  Input::AxisDirection axisDirection, int value)
 {
     // Act different in input sensing mode.
@@ -458,7 +501,7 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
         inputSensing(type, deviceID, button, axisDirection,  value);
         return;
     }
-    
+
     // Abort demo mode if a key is pressed during the race in demo mode
     if(dynamic_cast<DemoWorld*>(World::getWorld()))
     {
@@ -469,16 +512,16 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
 
     StateManager::ActivePlayer*   player = NULL;
     PlayerAction    action;
-    bool action_found = m_device_manager->translateInput(type, deviceID, 
+    bool action_found = m_device_manager->translateInput(type, deviceID,
                                                          button, axisDirection,
-                                                         value, m_mode, 
+                                                         &value, m_mode,
                                                          &player, &action);
 
-    // in menus, some keyboard keys are standard (before each player selected 
-    // his device). So if a key could not be mapped to any known binding, 
+    // in menus, some keyboard keys are standard (before each player selected
+    // his device). So if a key could not be mapped to any known binding,
     // fall back to check the defaults.
-    if (!action_found && 
-            StateManager::get()->getGameState() != GUIEngine::GAME && 
+    if (!action_found &&
+            StateManager::get()->getGameState() != GUIEngine::GAME &&
             type == Input::IT_KEYBOARD &&
             m_mode == MENU && m_device_manager->getAssignMode() == NO_ASSIGN)
     {
@@ -490,28 +533,29 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
         else if (button == KEY_RIGHT)  action = PA_MENU_RIGHT;
         else if (button == KEY_SPACE)  action = PA_MENU_SELECT;
         else if (button == KEY_RETURN) action = PA_MENU_SELECT;
+        else if (button == KEY_TAB)    action = PA_MENU_DOWN;
 
         if (button == KEY_RETURN && GUIEngine::ModalDialog::isADialogActive())
         {
             GUIEngine::ModalDialog::onEnterPressed();
         }
-        
+
         if (action != PA_BEFORE_FIRST)
         {
             action_found = true;
             player = NULL;
         }
     }
-    
+
     // do something with the key if it matches a binding
     if (action_found)
     {
         // If we're in the kart menu awaiting new players, do special things
         // when a device presses fire or rescue
         if (m_device_manager->getAssignMode() == DETECT_NEW)
-        {            
+        {
             // Player is unjoining
-            if ((player != NULL) && (action == PA_RESCUE || 
+            if ((player != NULL) && (action == PA_RESCUE ||
                                      action == PA_MENU_CANCEL ) )
             {
                 // returns true if the event was handled
@@ -538,11 +582,11 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
                     InputDevice *device = NULL;
                     if (type == Input::IT_KEYBOARD)
                     {
-                        //std::cout << "==== New Player Joining with Key " << 
+                        //std::cout << "==== New Player Joining with Key " <<
                         // button << " ====" << std::endl;
                         device = m_device_manager->getKeyboardFromBtnID(button);
                     }
-                    else if (type == Input::IT_STICKBUTTON || 
+                    else if (type == Input::IT_STICKBUTTON ||
                              type == Input::IT_STICKMOTION    )
                     {
                         device = m_device_manager->getGamePadFromIrrID(deviceID);
@@ -550,17 +594,17 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
 
                     if (device != NULL)
                     {
-                        KartSelectionScreen::getInstance()->playerJoin(device, 
+                        KartSelectionScreen::getInstance()->playerJoin(device,
                                                                        false );
                     }
                 }
-                return; // we're done here, ignore devices that aren't 
+                return; // we're done here, ignore devices that aren't
                         // associated with players
             }
         }
 
         // ... when in-game
-        if (StateManager::get()->getGameState() == GUIEngine::GAME && 
+        if (StateManager::get()->getGameState() == GUIEngine::GAME &&
              !GUIEngine::ModalDialog::isADialogActive()                  )
         {
             if (player == NULL)
@@ -578,7 +622,7 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
                     "Error, trying to process action for an unknown player\n";
                 return;
             }
-            
+
             Controller* controller = pk->getController();
             if (controller != NULL) controller->action(action, abs(value));
         }
@@ -593,15 +637,15 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
                 m_timer = 0;
             }
 
-            // When in master-only mode, we can safely assume that players 
-            // are set up, contrarly to early menus where we accept every 
+            // When in master-only mode, we can safely assume that players
+            // are set up, contrarly to early menus where we accept every
             // input because players are not set-up yet
             if (m_master_player_only && player == NULL)
             {
-                if (type == Input::IT_STICKMOTION || 
+                if (type == Input::IT_STICKMOTION ||
                     type == Input::IT_STICKBUTTON)
                 {
-                    GamePadDevice* gp = 
+                    GamePadDevice* gp =
                         getDeviceList()->getGamePadFromIrrID(deviceID);
 
                     if (gp != NULL &&
@@ -616,7 +660,7 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
                 }
                 return;
             }
-            
+
             // menu input
             if (!m_timer_in_use)
             {
@@ -625,36 +669,36 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
                     m_timer_in_use = true;
                     m_timer = 0.25;
                 }
-                                
-                // player may be NULL in early menus, before player setup has 
+
+                // player may be NULL in early menus, before player setup has
                 // been performed
                 int playerID = (player == NULL ? 0 : player->getID());
-                
-                // If only the master player can act, and this player is not 
+
+                // If only the master player can act, and this player is not
                 // the master, ignore his input
-                if (m_device_manager->getAssignMode() == ASSIGN && 
+                if (m_device_manager->getAssignMode() == ASSIGN &&
                     m_master_player_only &&
                     playerID != PLAYER_ID_GAME_MASTER)
                 {
                     //I18N: message shown when a player that isn't game master
-                    //I18N: tries to modify options that only the game master 
+                    //I18N: tries to modify options that only the game master
                     //I18N: is allowed to
                     GUIEngine::showMessage(
                         _("Only the Game Master may act at this point!"));
                     return;
                 }
-                
-                // all is good, pass the translated input event on to the 
+
+                // all is good, pass the translated input event on to the
                 // event handler
                 GUIEngine::EventHandler::get()
-                    ->processGUIAction(action, deviceID, abs(value), type, 
+                    ->processGUIAction(action, deviceID, abs(value), type,
                                        playerID);
             }
         }
     }
     else if (type == Input::IT_KEYBOARD)
     {
-        // keyboard press not handled by device manager / bindings. 
+        // keyboard press not handled by device manager / bindings.
         // Check static bindings...
         handleStaticAction( button, value );
     }
@@ -665,7 +709,7 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
 void InputManager::setMasterPlayerOnly(bool enabled)
 {
 #if INPUT_MODE_DEBUG
-    std::cout << 
+    std::cout <<
         "====== InputManager::setMasterPlayerOnly(" << enabled << ") ======\n";
 #endif
     m_master_player_only = enabled;
@@ -697,23 +741,24 @@ EventPropagation InputManager::input(const SEvent& event)
 {
     if (event.EventType == EET_JOYSTICK_INPUT_EVENT)
     {
-        // Axes - FIXME, instead of checking all of them, ask the bindings 
+        // Axes - FIXME, instead of checking all of them, ask the bindings
         // which ones to poll
-        for (int axis_id=0; axis_id<SEvent::SJoystickEvent::NUMBER_OF_AXES ; 
+        for (int axis_id=0; axis_id<SEvent::SJoystickEvent::NUMBER_OF_AXES ;
               axis_id++)
         {
             int value = event.JoystickEvent.Axis[axis_id];
 
             if (UserConfigParams::m_gamepad_debug)
             {
-                printf("axis motion: gamepad_id=%d axis=%d value=%d\n",
-                       event.JoystickEvent.Joystick, axis_id, value);
+                Log::info("InputManager",
+                          "axis motion: gamepad_id=%d axis=%d value=%d",
+                          event.JoystickEvent.Joystick, axis_id, value);
             }
 
-            dispatchInput(Input::IT_STICKMOTION, event.JoystickEvent.Joystick, 
+            dispatchInput(Input::IT_STICKMOTION, event.JoystickEvent.Joystick,
                           axis_id, Input::AD_NEUTRAL, value);
         }
-        
+
         if (event.JoystickEvent.POV == 65535)
         {
             dispatchInput(Input::IT_STICKMOTION, event.JoystickEvent.Joystick,
@@ -724,7 +769,7 @@ EventPropagation InputManager::input(const SEvent& event)
         else
         {
             // *0.017453925f is to convert degrees to radians
-            dispatchInput(Input::IT_STICKMOTION, event.JoystickEvent.Joystick, 
+            dispatchInput(Input::IT_STICKMOTION, event.JoystickEvent.Joystick,
                           Input::HAT_H_ID, Input::AD_NEUTRAL,
                           (int)(cos(event.JoystickEvent.POV*0.017453925f/100.0f)
                                 *Input::MAX_VALUE));
@@ -733,8 +778,8 @@ EventPropagation InputManager::input(const SEvent& event)
                           (int)(sin(event.JoystickEvent.POV*0.017453925f/100.0f)
                                 *Input::MAX_VALUE));
         }
-        
-        GamePadDevice* gp = 
+
+        GamePadDevice* gp =
             getDeviceList()->getGamePadFromIrrID(event.JoystickEvent.Joystick);
 
         if (gp == NULL)
@@ -746,18 +791,19 @@ EventPropagation InputManager::input(const SEvent& event)
         for(int i=0; i<gp->m_button_count; i++)
         {
             const bool isButtonPressed = event.JoystickEvent.IsButtonPressed(i);
-            
+
             // Only report button events when the state of the button changes
-            if ((!gp->isButtonPressed(i) &&  isButtonPressed) || 
+            if ((!gp->isButtonPressed(i) &&  isButtonPressed) ||
                  (gp->isButtonPressed(i) && !isButtonPressed)    )
             {
                 if (UserConfigParams::m_gamepad_debug)
                 {
-                    printf("button %i, status=%i\n", i, isButtonPressed);
+                    Log::info("InputManager", "button %i, status=%i",
+                              i, isButtonPressed);
                 }
-                
-                dispatchInput(Input::IT_STICKBUTTON, 
-                              event.JoystickEvent.Joystick, i, 
+
+                dispatchInput(Input::IT_STICKBUTTON,
+                              event.JoystickEvent.Joystick, i,
                               Input::AD_POSITIVE,
                               isButtonPressed ? Input::MAX_VALUE : 0);
             }
@@ -767,7 +813,7 @@ EventPropagation InputManager::input(const SEvent& event)
     }
     else if (event.EventType == EET_KEY_INPUT_EVENT)
     {
-        // On some systems (linux esp.) certain keys (e.g. [] ) have a 0 
+        // On some systems (linux esp.) certain keys (e.g. [] ) have a 0
         // Key value, but do have a value defined in the Char field.
         // So to distinguish them (otherwise [] would both be mapped to
         // the same value 0, which means we can't distinguish which key
@@ -786,7 +832,7 @@ EventPropagation InputManager::input(const SEvent& event)
                 return EVENT_BLOCK;
             }
             // 'backspace' in a text control must never be mapped, since user
-            // can be in a text area trying to erase text (and if it's mapped 
+            // can be in a text area trying to erase text (and if it's mapped
             // to rescue that would dismiss the dialog instead of erasing a
             // single letter). Same for spacebar. Same for letters.
             if (GUIEngine::isWithinATextBox())
@@ -800,12 +846,12 @@ EventPropagation InputManager::input(const SEvent& event)
                     return EVENT_LET;
                 }
             }
-            
+
             const bool wasInTextBox = GUIEngine::isWithinATextBox();
-            
+
             dispatchInput(Input::IT_KEYBOARD, event.KeyInput.Char, key,
                           Input::AD_POSITIVE, Input::MAX_VALUE);
-            
+
             // if this action took us into a text box, don't let event continue
             // (FIXME not the cleanest solution)
             if (!wasInTextBox && GUIEngine::isWithinATextBox())
@@ -817,8 +863,8 @@ EventPropagation InputManager::input(const SEvent& event)
         else
         {
             // 'backspace' in a text control must never be mapped, since user
-            // can be in a text area trying to erase text (and if it's mapped 
-            // to rescue that would dismiss the dialog instead of erasing a 
+            // can be in a text area trying to erase text (and if it's mapped
+            // to rescue that would dismiss the dialog instead of erasing a
             // single letter). Same for spacebar. Same for letters.
             if (GUIEngine::isWithinATextBox())
             {
@@ -831,8 +877,8 @@ EventPropagation InputManager::input(const SEvent& event)
                     return EVENT_LET;
                 }
             }
-            
-            dispatchInput(Input::IT_KEYBOARD, event.KeyInput.Char, key, 
+
+            dispatchInput(Input::IT_KEYBOARD, event.KeyInput.Char, key,
                           Input::AD_POSITIVE, 0);
             return EVENT_BLOCK; // Don't propagate key up events
         }
@@ -857,20 +903,20 @@ EventPropagation InputManager::input(const SEvent& event)
         EMIE_RMOUSE_LEFT_UP     Right mouse button was left up.
         EMIE_MMOUSE_LEFT_UP     Middle mouse button was left up.
         EMIE_MOUSE_MOVED    The mouse cursor changed its position.
-        EMIE_MOUSE_WHEEL    The mouse wheel was moved. Use Wheel value in 
-                            event data to find out in what direction and 
+        EMIE_MOUSE_WHEEL    The mouse wheel was moved. Use Wheel value in
+                            event data to find out in what direction and
                             how fast.
          */
     }
 #endif
-    
-    // block events in all modes but initial menus (except in text boxes to 
+
+    // block events in all modes but initial menus (except in text boxes to
     // allow typing, and except in modal dialogs in-game)
-    // FIXME: 1) that's awful logic 2) that's not what the code below does, 
+    // FIXME: 1) that's awful logic 2) that's not what the code below does,
     // events are never blocked in menus
-    if (getDeviceList()->getAssignMode() != NO_ASSIGN && 
+    if (getDeviceList()->getAssignMode() != NO_ASSIGN &&
         !GUIEngine::isWithinATextBox() &&
-        (!GUIEngine::ModalDialog::isADialogActive() && 
+        (!GUIEngine::ModalDialog::isADialogActive() &&
         StateManager::get()->getGameState() == GUIEngine::GAME))
     {
         return EVENT_BLOCK;
@@ -913,7 +959,7 @@ bool InputManager::isInMode(InputDriverMode expMode)
 void InputManager::setMode(InputDriverMode new_mode)
 {
     if (new_mode == m_mode) return; // no change
-    
+
     switch (new_mode)
     {
         case MENU:
@@ -935,7 +981,7 @@ void InputManager::setMode(InputDriverMode new_mode)
                     //irr_driver->showPointer();
                     m_mode = MENU;
                     break;
-                    
+
                 case BOOTSTRAP:
                     // Leaving boot strap mode.
 
@@ -952,15 +998,16 @@ void InputManager::setMode(InputDriverMode new_mode)
 
                     //irr_driver->showPointer();
                     m_sensed_input_high_gamepad.clear();
+                    m_sensed_input_zero_gamepad.clear();
                     m_sensed_input_high_kbd.clear();
 
-                    // The order is deliberate just in case someone starts 
-                    // to make STK multithreaded: m_sensed_input must not be 
+                    // The order is deliberate just in case someone starts
+                    // to make STK multithreaded: m_sensed_input must not be
                     // 0 when mode == INPUT_SENSE_PREFER_{AXIS,BUTTON}.
                     m_mode = MENU;
 
                     break;
-                    
+
                     /*
                 case LOWLEVEL:
                     // Leaving lowlevel mode.
@@ -975,7 +1022,7 @@ void InputManager::setMode(InputDriverMode new_mode)
                     // Something is broken.
                     //assert (false);
             }
-            
+
             break;
         case INGAME:
 #if INPUT_MODE_DEBUG
@@ -1003,7 +1050,7 @@ void InputManager::setMode(InputDriverMode new_mode)
             // We must be in menu mode now in order to switch.
             assert (m_mode == MENU);
 
-            // Reset the helper values for the relative mouse movement 
+            // Reset the helper values for the relative mouse movement
             // supresses to the notification of them as an input.
             m_mouse_val_x = m_mouse_val_y = 0;
             m_mode        = new_mode;
