@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2012  Joerg Henrichs
+//  Copyright (C) 2012-2013  Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@ class Attachment;
 class btKart;
 class btQuaternion;
 class btUprightConstraint;
-class Camera;
 class Controller;
 class Item;
 class KartModel;
@@ -40,7 +39,7 @@ class Skidding;
 class SlipStream;
 
 /** An abstract interface for the actual karts. Some functions are actually
- *  implemented here in order to allow inlining. 
+ *  implemented here in order to allow inlining.
  * \ingroup karts
  */
 class AbstractKart : public Moveable
@@ -52,6 +51,10 @@ private:
     float m_kart_width;
     /** Height of the kart, copy of the data from KartModel. */
     float m_kart_height;
+    /** Coordinate on up axis */
+    float m_kart_highest_point;
+    /** The position of all four wheels in the 3d model */
+    const Vec3* m_wheel_graphics_position;
 
     /** Index of kart in world. */
     unsigned int m_world_kart_id;
@@ -76,12 +79,12 @@ protected:
     AbstractKartAnimation *m_kart_animation;
 
 public:
-                   AbstractKart(const std::string& ident, 
+                   AbstractKart(const std::string& ident,
                                 int world_kart_id,
                                 int position, const btTransform& init_transform);
     virtual       ~AbstractKart();
     virtual void   reset();
-    virtual void   init(RaceManager::KartType type, bool is_first_kart) = 0;
+    virtual void   init(RaceManager::KartType type) = 0;
     // ========================================================================
     // Functions related to controlling the kart
     // ------------------------------------------------------------------------
@@ -101,7 +104,7 @@ public:
     // Access to the kart properties.
     // ------------------------------------------------------------------------
     /** Returns the kart properties of this kart. */
-    const KartProperties* getKartProperties() const 
+    const KartProperties* getKartProperties() const
                             { return m_kart_properties; }
     // ------------------------------------------------------------------------
     /** Sets the kart properties. */
@@ -117,7 +120,7 @@ public:
      *  speed. */
     virtual float getMaxSteerAngle () const = 0;
     // ------------------------------------------------------------------------
-    /** Returns the time till full steering is reached for this kart. 
+    /** Returns the time till full steering is reached for this kart.
      *  This can depend on the current steering value, which must be >= 0.
      */
     virtual float getTimeFullSteer(float steer) const = 0;
@@ -137,6 +140,9 @@ public:
     /** Returns this kart's kart model. */
     KartModel* getKartModel() { return m_kart_model;      }
     // ------------------------------------------------------------------------
+    /** Returns this kart's kart model. */
+    const KartModel* getKartModel() const { return m_kart_model;      }
+    // ------------------------------------------------------------------------
     /** Returns the length of the kart. */
     float getKartLength() const { return m_kart_length; }
     // ------------------------------------------------------------------------
@@ -146,8 +152,17 @@ public:
     /** Returns the width of the kart. */
     float getKartWidth() const {return m_kart_width; }
     // ------------------------------------------------------------------------
+    /** Returns the highest point of the kart (coordinate on up axis) */
+    float getHighestPoint() const { return m_kart_highest_point;  }
+    // ------------------------------------------------------------------------
     /** Returns true if this kart has no wheels. */
     bool isWheeless() const;
+    // ------------------------------------------------------------------------
+    /** Returns the position of a wheel relative to the kart.
+     *  \param i Index of the wheel: 0=front right, 1 = front left, 2 = rear
+     *           right, 3 = rear left.  */
+    const Vec3& getWheelGraphicsPosition(int i) const
+                {assert(i>=0 && i<4); return m_wheel_graphics_position[i];}
 
     // ========================================================================
     // Emergency animation related functions.
@@ -156,7 +171,7 @@ public:
      *  animation is being shown. */
     AbstractKartAnimation *getKartAnimation() { return m_kart_animation; }
     // ------------------------------------------------------------------------
-    const AbstractKartAnimation *getKartAnimation() const 
+    const AbstractKartAnimation *getKartAnimation() const
                                                    { return m_kart_animation; }
     // ------------------------------------------------------------------------
     /** Sets a new kart animation. */
@@ -168,7 +183,7 @@ public:
     /** Returns the index of this kart in world. */
     unsigned int   getWorldKartId() const         { return m_world_kart_id;   }
     // ------------------------------------------------------------------------
-    /** Saves the old controller in m_saved_controller and stores a new 
+    /** Saves the old controller in m_saved_controller and stores a new
      *  controller. The save controller is needed in case of a reset.
      *  \param controller The new controller to use (atm it's always an
      *         end controller). */
@@ -224,7 +239,7 @@ public:
      *  bonus and maluses that are currently applied. */
     virtual float getCurrentMaxSpeed() const = 0;
     // ------------------------------------------------------------------------
-    /** Returns how much increased speed time is left over in the given 
+    /** Returns how much increased speed time is left over in the given
      *  category. Not pure abstract, since there is no need to implement this
      *  e.g. in Ghost.
      *  \param category Which category to report on. */
@@ -238,14 +253,14 @@ public:
      *  \param fade_out_time How long the maximum speed will fade out linearly.
      */
     virtual void increaseMaxSpeed(unsigned int category, float add_speed,
-                                  float engine_force, float duration, 
+                                  float engine_force, float duration,
                                   float fade_out_time) = 0;
     // ------------------------------------------------------------------------
     /** Defines a slowdown, which is in fraction of top speed.
      *  \param category The category for which the speed is increased.
      *  \param max_speed_fraction Fraction of top speed to allow only.
      *  \param fade_in_time How long till maximum speed is capped. */
-    virtual void setSlowdown(unsigned int category, float max_speed_fraction, 
+    virtual void setSlowdown(unsigned int category, float max_speed_fraction,
                              float fade_in_time) = 0;
     // ------------------------------------------------------------------------
     /** Returns the remaining collected energy. */
@@ -280,35 +295,23 @@ public:
     /** Activates a slipstream effect, atm that is display some nitro. */
     virtual void setSlipstreamEffect(float f) = 0;
     // ------------------------------------------------------------------------
-    /** Sets the camera for this kart. Takes ownership of the camera and
-     *  will delete it. */
-    virtual void setCamera(Camera *camera) = 0;
-    // ------------------------------------------------------------------------
-    /** Returns the camera of this kart (or NULL if no camera is attached
-     *  to this kart). */
-    virtual Camera* getCamera() = 0;
-    // ------------------------------------------------------------------------
-    /** Returns the camera of this kart (or NULL if no camera is attached
-     *  to this kart). */
-    virtual const Camera* getCamera() const = 0;
-    // ------------------------------------------------------------------------
     /** Plays a beep sfx. */
     virtual void beep() = 0;
     // ------------------------------------------------------------------------
-    /** This function will play a particular character voice for this kart.  
-     *  It returns whether or not a character voice sample exists for the 
-     *  particular event.  If there is no voice sample, a default can be 
+    /** This function will play a particular character voice for this kart.
+     *  It returns whether or not a character voice sample exists for the
+     *  particular event.  If there is no voice sample, a default can be
      *  played instead. */
     virtual bool playCustomSFX(unsigned int type) = 0;
     // ------------------------------------------------------------------------
     /** Show fire to go with a zipper. */
     virtual void showZipperFire() = 0;
     // ------------------------------------------------------------------------
-    /** Sets zipper time, and apply one time additional speed boost. It can be 
+    /** Sets zipper time, and apply one time additional speed boost. It can be
      *  used with a specific material, in which case the zipper parmaters are
      *  taken from this material (parameters that are <0 will be using the
      *  kart-specific values from kart-properties. */
-    virtual void handleZipper(const Material *m=NULL, 
+    virtual void handleZipper(const Material *m=NULL,
                               bool play_sound=false) = 0;
     // ------------------------------------------------------------------------
     /** Returns true if this kart has finished the race. */
@@ -316,6 +319,11 @@ public:
     // ------------------------------------------------------------------------
     virtual void setEnergy(float val) = 0;
     // ------------------------------------------------------------------------
+    /** Return whether nitro is being used despite the nitro button not being
+     *  pressed due to minimal use time requirements
+     */
+    virtual float isOnMinNitroTime() const = 0;
+    // ------------------------------------------------------------------------
     /** Returns the current material the kart is on. */
     virtual const Material *getMaterial() const = 0;
     // ------------------------------------------------------------------------
@@ -342,18 +350,10 @@ public:
     /** Returns true if the kart is 'resting', i.e. (nearly) not moving. */
     virtual bool isInRest() const = 0;
     // ------------------------------------------------------------------------
-    /** Starts the engine sound effect. Called once the track intro phase is 
+    /** Starts the engine sound effect. Called once the track intro phase is
      *  over. */
     virtual void startEngineSFX() = 0;
     // ------------------------------------------------------------------------
-    /** Stores the current suspension length. This function is called from 
-     *  world after all karts are in resting position (see 
-     *  World::resetAllKarts), so that the default suspension rest length can 
-     *  be stored. This is then used later to move the wheels depending on 
-     *  actual suspension, so that when a kart is in rest, the wheels are at
-     *  the position at which they were modelled. */
-    virtual void setSuspensionLength() = 0;
-    // ------------------------------------------------------------------------
     /** This method is to be called every time the mass of the kart is updated,
      *  which includes attaching an anvil to the kart (and detaching). */
     virtual void updateWeight() = 0;
@@ -372,6 +372,17 @@ public:
     // ------------------------------------------------------------------------
     virtual void setInvulnerableTime(float t) = 0;
     // ------------------------------------------------------------------------
+    /** Returns if the kart is protected by a shield. */
+    virtual bool isShielded() const = 0;
+    // ------------------------------------------------------------------------
+    virtual void setShieldTime(float t) = 0;
+    // ------------------------------------------------------------------------
+    virtual float getShieldTime() const = 0;
+    // ------------------------------------------------------------------------
+    /** Decreases the kart's shield time. */
+    virtual void decreaseShieldTime() = 0;
+    // ------------------------------------------------------------------------
+
     /** Shows the star effect for a certain time. */
     virtual void showStarEffect(float t) = 0;
     // ------------------------------------------------------------------------
@@ -394,11 +405,10 @@ public:
         e.g. to calculate the starting point and direction of projectiles. */
     virtual btTransform getAlignedTransform(const float customPitch=-1) = 0;
     // -------------------------------------------------------------------------
-    virtual void onFirePressed() = 0;
-    // -------------------------------------------------------------------------
     /** Set a text that is displayed on top of a kart.
      */
     virtual void setOnScreenText(const wchar_t *text) = 0;
+
 };   // AbstractKart
 
 
