@@ -1,7 +1,7 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006-2009  Eduardo Hernandez Munoz
-//  Copyright (C) 2009, 2010 Joerg Henrichs
+//  Copyright (C) 2006-2013  Eduardo Hernandez Munoz
+//  Copyright (C) 2009-2013  Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -31,17 +31,71 @@
 
 bool AIBaseController::m_ai_debug = false;
 
+/**
+This is the base class for all AIs. At this stage there are two similar
+AIs: one is the SkiddingAI, which is the AI used in lap based races
+(including follow-the-leader mode), the other one is the end controller,
+I.e. the controller that takes over from a player (or AI) when the race is
+finished.
+
+This base class defines some basic operations:
+- It takes care on which part of the QuadGraph the AI currently is.
+- It determines which path the AI should take (in case of shortcuts
+  or forks in the road).
+
+At race start and every time a new lap is started, the AI will compute the
+path the kart is taking this lap (computePath). At this stage the decision
+which road in case of shortcut to take is purely random. It stores the
+information in two arrays:
+  m_successor_index[i] stores which successor to take from node i.
+       The successor is a number between 0 and number_of_successors - 1.
+  m_next_node_index[i] stores the actual index of the graph node that
+       follows after node i.
+Depending on operation one of the other data is more useful, so this
+class stores both information to avoid looking it up over and over.
+Once this is done (still in computePath), the array m_all_look_aheads is
+computed. This array stores for each quad a list of the next (atm) 10 quads.
+This is used when the AI is selecting where to drive next, and it will just
+pass the list of next quads to findRoadSector.
+
+Note that the quad graph information is stored for every quad in the quad
+graph, even if the quad is not on the path chosen. This is necessary since
+it can happen that a kart ends up on a path not choses (e.g. perhaps it was
+pushed on that part, or couldn't get a sharp corner).
+
+In update(), which gets called one per frame per AI, this object will
+determine the quad the kart is currently on (which is then used to determine
+where the kart will be driving to). This uses the m_all_look_aheads to
+speed up this process (since the kart is likely to be either on the same
+quad as it was before, or the next quad in the m_all_look_aheads list).
+
+It will also check if the kart is stuck:
+this is done by maintaining a list of times when the kart hits the track. If
+(atm) more than 3 collisions happen in 1.5 seconds, the kart is considered
+stuck and will trigger a rescue (due to the pushback from the track it will
+take some time if a kart is really stuck before it will hit the track again).
+
+This base class also contains some convenience functions which are useful
+in all AIs, e.g.:
+-  steerToPoint: determine the steering angle to use depending on the
+   current location and the point the kart is driving to.
+-  normalizeAngle: To normalise the steering angle to be in [-PI,PI].
+-  setSteering: Converts the steering angle into a steering fraction
+   in [-1,1].
+
+*/
 AIBaseController::AIBaseController(AbstractKart *kart,
-                                   StateManager::ActivePlayer *player) 
+                                   StateManager::ActivePlayer *player)
                 : Controller(kart, player)
 {
     m_kart          = kart;
     m_kart_length   = m_kart->getKartLength();
     m_kart_width    = m_kart->getKartWidth();
-    m_ai_properties = 
+    m_ai_properties =
         m_kart->getKartProperties()->getAIPropertiesForDifficulty();
 
-    if(race_manager->getMinorMode()!=RaceManager::MINOR_MODE_3_STRIKES)
+    if(race_manager->getMinorMode()!=RaceManager::MINOR_MODE_3_STRIKES &&
+       race_manager->getMinorMode()!=RaceManager::MINOR_MODE_SOCCER)
     {
         m_world     = dynamic_cast<LinearWorld*>(World::getWorld());
         m_track     = m_world->getTrack();
@@ -65,13 +119,13 @@ AIBaseController::AIBaseController(AbstractKart *kart,
 //-----------------------------------------------------------------------------
 void AIBaseController::reset()
 {
-	m_stuck_trigger_rescue = false;
-	m_collision_times.clear();
-}   // reset 
+    m_stuck_trigger_rescue = false;
+    m_collision_times.clear();
+}   // reset
 
 //-----------------------------------------------------------------------------
-/** In debug mode when the user specified --ai-debug on the command line set 
- *  the name of the controller as on-screen text, so that the different AI 
+/** In debug mode when the user specified --ai-debug on the command line set
+ *  the name of the controller as on-screen text, so that the different AI
  *  controllers can be distinguished.
  *  \param name Name of the controller.
 */
@@ -97,7 +151,13 @@ void  AIBaseController::newLap(int lap)
 }   // newLap
 
 //-----------------------------------------------------------------------------
-/** Computes a path for the AI to follow.
+/** Computes a path for the AI to follow. This function is called at race
+ *  start and every time a new lap is started. Recomputing the path every
+ *  time will mean that the kart will not always take the same path, but
+ *  (potentially) vary from lap to lap. At this stage the decision is done
+ *  randomly. The AI could be improved by collecting more information about
+ *  each branch of a track, and selecting the 'appropriate' one (e.g. if the
+ *  AI is far ahead, chose a longer/slower path).
  */
 void AIBaseController::computePath()
 {
@@ -113,11 +173,11 @@ void AIBaseController::computePath()
         // might not have a successor (since the first and last edge of
         // a hidden shortcut is ignored). Since in the case that the AI
         // ends up on a short cut (e.g. by accident) and doesn't have an
-        // allowed way to drive, it should still be able to drive, so add 
+        // allowed way to drive, it should still be able to drive, so add
         // the non-AI successors of that node in this case.
         if(next.size()==0)
             QuadGraph::get()->getSuccessors(i, next, /*for_ai*/false);
-        // For now pick one part on random, which is not adjusted during the 
+        // For now pick one part on random, which is not adjusted during the
         // race. Long term statistics might be gathered to determine the
         // best way, potentially depending on race position etc.
         int r = rand();
@@ -132,10 +192,10 @@ void AIBaseController::computePath()
     const unsigned int look_ahead=10;
     // Now compute for each node in the graph the list of the next 'look_ahead'
     // graph nodes. This is the list of node that is tested in checkCrashes.
-    // If the look_ahead is too big, the AI can skip loops (see 
+    // If the look_ahead is too big, the AI can skip loops (see
     // QuadGraph::findRoadSector for details), if it's too short the AI won't
     // find too good a driveline. Note that in general this list should
-    // be computed recursively, but since the AI for now is using only 
+    // be computed recursively, but since the AI for now is using only
     // (randomly picked) path this is fine
     m_all_look_aheads.resize(QuadGraph::get()->getNumNodes());
     for(unsigned int i=0; i<QuadGraph::get()->getNumNodes(); i++)
@@ -160,7 +220,7 @@ void AIBaseController::computePath()
  */
 void AIBaseController::update(float dt)
 {
-	m_stuck_trigger_rescue = false;
+    m_stuck_trigger_rescue = false;
 
     if(QuadGraph::get())
     {
@@ -168,7 +228,7 @@ void AIBaseController::update(float dt)
         int old_node = m_track_node;
         if(m_track_node!=QuadGraph::UNKNOWN_SECTOR)
         {
-            QuadGraph::get()->findRoadSector(m_kart->getXYZ(), &m_track_node, 
+            QuadGraph::get()->findRoadSector(m_kart->getXYZ(), &m_track_node,
                 &m_all_look_aheads[m_track_node]);
         }
         // If we can't find a proper place on the track, to a broader search
@@ -183,66 +243,64 @@ void AIBaseController::update(float dt)
             m_next_node_index[m_track_node]==-1)
             m_track_node = old_node;
     }
-    if(m_controls->m_fire)
-        m_kart->onFirePressed();
 }   // update
 
 //-----------------------------------------------------------------------------
 /** This is called when the kart crashed with the terrain. This subroutine
  *  tries to detect if the AI is stuck by determining if a certain number
- *  of collisions happened in a certain amount of time, and if so rescues 
+ *  of collisions happened in a certain amount of time, and if so rescues
  *  the kart.
- *  \paran m Pointer to the material that was hit (NULL if no specific 
+ *  \paran m Pointer to the material that was hit (NULL if no specific
  *         material was used for the part of the track that was hit).
  */
 void AIBaseController::crashed(const Material *m)
 {
-	// Defines how many collision in what time will trigger a rescue.
-	// Note that typically it takes ~0.5 seconds for the AI to hit
-	// the track again if it is stuck (i.e. time for the push back plus
-	// time for the AI to accelerate and hit the terrain again).
-	const unsigned int NUM_COLLISION = 3;
-	const float COLLISION_TIME       = 1.5f;
+    // Defines how many collision in what time will trigger a rescue.
+    // Note that typically it takes ~0.5 seconds for the AI to hit
+    // the track again if it is stuck (i.e. time for the push back plus
+    // time for the AI to accelerate and hit the terrain again).
+    const unsigned int NUM_COLLISION = 3;
+    const float COLLISION_TIME       = 1.5f;
 
-	float time = World::getWorld()->getTime();
-	if(m_collision_times.size()==0)
-	{
-		m_collision_times.push_back(time);
-		return;
-	}
+    float time = World::getWorld()->getTime();
+    if(m_collision_times.size()==0)
+    {
+        m_collision_times.push_back(time);
+        return;
+    }
 
-	// Filter out multiple collisions report caused by single collision
-	// (bullet can report a collision more than once per frame, and 
-	// resolving it can take a few frames as well, causing more reported
-	// collisions to happen). The time of 0.2 seconds was experimentally
-	// found, typically it takes 0.5 seconds for a kart to be pushed back
-	// from the terrain and accelerate to hit the same terrain again.
-	if(time - m_collision_times.back() < 0.2f)
-		return;
+    // Filter out multiple collisions report caused by single collision
+    // (bullet can report a collision more than once per frame, and
+    // resolving it can take a few frames as well, causing more reported
+    // collisions to happen). The time of 0.2 seconds was experimentally
+    // found, typically it takes 0.5 seconds for a kart to be pushed back
+    // from the terrain and accelerate to hit the same terrain again.
+    if(time - m_collision_times.back() < 0.2f)
+        return;
 
 
-	// Remove all outdated entries, i.e. entries that are older than the
-	// collision time plus 1 second. Older entries must be deleted, 
-	// otherwise a collision that happened (say) 10 seconds ago could
-	// contribute to a stuck condition.
-	while(m_collision_times.size()>0 && 
-		   time - m_collision_times[0] > 1.0f+COLLISION_TIME)
-		   m_collision_times.erase(m_collision_times.begin());
+    // Remove all outdated entries, i.e. entries that are older than the
+    // collision time plus 1 second. Older entries must be deleted,
+    // otherwise a collision that happened (say) 10 seconds ago could
+    // contribute to a stuck condition.
+    while(m_collision_times.size()>0 &&
+           time - m_collision_times[0] > 1.0f+COLLISION_TIME)
+           m_collision_times.erase(m_collision_times.begin());
 
-	m_collision_times.push_back(time);
+    m_collision_times.push_back(time);
 
-	// Now detect if there are enough collision records in the
-	// specified time interval.
-	if(time - m_collision_times.front() > COLLISION_TIME
-		&& m_collision_times.size()>=NUM_COLLISION)
-	{
-		// We can't call m_kart->forceRescue here, since crased() is
-		// called during physics processing, and forceRescue() removes the
-		// chassis from the physics world, which would then cause
-		// inconsistencies and potentially a crash during the physics 
-		// processing. So only set a flag, which is tested during update.
-		m_stuck_trigger_rescue = true;
-	}
+    // Now detect if there are enough collision records in the
+    // specified time interval.
+    if(time - m_collision_times.front() > COLLISION_TIME
+        && m_collision_times.size()>=NUM_COLLISION)
+    {
+        // We can't call m_kart->forceRescue here, since crased() is
+        // called during physics processing, and forceRescue() removes the
+        // chassis from the physics world, which would then cause
+        // inconsistencies and potentially a crash during the physics
+        // processing. So only set a flag, which is tested during update.
+        m_stuck_trigger_rescue = true;
+    }
 
 }   // crashed(Material)
 
@@ -250,7 +308,7 @@ void AIBaseController::crashed(const Material *m)
 /** Returns the next sector of the given sector index. This is used
  *  for branches in the quad graph to select which way the AI kart should
  *  go. This is a very simple implementation that always returns the first
- *  successor.
+ *  successor, but it can be overridden to allow a better selection.
  *  \param index Index of the graph node for which the successor is searched.
  *  \return Returns the successor of this graph node.
  */
@@ -266,10 +324,10 @@ unsigned int AIBaseController::getNextSector(unsigned int index)
  ** attached to this kart into account by modifying the actual steer angle
  *  somewhat to simulate driving without seeing.
  */
-float AIBaseController::steerToAngle(const unsigned int sector, 
+float AIBaseController::steerToAngle(const unsigned int sector,
                                      const float add_angle)
 {
-    float angle = QuadGraph::get()->getAngleToNext(sector, 
+    float angle = QuadGraph::get()->getAngleToNext(sector,
                                                    getNextSector(sector));
 
     //Desired angle minus current angle equals how many angles to turn
@@ -303,12 +361,12 @@ float AIBaseController::steerToPoint(const Vec3 &point)
 
     // The point the kart is aiming at can be reached 'incorrectly' if the
     // point is below the y=x line: Instead of aiming at that point directly
-    // the point will be reached on its way 'back' after a more than 90 
+    // the point will be reached on its way 'back' after a more than 90
     // degree turn in the circle, i.e.:
     // |                 So the point p (belolw the y=x line) can not be
     // |  ---\           reached on any circle directly, so it is reached
     // | /    \          on the indicated way. Since this is not the way
-    // |/      p         we expect a kart to drive (it will result in the 
+    // |/      p         we expect a kart to drive (it will result in the
     // +--------------   kart doing slaloms, not driving straight), the
     // kart will trigger skidding to allow for sharper turns, and hopefully
     // the situation will change so that the point p can then be reached
@@ -331,22 +389,22 @@ float AIBaseController::steerToPoint(const Vec3 &point)
     // (0,0) facing straight ahead. The center of the rotation is then
     // on the X axis and can be computed by the fact that the distance
     // to the kart and to the point to aim at must be the same:
-    // r*r = (r-x)*(r-x) + y*y  
+    // r*r = (r-x)*(r-x) + y*y
     // where r is the radius (= position on the X axis), and x, y are the
     // local coordinates of the point to aim at. Solving for r
     // results in r = (x*x+y*y)/2x
     float radius = (lc.getX()*lc.getX() + lc.getZ()*lc.getZ())
-                 / (2.0f*lc.getX());    
+                 / (2.0f*lc.getX());
 
     // sin(steern_angle) = wheel_base / radius:
     float sin_steer_angle = m_kart->getKartProperties()->getWheelBase()/radius;
 
-    // If the wheel base is too long (i.e. the minimum radius is too large 
+    // If the wheel base is too long (i.e. the minimum radius is too large
     // to actually reach the target), make sure that skidding is used
     if(sin_steer_angle <= -1.0f)
         return -m_kart->getMaxSteerAngle()
                *m_ai_properties->m_skidding_threshold-0.1f;
-    if(sin_steer_angle >=  1.0f) 
+    if(sin_steer_angle >=  1.0f)
         return  m_kart->getMaxSteerAngle()
                *m_ai_properties->m_skidding_threshold+0.1f;
     float steer_angle     = asin(sin_steer_angle);
@@ -381,11 +439,11 @@ float AIBaseController::normalizeAngle(float angle)
 }   // normalizeAngle
 
 //-----------------------------------------------------------------------------
-/** Converts the steering angle to a lr steering in the range of -1 to 1. 
- *  If the steering angle is too great, it will also trigger skidding. This 
+/** Converts the steering angle to a lr steering in the range of -1 to 1.
+ *  If the steering angle is too great, it will also trigger skidding. This
  *  function uses a 'time till full steer' value specifying the time it takes
- *  for the wheel to reach full left/right steering similar to player karts 
- *  when using a digital input device. The parameter is defined in the kart 
+ *  for the wheel to reach full left/right steering similar to player karts
+ *  when using a digital input device. The parameter is defined in the kart
  *  properties and helps somewhat to make AI karts more 'pushable' (since
  *  otherwise the karts counter-steer to fast).
  *  It also takes the effect of a plunger into account by restricting the
@@ -399,8 +457,8 @@ void AIBaseController::setSteering(float angle, float dt)
     if(!doSkid(steer_fraction))
         m_controls->m_skid = KartControl::SC_NONE;
     else
-        m_controls->m_skid = steer_fraction > 0 ? KartControl::SC_RIGHT 
-                                                : KartControl::SC_LEFT; 
+        m_controls->m_skid = steer_fraction > 0 ? KartControl::SC_RIGHT
+                                                : KartControl::SC_LEFT;
     float old_steer      = m_controls->m_steer;
 
     if     (steer_fraction >  1.0f) steer_fraction =  1.0f;
@@ -411,17 +469,17 @@ void AIBaseController::setSteering(float angle, float dt)
         if     (steer_fraction >  0.5f) steer_fraction =  0.5f;
         else if(steer_fraction < -0.5f) steer_fraction = -0.5f;
     }
-    
+
     // The AI has its own 'time full steer' value (which is the time
     float max_steer_change = dt/m_ai_properties->m_time_full_steer;
     if(old_steer < steer_fraction)
     {
-        m_controls->m_steer = (old_steer+max_steer_change > steer_fraction) 
+        m_controls->m_steer = (old_steer+max_steer_change > steer_fraction)
                            ? steer_fraction : old_steer+max_steer_change;
     }
     else
     {
-        m_controls->m_steer = (old_steer-max_steer_change < steer_fraction) 
+        m_controls->m_steer = (old_steer-max_steer_change < steer_fraction)
                            ? steer_fraction : old_steer-max_steer_change;
     }
 }   // setSteering
@@ -430,7 +488,7 @@ void AIBaseController::setSteering(float angle, float dt)
 /** Determines if the kart should skid. The base implementation enables
  *  skidding if a sharp turn is needed (which is for the old skidding
  *  implementation).
- *  \param steer_fraction The steering fraction as computed by the 
+ *  \param steer_fraction The steering fraction as computed by the
  *          AIBaseController.
  *  \return True if the kart should skid.
  */
@@ -451,10 +509,10 @@ bool AIBaseController::doSkid(float steer_fraction)
     return fabsf(steer_fraction)>=m_ai_properties->m_skidding_threshold;
 }   // doSkid
 // ------------------------------------------------------------------------
-/** Certain AI levels will not receive a slipstream bonus in order to 
+/** Certain AI levels will not receive a slipstream bonus in order to
  *  be not as hard.
  */
-bool AIBaseController::disableSlipstreamBonus() const 
+bool AIBaseController::disableSlipstreamBonus() const
 {
     return m_ai_properties->disableSlipstreamUsage();
 }   // disableSlipstreamBonus
