@@ -1,7 +1,7 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
-//  Copyright (C) 2006-2009 Joerg Henrichs, Steve Baker
+//  Copyright (C) 2004-2013 Steve Baker <sjbaker1@airmail.net>
+//  Copyright (C) 2006-2013 Joerg Henrichs, Steve Baker
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 #include "config/stk_config.hpp"
 #include "graphics/camera.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/post_processing.hpp"
 #include "input/input_manager.hpp"
 #include "items/attachment.hpp"
 #include "items/item.hpp"
@@ -37,6 +38,7 @@
 #include "race/history.hpp"
 #include "states_screens/race_gui_base.hpp"
 #include "utils/constants.hpp"
+#include "utils/log.hpp"
 #include "utils/translation.hpp"
 
 /** The constructor for a player kart.
@@ -46,23 +48,23 @@
  *  \param init_pos The start coordinates and heading of the kart.
  *  \param player_index  Index of the player kart.
  */
-PlayerController::PlayerController(AbstractKart *kart, 
-                                   StateManager::ActivePlayer *player, 
-                                   unsigned int player_index)      
+PlayerController::PlayerController(AbstractKart *kart,
+                                   StateManager::ActivePlayer *player,
+                                   unsigned int player_index)
                 : Controller(kart)
 {
     assert(player != NULL);
     m_player       = player;
     m_player->setKart(kart);
     m_penalty_time = 0.0f;
-    kart->setCamera(new Camera(player_index, kart));
-    kart->getCamera()->setMode(Camera::CM_NORMAL);
-
-    m_bzzt_sound  = sfx_manager->createSoundSource( "bzzt" );
-    m_wee_sound   = sfx_manager->createSoundSource( "wee"  );
-    m_ugh_sound   = sfx_manager->createSoundSource( "ugh"  );
-    m_grab_sound  = sfx_manager->createSoundSource( "grab_collectable" );
-    m_full_sound  = sfx_manager->createSoundSource( "energy_bar_full" );
+    // Keep a pointer to the camera to remove the need to search for
+    // the right camera once per frame later.
+    m_camera       = Camera::createCamera(kart);
+    m_bzzt_sound   = sfx_manager->createSoundSource( "bzzt" );
+    m_wee_sound    = sfx_manager->createSoundSource( "wee"  );
+    m_ugh_sound    = sfx_manager->createSoundSource( "ugh"  );
+    m_grab_sound   = sfx_manager->createSoundSource( "grab_collectable" );
+    m_full_sound   = sfx_manager->createSoundSource( "energy_bar_full" );
 
     reset();
 }   // PlayerController
@@ -95,7 +97,7 @@ void PlayerController::reset()
 
 // ----------------------------------------------------------------------------
 /** Resets the state of control keys. This is used after the in-game menu to
- *  avoid that any keys pressed at the time the menu is opened are still 
+ *  avoid that any keys pressed at the time the menu is opened are still
  *  considered to be pressed.
  */
 void PlayerController::resetInputState()
@@ -111,9 +113,9 @@ void PlayerController::resetInputState()
 
 // ----------------------------------------------------------------------------
 /** This function interprets a kart action and value, and set the corresponding
- *  entries in the kart control data structure. This function handles esp. 
+ *  entries in the kart control data structure. This function handles esp.
  *  cases like 'press left, press right, release right' - in this case after
- *  releasing right, the steering must switch to left again. Similarly it 
+ *  releasing right, the steering must switch to left again. Similarly it
  *  handles 'press left, press right, release left' (in which case still
  *  right must be selected). Similarly for braking and acceleration.
  * \param action  The action to be executed.
@@ -193,13 +195,7 @@ void PlayerController::action(PlayerAction action, int value)
         break;
     case PA_FIRE:
     {
-        bool wasFirePressed = m_controls->m_fire;
         m_controls->m_fire = (value!=0);
-        if (m_controls->m_fire && !wasFirePressed)
-        {
-            World::getWorld()->onFirePressed(this);
-            m_kart->onFirePressed();
-        }
         break;
     }
     case PA_LOOK_BACK:
@@ -219,7 +215,7 @@ void PlayerController::action(PlayerAction action, int value)
     case PA_PAUSE_RACE:
         if (value != 0) StateManager::get()->escapePressed();
         break;
-    default: 
+    default:
        break;
     }
 
@@ -232,7 +228,7 @@ void PlayerController::steer(float dt, int steer_val)
 {
     if(UserConfigParams::m_gamepad_debug)
     {
-        printf("steering: steer_val %d ", steer_val);
+        Log::debug("PlayerController", "steering: steer_val %d ", steer_val);
         RaceGUIBase* gui_base = World::getWorld()->getRaceGUI();
         gui_base->clearAllMessages();
         gui_base->addMessage(StringUtils::insertValues(L"steer_val %i", steer_val), m_kart, 1.0f,
@@ -246,7 +242,7 @@ void PlayerController::steer(float dt, int steer_val)
         m_controls->m_steer = 0;
     }
 
-    
+
     // Amount the steering is changed for digital devices.
     // If the steering is 'back to straight', a different steering
     // change speed is used.
@@ -285,7 +281,7 @@ void PlayerController::steer(float dt, int steer_val)
     }   // no key is pressed
     if(UserConfigParams::m_gamepad_debug)
     {
-        printf("  set to: %f\n", m_controls->m_steer);
+        Log::debug("PlayerController", "  set to: %f\n", m_controls->m_steer);
     }
 
     m_controls->m_steer = std::min(1.0f, std::max(-1.0f, m_controls->m_steer));
@@ -306,8 +302,15 @@ void PlayerController::skidBonusTriggered()
  */
 void PlayerController::update(float dt)
 {
-    // Don't do steering if it's replay. In position only replay it doesn't 
-    // matter, but if it's physics replay the gradual steering causes 
+    if (UserConfigParams::m_gamepad_debug)
+    {
+        // Print a dividing line so that it's easier to see which events
+        // get received in which order in the one frame.
+        Log::debug("PlayerController", "irr_driver", "-------------------------------------\n");
+    }
+
+    // Don't do steering if it's replay. In position only replay it doesn't
+    // matter, but if it's physics replay the gradual steering causes
     // incorrect results, since the stored values are already adjusted.
     if (!history->replayHistory())
         steer(dt, m_steer_val);
@@ -349,25 +352,19 @@ void PlayerController::update(float dt)
         return;
     }
 
-    if (m_controls->m_fire && !m_kart->getKartAnimation())
-    {
-        if (m_kart->getPowerup()->getType()==PowerupManager::POWERUP_NOTHING) 
-            m_kart->beep();
-    }
-
     // look backward when the player requests or
     // if automatic reverse camera is active
-    if (m_kart->getCamera()->getMode() != Camera::CM_FINAL)
+    if (m_camera->getMode() != Camera::CM_FINAL)
     {
-        if (m_controls->m_look_back || (UserConfigParams::m_reverse_look_threshold>0 && 
+        if (m_controls->m_look_back || (UserConfigParams::m_reverse_look_threshold>0 &&
             m_kart->getSpeed()<-UserConfigParams::m_reverse_look_threshold))
         {
-            m_kart->getCamera()->setMode(Camera::CM_REVERSE);
+            m_camera->setMode(Camera::CM_REVERSE);
         }
         else
         {
-            if (m_kart->getCamera()->getMode() == Camera::CM_REVERSE)
-                m_kart->getCamera()->setMode(Camera::CM_NORMAL);
+            if (m_camera->getMode() == Camera::CM_REVERSE)
+                m_camera->setMode(Camera::CM_NORMAL);
         }
     }
 
@@ -382,7 +379,7 @@ void PlayerController::update(float dt)
         new RescueAnimation(m_kart);
         m_controls->m_rescue=false;
     }
-    if (m_kart->getKartAnimation() && 
+    if (m_kart->getKartAnimation() &&
         m_kart->getAttachment()->getType() != Attachment::ATTACH_TINYTUX)
     {
         m_bzzt_sound->play();
@@ -414,9 +411,12 @@ void PlayerController::setPosition(int p)
 //-----------------------------------------------------------------------------
 /** Called when a kart finishes race.
  *  /param time Finishing time for this kart.
- */
+ d*/
 void PlayerController::finishedRace(float time)
 {
+    // This will implicitely trigger setting the first end camera to be active
+    m_camera->setMode(Camera::CM_FINAL);
+
 }   // finishedRace
 
 //-----------------------------------------------------------------------------
@@ -434,7 +434,7 @@ void PlayerController::handleZipper(bool play_sound)
     }
 
     // Apply the motion blur according to the speed of the kart
-    irr_driver->getPostProcessing()->giveBoost();
+    irr_driver->getPostProcessing()->giveBoost(m_camera->getIndex());
 
     m_kart->showZipperFire();
 
@@ -444,9 +444,9 @@ void PlayerController::handleZipper(bool play_sound)
 /** Called when a kart hits an item.
  *  \param item Item that was collected.
  *  \param add_info Additional info to be used then handling the item. If
- *                  this is -1 (default), the item type is selected 
- *                  randomly. Otherwise it contains the powerup or 
- *                  attachment for the kart. This is used in network mode to 
+ *                  this is -1 (default), the item type is selected
+ *                  randomly. Otherwise it contains the powerup or
+ *                  attachment for the kart. This is used in network mode to
  *                  let the server determine the powerup/attachment for
  *                  the clients.
  */
@@ -480,7 +480,7 @@ void PlayerController::collectedItem(const Item &item, int add_info, float old_e
             break;
         default:
             m_grab_sound->play();
-            break; 
+            break;
         }
     }
 }   // collectedItem
