@@ -1,7 +1,7 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
-//  Copyright (C) 2006 SuperTuxKart-Team, Joerg Henrichs, Steve Baker
+//  Copyright (C) 2004-2013 Steve Baker <sjbaker1@airmail.net>
+//  Copyright (C) 2006-2013 SuperTuxKart-Team, Joerg Henrichs, Steve Baker
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -38,15 +38,14 @@ class btKart;
 class btUprightConstraint;
 
 class Attachment;
-class Camera;
 class Controller;
 class Item;
 class AbstractKartAnimation;
+class HitEffect;
 class KartGFX;
 class MaxSpeed;
 class ParticleEmitter;
 class ParticleKind;
-class Rain;
 class SFXBase;
 class Shadow;
 class Skidding;
@@ -54,8 +53,8 @@ class SkidMarks;
 class SlipStream;
 class Stars;
 
-/** The main kart class. All type of karts are of this object, but with 
- *  different controllers. The controllers are what turn a kart into a 
+/** The main kart class. All type of karts are of this object, but with
+ *  different controllers. The controllers are what turn a kart into a
  *  player kart (i.e. the controller handle input), or an AI kart (the
  *  controller runs the AI code to set steering etc).
  *  Kart has two base classes: the most important one is moveable (which
@@ -78,17 +77,17 @@ private:
 
     /** True if kart is flying (for debug purposes only). */
     bool m_flying;
-    
+
     /** Set when hitting bubblegum */
     bool m_has_caught_nolok_bubblegum;
-    
+
     /** Reset position. */
     btTransform  m_reset_transform;
 
     /** This object handles all skidding. */
     Skidding *m_skidding;
 
-    /** The main controller of this object, used for driving. This 
+    /** The main controller of this object, used for driving. This
      *  controller is used to run the kart. It will be replaced
      *  with an end kart controller when the kart finishes the race. */
     Controller  *m_controller;
@@ -109,26 +108,17 @@ private:
 
     /** For stars rotating around head effect */
     Stars *m_stars_effect;
-    
 
-protected:       // Used by the AI atm    
-    /** The camera for each kart. Not all karts have cameras (e.g. AI karts
-     *  usually don't), but there are exceptions: e.g. after the end of a
-     *  race an AI kart is replacing the kart for a player.
-     */
-    Camera      *m_camera;
-    
-private:
-    /** True if the kart hasn't moved since 'ready-set-go' - used to 
+    /** True if the kart hasn't moved since 'ready-set-go' - used to
      *  determine startup boost. */
     bool         m_has_started;
 
     /**<Maximum engine rpm's for the current gear*/
     float        m_max_gear_rpm;
 
-    /** A short time after a collision acceleration is disabled to allow 
+    /** A short time after a collision acceleration is disabled to allow
      *  the karts to bounce back*/
-    float        m_bounce_back_time;   
+    float        m_bounce_back_time;
 
     /** Time a kart is invulnerable. */
     float        m_invulnerable_time;
@@ -136,12 +126,20 @@ private:
     /** How long a kart is being squashed. If this is >0
      *  the kart is squashed. */
     float        m_squash_time;
-    
-    /** If > 0 then bubble gum effect is on */
+
+    /** Current leaning of the kart. */
+    float        m_current_lean;
+
+    /** If > 0 then bubble gum effect is on. This is the sliding when hitting a gum on the floor, not the shield. */
     float        m_bubblegum_time;
 
     /** The torque to apply after hitting a bubble gum. */
     float        m_bubblegum_torque;
+    
+    /** True if fire button was pushed and not released */
+    bool         m_fire_clicked;
+
+
 
     // Bullet physics parameters
     // -------------------------
@@ -156,15 +154,21 @@ private:
 
     // Graphical effects
     // -----------------
+    /** Time a kart is jumping. */
+    float            m_jump_time;
+    
+    /** Is time flying activated */
+    bool             m_is_jumping;
+    
     /** The shadow of a kart. */
     Shadow          *m_shadow;
-    
+
     /** If a kart is flying, the shadow is disabled (since it is
      *  stuck to the kart, i.e. the shadow would be flying, too). */
     bool             m_shadow_enabled;
-    
+
     ParticleEmitter *m_sky_particles_emitter;
-    
+
     /** All particle effects. */
     KartGFX         *m_kart_gfx;
 
@@ -173,20 +177,12 @@ private:
 
     /** Handles all slipstreaming. */
     SlipStream      *m_slipstream;
-    
-    Rain            *m_rain;
 
     /** Rotation compared to the start position, same for all wheels */
     float           m_wheel_rotation;
 
     /** Rotation change in the last time delta, same for all wheels */
     float           m_wheel_rotation_dt;
-    
-    /** For each wheel it stores the suspension length after the karts are at 
-     *  the start position, i.e. the suspension will be somewhat compressed.
-     *  The bullet suspensionRestLength is the value when the suspension is not
-     *  at all compressed. */
-    float           m_default_suspension_length[4];
 
     /** The skidmarks object for this kart. */
     SkidMarks      *m_skidmarks;
@@ -194,7 +190,7 @@ private:
     float           m_finish_time;
     bool            m_finished_race;
 
-    /** When a kart has its view blocked by the plunger, this variable will be 
+    /** When a kart has its view blocked by the plunger, this variable will be
      *  > 0 the number it contains is the time left before removing plunger. */
     float         m_view_blocked_by_plunger;
     float         m_speed;
@@ -209,7 +205,11 @@ private:
     SFXBase      *m_previous_terrain_sound;
     SFXBase      *m_skid_sound;
     SFXBase      *m_goo_sound;
+    SFXBase      *m_boing_sound;
     float         m_time_last_crash;
+    
+    /** To prevent using nitro in too short bursts */
+    float         m_min_nitro_time;
 
     void          updatePhysics(float dt);
     void          handleMaterialSFX(const Material *material);
@@ -220,32 +220,30 @@ private:
     void          updateEngineSFX();
     void          updateNitro(float dt);
     float         getActualWheelForce();
-    void          crashed();
-    void          loadData(RaceManager::KartType type, bool is_first_kart,
-                           bool animatedModel);
-    
+    void          crashed(const Material* m, AbstractKart *k);
+    void          loadData(RaceManager::KartType type, bool animatedModel);
+
 public:
                    Kart(const std::string& ident, unsigned int world_kart_id,
                         int position, const btTransform& init_transform);
     virtual       ~Kart();
-    virtual void   init(RaceManager::KartType type, bool is_first_kart);
-    virtual void   updateGraphics(float dt, const Vec3& off_xyz,  
+    virtual void   init(RaceManager::KartType type);
+    virtual void   updateGraphics(float dt, const Vec3& off_xyz,
                                   const btQuaternion& off_rotation);
     virtual void   createPhysics    ();
     virtual void   updateWeight     ();
     virtual bool   isInRest         () const;
-    virtual void   setSuspensionLength();
     virtual void   applyEngineForce (float force);
 
     virtual void flyUp();
     virtual void flyDown();
-    
+
     virtual void   startEngineSFX   ();
     virtual void   adjustSpeed      (float f);
     virtual void   increaseMaxSpeed(unsigned int category, float add_speed,
-                                    float engine_force, float duration, 
+                                    float engine_force, float duration,
                                     float fade_out_time);
-    virtual void   setSlowdown(unsigned int category, float max_speed_fraction, 
+    virtual void   setSlowdown(unsigned int category, float max_speed_fraction,
                                float fade_in_time);
     virtual float getSpeedIncreaseTimeLeft(unsigned int category) const;
     virtual void  collectedItem(Item *item, int random_attachment);
@@ -255,7 +253,7 @@ public:
     virtual float getTerrainPitch(float heading) const;
 
     virtual void   reset            ();
-    virtual void   handleZipper     (const Material *m=NULL, 
+    virtual void   handleZipper     (const Material *m=NULL,
                                      bool play_sound=false);
     virtual void   setSquash        (float time, float slowdown);
 
@@ -290,18 +288,6 @@ public:
     /** Returns a points to this kart's graphical effects. */
     KartGFX*       getKartGFX()                   { return m_kart_gfx;        }
     // ------------------------------------------------------------------------
-    /** Returns the camera of this kart (or NULL if no camera is attached
-     *  to this kart). */
-    virtual Camera* getCamera         ()       {return m_camera;}
-    // ------------------------------------------------------------------------
-    /** Returns the camera of this kart (or NULL if no camera is attached
-     *  to this kart) - const version. */
-    virtual const Camera*  getCamera  () const {return m_camera;}
-    // ------------------------------------------------------------------------
-    /** Sets the camera for this kart. Takes ownership of the camera and
-     *  will delete it. */
-    virtual void setCamera(Camera *camera);
-    // ------------------------------------------------------------------------
     /** Returns the remaining collected energy. */
     virtual float  getEnergy           () const { return m_collected_energy; }
     // ------------------------------------------------------------------------
@@ -323,13 +309,7 @@ public:
     // ------------------------------------------------------------------------
     /** Sets that the view is blocked by a plunger. The duration depends on
      *  the difficulty, see KartPorperties getPlungerInFaceTime. */
-    virtual void   blockViewWithPlunger()   
-    { 
-        // Avoid that a plunger extends the plunger time
-        if(m_view_blocked_by_plunger<=0)
-            m_view_blocked_by_plunger = 
-                                   m_kart_properties->getPlungerInFaceTime();
-    }   // blockViewWithPlunger
+    virtual void   blockViewWithPlunger();
     // -------------------------------------------------------------------------
     /** Returns a bullet transform object located at the kart's position
         and oriented in the direction the kart is going. Can be useful
@@ -337,14 +317,14 @@ public:
     virtual btTransform getAlignedTransform(const float customPitch=-1);
     // -------------------------------------------------------------------------
     /** Returns the color used for this kart. */
-    const video::SColor &getColor() const 
+    const video::SColor &getColor() const
                                         {return m_kart_properties->getColor();}
     // ------------------------------------------------------------------------
     /** Returns the time till full steering is reached for this kart.
      *  \param steer Current steer value (must be >=0), on which the time till
      *         full steer depends. */
-    virtual float getTimeFullSteer(float steer) const 
-    { 
+    virtual float getTimeFullSteer(float steer) const
+    {
         return m_kart_properties->getTimeFullSteer(steer);
     }   // getTimeFullSteer
     // ------------------------------------------------------------------------
@@ -365,7 +345,7 @@ public:
     virtual btKart    *getVehicle      () const {return m_vehicle;                }
     // ------------------------------------------------------------------------
     /** Returns the upright constraint for this kart. */
-    virtual btUprightConstraint *getUprightConstraint() const 
+    virtual btUprightConstraint *getUprightConstraint() const
                                                   {return m_uprightConstraint;}
     // ------------------------------------------------------------------------
     /** Returns the speed of the kart in meters/second. */
@@ -408,14 +388,32 @@ public:
     virtual void eliminate();
     // ------------------------------------------------------------------------
     /** Makes a kart invulnerable for a certain amount of time. */
-    virtual void  setInvulnerableTime(float t) { m_invulnerable_time = t; };
+    virtual void  setInvulnerableTime(float t) { m_invulnerable_time = t; }
     // ------------------------------------------------------------------------
     /** Returns if the kart is invulnerable. */
     virtual bool   isInvulnerable() const { return m_invulnerable_time > 0; }
     // ------------------------------------------------------------------------
+    /** Enables a kart shield protection for a certain amount of time. */
+    virtual void setShieldTime(float t);
+    // ------------------------------------------------------------------------
+    /** Returns if the kart is protected by a shield. */
+    virtual bool isShielded() const;
+    // ------------------------------------------------------------------------
+    /** Returns the remaining time the kart is protected by a shield. */
+    virtual float getShieldTime() const;
+    // ------------------------------------------------------------------------
+    /** Decreases the kart's shield time. */
+    virtual void decreaseShieldTime();
+    // ------------------------------------------------------------------------
+
     /** Sets the energy the kart has collected. */
     virtual void   setEnergy(float val) { m_collected_energy = val; }
     // ------------------------------------------------------------------------
+    /** Return whether nitro is being used despite the nitro button not being
+     *  pressed due to minimal use time requirements
+     */
+    virtual float isOnMinNitroTime() const { return m_min_nitro_time > 0.0f; }
+    // ------------------------------------------------------------------------
     /** Returns if the kart is currently being squashed. */
     virtual bool   isSquashed() const { return m_squash_time >0; }
     // ------------------------------------------------------------------------
@@ -425,8 +423,6 @@ public:
     /** Returns the terrain info oject. */
     TerrainInfo *getTerrainInfo() { return m_terrain_info; }
     // ------------------------------------------------------------------------
-    virtual void onFirePressed();
-    // ------------------------------------------------------------------------
     virtual void setOnScreenText(const wchar_t *text);
 };   // Kart
 
