@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -18,12 +18,8 @@
 
 #include "karts/kart_properties.hpp"
 
-#include <iostream>
-#include <stdexcept>
-#include <string>
-
+#include "addons/addon.hpp"
 #include "config/stk_config.hpp"
-#include "config/user_config.hpp"
 #include "graphics/irr_driver.hpp"
 #include "graphics/material_manager.hpp"
 #include "io/file_manager.hpp"
@@ -33,9 +29,15 @@
 #include "modes/world.hpp"
 #include "io/xml_node.hpp"
 #include "utils/constants.hpp"
+#include "utils/log.hpp"
 #include "utils/string_utils.hpp"
 #include "utils/translation.hpp"
 
+#include <iostream>
+#include <stdexcept>
+#include <string>
+
+
 float KartProperties::UNDEFINED = -99.9f;
 
 /** The constructor initialises all values with invalid values. It can later
@@ -60,37 +62,41 @@ KartProperties::KartProperties(const std::string &filename)
 
     // Set all other values to undefined, so that it can later be tested
     // if everything is defined properly.
-    m_mass = m_brake_factor = m_engine_power[0] = m_engine_power[1] =
-        m_engine_power[2] = m_max_speed[0] = m_max_speed[1] = m_max_speed[2] =
+    m_mass = m_brake_factor =
         m_time_reset_steer = m_nitro_consumption = m_nitro_engine_force =
-        m_nitro_small_container = m_nitro_big_container = m_nitro_max = 
+        m_nitro_small_container = m_nitro_big_container = m_nitro_max =
         m_nitro_max_speed_increase = m_nitro_duration = m_nitro_fade_out_time =
         m_suspension_stiffness = m_wheel_damping_relaxation = m_wheel_base =
         m_wheel_damping_compression = m_friction_slip = m_roll_influence =
         m_wheel_radius = m_chassis_linear_damping = m_max_suspension_force =
         m_chassis_angular_damping = m_suspension_rest =
-        m_max_speed_reverse_ratio = 
-        m_rescue_vert_offset = m_upright_tolerance = 
-        m_collision_terrain_impulse = m_collision_impulse = m_restitution =
-        m_collision_impulse_time = 
+        m_max_speed_reverse_ratio = m_rescue_vert_offset =
+        m_upright_tolerance = m_collision_terrain_impulse =
+        m_collision_impulse = m_restitution = m_collision_impulse_time =
         m_upright_max_force = m_suspension_travel_cm =
-        m_track_connection_accel =
-        m_rubber_band_max_length = m_rubber_band_force =
-        m_rubber_band_duration = m_rubber_band_speed_increase = 
-        m_rubber_band_fade_out_time = m_plunger_in_face_duration[0] =
-        m_plunger_in_face_duration[1] = m_plunger_in_face_duration[2] =
+        m_track_connection_accel = m_rubber_band_max_length =
+        m_rubber_band_force = m_rubber_band_duration =
+        m_rubber_band_speed_increase = m_rubber_band_fade_out_time =
         m_zipper_time = m_zipper_force = m_zipper_speed_gain =
-        m_zipper_max_speed_increase = m_zipper_fade_out_time =       
+        m_zipper_max_speed_increase = m_zipper_fade_out_time =
         m_slipstream_length = m_slipstream_width = m_slipstream_collect_time =
         m_slipstream_use_time = m_slipstream_add_power =
         m_slipstream_min_speed = m_slipstream_max_speed_increase =
         m_slipstream_duration = m_slipstream_fade_out_time =
-        m_camera_distance = m_camera_forward_up_angle = 
+        m_camera_distance = m_camera_forward_up_angle =
         m_camera_backward_up_angle = m_explosion_invulnerability_time =
         m_rescue_time = m_rescue_height = m_explosion_time =
-        m_explosion_radius =
+        m_explosion_radius = m_max_lean = m_lean_speed =
         m_swatter_distance2 = m_swatter_duration = m_squash_slowdown =
-        m_squash_duration = m_downward_impulse_factor = UNDEFINED;
+        m_squash_duration = m_downward_impulse_factor = 
+        m_bubblegum_fade_in_time = m_bubblegum_speed_fraction =
+        m_bubblegum_time = m_bubblegum_torque = m_jump_animation_time = 
+            UNDEFINED;
+
+    m_engine_power.resize(RaceManager::DIFFICULTY_COUNT, UNDEFINED);
+    m_max_speed.resize(RaceManager::DIFFICULTY_COUNT, UNDEFINED);
+    m_plunger_in_face_duration.resize(RaceManager::DIFFICULTY_COUNT,
+                                      UNDEFINED);
 
     m_terrain_impulse_type   = IMPULSE_NONE;
     m_gravity_center_shift   = Vec3(UNDEFINED);
@@ -102,8 +108,9 @@ KartProperties::KartProperties(const std::string &filename)
     m_engine_sfx_type        = "engine_small";
     m_kart_model             = NULL;
     m_has_rand_wheels        = false;
+    m_nitro_min_consumption  = 0.53f;
     // The default constructor for stk_config uses filename=""
-    if (filename != "") 
+    if (filename != "")
     {
         m_skidding_properties = NULL;
         for(unsigned int i=0; i<RaceManager::DIFFICULTY_COUNT; i++)
@@ -171,8 +178,13 @@ void KartProperties::load(const std::string &filename, const std::string &node)
     m_kart_model  = new KartModel(/*is_master*/true);
 
     const XMLNode * root = 0;
-    m_root  = StringUtils::getPath(filename);
+    m_root  = StringUtils::getPath(filename)+"/";
     m_ident = StringUtils::getBasename(StringUtils::getPath(filename));
+    // If this is an addon kart, add "addon_" to the identifier - just in
+    // case that an addon kart has the same directory name (and therefore
+    // identifier) as an included kart.
+    if(Addon::isAddon(filename))
+        m_ident = Addon::createAddonId(m_ident);
     try
     {
         root = new XMLNode(filename);
@@ -181,7 +193,7 @@ void KartProperties::load(const std::string &filename, const std::string &node)
             std::ostringstream msg;
             msg << "Couldn't load kart properties '" << filename <<
                 "': no kart node.";
-            
+
             delete m_kart_model;
             throw std::runtime_error(msg.str());
         }
@@ -189,9 +201,9 @@ void KartProperties::load(const std::string &filename, const std::string &node)
     }
     catch(std::exception& err)
     {
-        fprintf(stderr, "Error while parsing KartProperties '%s':\n",
-                filename.c_str());
-        fprintf(stderr, "%s\n", err.what());
+        Log::error("KartProperties", "Error while parsing KartProperties '%s':\n",
+                   filename.c_str());
+        Log::error("KartProperties", "%s\n", err.what());
     }
     if(root) delete root;
 
@@ -201,23 +213,28 @@ void KartProperties::load(const std::string &filename, const std::string &node)
 
 
     // Load material
-    std::string materials_file = m_root+"/materials.xml";
+    std::string materials_file = m_root+"materials.xml";
     file_manager->pushModelSearchPath  (m_root);
     file_manager->pushTextureSearchPath(m_root);
 
+    irr_driver->setTextureErrorMessage("Error while loading kart '%s':",
+                                       m_name);
+
     // addShared makes sure that these textures/material infos stay in memory
     material_manager->addSharedMaterial(materials_file);
 
-    m_icon_file = m_root+"/"+m_icon_file;
+    m_icon_file = m_root+m_icon_file;
 
     // Make permanent is important, since otherwise icons can get deleted
     // (e.g. when freeing temp. materials from a track, the last icon
     //  would get deleted, too.
     m_icon_material = material_manager->getMaterial(m_icon_file,
-                                                    /*is_full+path*/true, 
-                                                    /*make_permanent*/true);
+                                                    /*is_full_path*/true,
+                                                    /*make_permanent*/true,
+                                                    /*complain_if_not_found*/true,
+                                                    /*strip_path*/false);
     if(m_minimap_icon_file!="")
-        m_minimap_icon = irr_driver->getTexture(m_root+"/"+m_minimap_icon_file);
+        m_minimap_icon = irr_driver->getTexture(m_root+m_minimap_icon_file);
     else
         m_minimap_icon = NULL;
 
@@ -234,7 +251,7 @@ void KartProperties::load(const std::string &filename, const std::string &node)
             throw std::runtime_error("Cannot load kart models");
         }
     }
-    
+
     if(m_gravity_center_shift.getX()==UNDEFINED)
     {
         m_gravity_center_shift.setX(0);
@@ -250,11 +267,13 @@ void KartProperties::load(const std::string &filename, const std::string &node)
     // Now convert the turn radius into turn angle:
     for(unsigned int i=0; i<m_turn_angle_at_speed.size(); i++)
     {
-        m_turn_angle_at_speed.setY( i, 
+        m_turn_angle_at_speed.setY( i,
                             sin(m_wheel_base/m_turn_angle_at_speed.getY(i)) );
     }
 
     m_shadow_texture = irr_driver->getTexture(m_shadow_file);
+
+    irr_driver->unsetTextureErrorMessage();
     file_manager->popTextureSearchPath();
     file_manager->popModelSearchPath();
 
@@ -267,11 +286,11 @@ void KartProperties::load(const std::string &filename, const std::string &node)
 void KartProperties::getAllData(const XMLNode * root)
 {
     root->get("version", &m_version);
-    
+
     root->get("name",              &m_name             );
-    
+
     root->get("icon-file",         &m_icon_file        );
-    
+
     root->get("minimap-icon-file", &m_minimap_icon_file);
 
     root->get("shadow-file",       &m_shadow_file      );
@@ -300,6 +319,15 @@ void KartProperties::getAllData(const XMLNode * root)
         nitro_node->get("duration",           &m_nitro_duration          );
         nitro_node->get("fade-out-time",      &m_nitro_fade_out_time     );
         nitro_node->get("max",                &m_nitro_max               );
+        nitro_node->get("min-consumption-time", &m_nitro_min_consumption   );
+    }
+
+    if(const XMLNode *bubble_node = root->getNode("bubblegum"))
+    {
+        bubble_node->get("time",           &m_bubblegum_time          );
+        bubble_node->get("speed-fraction", &m_bubblegum_speed_fraction);
+        bubble_node->get("fade-in-time",   &m_bubblegum_fade_in_time  );
+        bubble_node->get("torque",         &m_bubblegum_torque        );
     }
 
     if(const XMLNode *rescue_node = root->getNode("rescue"))
@@ -313,7 +341,7 @@ void KartProperties::getAllData(const XMLNode * root)
     {
         explosion_node->get("time",   &m_explosion_time  );
         explosion_node->get("radius", &m_explosion_radius);
-        explosion_node->get("invulnerability-time", 
+        explosion_node->get("invulnerability-time",
                         &m_explosion_invulnerability_time);
     }
 
@@ -330,6 +358,8 @@ void KartProperties::getAllData(const XMLNode * root)
         m_ai_properties[RaceManager::DIFFICULTY_MEDIUM]->load(medium);
         const XMLNode *hard = ai_node->getNode("hard");
         m_ai_properties[RaceManager::DIFFICULTY_HARD]->load(hard);
+        const XMLNode *best = ai_node->getNode("best");
+        m_ai_properties[RaceManager::DIFFICULTY_BEST]->load(best);
     }
 
     if(const XMLNode *slipstream_node = root->getNode("slipstream"))
@@ -340,7 +370,7 @@ void KartProperties::getAllData(const XMLNode * root)
         slipstream_node->get("use-time",     &m_slipstream_use_time          );
         slipstream_node->get("add-power",    &m_slipstream_add_power         );
         slipstream_node->get("min-speed",    &m_slipstream_min_speed         );
-        slipstream_node->get("max-speed-increase", 
+        slipstream_node->get("max-speed-increase",
                                              &m_slipstream_max_speed_increase);
         slipstream_node->get("duration",     &m_slipstream_duration          );
         slipstream_node->get("fade-out-time",&m_slipstream_fade_out_time     );
@@ -359,32 +389,20 @@ void KartProperties::getAllData(const XMLNode * root)
     {
         engine_node->get("brake-factor", &m_brake_factor);
         engine_node->get("max-speed-reverse-ratio", &m_max_speed_reverse_ratio);
-        std::vector<float> v;
-        if( engine_node->get("power", &v))
+        engine_node->get("power", &m_engine_power);
+        if(m_engine_power.size()!=RaceManager::DIFFICULTY_COUNT)
         {
-            if(v.size()!=3)
-                printf("Incorrect engine-power specifications for kart '%s'\n",
-                getIdent().c_str());
-            else
-            {
-                m_engine_power[0] = v[0];
-                m_engine_power[1] = v[1];
-                m_engine_power[2] = v[2];
-            }
-        }   // if engine-power
-        v.clear();
-        if( engine_node->get("max-speed", &v))
+            Log::fatal("KartProperties",
+                       "Incorrect engine-power specifications for kart '%s'\n",
+                       getIdent().c_str());
+        }
+        engine_node->get("max-speed", &m_max_speed);
+        if(m_max_speed.size()!=RaceManager::DIFFICULTY_COUNT)
         {
-            if(v.size()!=3)
-                printf("Incorrect max-speed specifications for kart '%s'\n",
-                getIdent().c_str());
-            else
-            {
-                m_max_speed[0] = v[0];
-                m_max_speed[1] = v[1];
-                m_max_speed[2] = v[2];
-            }
-        }   // if max-speed
+            Log::fatal("KartProperties",
+                       "Incorrect max-speed specifications for kart '%s'\n",
+                       getIdent().c_str());
+        }
     }   // if getNode("engine")
 
     if(const XMLNode *gear_node = root->getNode("gear"))
@@ -413,20 +431,25 @@ void KartProperties::getAllData(const XMLNode * root)
         wheels_node->get("radius",              &m_wheel_radius             );
     }
 
+    if(const XMLNode *speed_weighted_objects_node = root->getNode("speed-weighted-objects"))
+    {
+        m_speed_weighted_object_properties.loadFromXMLNode(speed_weighted_objects_node);
+    }
+
     if(const XMLNode *friction_node = root->getNode("friction"))
         friction_node->get("slip", &m_friction_slip);
 
     if(const XMLNode *stability_node = root->getNode("stability"))
     {
-        stability_node->get("roll-influence",          
+        stability_node->get("roll-influence",
                                                    &m_roll_influence         );
         stability_node->get("chassis-linear-damping",
                                                    &m_chassis_linear_damping );
-        stability_node->get("chassis-angular-damping", 
+        stability_node->get("chassis-angular-damping",
                                                    &m_chassis_angular_damping);
-        stability_node->get("downward-impulse-factor", 
+        stability_node->get("downward-impulse-factor",
                                                    &m_downward_impulse_factor);
-        stability_node->get("track-connection-accel", 
+        stability_node->get("track-connection-accel",
                                                    &m_track_connection_accel );
     }
 
@@ -454,13 +477,13 @@ void KartProperties::getAllData(const XMLNode * root)
             m_terrain_impulse_type = IMPULSE_TO_DRIVELINE;
         else
         {
-            fprintf(stderr, "Missing or incorrect value for impulse-type: '%s'.\n",
-                            s.c_str());
-            exit(-1);
+            Log::fatal("KartProperties",
+                       "Missing or incorrect value for impulse-type: '%s'.\n",
+                       s.c_str());
         }
     }
 
-    //TODO: wheel front right and wheel front left is not loaded, yet is 
+    //TODO: wheel front right and wheel front left is not loaded, yet is
     //TODO: listed as an attribute in the xml file after wheel-radius
     //TODO: same goes for their rear equivalents
 
@@ -471,17 +494,11 @@ void KartProperties::getAllData(const XMLNode * root)
         plunger_node->get("band-duration",      &m_rubber_band_duration      );
         plunger_node->get("band-speed-increase",&m_rubber_band_speed_increase);
         plunger_node->get("band-fade-out-time", &m_rubber_band_fade_out_time );
-        std::vector<float> v;
-        plunger_node->get("in-face-time",    &v);
-        if(v.size()!=3)
+        plunger_node->get("in-face-time", &m_plunger_in_face_duration);
+        if(m_plunger_in_face_duration.size()!=RaceManager::DIFFICULTY_COUNT)
         {
-            fprintf(stderr, "[KartProperties] ERROR: Invalid plunger in-face-time specification.");
-        }
-        else
-        {
-            m_plunger_in_face_duration[0] = v[0];
-            m_plunger_in_face_duration[1] = v[1];
-            m_plunger_in_face_duration[2] = v[2];
+            Log::fatal("KartProperties",
+                       "Invalid plunger in-face-time specification.");
         }
     }
 
@@ -507,6 +524,19 @@ void KartProperties::getAllData(const XMLNode * root)
         }
     }
 
+    if(const XMLNode *lean_node= root->getNode("lean"))
+    {
+        lean_node->get("max",   &m_max_lean  );
+        lean_node->get("speed", &m_lean_speed);
+        m_max_lean   *= DEGREE_TO_RAD;
+        m_lean_speed *= DEGREE_TO_RAD;
+    }
+
+    if(const XMLNode *jump_node= root->getNode("jump"))
+    {
+        jump_node->get("animation-time", &m_jump_animation_time);
+    }
+
     if(const XMLNode *camera_node= root->getNode("camera"))
     {
         camera_node->get("distance", &m_camera_distance);
@@ -530,8 +560,8 @@ void KartProperties::getAllData(const XMLNode * root)
         else if (s == "small") m_engine_sfx_type = "engine_small";
         else
         {
-            std::cerr << "[KartProperties::getAllData()] WARNING : Kart " << m_name.c_str()
-                      << " has invalid engine : " << s << "\n";
+            Log::warn("KartProperties", "Kart '%s' has invalid engine '%s'.",
+                       m_name.c_str(), s.c_str());
             m_engine_sfx_type = "engine_small";
         }
 
@@ -573,30 +603,33 @@ void KartProperties::checkAllSet(const std::string &filename)
 {
     if(m_gear_switch_ratio.size()==0)
     {
-        fprintf(stderr,"Missing default value for 'gear-switch-ratio' in '%s'.\n",
-                filename.c_str());
-        exit(-1);
+        Log::fatal("KartProperties",
+                   "Missing default value for 'gear-switch-ratio' in '%s'.\n",
+                   filename.c_str());
     }
     if(m_gear_power_increase.size()==0)
     {
-        fprintf(stderr,"Missing default value for 'gear-power-increase' in '%s'.\n",
+        Log::fatal("KartProperties",
+                   "Missing default value for 'gear-power-increase' in '%s'.\n",
                 filename.c_str());
-        exit(-1);
     }
     if(m_gear_switch_ratio.size()!=m_gear_power_increase.size())    {
-        fprintf(stderr,"Number of entries for 'gear-switch-ratio' and 'gear-power-increase\n");
-        fprintf(stderr,"in '%s' must be equal.\n", filename.c_str());
-        exit(-1);
+        Log::error("KartProperties",
+                   "Number of entries for 'gear-switch-ratio' and "
+                   "'gear-power-increase\n");
+        Log::fatal("KartProperties", "in '%s' must be equal.\n",
+                    filename.c_str());
     }
     if(m_startup_boost.size()!=m_startup_times.size())
     {
-        fprintf(stderr, "Number of entried for 'startup times' and 'startup-boost\n");
-        fprintf(stderr, "must be identical.\n");
-        exit(-1);
+        Log::error("KartProperties",
+                 "Number of entried for 'startup times' and 'startup-boost\n");
+        Log::fatal("KartProperties", "must be identical.\n");
     }
-#define CHECK_NEG(  a,strA) if(a<=UNDEFINED) {                         \
-        fprintf(stderr,"Missing default value for '%s' in '%s'.\n",    \
-                strA,filename.c_str());exit(-1);                       \
+#define CHECK_NEG(  a,strA) if(a<=UNDEFINED) {                      \
+        Log::fatal("KartProperties",                                \
+                    "Missing default value for '%s' in '%s'.\n",    \
+                    strA,filename.c_str());                \
     }
 
     CHECK_NEG(m_mass,                       "mass"                          );
@@ -610,12 +643,6 @@ void KartProperties::checkAllSet(const std::string &filename)
     CHECK_NEG(m_chassis_angular_damping, "stability chassis-angular-damping");
     CHECK_NEG(m_downward_impulse_factor, "stability downward-impulse-factor");
     CHECK_NEG(m_track_connection_accel,  "stability track-connection-accel" );
-    CHECK_NEG(m_engine_power[0],            "engine power[0]"               );
-    CHECK_NEG(m_engine_power[1],            "engine power[1]"               );
-    CHECK_NEG(m_engine_power[2],            "engine power[2]"               );
-    CHECK_NEG(m_max_speed[0],               "engine maximum-speed[0]"       );
-    CHECK_NEG(m_max_speed[1],               "engine maximum-speed[1]"       );
-    CHECK_NEG(m_max_speed[2],               "engine maximum-speed[2]"       );
     CHECK_NEG(m_max_speed_reverse_ratio,    "engine max-speed-reverse-ratio");
     CHECK_NEG(m_brake_factor,               "engine brake-factor"           );
     CHECK_NEG(m_suspension_stiffness,       "suspension stiffness"          );
@@ -631,9 +658,6 @@ void KartProperties::checkAllSet(const std::string &filename)
     CHECK_NEG(m_bevel_factor.getZ(),        "collision bevel-factor"        );
     CHECK_NEG(m_upright_tolerance,          "upright tolerance"             );
     CHECK_NEG(m_upright_max_force,          "upright max-force"             );
-    CHECK_NEG(m_plunger_in_face_duration[0],"plunger in-face-time[0]"       );
-    CHECK_NEG(m_plunger_in_face_duration[1],"plunger in-face-time[1]"       );
-    CHECK_NEG(m_plunger_in_face_duration[2],"plunger in-face-time[2]"       );
     CHECK_NEG(m_rubber_band_max_length,     "plunger band-max-length"       );
     CHECK_NEG(m_rubber_band_force,          "plunger band-force"            );
     CHECK_NEG(m_rubber_band_duration,       "plunger band-duration"         );
@@ -650,7 +674,7 @@ void KartProperties::checkAllSet(const std::string &filename)
     CHECK_NEG(m_slipstream_use_time,        "slipstream use-time"           );
     CHECK_NEG(m_slipstream_add_power,       "slipstream add-power"          );
     CHECK_NEG(m_slipstream_min_speed,       "slipstream min-speed"          );
-    CHECK_NEG(m_slipstream_max_speed_increase, 
+    CHECK_NEG(m_slipstream_max_speed_increase,
                                             "slipstream max-speed-increase" );
     CHECK_NEG(m_slipstream_duration,        "slipstream duration"           );
     CHECK_NEG(m_slipstream_fade_out_time,   "slipstream fade-out-time"      );
@@ -665,19 +689,36 @@ void KartProperties::checkAllSet(const std::string &filename)
     CHECK_NEG(m_nitro_duration,             "nitro duration"                );
     CHECK_NEG(m_nitro_fade_out_time,        "nitro fade-out-time"           );
     CHECK_NEG(m_nitro_max,                  "nitro max"                     );
+    CHECK_NEG(m_bubblegum_time,             "bubblegum time"                );
+    CHECK_NEG(m_bubblegum_speed_fraction,   "bubblegum speed-fraction"      );
+    CHECK_NEG(m_bubblegum_fade_in_time  ,   "bubblegum fade-in-time"        );
+    CHECK_NEG(m_bubblegum_torque,           "bubblegum  torque"             );
+
     CHECK_NEG(m_swatter_distance2,          "swatter distance"              );
     CHECK_NEG(m_swatter_duration,           "swatter duration"              );
-    CHECK_NEG(m_squash_duration,            "squash-duration"               );
-    CHECK_NEG(m_squash_slowdown,            "squash-slowdown"               );
+    CHECK_NEG(m_squash_duration,            "swatter squash-duration"       );
+    CHECK_NEG(m_squash_slowdown,            "swatter squash-slowdown"       );
+    CHECK_NEG(m_max_lean,                   "lean max"                      );
+    CHECK_NEG(m_lean_speed,                 "lean speed"                    );
 
     CHECK_NEG(m_rescue_height,              "rescue height"                 );
     CHECK_NEG(m_rescue_time,                "rescue time"                   );
     CHECK_NEG(m_rescue_vert_offset,         "rescue vert-offset"            );
     CHECK_NEG(m_explosion_time,             "explosion time"                );
-    CHECK_NEG(m_explosion_invulnerability_time, 
+    CHECK_NEG(m_explosion_invulnerability_time,
                                             "explosion invulnerability-time");
     CHECK_NEG(m_explosion_radius,           "explosion radius"              );
 
+    for(unsigned int i=RaceManager::DIFFICULTY_FIRST;
+        i<=RaceManager::DIFFICULTY_LAST; i++)
+    {
+        CHECK_NEG(m_max_speed[i], "engine maximum-speed[0]");
+        CHECK_NEG(m_engine_power[i], "engine power" );
+        CHECK_NEG(m_plunger_in_face_duration[i],"plunger in-face-time");
+    }
+
+    m_speed_weighted_object_properties.checkAllSet();
+
     m_skidding_properties->checkAllSet(filename);
     for(unsigned int i=0; i<RaceManager::DIFFICULTY_COUNT; i++)
         m_ai_properties[i]->checkAllSet(filename);
