@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -42,12 +42,12 @@ class Material;
 class SkiddingProperties;
 class XMLNode;
 
-/** 
+/**
  *  \brief This class stores the properties of a kart.
  *  This includes size, name, identifier, physical properties etc.
  *  It is atm also the base class for STKConfig, which stores the default values
  *  for all physics constants.
- *  Note that KartProperies is copied (when setting the default values from 
+ *  Note that KartProperies is copied (when setting the default values from
  *  stk_config.
  *
  * \ingroup karts
@@ -86,6 +86,8 @@ private:
 
     /** List of all groups the kart belongs to. */
     std::vector<std::string> m_groups;
+
+    /** Dummy value to detect unset properties. */
     static float UNDEFINED;
 
     /** Version of the .kart file. */
@@ -123,7 +125,7 @@ private:
     float m_mass;
 
     /** Maximum force from engine for eachdifficulty. */
-    float m_engine_power[3];
+    std::vector<float> m_engine_power;
 
     /** Braking factor * engine_power braking force. */
     float m_brake_factor;
@@ -158,22 +160,27 @@ private:
 
     /** Duration a zipper is active. */
     float m_zipper_time;
-    
+
     /** Fade out time for a zipper. */
     float m_zipper_fade_out_time;
 
     /** Additional force added to the acceleration. */
     float m_zipper_force;
-    
+
     /** Initial one time speed gain. */
     float m_zipper_speed_gain;
-    
+
     /** Absolute increase of the kart's maximum speed (in m/s). */
     float m_zipper_max_speed_increase;
 
     /** Vertical offset after rescue. */
     float m_rescue_vert_offset;
 
+    /** Minimum time during which nitro is consumed when pressing
+     *  the nitro key (to prevent using in very small bursts)
+     */
+    float m_nitro_min_consumption;
+
     /** Filename of the wheel models. */
     std::string m_wheel_filename[4];
     /**  Radius of the graphical wheels.  */
@@ -192,7 +199,7 @@ private:
     /** Fade out time when the rubber band is removed. */
     float       m_rubber_band_fade_out_time;
      /**Duration of plunger in face depending on difficulty. */
-    float       m_plunger_in_face_duration[3];
+    std::vector<float>  m_plunger_in_face_duration;
     /** Wheel base of the kart. */
     float       m_wheel_base;
     /** Nitro consumption. */
@@ -205,14 +212,24 @@ private:
     float       m_nitro_max_speed_increase;
     /** Additional engine force to affect the kart. */
     float       m_nitro_engine_force;
-    /**  How long the increased nitro max speed will be valid after 
+    /**  How long the increased nitro max speed will be valid after
      *  the kart stops using nitro (and the fade-out-time starts). */
     float       m_nitro_duration;
-    /** Duration during which the increased maximum speed 
+    /** Duration during which the increased maximum speed
      *  due to nitro fades out. */
     float       m_nitro_fade_out_time;
     /** Maximum nitro a kart can collect. */
     float       m_nitro_max;
+    /** Bubble gum diration. */
+    float       m_bubblegum_time;
+    /** Torque to add when a bubble gum was hit in order to make the kart go 
+     *  sideways a bit. */
+    float       m_bubblegum_torque;
+    /** Fraction of top speed that can be reached maximum after hitting a
+     *  bubble gum. */
+    float       m_bubblegum_speed_fraction;
+    /** How long to fade in the slowdown for a bubble gum. */
+    float       m_bubblegum_fade_in_time;
     /** Square of the maximum distance a swatter can operate. */
     float       m_swatter_distance2;
     /** How long the swatter lasts. */
@@ -223,6 +240,17 @@ private:
      *  is max_speed*m_squash_slowdown. */
     float       m_squash_slowdown;
 
+    /** The maximum roll a kart graphics should show when driving in a fast
+     *  curve. This is read in as degrees, but stored in radians. */
+     float      m_max_lean;
+
+     /** The speed with which the roll (when leaning in a curve) changes
+      *  (in radians/second). */
+     float      m_lean_speed;
+     
+     /** How long a jump must be in order to trigger the jump animation. */
+     float      m_jump_animation_time;
+
     /** Engine sound effect. */
     std::string m_engine_sfx_type;
 
@@ -236,6 +264,9 @@ private:
     float m_roll_influence;
     float m_wheel_radius;
 
+    /** Parameters for the speed-weighted objects */
+    SpeedWeightedObject::Properties   m_speed_weighted_object_properties;
+
     /** An impulse pushing the kart down which is proportional to speed. So
      *  the actual impulse is  speed * m_downward_impulse_factor. Set it to
      *  0 to disable completely. Based on
@@ -243,7 +274,7 @@ private:
      *  &p=21240&hilit=vehicle#p21240  */
     float m_downward_impulse_factor;
 
-    /** Artifical acceleration that pulls a kart down onto the track if one 
+    /** Artifical acceleration that pulls a kart down onto the track if one
      *  axis loses contact with the track. */
     float m_track_connection_accel;
 
@@ -253,7 +284,9 @@ private:
     /** Angular damping to prevent it from turning too easily. */
     float m_chassis_angular_damping;
 
-    float m_max_speed[3];
+    /** The maximum speed at each difficulty. */
+    std::vector<float> m_max_speed;
+
     float m_max_speed_reverse_ratio;
 
     /** Shift of center of gravity. */
@@ -274,7 +307,7 @@ public:
      *  driveline. The later works nice as long as the kart is driving
      *  on the main track, but can work very bad if the kart is drivling
      *  off-track (and a wrong driveline is selected). */
-    enum TerrainImpulseType {IMPULSE_NONE, IMPULSE_NORMAL, 
+    enum TerrainImpulseType {IMPULSE_NONE, IMPULSE_NORMAL,
                              IMPULSE_TO_DRIVELINE};
 private:
     TerrainImpulseType m_terrain_impulse_type;
@@ -282,7 +315,7 @@ private:
     /** An additional impulse to push a kart away if it hits terrain */
     float m_collision_terrain_impulse;
 
-    /** An additiojnal artificial impulse that pushes two karts in a 
+    /** An additiojnal artificial impulse that pushes two karts in a
      *  side-side collision away from each other. */
     float m_collision_impulse;
 
@@ -307,7 +340,7 @@ private:
     float m_slipstream_add_power;
     /** Minimum speed for slipstream to take effect. */
     float m_slipstream_min_speed;
-    /** How much the speed of the kart might exceed its 
+    /** How much the speed of the kart might exceed its
      *  normal maximum speed. */
     float m_slipstream_max_speed_increase;
     /** How long the higher speed lasts after slipstream stopped working. */
@@ -318,11 +351,11 @@ private:
     /** Distance of normal camera from kart. */
     float m_camera_distance;
 
-    /** Up angle of the camera in relation to the pitch of the kart when 
+    /** Up angle of the camera in relation to the pitch of the kart when
      *  driving forwards. */
     float m_camera_forward_up_angle;
 
-    /** Up angle of the camera in relation to the pitch of the kart when 
+    /** Up angle of the camera in relation to the pitch of the kart when
      *  driving backwards. */
     float m_camera_backward_up_angle;
 
@@ -335,7 +368,7 @@ private:
        m_gear_switch_ratio). */
     std::vector<float> m_gear_power_increase;
 
-    /** If the kart starts within the specified time at index I after 'go', 
+    /** If the kart starts within the specified time at index I after 'go',
      *  it receives the speed boost from m_startup_boost[I]. */
     std::vector<float> m_startup_times;
 
@@ -356,7 +389,7 @@ public:
     float getStartupBoost   () const;
 
     // ------------------------------------------------------------------------
-    /** Returns the (maximum) speed for a given turn radius. 
+    /** Returns the (maximum) speed for a given turn radius.
      *  \param radius The radius for which the speed needs to be computed. */
     float getSpeedForTurnRadius(float radius) const {
         float angle = sin(m_wheel_base / radius);
@@ -364,7 +397,7 @@ public:
     }   // getSpeedForTurnRadius
     // ------------------------------------------------------------------------
     /** Returns the maximum steering angle (depending on speed). */
-    float getMaxSteerAngle(float speed) const { 
+    float getMaxSteerAngle(float speed) const {
         return m_turn_angle_at_speed.get(speed);
     }   // getMaxSteerAngle
 
@@ -372,78 +405,106 @@ public:
     /** Returns the material for the kart icons. */
     Material*     getIconMaterial    () const {return m_icon_material;        }
 
+    // ------------------------------------------------------------------------
     /** Returns the texture to use in the minimap, or NULL if not defined. */
     video::ITexture *getMinimapIcon  () const {return m_minimap_icon;         }
 
+    // ------------------------------------------------------------------------
     /** Returns a pointer to the KartModel object. */
-    KartModel*    getKartModelCopy   () const 
+    KartModel*    getKartModelCopy   () const
                                             {return m_kart_model->makeCopy(); }
 
+    // ------------------------------------------------------------------------
     /** Returns a pointer to the main KartModel object. This copy
      *  should not be modified, not attachModel be called on it. */
     const KartModel& getMasterKartModel() const {return *m_kart_model;        }
 
-    /** Returns the name of this kart. 
+    // ------------------------------------------------------------------------
+    /** Sets the name of a mesh to be used for this kart.
+     *  \param hat_name Name of the mesh.
+     */
+    void setHatMeshName(const std::string &hat_name)
+    {
+        m_kart_model->setHatMeshName(hat_name);
+    }   // setHatMeshName
+    // ------------------------------------------------------------------------
+    /** Returns the name of this kart.
         \note Pass it through fridibi as needed, this is the LTR name
       */
-    const wchar_t* getName() const 
+    const wchar_t* getName() const
     {
-        return translations->w_gettext(m_name.c_str()); 
+        return translations->w_gettext(m_name.c_str());
     }
 
+    // ------------------------------------------------------------------------
     const std::string getNonTranslatedName() const {return m_name;}
 
+    // ------------------------------------------------------------------------
     /** Returns the internal identifier of this kart. */
     const std::string& getIdent      () const {return m_ident;                }
 
+    // ------------------------------------------------------------------------
     /** Returns the shadow texture to use. */
     video::ITexture *getShadowTexture() const {return m_shadow_texture;       }
 
+    // ------------------------------------------------------------------------
     /** Returns the absolute path of the icon file of this kart. */
     const std::string& getAbsoluteIconFile() const      { return m_icon_file; }
 
+    // ------------------------------------------------------------------------
     /** Returns custom sound effects for this kart. */
-    const int          getCustomSfxId (SFXManager::CustomSFX type) 
+    const int          getCustomSfxId (SFXManager::CustomSFX type)
                                        const  {return m_custom_sfx_id[type];  }
 
+    // ------------------------------------------------------------------------
     /** Returns the version of the .kart file. */
     int   getVersion                () const {return m_version;               }
 
+    // ------------------------------------------------------------------------
     /** Returns the dot color to use for this kart in the race gui. */
     const video::SColor &getColor   () const {return m_color;                 }
 
+    // ------------------------------------------------------------------------
     /** Returns the number of edges for the polygon used to draw the dot of
      *  this kart on the mini map of the race gui. */
     int   getShape                  () const {return m_shape;                 }
 
-    /** REturns the list of groups this kart belongs to. */
+    // ------------------------------------------------------------------------
+    /** Returns the list of groups this kart belongs to. */
     const std::vector<std::string>&
                   getGroups         () const {return m_groups;                }
+    // ------------------------------------------------------------------------
     /** Returns the mass of this kart. */
     float getMass                   () const {return m_mass;                  }
+    // ------------------------------------------------------------------------
     /** Returns the maximum engine power depending on difficulty. */
-    float getMaxPower               () const 
+    float getMaxPower               () const
                         {return m_engine_power[race_manager->getDifficulty()];}
 
-    /** Returns the time the kart needs to fully steer in one direction from 
+    // ------------------------------------------------------------------------
+    /** Returns the time the kart needs to fully steer in one direction from
      *  steering straight depending on the current steering value.
      *  \param steer Current steering value, must be >=0. */
-    float getTimeFullSteer(float steer) const 
+    float getTimeFullSteer(float steer) const
     {
         assert(steer>=0);
-        return m_time_full_steer.get(steer); 
+        return m_time_full_steer.get(steer);
     }   // getTimeFullSteer
 
+    // ------------------------------------------------------------------------
     /** Returns the time the kart needs to go back to steering straight from
      *  full steer. */
     float getTimeResetSteer         () const { return m_time_reset_steer;     }
+    // ------------------------------------------------------------------------
     /** Get braking information. */
     float getBrakeFactor            () const {return m_brake_factor;          }
 
+    // ------------------------------------------------------------------------
     /** Get maximum reverse speed ratio. */
-    float getMaxSpeedReverseRatio   () const 
+    float getMaxSpeedReverseRatio   () const
                                           {return m_max_speed_reverse_ratio;  }
 
+    // ------------------------------------------------------------------------
     /** Returns the engine type (used to change sfx depending on kart size). */
     const std::string& getEngineSfxType    () const {return m_engine_sfx_type;}
 
@@ -452,267 +513,372 @@ public:
     /** Returns the suspension stiffness. */
     float getSuspensionStiffness    () const {return m_suspension_stiffness;  }
 
+    // ------------------------------------------------------------------------
     /** Returns damping relaxation. */
-    float getWheelDampingRelaxation () const 
+    float getWheelDampingRelaxation () const
                                           {return m_wheel_damping_relaxation; }
 
+    // ------------------------------------------------------------------------
     /** Returns the wheel damping compression. */
-    float getWheelDampingCompression() const 
+    float getWheelDampingCompression() const
                                           {return m_wheel_damping_compression;}
 
+    // ------------------------------------------------------------------------
     /** Returns maximum suspension force. */
     float getMaxSuspensionForce() const {return m_max_suspension_force; }
 
+    // ------------------------------------------------------------------------
     /** Returns friction slip. */
     float getFrictionSlip           () const {return m_friction_slip;         }
 
+    // ------------------------------------------------------------------------
     /** Returns roll influence. */
     float getRollInfluence          () const {return m_roll_influence;        }
 
+    // ------------------------------------------------------------------------
     /** Returns wheel radius. */
     float getWheelRadius            () const {return m_wheel_radius;          }
 
+    // ------------------------------------------------------------------------
+    /** Returns parameters for the speed-weighted objects */
+    const SpeedWeightedObject::Properties& getSpeedWeightedObjectProperties() const {return m_speed_weighted_object_properties;}
+    
+    // ------------------------------------------------------------------------
     /** Returns the wheel base (distance front to rear axis). */
     float getWheelBase              () const {return m_wheel_base;            }
 
+    // ------------------------------------------------------------------------
     /** Returns linear damping of chassis. */
     float getChassisLinearDamping   () const {return m_chassis_linear_damping;}
 
+    // ------------------------------------------------------------------------
     /** Returns angular damping of chassis. */
-    float getChassisAngularDamping  () const 
+    float getChassisAngularDamping  () const
                                            {return m_chassis_angular_damping; }
 
+    // ------------------------------------------------------------------------
     /** Artifical downward impulse every frame. */
     float getDownwardImpulseFactor() const { return m_downward_impulse_factor;}
 
+    // ------------------------------------------------------------------------
     /** Returns artificial acceleration to keep wheels on track. */
     float getTrackConnectionAccel   () const {return m_track_connection_accel;}
 
+    // ------------------------------------------------------------------------
     /** Returns the maximum speed dependent on the difficult level. */
     float getMaxSpeed               () const {return
                                    m_max_speed[race_manager->getDifficulty()];}
 
+    // ------------------------------------------------------------------------
     /** Returns the nitro consumption. */
     float getNitroConsumption       () const {return m_nitro_consumption;     }
 
+    // ------------------------------------------------------------------------
     /** Returns the amount of nitro for a small container. */
     float getNitroSmallContainer    () const {return m_nitro_small_container; }
 
+    // ------------------------------------------------------------------------
     /** Returns the amount of nitro for a big container. */
     float getNitroBigContainer      () const {return m_nitro_big_container;   }
 
+    // ------------------------------------------------------------------------
     /** Returns the increase of maximum speed due to nitro. */
-    float getNitroMaxSpeedIncrease  () const 
+    float getNitroMaxSpeedIncrease  () const
                                           {return m_nitro_max_speed_increase; }
 
+    // ------------------------------------------------------------------------
     float getNitroEngineForce       () const {return m_nitro_engine_force;    }
+    // ------------------------------------------------------------------------
     /** Returns how long the increased nitro max speed will be valid after
      *  the kart stops using nitro (and the fade-out-time starts). */
     float getNitroDuration          () const {return m_nitro_duration;        }
 
-    /** Returns the duration during which the increased maximum speed 
+    // ------------------------------------------------------------------------
+    /** Returns the duration during which the increased maximum speed
      *  due to nitro fades out. */
     float getNitroFadeOutTime       () const {return m_nitro_fade_out_time;   }
 
+    // ------------------------------------------------------------------------
     /** Returns the maximum amount of nitro a kart can store. */
     float getNitroMax               () const {return m_nitro_max;             }
-
-    /** Returns a shift of the center of mass (lowering the center of mass 
+    // ------------------------------------------------------------------------
+    /** Returns how long a bubble gum is active. */
+    float getBubblegumTime() const { return m_bubblegum_time; }
+    // ------------------------------------------------------------------------
+    /** Returns the torque to add when a bubble gum was hit . */
+    float getBubblegumTorque() const { return m_bubblegum_torque; }
+    // ------------------------------------------------------------------------
+    /** Returns the fraction of top speed that can be reached maximum after 
+     *  hitting a bubble gum. */
+    float getBubblegumSpeedFraction() const {return m_bubblegum_speed_fraction;}
+    // ------------------------------------------------------------------------
+    /** Returns how long to fade in the slowdown for a bubble gum. */
+    float getBubblegumFadeInTime() const { return m_bubblegum_fade_in_time; }
+    // ------------------------------------------------------------------------
+    /** Returns a shift of the center of mass (lowering the center of mass
      *  makes the karts more stable. */
     const Vec3&getGravityCenterShift() const {return m_gravity_center_shift;  }
 
+    // ------------------------------------------------------------------------
     /** Retusn suspension rest length. */
     float getSuspensionRest         () const {return m_suspension_rest;       }
 
+    // ------------------------------------------------------------------------
     /** Returns the amount the suspension can extend. */
     float getSuspensionTravelCM     () const {return m_suspension_travel_cm;  }
 
+    // ------------------------------------------------------------------------
     /** Returns if the spring should be exponentially dampened. */
     bool getExpSpringResponse() const {return m_exp_spring_response; }
 
-    /** Returns an artificial impulse to push karts away from the terrain 
+    // ------------------------------------------------------------------------
+    /** Returns an artificial impulse to push karts away from the terrain
      *  it hits. */
-    float getCollisionTerrainImpulse() const 
+    float getCollisionTerrainImpulse() const
                                           {return m_collision_terrain_impulse;}
 
+    // ------------------------------------------------------------------------
     /** Returns what kind of impulse STK should use in case of a kart-track
      *  collision. */
     TerrainImpulseType getTerrainImpulseType() const
                                              { return m_terrain_impulse_type; }
+    // ------------------------------------------------------------------------
     /** Returns the (artificial) collision impulse this kart will apply
      *  to another kart in case of a non-frontal collision. */
     float getCollisionImpulse       () const {return m_collision_impulse;}
 
+    // ------------------------------------------------------------------------
     /** Returns how long the collision impulse should be applied. */
     float getCollisionImpulseTime() const { return m_collision_impulse_time;}
 
+    // ------------------------------------------------------------------------
     /** Returns the restitution factor for this kart. */
     float getRestitution            () const { return m_restitution; }
 
+    // ------------------------------------------------------------------------
     /** Returns the vertical offset when rescuing karts to avoid karts being
      *  rescued in (or under) the track. */
     float getVertRescueOffset       () const {return m_rescue_vert_offset;    }
 
+    // ------------------------------------------------------------------------
     /** Returns the time a kart is rised during a rescue. */
     float getRescueTime             () const {return m_rescue_time;           }
 
+    // ------------------------------------------------------------------------
     /** Returns the height a kart is moved to during a rescue. */
     float getRescueHeight           () const {return m_rescue_height;         }
 
+    // ------------------------------------------------------------------------
     /** Returns the time an explosion animation is shown. */
     float getExplosionTime          () const {return m_explosion_time;        }
 
+    // ------------------------------------------------------------------------
     /** Returns the height of the explosion animation. */
     float getExplosionRadius        () const {return m_explosion_radius;      }
 
-    /** Returns how long a kart is invulnerable after being hit by an 
+    // ------------------------------------------------------------------------
+    /** Returns how long a kart is invulnerable after being hit by an
         explosion. */
-    float getExplosionInvulnerabilityTime() const 
+    float getExplosionInvulnerabilityTime() const
                                    { return m_explosion_invulnerability_time; }
 
+    // ------------------------------------------------------------------------
     /** Returns how much a kart can roll/pitch before the upright constraint
      *  counteracts. */
     float getUprightTolerance       () const {return m_upright_tolerance;     }
 
+    // ------------------------------------------------------------------------
     /** Returns the maximum value of the upright counteracting force. */
     float getUprightMaxForce        () const {return m_upright_max_force;     }
 
+    // ------------------------------------------------------------------------
     /** Returns the maximum length of a rubber band before it breaks. */
     float getRubberBandMaxLength    () const {return m_rubber_band_max_length;}
 
+    // ------------------------------------------------------------------------
     /** Returns force a rubber band has when attached to a kart. */
     float getRubberBandForce        () const {return m_rubber_band_force;     }
 
+    // ------------------------------------------------------------------------
     /** Returns the duration a rubber band is active for. */
     float getRubberBandDuration     () const {return m_rubber_band_duration;  }
 
-    /** Returns the increase of maximum speed while a rubber band is 
+    // ------------------------------------------------------------------------
+    /** Returns the increase of maximum speed while a rubber band is
      *  pulling. */
-    float getRubberBandSpeedIncrease() const 
+    float getRubberBandSpeedIncrease() const
     {
         return m_rubber_band_speed_increase;
     }
 
+    // ------------------------------------------------------------------------
     /** Return the fade out time once a rubber band is removed. */
-    float getRubberBandFadeOutTime() const 
+    float getRubberBandFadeOutTime() const
     {
         return m_rubber_band_fade_out_time;
     }
 
+    // ------------------------------------------------------------------------
     /** Returns duration of a plunger in your face. */
-    float getPlungerInFaceTime      () const 
+    float getPlungerInFaceTime      () const
             {return m_plunger_in_face_duration[race_manager->getDifficulty()];}
 
+    // ------------------------------------------------------------------------
     /** Returns the time a zipper is active. */
     float getZipperTime             () const {return m_zipper_time;           }
 
+    // ------------------------------------------------------------------------
     /** Returns the time a zipper is active. */
     float getZipperFadeOutTime     () const {return m_zipper_fade_out_time;   }
 
+    // ------------------------------------------------------------------------
     /** Returns the additional force added applied to the kart. */
     float getZipperForce            () const { return m_zipper_force;         }
 
+    // ------------------------------------------------------------------------
     /** Returns the initial zipper speed gain. */
     float getZipperSpeedGain        () const { return m_zipper_speed_gain;    }
 
-    /** Returns the increase of the maximum speed of the kart 
+    // ------------------------------------------------------------------------
+    /** Returns the increase of the maximum speed of the kart
      *  if a zipper is active. */
-    float getZipperMaxSpeedIncrease () const 
+    float getZipperMaxSpeedIncrease () const
                                          { return m_zipper_max_speed_increase;}
 
+    // ------------------------------------------------------------------------
     /** Returns how far behind a kart slipstreaming works. */
     float getSlipstreamLength       () const {return m_slipstream_length;     }
 
+    // ------------------------------------------------------------------------
     /** Returns how wide the slipstream area is at the end. */
     float getSlipstreamWidth        () const {return m_slipstream_width;      }
 
+    // ------------------------------------------------------------------------
     /** Returns time after which slipstream has maximum effect. */
-    float getSlipstreamCollectTime  () const 
+    float getSlipstreamCollectTime  () const
                                           {return m_slipstream_collect_time;  }
 
+    // ------------------------------------------------------------------------
     /** Returns time after which slipstream has maximum effect. */
     float getSlipstreamUseTime      () const {return m_slipstream_use_time;   }
 
+    // ------------------------------------------------------------------------
     /** Returns additional power due to slipstreaming. */
     float getSlipstreamAddPower     () const {return m_slipstream_add_power;  }
 
+    // ------------------------------------------------------------------------
     /** Returns the minimum slipstream speed. */
     float getSlipstreamMinSpeed     () const {return m_slipstream_min_speed;  }
 
-    /** Returns the increase of the maximum speed of a kart 
+    // ------------------------------------------------------------------------
+    /** Returns the increase of the maximum speed of a kart
      *  due to slipstream. */
-    float getSlipstreamMaxSpeedIncrease() const 
+    float getSlipstreamMaxSpeedIncrease() const
                                     { return m_slipstream_max_speed_increase; }
-    /** Returns how long the higher speed lasts after slipstream 
+    // ------------------------------------------------------------------------
+    /** Returns how long the higher speed lasts after slipstream
      *  stopped working. */
     float getSlipstreamDuration     () const { return m_slipstream_duration;  }
 
-    /** Returns how long the slip stream speed increase will gradually 
+    // ------------------------------------------------------------------------
+    /** Returns how long the slip stream speed increase will gradually
      *  be reduced. */
-    float getSlipstreamFadeOutTime  () const 
+    float getSlipstreamFadeOutTime  () const
                                          { return m_slipstream_fade_out_time; }
 
+    // ------------------------------------------------------------------------
     /** Returns the scale factor by which the shadow plane
      *  had to be set. */
     float getShadowScale            () const {return m_shadow_scale;          }
-    
+
+    // ------------------------------------------------------------------------
     /** Returns the scale factor by which the shadow plane
      *  had to be set. */
     float getShadowXOffset          () const {return m_shadow_x_offset;       }
-    
+
+    // ------------------------------------------------------------------------
     /** Returns the scale factor by which the shadow plane
      *  had to be set. */
     float getShadowYOffset          () const {return m_shadow_y_offset;       }
-    
+
+    // ------------------------------------------------------------------------
     /** Returns a pointer to the skidding properties. */
-    const SkiddingProperties *getSkiddingProperties() const 
+    const SkiddingProperties *getSkiddingProperties() const
                                               { return m_skidding_properties; }
 
+    // ------------------------------------------------------------------------
     /** Returns a pointer to the AI properties. */
-    const AIProperties *getAIPropertiesForDifficulty() const 
+    const AIProperties *getAIPropertiesForDifficulty() const
     {
         return m_ai_properties[race_manager->getDifficulty()];
     }   // getAIProperties
 
+    // ------------------------------------------------------------------------
     /** Returns ratio of current speed to max speed at which the gear will
      *  change (for our simualated gears = simple change of engine power). */
     const std::vector<float>&
           getGearSwitchRatio        () const {return m_gear_switch_ratio;     }
 
+    // ------------------------------------------------------------------------
     /** Returns the power increase depending on gear. */
     const std::vector<float>&
           getGearPowerIncrease      () const {return m_gear_power_increase;   }
 
+    // ------------------------------------------------------------------------
     /** Returns distance between kart and camera. */
     float getCameraDistance         () const {return m_camera_distance;       }
 
+    // ------------------------------------------------------------------------
     /** Returns the angle the camera has relative to the pitch of the kart. */
-    float getCameraForwardUpAngle   () const 
+    float getCameraForwardUpAngle   () const
                                            {return m_camera_forward_up_angle; }
 
+    // ------------------------------------------------------------------------
     /** Returns the angle the camera has relative to the pitch of the kart. */
-    float getCameraBackwardUpAngle  () const 
+    float getCameraBackwardUpAngle  () const
                                           {return m_camera_backward_up_angle; }
 
+    // ------------------------------------------------------------------------
     /** Returns the full path where the files for this kart are stored. */
     const std::string& getKartDir   () const {return m_root;                  }
 
-    /** Returns the square of the maximum distance at which a swatter 
+    // ------------------------------------------------------------------------
+    /** Returns the square of the maximum distance at which a swatter
      *  can hit karts. */
     float getSwatterDistance2() const { return m_swatter_distance2; }
 
+    // ------------------------------------------------------------------------
     /** Returns how long a swatter will stay attached/ready to be used. */
     float getSwatterDuration() const { return m_swatter_duration; }
 
+    // ------------------------------------------------------------------------
     /** Returns how long a kart remains squashed. */
     float getSquashDuration() const {return m_squash_duration; }
 
+    // ------------------------------------------------------------------------
     /** Returns the slowdown of a kart that is squashed. */
     float getSquashSlowdown() const {return m_squash_slowdown; }
 
+    // ------------------------------------------------------------------------
+    /** The maximum leaning a kart should show (In radians). */
+    float getMaxLean() const { return m_max_lean; }
+
+    // ------------------------------------------------------------------------
+    /** The speed with which a kart should lean (in radians/s). */
+    float getLeanSpeed() const { return m_lean_speed; }
+    // ------------------------------------------------------------------------
+    /** Return show long a jump must last in order to play the jump 
+     *  animation. */
+    float getJumpAnimationTime() const { return m_jump_animation_time; }
+    // ------------------------------------------------------------------------
     /** Returns true if wheels should have random rotation at start. */
     bool hasRandomWheels() const { return m_has_rand_wheels; }
-
+    // ------------------------------------------------------------------------
+    /** Returns minimum time during which nitro is consumed when pressing nitro
+     *  key, to prevent using nitro in very short bursts
+     */
+    float getNitroMinConsumptionTime() const { return m_nitro_min_consumption; }
+    // ------------------------------------------------------------------------
     /** Returns the bevel factor (!=0 indicates to use a bevelled box). */
     const Vec3 &getBevelFactor() const { return m_bevel_factor; }
 };   // KartProperties
