@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2012  Joerg Henrichs
+//  Copyright (C) 2012-2013  Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -29,15 +29,16 @@
 #include "modes/world.hpp"
 #include "physics/btKart.hpp"
 #include "tracks/track.hpp"
+#include "utils/log.hpp"
 
 /** Constructor of the skidding object.
  */
 Skidding::Skidding(Kart *kart, const SkiddingProperties *sp)
 {
 #ifdef SKID_DEBUG
-    m_predicted_curve = new ShowCurve(0.05f, 0.05f, 
+    m_predicted_curve = new ShowCurve(0.05f, 0.05f,
                                       irr::video::SColor(128, 0, 128, 0));
-    m_actual_curve    = new ShowCurve(0.05f, 0.05f, 
+    m_actual_curve    = new ShowCurve(0.05f, 0.05f,
                                       irr::video::SColor(128, 0, 0, 128));
     m_predicted_curve->setVisible(false);
     m_actual_curve->setVisible(false);
@@ -71,14 +72,16 @@ void Skidding::reset()
     m_gfx_jump_offset     = 0.0f;
     m_remaining_jump_time = 0.0f;
     m_jump_speed          = 0.0f;
-    m_kart->getKartGFX()->setCreationRateAbsolute(KartGFX::KGFX_SKID, 0);
+    m_kart->getKartGFX()->setCreationRateAbsolute(KartGFX::KGFX_SKIDL, 0);
+    m_kart->getKartGFX()->setCreationRateAbsolute(KartGFX::KGFX_SKIDR, 0);
+    m_kart->getControls().m_skid = KartControl::SC_NONE;
 }   // reset
 
 // ----------------------------------------------------------------------------
-/** Computes the actual steering fraction to be used in the physics, and 
+/** Computes the actual steering fraction to be used in the physics, and
  *  stores it in m_real_skidding. This is later used by kart to set the
  *  physical steering. The real steering takes skidding into account: if the
- *  kart skids either left or right, the steering fraction is bound by 
+ *  kart skids either left or right, the steering fraction is bound by
  *  reduce-turn-min and reduce-turn-max.
  */
 void Skidding::updateSteering(float steer, float dt)
@@ -113,10 +116,19 @@ void Skidding::updateSteering(float steer, float dt)
                 m_visual_rotation = f;
         }
         break;
+    case SKID_BREAK:
+        m_real_steering = steer;
+        if (m_visual_rotation > 0.05f) m_visual_rotation -= 0.05f;
+        else if (m_visual_rotation < -0.05f) m_visual_rotation += 0.05f;
+        else
+        {
+            reset();
+        }        
+        break;
     case SKID_ACCUMULATE_RIGHT:
         {
             float f = (1.0f+steer)*0.5f;   // map [-1,1] --> [0, 1]
-            m_real_steering   = m_skid_reduce_turn_min+ 
+            m_real_steering   = m_skid_reduce_turn_min+
                                 m_skid_reduce_turn_delta*f;
             if(m_skid_time < m_skid_visual_time)
                 m_visual_rotation = m_skid_visual*m_real_steering*m_skid_time
@@ -137,16 +149,18 @@ void Skidding::updateSteering(float steer, float dt)
                 m_visual_rotation = m_skid_visual * m_real_steering;
             break;
         }
+        
+
     }   // switch m_skid_state
 
 }   // updateSteering
 
 // ----------------------------------------------------------------------------
 /** Returns the steering value necessary to set in KartControls.m_steer in
- *  order to actually to steer the specified amount in 'steering'. 
+ *  order to actually to steer the specified amount in 'steering'.
  *  If the kart is not skidding, the return value is just
- *  'steering'. Otherwise the return value will be (depending on current 
- *  skidding direction) 'steering1', so that when the kart 
+ *  'steering'. Otherwise the return value will be (depending on current
+ *  skidding direction) 'steering1', so that when the kart
  *  steers 'steering1', it will de facto steer by the original 'steering'
  *  amount. This function might return a result outside of [-1,1] if the
  *  specified steering can not be reached (e.g. due to skidding)
@@ -158,11 +172,12 @@ float Skidding::getSteeringWhenSkidding(float steering) const
     case SKID_OLD:            assert(false); break;
     case SKID_SHOW_GFX_LEFT:
     case SKID_SHOW_GFX_RIGHT:
+    case SKID_BREAK:
     case SKID_NONE:           return steering;
         break;
     case SKID_ACCUMULATE_RIGHT:
         {
-            float f = (steering - m_skid_reduce_turn_min) 
+            float f = (steering - m_skid_reduce_turn_min)
                    /  m_skid_reduce_turn_delta;
             return f *2.0f-1.0f;
             break;
@@ -186,7 +201,7 @@ float Skidding::getSteeringWhenSkidding(float steering) const
  *               the kart's max steering angle.
  *  \param skidding  True if the skid button is pressed.
  */
-void Skidding::update(float dt, bool is_on_ground, 
+void Skidding::update(float dt, bool is_on_ground,
                       float steering, KartControl::SkidControl skidding)
 {
     // If a kart animation is shown, stop all skidding bonuses.
@@ -195,10 +210,20 @@ void Skidding::update(float dt, bool is_on_ground,
         reset();
         return;
     }
+
+    // No skidding backwards or while stopped
+    if(m_kart->getSpeed() < 0.001f && 
+       m_skid_state != SKID_NONE && m_skid_state != SKID_BREAK)
+    {
+        m_skid_state = SKID_BREAK;
+        m_kart->getKartGFX()->setCreationRateAbsolute(KartGFX::KGFX_SKIDL, 0);
+        m_kart->getKartGFX()->setCreationRateAbsolute(KartGFX::KGFX_SKIDR, 0);
+    }
+
     m_skid_bonus_ready = false;
     if (is_on_ground)
     {
-        if((fabs(steering) > 0.001f) && 
+        if((fabs(steering) > 0.001f) &&
             m_kart->getSpeed()>m_min_skid_speed &&
             (skidding==KartControl::SC_LEFT||skidding==KartControl::SC_RIGHT))
         {
@@ -216,7 +241,7 @@ void Skidding::update(float dt, bool is_on_ground,
 
     if(m_skid_factor>m_skid_max)
         m_skid_factor = m_skid_max;
-    else 
+    else
         if(m_skid_factor<1.0f) m_skid_factor = 1.0f;
 
     // FIXME hiker: remove once the new skidding code is finished.
@@ -249,21 +274,21 @@ void Skidding::update(float dt, bool is_on_ground,
     //    is reached when the skid key is pressed.
     // SKID_ACCUMULATE_{LEFT,RIGHT}:
     //    The kart is still skidding. The skidding time will be
-    //    accumulated in m_skid_time, and once the minimum time for a 
+    //    accumulated in m_skid_time, and once the minimum time for a
     //    bonus is reached, the "bonus gfx now available" gfx is shown.
     //    If the skid button is not pressed anymore, this will trigger
-    //    a potential bonus. Also the rotation of the physical body to 
-    //    be in synch with the graphical kart is started (which is 
-    //    independently handled in the kart physics). 
+    //    a potential bonus. Also the rotation of the physical body to
+    //    be in synch with the graphical kart is started (which is
+    //    independently handled in the kart physics).
     // SKID_SHOW_GFX_{LEFT<RIGHT}
     //    Shows the skidding gfx while the bonus is available.
-    // FIXME: what should we do if skid key is pressed while still in 
+    // FIXME: what should we do if skid key is pressed while still in
     //   SKID_SHOW_GFX??? Adjusting the body rotation is difficult.
     //   For now skidding will only start again once SKID_SHOW_GFX
     //   is changed to SKID_NONE.
     switch(m_skid_state)
     {
-    case SKID_NONE: 
+    case SKID_NONE:
         {
             if(skidding!=KartControl::SC_LEFT &&
                 skidding!=KartControl::SC_RIGHT)
@@ -273,24 +298,24 @@ void Skidding::update(float dt, bool is_on_ground,
             if(m_remaining_jump_time>0 ||
                 m_kart->getSpeed() <m_min_skid_speed) break;
 
-            m_skid_state = skidding==KartControl::SC_RIGHT 
+            m_skid_state = skidding==KartControl::SC_RIGHT
                          ? SKID_ACCUMULATE_RIGHT
                          : SKID_ACCUMULATE_LEFT;
-            // Add a little jump to the kart. Determine the vertical speed 
+            // Add a little jump to the kart. Determine the vertical speed
             // necessary for the kart to go 0.5*jump_time up (then it needs
             // the same time to come down again), based on v = gravity * t.
-            // Then use this speed to determine the impulse necessary to 
+            // Then use this speed to determine the impulse necessary to
             // reach this speed.
-            float v = World::getWorld()->getTrack()->getGravity() 
+            float v = World::getWorld()->getTrack()->getGravity()
                     * 0.5f*m_physical_jump_time;
             btVector3 imp(0, v / m_kart->getBody()->getInvMass(),0);
             m_kart->getVehicle()->getRigidBody()->applyCentralImpulse(imp);
 
             // Some karts might use a graphical-only jump. Set it up:
-            m_jump_speed = World::getWorld()->getTrack()->getGravity() 
+            m_jump_speed = World::getWorld()->getTrack()->getGravity()
                          * 0.5f*m_graphical_jump_time;
             m_remaining_jump_time = m_graphical_jump_time;
-             
+
 #ifdef SKID_DEBUG
 #define SPEED 20.0f
             updateSteering(steering, dt);
@@ -319,7 +344,7 @@ void Skidding::update(float dt, bool is_on_ground,
                 Vec3 xyz(real_x, 0.2f, sqrt(r*r-(r-x)*(r-x))*(1.0f+SPEED/150.0f)
                           *(1+(angle/m_kart->getKartProperties()->getMaxSteerAngle(SPEED)-0.6f)*0.1f));
                 Vec3 xyz1=m_kart->getTrans()(xyz);
-                printf("predict %f %f %f speed %f angle %f\n", 
+                Log::debug("Skidding", "predict %f %f %f speed %f angle %f",
                     xyz1.getX(), xyz1.getY(), xyz1.getZ(),
                     m_kart->getSpeed(), angle);
                 m_predicted_curve->addPoint(xyz);
@@ -328,6 +353,11 @@ void Skidding::update(float dt, bool is_on_ground,
 #endif
             m_skid_time  = 0;   // fallthrough
         }
+    case SKID_BREAK:
+        {
+            updateSteering(steering, dt);
+            break;
+        }
     case SKID_ACCUMULATE_LEFT:
     case SKID_ACCUMULATE_RIGHT:
         {
@@ -343,9 +373,9 @@ void Skidding::update(float dt, bool is_on_ground,
             }
 
             m_actual_curve->addPoint(m_kart->getXYZ());
-            printf("actual %f %f %f turn %f speed %f angle %f\n",
+            Log::debug("Skidding", "actual %f %f %f turn %f speed %f angle %f",
                 m_kart->getXYZ().getX(),m_kart->getXYZ().getY(),m_kart->getXYZ().getZ(),
-                m_real_steering, m_kart->getSpeed(), 
+                m_real_steering, m_kart->getSpeed(),
                 m_kart->getKartProperties()->getMaxSteerAngle(m_kart->getSpeed()));
 #endif
             m_skid_time += dt;
@@ -353,7 +383,7 @@ void Skidding::update(float dt, bool is_on_ground,
             unsigned int level = getSkidBonus(&bonus_time, &bonus_speed,
                                               &bonus_force);
             // If at least level 1 bonus is reached, show appropriate gfx
-            if(level>0) 
+            if(level>0)
             {
                 m_skid_bonus_ready = true;
                 m_kart->getKartGFX()->setSkidLevel(level);
@@ -362,7 +392,7 @@ void Skidding::update(float dt, bool is_on_ground,
             // SKID_SHOW_GFX_*
             if(skidding == KartControl::SC_NONE)
             {
-                m_skid_state = m_skid_state == SKID_ACCUMULATE_LEFT 
+                m_skid_state = m_skid_state == SKID_ACCUMULATE_LEFT
                              ? SKID_SHOW_GFX_LEFT
                              : SKID_SHOW_GFX_RIGHT;
                 float t = std::min(m_skid_time, m_skid_visual_time);
@@ -376,16 +406,21 @@ void Skidding::update(float dt, bool is_on_ground,
                 if(bonus_time>0)
                 {
                     m_kart->getKartGFX()
-                          ->setCreationRateRelative(KartGFX::KGFX_SKID, 1.0f);
+                          ->setCreationRateRelative(KartGFX::KGFX_SKIDL, 1.0f);
+                    m_kart->getKartGFX()
+                          ->setCreationRateRelative(KartGFX::KGFX_SKIDR, 1.0f);
                     m_kart->m_max_speed->
                         instantSpeedIncrease(MaxSpeed::MS_INCREASE_SKIDDING,
                                              bonus_speed, bonus_speed,
-                                             bonus_force, bonus_time, 
+                                             bonus_force, bonus_time,
                                              /*fade-out-time*/ 1.0f);
                 }
-                else
+                else {
                     m_kart->getKartGFX()
-                          ->setCreationRateAbsolute(KartGFX::KGFX_SKID, 0);
+                          ->setCreationRateAbsolute(KartGFX::KGFX_SKIDL, 0);
+                    m_kart->getKartGFX()
+                          ->setCreationRateAbsolute(KartGFX::KGFX_SKIDR, 0);
+            }
             }
             break;
         }   // case
@@ -396,7 +431,9 @@ void Skidding::update(float dt, bool is_on_ground,
         {
             m_skid_time = 0;
             m_kart->getKartGFX()
-                  ->setCreationRateAbsolute(KartGFX::KGFX_SKID, 0);
+                  ->setCreationRateAbsolute(KartGFX::KGFX_SKIDL, 0);
+            m_kart->getKartGFX()
+                  ->setCreationRateAbsolute(KartGFX::KGFX_SKIDR, 0);
             m_skid_state = SKID_NONE;
         }
     }   // switch
@@ -411,7 +448,7 @@ void Skidding::update(float dt, bool is_on_ground,
  *  \param bonus_force Additional engine force.
  *  \return The bonus level: 0 = no bonus, 1 = first entry in bonus array etc.
  */
-unsigned int Skidding::getSkidBonus(float *bonus_time, 
+unsigned int Skidding::getSkidBonus(float *bonus_time,
                                     float *bonus_speed,
                                     float *bonus_force) const
 {
