@@ -1,6 +1,7 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2004-2013 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -39,6 +40,9 @@ public:
         ~MainLoop();
     void run();
     void abort();
+    // ------------------------------------------------------------------------
+    /** Returns true if STK is to be stoppe. */
+    bool isAborted() const { return m_abort; }
 };   // MainLoop
 
 extern MainLoop* main_loop;
