@@ -1,5 +1,5 @@
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004 SuperTuxKart-Team
+//  Copyright (C) 2004-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 #include "karts/abstract_kart.hpp"
 #include "karts/controller/controller.hpp"
 #include "karts/kart_properties.hpp"
-#include "network/network_manager.hpp" 
+#include "network/network_manager.hpp"
 #include "physics/physics.hpp"
 #include "race/history.hpp"
 #include "states_screens/race_gui_base.hpp"
@@ -41,7 +41,6 @@
  */
 LinearWorld::LinearWorld() : WorldWithRank()
 {
-    m_kart_display_info    = NULL;
     m_last_lap_sfx         = sfx_manager->createSoundSource("last_lap_fanfare");
     m_last_lap_sfx_played  = false;
     m_last_lap_sfx_playing = false;
@@ -56,41 +55,33 @@ LinearWorld::LinearWorld() : WorldWithRank()
 void LinearWorld::init()
 {
     WorldWithRank::init();
-    
+
     assert(!m_track->isArena());
-    
+    assert(!m_track->isSoccer());
+
     m_last_lap_sfx_played           = false;
     m_last_lap_sfx_playing          = false;
-    const unsigned int kart_amount  = m_karts.size();
-    
-    m_kart_display_info = new RaceGUIBase::KartIconDisplayInfo[kart_amount];
-
-    for(unsigned int n=0; n<kart_amount; n++)
-    {
-        KartInfo info;
-        info.getSector()->update(m_karts[n]->getXYZ());
-        m_kart_info.push_back(info);
-    }   // next kart
 
+    // The values are initialised in reset()
+    m_kart_info.resize(m_karts.size());
 }   // init
 
 //-----------------------------------------------------------------------------
+/** The destructor frees al data structures.
+ */
 LinearWorld::~LinearWorld()
 {
     sfx_manager->deleteSFX(m_last_lap_sfx);
-
-    // In case that a track is not found, m_kart_display info was never
-    // initialised.
-    if(m_kart_display_info)
-        delete[] m_kart_display_info;
 }   // ~LinearWorld
 
 //-----------------------------------------------------------------------------
-void LinearWorld::restartRace()
+/** Called before a race is started (or restarted). It resets the data
+ *  structures that keep track of position and distance long track of
+ *  all karts.
+ */
+void LinearWorld::reset()
 {
-    WorldWithRank::restartRace();
-    //if(m_last_lap_sfx->getStatus()== SFXManager::SFX_PLAYING)
-    //    m_last_lap_sfx->stop();
+    WorldWithRank::reset();
     m_last_lap_sfx_played = false;
     m_last_lap_sfx_playing = false;
 
@@ -98,24 +89,45 @@ void LinearWorld::restartRace()
     for(unsigned int i=0; i<kart_amount; i++)
     {
         m_kart_info[i].reset();
-        m_kart_info[i].getSector()->update(m_karts[i]->getXYZ());
+        m_kart_info[i].getTrackSector()->update(m_karts[i]->getXYZ());
     }   // next kart
 
-    // First all kart infos must be updated before the kart position can be 
+    // At the moment the last kart would be the one that is furthest away
+    // from the start line, i.e. it would determine the amount by which
+    // the track length must be extended (to avoid negative numbers in
+    // estimateFinishTimeForKart()). On the other hand future game modes
+    // might not follow this rule, so check the distance for all karts here:
+    m_distance_increase = m_track->getTrackLength();
+    for(unsigned int i=0; i<kart_amount; i++)
+    {
+        m_distance_increase = std::min(m_distance_increase,
+                                       getDistanceDownTrackForKart(i));
+    }
+    // Track length - minimum distance is how much the track length must
+    // be increased to avoid negative values in estimateFinishTimeForKart
+    // Increase this value somewhat in case that a kart drivess/slides
+    // backwards a little bit at start.
+    m_distance_increase = m_track->getTrackLength() - m_distance_increase
+                        + 5.0f;
+
+    if(m_distance_increase<0) m_distance_increase = 1.0f;  // shouldn't happen
+
+    // First all kart infos must be updated before the kart position can be
     // recomputed, since otherwise 'new' (initialised) valued will be compared
     // with old values.
     updateRacePosition();
-    
+
 #ifdef DEBUG
-    //FIXME: this could be defined somewhere in a central header so it can 
+    //FIXME: this could be defined somewhere in a central header so it can
     //       be used everywhere
 #define assertExpr( ARG1, OP, ARG2 ) if (!(ARG1 OP ARG2)) \
         { \
-            std::cerr << "Failed assert " << #ARG1 << #OP << #ARG2 << " @ " << __FILE__ << ":" << __LINE__ \
+            std::cerr << "Failed assert " << #ARG1 << #OP << #ARG2 << " @ " \
+                      << __FILE__ << ":" << __LINE__ \
                       << "; values are (" << ARG1 << #OP << ARG2 << ")\n"; \
             assert(false); \
         }
-    
+
     for (unsigned int i=0; i<kart_amount; i++)
     {
         for (unsigned int j=i+1; j<kart_amount; j++)
@@ -125,8 +137,8 @@ void LinearWorld::restartRace()
         }
     }
 #endif
-    
-}   // restartRace
+
+}   // reset
 
 //-----------------------------------------------------------------------------
 /** General update function called once per frame. This updates the kart
@@ -138,15 +150,15 @@ void LinearWorld::update(float dt)
     // run generic parent stuff that applies to all modes. It
     // especially updates the kart positions.
     WorldWithRank::update(dt);
-    
-    if (m_last_lap_sfx_playing && 
+
+    if (m_last_lap_sfx_playing &&
         m_last_lap_sfx->getStatus() != SFXManager::SFX_PLAYING)
     {
         if(music_manager->getCurrentMusic())
             music_manager->getCurrentMusic()->resetTemporaryVolume();
         m_last_lap_sfx_playing = false;
     }
-    
+
     const unsigned int kart_amount = getNumKarts();
 
     // Do stuff specific to this subtype of race.
@@ -156,12 +168,12 @@ void LinearWorld::update(float dt)
         KartInfo& kart_info = m_kart_info[n];
         AbstractKart* kart = m_karts[n];
 
-        // Nothing to do for karts that are currently being 
+        // Nothing to do for karts that are currently being
         // rescued or eliminated
         if(kart->getKartAnimation()) continue;
 
-        kart_info.getSector()->update(kart->getXYZ());
-        kart_info.m_overall_distance = kart_info.m_race_lap 
+        kart_info.getTrackSector()->update(kart->getXYZ());
+        kart_info.m_overall_distance = kart_info.m_race_lap
                                      * m_track->getTrackLength()
                         + getDistanceDownTrackForKart(kart->getWorldKartId());
     }   // for n
@@ -172,23 +184,23 @@ void LinearWorld::update(float dt)
     // ---------------------------------------------------------------
     WorldWithRank::updateTrack(dt);
     updateRacePosition();
-    
+
     for (unsigned int i=0; i<kart_amount; i++)
     {
         // ---------- update rank ------
-        if (m_karts[i]->hasFinishedRace() || 
+        if (m_karts[i]->hasFinishedRace() ||
             m_karts[i]->isEliminated()       ) continue;
 
         // During the last lap update the estimated finish time.
         // This is used to play the faster music, and by the AI
         if (m_kart_info[i].m_race_lap == race_manager->getNumLaps()-1)
         {
-            m_kart_info[i].m_estimated_finish = 
+            m_kart_info[i].m_estimated_finish =
                 estimateFinishTimeForKart(m_karts[i]);
         }
         checkForWrongDirection(i);
     }
-    
+
 #ifdef DEBUG
     // Debug output in case that the double position error occurs again.
     std::vector<int> pos_used;
@@ -206,7 +218,7 @@ void LinearWorld::update(float dt)
                     m_kart_info[j].m_race_lap,
                     getDistanceDownTrackForKart(m_karts[j]->getWorldKartId()),
                     m_kart_info[j].m_overall_distance,
-                    (m_karts[j]->getPosition() == m_karts[i]->getPosition() 
+                    (m_karts[j]->getPosition() == m_karts[i]->getPosition()
                      ? "<--- !!!" : "")                                      );
             }
         }
@@ -224,18 +236,18 @@ void LinearWorld::newLap(unsigned int kart_index)
     KartInfo &kart_info = m_kart_info[kart_index];
     AbstractKart *kart  = m_karts[kart_index];
 
-    // Only update the kart controller if a kart that has already finished 
-    // the race crosses the start line again. This avoids 'fastest lap' 
+    // Only update the kart controller if a kart that has already finished
+    // the race crosses the start line again. This avoids 'fastest lap'
     // messages if the end controller does a fastest lap, but especially
     // allows the end controller to switch end cameras
-    if(kart->hasFinishedRace()) 
+    if(kart->hasFinishedRace())
     {
         kart->getController()->newLap(kart_info.m_race_lap);
         return;
     }
 
     const int lap_count = race_manager->getNumLaps();
-    
+
     // Only increase the lap counter and set the new time if the
     // kart hasn't already finished the race (otherwise the race_gui
     // will begin another countdown).
@@ -244,7 +256,7 @@ void LinearWorld::newLap(unsigned int kart_index)
         assert(kart->getWorldKartId()==kart_index);
         kart_info.m_time_at_last_lap=getTime();
         kart_info.m_race_lap++;
-        m_kart_info[kart_index].m_overall_distance = 
+        m_kart_info[kart_index].m_overall_distance =
               m_kart_info[kart_index].m_race_lap * m_track->getTrackLength()
             + getDistanceDownTrackForKart(kart->getWorldKartId());
     }
@@ -260,7 +272,7 @@ void LinearWorld::newLap(unsigned int kart_index)
                 m_last_lap_sfx->play();
                 m_last_lap_sfx_played = true;
                 m_last_lap_sfx_playing = true;
-            
+
                 // In case that no music is defined
                 if(music_manager->getCurrentMusic() &&
                     music_manager->getMasterMusicVolume() > 0.2f)
@@ -275,18 +287,19 @@ void LinearWorld::newLap(unsigned int kart_index)
             }
         }
     }
-    else if (raceHasLaps() && kart_info.m_race_lap > 0 && kart_info.m_race_lap+1 < lap_count)
+    else if (raceHasLaps() && kart_info.m_race_lap > 0 &&
+             kart_info.m_race_lap+1 < lap_count)
     {
         m_race_gui->addMessage(_("Lap %i", kart_info.m_race_lap+1),
-                               kart, 3.0f, video::SColor(255, 210, 100, 50), 
+                               kart, 3.0f, video::SColor(255, 210, 100, 50),
                                true);
     }
 
-    // The race positions must be updated here: consider the situation where 
+    // The race positions must be updated here: consider the situation where
     // the first kart does not cross the finish line in its last lap, instead
-    // it passes it, the kart reverses and crosses the finishing line 
+    // it passes it, the kart reverses and crosses the finishing line
     // backwards. Just before crossing the finishing line the kart will be on
-    // the last lap, but with a distance along the track close to zero. 
+    // the last lap, but with a distance along the track close to zero.
     // Therefore its position will be wrong. If the race position gets updated
     // after increasing the number of laps (but before tagging the kart to have
     // finished the race) the position will be correct (since the kart now
@@ -299,7 +312,7 @@ void LinearWorld::newLap(unsigned int kart_index)
     // Similarly the situation can happen if the distance along track should
     // go back to zero before actually crossing the finishing line. While this
     // should not happen, it could potentially be caused by floating point
-    // errors. In this case the call to updateRacePosition will avoid 
+    // errors. In this case the call to updateRacePosition will avoid
     // duplicated race positions as well.
     updateRacePosition();
 
@@ -329,20 +342,20 @@ void LinearWorld::newLap(unsigned int kart_index)
         kart_info.m_race_lap>0)
     {
         m_fastest_lap = time_per_lap;
-        
+
         std::string s = StringUtils::timeToString(time_per_lap);
-        
+
         irr::core::stringw m_fastest_lap_message;
         //I18N: as in "fastest lap: 60 seconds by Wilber"
-        m_fastest_lap_message += _C("fastest_lap", "%s by %s", s.c_str(), 
+        m_fastest_lap_message += _C("fastest_lap", "%s by %s", s.c_str(),
                                     core::stringw(kart->getName()));
-        
+
         m_race_gui->addMessage(m_fastest_lap_message, NULL,
                                3.0f, video::SColor(255, 255, 255, 255), false);
-        
+
         m_race_gui->addMessage(_("New fastest lap"), NULL,
                                3.0f, video::SColor(255, 255, 255, 255), false);
-        
+
     } // end if new fastest lap
 
     kart_info.m_lap_start_time = getTime();
@@ -350,32 +363,32 @@ void LinearWorld::newLap(unsigned int kart_index)
 }   // newLap
 
 //-----------------------------------------------------------------------------
-/** Gets the sector a kart is on. This return UNKNOWN_SECTOR if the kart_id
- *  is larger than the current kart info. This is necessary in the case that
- *  a collision with the track happens during resetAllKarts: at this time 
+/** Gets the sector a kart is on. This function returns UNKNOWN_SECTOR if the
+ *  kart_id is larger than the current kart info. This is necessary in the case
+ *  that a collision with the track happens during resetAllKarts: at this time
  *  m_kart_info is not initialised (and has size 0), so it would trigger this
- *  assert. While this normally does not happen, it is useful for track 
+ *  assert. While this normally does not happen, it is useful for track
  *  designers that STK does not crash.
- *  \param kart_id The world kart id of the kart for which to return 
+ *  \param kart_id The world kart id of the kart for which to return
  *                 the sector.
  */
 int LinearWorld::getSectorForKart(const AbstractKart *kart) const
 {
     if(kart->getWorldKartId()>=m_kart_info.size())
         return QuadGraph::UNKNOWN_SECTOR;
-    return m_kart_info[kart->getWorldKartId()].getSector()
+    return m_kart_info[kart->getWorldKartId()].getTrackSector()
           ->getCurrentGraphNode();
 }   // getSectorForKart
 
 //-----------------------------------------------------------------------------
-/** Returns the distance the kart has travelled along the track since 
+/** Returns the distance the kart has travelled along the track since
  *  crossing the start line..
  *  \param kart_id Index of the kart.
  */
 float LinearWorld::getDistanceDownTrackForKart(const int kart_id) const
 {
     assert(kart_id < (int)m_kart_info.size());
-    return m_kart_info[kart_id].getSector()->getDistanceFromStart();
+    return m_kart_info[kart_id].getTrackSector()->getDistanceFromStart();
 }   // getDistanceDownTrackForKart
 
 //-----------------------------------------------------------------------------
@@ -386,7 +399,7 @@ float LinearWorld::getDistanceDownTrackForKart(const int kart_id) const
 float LinearWorld::getDistanceToCenterForKart(const int kart_id) const
 {
     assert(kart_id < (int)m_kart_info.size());
-    return m_kart_info[kart_id].getSector()->getDistanceToCenter();
+    return m_kart_info[kart_id].getTrackSector()->getDistanceToCenter();
 }   // getDistanceToCenterForKart
 
 //-----------------------------------------------------------------------------
@@ -415,7 +428,8 @@ float LinearWorld::getTimeAtLapForKart(const int kart_id) const
 }   // getTimeAtLapForKart
 
 //-----------------------------------------------------------------------------
-RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
+void LinearWorld::getKartsDisplayInfo(
+                           std::vector<RaceGUIBase::KartIconDisplayInfo> *info)
 {
     int   laps_of_leader       = -1;
     float time_of_leader       = -1;
@@ -425,13 +439,11 @@ RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
     const unsigned int kart_amount = getNumKarts();
     for(unsigned int i = 0; i < kart_amount ; i++)
     {
-        RaceGUIBase::KartIconDisplayInfo& rank_info = m_kart_display_info[i];
+        RaceGUIBase::KartIconDisplayInfo& rank_info = (*info)[i];
         AbstractKart* kart = m_karts[i];
 
         // reset color
-        rank_info.r = 1.0;
-        rank_info.g = 1.0;
-        rank_info.b = 1.0;
+        rank_info.m_color = video::SColor(255, 255, 255, 255);
         rank_info.lap = -1;
 
         if(kart->isEliminated()) continue;
@@ -441,7 +453,7 @@ RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
 
         if(current_lap > laps_of_leader)
         {
-            // more laps than current leader --> new leader and 
+            // more laps than current leader --> new leader and
             // new time computation
             laps_of_leader = current_lap;
             time_of_leader = lap_time;
@@ -455,7 +467,7 @@ RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
     // we now know the best time of the lap. fill the remaining bits of info
     for(unsigned int i = 0; i < kart_amount ; i++)
     {
-        RaceGUIBase::KartIconDisplayInfo& rank_info = m_kart_display_info[i];
+        RaceGUIBase::KartIconDisplayInfo& rank_info = (*info)[i];
         KartInfo& kart_info = m_kart_info[i];
         AbstractKart* kart = m_karts[i];
 
@@ -463,14 +475,14 @@ RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
 
         // Don't compare times when crossing the start line first
         if(laps_of_leader>0                                                &&
-           (getTime() - getTimeAtLapForKart(kart->getWorldKartId())<5.0f || 
+           (getTime() - getTimeAtLapForKart(kart->getWorldKartId())<5.0f ||
             rank_info.lap != laps_of_leader)                               &&
             raceHasLaps())
         {  // Display for 5 seconds
             std::string str;
             if(position == 1)
             {
-                str = " " + StringUtils::timeToString( 
+                str = " " + StringUtils::timeToString(
                                  getTimeAtLapForKart(kart->getWorldKartId()) );
             }
             else
@@ -493,26 +505,46 @@ RaceGUIBase::KartIconDisplayInfo* LinearWorld::getKartsDisplayInfo()
 
         if(kart_info.m_race_lap>=numLaps)
         {  // kart is finished, display in green
-            rank_info.g = rank_info.b = 0;
+            rank_info.m_color.setGreen(0);
+            rank_info.m_color.setBlue(0);
         }
         else if(kart_info.m_race_lap>=0 && numLaps>1)
         {
-            rank_info.g = rank_info.b =
-                1.0f-(float)kart_info.m_race_lap/((float)numLaps-1.0f);
+            int col = (int)(255*(1.0f-(float)kart_info.m_race_lap
+                                    /((float)numLaps-1.0f)        ));
+            rank_info.m_color.setBlue(col);
+            rank_info.m_color.setGreen(col);
         }
     }   // next kart
 
 
-    return m_kart_display_info;
 }   // getKartsDisplayInfo
 
 //-----------------------------------------------------------------------------
-/** Estimate the arrival time of any karts that haven't arrived yet by using 
- *  their average speed up to now and the distance still to race. This 
- *  approach guarantees that the order of the karts won't change anymore 
- *  (karts ahead will have a higher average speed and therefore finish the 
- *  race earlier than karts further behind), so the position doesn't have to
- *  be updated to get the correct scoring.
+/** Estimate the arrival time of any karts that haven't arrived yet by using
+ *  their average speed up to now and the distance still to race. This
+ *  approach guarantees that the order of the karts won't change anymore
+ *  (karts ahead will have covered more distance and have therefore a higher
+ *  average speed and therefore finish the race earlier than karts further
+ *  behind), so the position doesn't have to be updated to get the correct
+ *  scoring.
+ *  As so often the devil is in the details: a kart that hasn't crossed the
+ *  starting line has a negative distance (when it is crossing the start line
+ *  its distance becomes 0), which can result in a negative average speed
+ *  (and therefore incorrect estimates). This is partly taken care of by
+ *  adding m_distance_increase to the distance covered by a kart. The value
+ *  of m_distance_increase is a bit more than the distance the last kart
+ *  has from the start line at start time. This guarantees that the distance
+ *  used in computing the average speed is positive in most cases. Only
+ *  exception is if a kart is driving backwards on purpose. While this
+ *  shouldn't happen (the AI doesn't do it, and if it's a player the game
+ *  won't finish so the time estimation won't be called and so the incorrect
+ *  result won't be displayed), this is still taken care of: if the average
+ *  speed is negative, the estimated arrival time of the kart is set to
+ *  99:00 plus kart position. This means that even in this case all karts
+ *  will have a different arrival time.
+ *  \pre The position of the karts are set according to the distance they
+ *       have covered.
  *  \param kart The kart for which to estimate the finishing times.
  */
 float LinearWorld::estimateFinishTimeForKart(AbstractKart* kart)
@@ -521,98 +553,92 @@ float LinearWorld::estimateFinishTimeForKart(AbstractKart* kart)
 
     float full_distance = race_manager->getNumLaps()
                         * m_track->getTrackLength();
+
     if(full_distance == 0)
-        full_distance = 1;  // For 0 lap races avoid warning below
+        full_distance = 1.0f;   // For 0 lap races avoid warning below
+
 #ifdef DEBUG
-    if(full_distance < kart_info.m_overall_distance)
+    if(kart_info.m_overall_distance > full_distance)
     {
         printf("WARNING: full distance < distance covered for kart '%s':\n",
                kart->getIdent().c_str());
         printf("%f < %f\n", full_distance, kart_info.m_overall_distance);
     }
 #endif
-    // Avoid potential problems (floating point issues, coding bug?) if a 
+    // Avoid potential problems (floating point issues, coding bug?) if a
     // kart has driven more than the full distance, but not finished:
     // Return the current time plus initial position to spread arrival
     // times a bit. This code should generally not be used at all, it's
     // just here to avoid invalid finishing times.
-    if(full_distance < kart_info.m_overall_distance)
+    if(kart_info.m_overall_distance > full_distance)
         return getTime() + kart->getInitialPosition();
-    
+
     // Finish time is the time needed for the whole race with
-    // the computed average speed computed.
-    float average_speed = getTime()==0 
-                        ? 1.0f 
-                        : kart_info.m_overall_distance/getTime();
+    // the computed average speed computed. The distance is always positive
+    // due to the way m_distance_increase was computed, so average speed
+    // is also always positive.
+    float average_speed = getTime()==0
+                        ? 1.0f
+                        : (m_distance_increase + kart_info.m_overall_distance)
+                          / getTime();
 
-    // Overall distance is negative before the starting line is crossed
-    // for the first time. Unlikely to happen in real races, but will
-    // happen in 0-laps races.
-    if(average_speed<0) 
-        average_speed = 1.0f;
+    // Avoid NAN or invalid results when average_speed is very low
+    // or negative (which can happen if a kart drives backwards and
+    // m_overall distance becomes smaller than -m_distance_increase).
+    // In this case set the time to 99 minutes, offset by kart
+    // position (to spread arrival times for all karts that arrive
+    // even later). This works for up to 60 karts (otherwise the
+    // time displayed would become too long: 100:xx:yy).
+    if(average_speed<0.01f)
+        return 99*60.0f + kart->getPosition();
 
-    // Avoid NAN as results when average_speed is low
-    // Instead just set time to 99:59:00
-    if(average_speed<0.1f)
-        return 99*60+59;
-    return getTime() + (full_distance - kart_info.m_overall_distance)  
-                     / average_speed;
+    float est_time = getTime() + (full_distance - kart_info.m_overall_distance)
+                                 / average_speed;
 
+    // Avoid times > 99:00 - in this case use kart position to spread
+    // arrival time so that each kart has a unique value. The pre-condition
+    // guarantees that this works correctly (higher position -> less distance
+    // covered -> later arrival time).
+    if(est_time>99*60.0f)
+        return 99*60.0f + kart->getPosition();
+
+    return est_time;
 }   // estimateFinishTimeForKart
 
-//-----------------------------------------------------------------------------
-/** Decide where to drop a rescued kart
+// ------------------------------------------------------------------------
+/** Returns the number of rescue positions on a given track, which in 
+ *  linear races is just the number of driveline quads.
   */
-void LinearWorld::moveKartAfterRescue(AbstractKart* kart)
+unsigned int LinearWorld::getNumberOfRescuePositions() const
+{
+    return QuadGraph::get()->getNumNodes();
+}   // getNumberOfRescuePositions
+
+// ------------------------------------------------------------------------
+unsigned int LinearWorld::getRescuePositionIndex(AbstractKart *kart)
 {
     KartInfo& info = m_kart_info[kart->getWorldKartId()];
 
-    info.getSector()->rescue();
-    int sector = info.getSector()->getCurrentGraphNode();
-    kart->setXYZ( QuadGraph::get()
-                  ->getQuadOfNode(sector).getCenter());
-
-    btQuaternion heading(btVector3(0.0f, 1.0f, 0.0f),
-                         m_track->getAngle(sector) );
-    kart->setRotation(heading);
-
-    // A certain epsilon is added here to the Z coordinate, in case
-    // that the drivelines are somewhat under the track. Otherwise, the
-    // kart might be placed a little bit under the track, triggering
-    // a rescue, ... (experimentally found value)
-    float epsilon = 0.5f * kart->getKartHeight();
+    info.getTrackSector()->rescue();
+    // Setting XYZ for the kart is important since otherwise the kart
+    // will not detect the right material again when doing the next
+    // raycast to detect where it is driving on (--> potential rescue loop)
+    return info.getTrackSector()->getCurrentGraphNode();
+}   // getRescuePositionIndex
 
+// ------------------------------------------------------------------------
+btTransform LinearWorld::getRescueTransform(unsigned int index) const
+{
+    const Vec3 &xyz = QuadGraph::get()->getQuadOfNode(index).getCenter();
     btTransform pos;
-    pos.setOrigin(kart->getXYZ()+btVector3(0, kart->getKartHeight() + epsilon,
-                                           0));
+    pos.setOrigin(xyz);
     pos.setRotation(btQuaternion(btVector3(0.0f, 1.0f, 0.0f),
-                    m_track->getAngle(sector)));
-
-    kart->getBody()->setCenterOfMassTransform(pos);
-
-    //project kart to surface of track
-    bool kart_over_ground = m_physics->projectKartDownwards(kart);
-
-    if (kart_over_ground)
-    {
-        //add vertical offset so that the kart starts off above the track
-        float vertical_offset = 
-              kart->getKartProperties()->getVertRescueOffset() 
-            * kart->getKartHeight();
-        kart->getBody()->translate(btVector3(0, vertical_offset, 0));
-    }
-    else
-    {
-        fprintf(stderr, "WARNING: invalid position after rescue for kart %s "
-                        "on track %s.\n",
-                (kart->getIdent().c_str()), m_track->getIdent().c_str());
-    }
-
-
-}   // moveKartAfterRescue
+                    m_track->getAngle(index)));
+    return pos;
+}   // getRescueTransform
 
 //-----------------------------------------------------------------------------
-/** Find the position (rank) of every kart. ATM it uses a stable O(n^2) 
+/** Find the position (rank) of every kart. ATM it uses a stable O(n^2)
  *  algorithm by counting for each kart how many other karts are ahead of
  *  it.
  */
@@ -625,7 +651,7 @@ void LinearWorld::updateRacePosition()
 #ifdef DEBUG
     bool rank_changed = false;
 #endif
-    
+
     // NOTE: if you do any changes to this loop, the next loop (see
     // DEBUG_KART_RANK below) needs to have the same changes applied
     // so that debug output is still correct!!!!!!!!!!!
@@ -645,19 +671,19 @@ void LinearWorld::updateRacePosition()
             continue;
         }
         KartInfo& kart_info = m_kart_info[i];
-        
+
         int p = 1 ;
 
         const unsigned int my_id = kart->getWorldKartId();
         const float my_distance  = m_kart_info[my_id].m_overall_distance;
 
-        // Count karts ahead of the current kart, i.e. kart that are 
+        // Count karts ahead of the current kart, i.e. kart that are
         // already finished or have covered a larger overall distance.
         for (unsigned int j = 0 ; j < kart_amount ; j++)
         {
             // don't compare a kart with itself and ignore eliminated karts
             if(j == my_id || m_karts[j]->isEliminated())
-                continue; 
+                continue;
 
             // If the other kart has:
             // - finished the race (but this kart hasn't)
@@ -685,23 +711,23 @@ void LinearWorld::updateRacePosition()
             std::cerr <<  "Info used to decide ranking :\n";
             for (unsigned int d=0; d<kart_amount; d++)
             {
-                std::cerr << "   kart " << m_karts[d]->getIdent() 
+                std::cerr << "   kart " << m_karts[d]->getIdent()
                           << " has finished(" << m_karts[d]->hasFinishedRace()
-                          << "), is at lap (" << getLapForKart(d) 
+                          << "), is at lap (" << getLapForKart(d)
                           << "), is at distance("
                           << m_kart_info[d].m_overall_distance
                           << "), is eliminated(" << m_karts[d]->isEliminated()
                           << ")" << std::endl;
             }
-                        
+
             std::cerr <<  "Who has each ranking so far :\n";
             for (unsigned int d=0; d<i; d++)
             {
-                std::cerr << "    " << m_karts[d]->getIdent() << " has rank " 
+                std::cerr << "    " << m_karts[d]->getIdent() << " has rank "
                           << m_karts[d]->getPosition() << std::endl;
             }
-            
-            std::cerr << "    --> And " << kart->getIdent() 
+
+            std::cerr << "    --> And " << kart->getIdent()
                       << " is being set at rank " << p << std::endl;
             history->Save();
             assert(false);
@@ -723,7 +749,7 @@ void LinearWorld::updateRacePosition()
         }
     }   // for i<kart_amount
 
-    // Define this to get a detailled analyses each time a race position 
+    // Define this to get a detailled analyses each time a race position
     // changes.
 #ifdef DEBUG
 #undef DEBUG_KART_RANK
@@ -734,7 +760,7 @@ void LinearWorld::updateRacePosition()
         for (unsigned int i=0; i<kart_amount; i++)
         {
             AbstractKart* kart = m_karts[i];
-            std::cout << "counting karts ahead of " << kart->getIdent() 
+            std::cout << "counting karts ahead of " << kart->getIdent()
                 << " (laps "           << m_kart_info[i].m_race_lap
                 << ", progress "       << m_kart_info[i].m_overall_distance
                 << " finished "        << kart->hasFinishedRace()
@@ -759,7 +785,7 @@ void LinearWorld::updateRacePosition()
                 {
                     std::cout << "    " << p << " : " << m_karts[j]->getIdent()
                               << " because it is eliminated.\n";
-                    continue; 
+                    continue;
                 }
                 if(!kart->hasFinishedRace() && m_karts[j]->hasFinishedRace())
                 {
@@ -772,16 +798,16 @@ void LinearWorld::updateRacePosition()
                 {
                     p++;
                     std::cout << "    " << p << " : " << m_karts[j]->getIdent()
-                        << " because it is ahead " 
+                        << " because it is ahead "
                         << m_kart_info[j].m_overall_distance <<".\n";
                     continue;
                 }
                 if(m_kart_info[j].m_overall_distance == my_distance &&
-                    m_karts[j]->getInitialPosition()<kart->getInitialPosition() )
+                   m_karts[j]->getInitialPosition()<kart->getInitialPosition())
                 {
                     p++;
                     std::cout << "    " << p << " : " << m_karts[j]->getIdent()
-                        << " has same distance, but started ahead " 
+                        << " has same distance, but started ahead "
                         << m_karts[j]->getInitialPosition()<<".\n";
                 }
             }   // next kart j
@@ -802,14 +828,14 @@ void LinearWorld::updateRacePosition()
 void LinearWorld::checkForWrongDirection(unsigned int i)
 {
     if(!m_karts[i]->getController()->isPlayerController()) return;
-    if(!m_kart_info[i].getSector()->isOnRoad()||
+    if(!m_kart_info[i].getTrackSector()->isOnRoad()||
         m_karts[i]->getKartAnimation()) return;
 
     const AbstractKart *kart=m_karts[i];
     // If the kart can go in more than one directions from the current track
     // don't do any reverse message handling, since it is likely that there
     // will be one direction in which it isn't going backwards anyway.
-    int sector = m_kart_info[i].getSector()->getCurrentGraphNode();
+    int sector = m_kart_info[i].getTrackSector()->getCurrentGraphNode();
     if(QuadGraph::get()->getNumberOfSuccessors(sector)>1)
         return;
 
