@@ -1,5 +1,7 @@
+
+
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -40,7 +42,7 @@ ThreeStrikesBattle::ThreeStrikesBattle() : WorldWithRank()
     WorldStatus::setClockMode(CLOCK_CHRONO);
     m_use_highscores = false;
     m_insert_tire = 0;
-    
+
     m_tire = irr_driver->getMesh( file_manager->getModelFile("tire.b3d") );
     irr_driver->grabAllTextures(m_tire);
 }   // ThreeStrikesBattle
@@ -53,34 +55,13 @@ void ThreeStrikesBattle::init()
 {
     WorldWithRank::init();
     m_display_rank = false;
-    
+
     // check for possible problems if AI karts were incorrectly added
     if(getNumKarts() > race_manager->getNumPlayers())
     {
-        fprintf(stderr, "No AI exists for this game mode\n");
-        exit(1);
+        Log::fatal("Three Strikes Battle", "No AI exists for this game mode");
     }
- 
-    const unsigned int kart_amount = m_karts.size();
-    m_kart_display_info = new RaceGUIBase::KartIconDisplayInfo[kart_amount];
-    
-    for(unsigned int n=0; n<kart_amount; n++)
-    {
-        // create the struct that ill hold each player's lives
-        BattleInfo info;
-        info.m_lives         = 3;
-        m_kart_info.push_back(info);
-        
-        // no positions in this mode
-        m_karts[n]->setPosition(-1);
-    }// next kart
-    
-    
-    BattleEvent evt;
-    evt.m_time = 0.0f;
-    evt.m_kart_info = m_kart_info;
-    m_battle_events.push_back(evt);    
-    
+    m_kart_info.resize(m_karts.size());
 }   // ThreeStrikesBattle
 
 //-----------------------------------------------------------------------------
@@ -90,8 +71,6 @@ void ThreeStrikesBattle::init()
 ThreeStrikesBattle::~ThreeStrikesBattle()
 {
     m_tires.clearWithoutDeleting();
-    
-    delete[] m_kart_display_info;
 
     irr_driver->grabAllTextures(m_tire);
     // Remove the mesh from the cache so that the mesh is properly
@@ -101,7 +80,60 @@ ThreeStrikesBattle::~ThreeStrikesBattle()
 }   // ~ThreeStrikesBattle
 
 //-----------------------------------------------------------------------------
-/** Adds two tires to each of the kart. The tires are used to represent 
+/** Called when a battle is restarted.
+ */
+void ThreeStrikesBattle::reset()
+{
+    WorldWithRank::reset();
+
+    const unsigned int kart_amount = m_karts.size();
+
+    for(unsigned int n=0; n<kart_amount; n++)
+    {
+        m_kart_info[n].m_lives  = 3;
+
+        // no positions in this mode
+        m_karts[n]->setPosition(-1);
+
+        scene::ISceneNode* kart_node = m_karts[n]->getNode();
+
+        // FIXME: sorry for this ugly const_cast, irrlicht doesn't seem to allow getting a writable list of children, wtf??
+        core::list<scene::ISceneNode*>& children = const_cast<core::list<scene::ISceneNode*>&>(kart_node->getChildren());
+        for (core::list<scene::ISceneNode*>::Iterator it = children.begin(); it != children.end(); it++)
+        {
+            scene::ISceneNode* curr = *it;
+
+            if (core::stringc(curr->getName()) == "tire1")
+            {
+                curr->setVisible(true);
+            }
+            else if (core::stringc(curr->getName()) == "tire2")
+            {
+                curr->setVisible(true);
+            }
+        }
+
+    }// next kart
+
+    // remove old battle events
+    m_battle_events.clear();
+
+    // add initial battle event
+    BattleEvent evt;
+    evt.m_time = 0.0f;
+    evt.m_kart_info = m_kart_info;
+    m_battle_events.push_back(evt);
+
+    TrackObject *obj;
+    for_in(obj, m_tires)
+    {
+        m_track->getTrackObjectManager()->removeObject(obj);
+    }
+    m_tires.clearWithoutDeleting();
+}   // reset
+
+//-----------------------------------------------------------------------------
+/** Adds two tires to each of the kart. The tires are used to represent
  *  lifes.
  *  \param kart The pointer to the kart (not used here).
  *  \param node The scene node of this kart.
@@ -109,7 +141,7 @@ ThreeStrikesBattle::~ThreeStrikesBattle()
 void ThreeStrikesBattle::kartAdded(AbstractKart* kart, scene::ISceneNode* node)
 {
     float coord = -kart->getKartLength()*0.5f;
-    
+
     scene::IMeshSceneNode* tire_node = irr_driver->addMesh(m_tire, node);
     tire_node->setPosition(core::vector3df(-0.16f, 0.3f, coord - 0.25f));
     tire_node->setScale(core::vector3df(0.4f, 0.4f, 0.4f));
@@ -124,25 +156,22 @@ void ThreeStrikesBattle::kartAdded(AbstractKart* kart, scene::ISceneNode* node)
 }   // kartAdded
 
 //-----------------------------------------------------------------------------
-/** Called when a kart is hit. 
+/** Called when a kart is hit.
  *  \param kart_id The world kart id of the kart that was hit.
  */
-void ThreeStrikesBattle::kartHit(const int kart_id)
+void ThreeStrikesBattle::kartHit(const unsigned int kart_id)
 {
-    assert(kart_id >= 0);
-    assert(kart_id < (int)m_karts.size());
-    
+    assert(kart_id < m_karts.size());
     // make kart lose a life
     m_kart_info[kart_id].m_lives--;
-    
+
     // record event
     BattleEvent evt;
     evt.m_time = getTime();
     evt.m_kart_info = m_kart_info;
-    m_battle_events.push_back(evt);   
-    
+    m_battle_events.push_back(evt);
+
     updateKartRanks();
-        
     // check if kart is 'dead'
     if (m_kart_info[kart_id].m_lives < 1)
     {
@@ -154,31 +183,60 @@ void ThreeStrikesBattle::kartHit(const int kart_id)
         if(wheels[2]) wheels[2]->setVisible(false);
         if(wheels[3]) wheels[3]->setVisible(false);
         eliminateKart(kart_id, /*notify_of_elimination*/ true);
+        // Find a camera of the kart with the most lives ("leader"), and
+        // attach all cameras for this kart to the leader.
+        int max_lives = 0;
+        AbstractKart *leader = NULL;
+        for(unsigned int i=0; i<getNumKarts(); i++)
+        {
+            AbstractKart * const kart = getKart(i);
+            if(kart->isEliminated() || kart->hasFinishedRace() ||
+                kart->getWorldKartId()==kart_id) continue;
+            if(m_kart_info[i].m_lives > max_lives)
+            {
+                leader = kart;
+                max_lives = m_kart_info[i].m_lives;
+            }
+        }
+        // leader could be 0 if the last two karts hit each other in
+        // the same frame
+        if(leader)
+        {
+            for(unsigned int i=0; i<Camera::getNumCameras(); i++)
+            {
+                Camera *camera = Camera::getCamera(i);
+                if(camera->getKart()->getWorldKartId()==kart_id)
+                {
+                    camera->setMode(Camera::CM_NORMAL);
+                    camera->setKart(leader);
+                }
+            }   // for in < number of cameras
+        }   // if leader
         m_insert_tire = 4;
     }
-    
+
     const unsigned int NUM_KARTS = getNumKarts();
     int num_karts_many_lives = 0;
- 
+
     for (unsigned int n = 0; n < NUM_KARTS; ++n)
     {
         if (m_kart_info[n].m_lives > 1) num_karts_many_lives++;
     }
-    
+
     // when almost over, use fast music
     if (num_karts_many_lives<=1 && !m_faster_music_active)
     {
         music_manager->switchToFastMusic();
         m_faster_music_active = true;
     }
-    
+
     scene::ISceneNode* kart_node = m_karts[kart_id]->getNode();
-    
-    // FIXME: sorry for this ugly const_cast, irrlicht doesn't seem to allow 
+
+    // FIXME: sorry for this ugly const_cast, irrlicht doesn't seem to allow
     // getting a writable list of children, wtf??
-    core::list<scene::ISceneNode*>& children = 
+    core::list<scene::ISceneNode*>& children =
         const_cast<core::list<scene::ISceneNode*>&>(kart_node->getChildren());
-    for (core::list<scene::ISceneNode*>::Iterator it = children.begin(); 
+    for (core::list<scene::ISceneNode*>::Iterator it = children.begin();
                                                   it != children.end(); it++)
     {
         scene::ISceneNode* curr = *it;
@@ -192,11 +250,11 @@ void ThreeStrikesBattle::kartHit(const int kart_id)
             curr->setVisible(m_kart_info[kart_id].m_lives >= 2);
         }
     }
-    
+
     // schedule a tire to be thrown away (but can't do it in this callback
     // because the caller is currently iterating the list of track objects)
     m_insert_tire++;
-    core::vector3df wheel_pos(m_karts[kart_id]->getKartWidth()*0.5f, 
+    core::vector3df wheel_pos(m_karts[kart_id]->getKartWidth()*0.5f,
                               0.0f, 0.0f);
     m_tire_position = kart_node->getPosition() + wheel_pos;
     m_tire_rotation = 0;
@@ -214,7 +272,7 @@ void ThreeStrikesBattle::kartHit(const int kart_id)
         m_tire_radius[i] = m_karts[kart_id]->getKartModel()
                                            ->getWheelGraphicsRadius(i);
     }
- 
+
     m_tire_dir = m_karts[kart_id]->getKartProperties()->getKartDir();
     if(m_insert_tire == 5 && m_karts[kart_id]->isWheeless())
         m_insert_tire = 0;
@@ -231,34 +289,34 @@ const std::string& ThreeStrikesBattle::getIdent() const
 
 //-----------------------------------------------------------------------------
 /** Update the world and the track.
- *  \param dt Time step size. 
+ *  \param dt Time step size.
  */
 void ThreeStrikesBattle::update(float dt)
 {
     WorldWithRank::update(dt);
     WorldWithRank::updateTrack(dt);
-    
+
     core::vector3df tire_offset;
     std::string tire;
     float scale = 0.5f;
     float radius = 0.5f;
-    PhysicalObject::bodyTypes tire_model;
+    PhysicalObject::bodyTypes body_shape;
 
     // insert blown away tire(s) now if was requested
     while (m_insert_tire > 0)
-    {        
+    {
         if(m_insert_tire == 1)
         {
             tire_offset = core::vector3df(0.0f, 0.0f, 0.0f);
             tire = file_manager->getModelFile("tire.b3d");
             scale = 0.5f;
             radius = 0.5f;
-            tire_model = PhysicalObject::MP_CYLINDER_Y;
+            body_shape = PhysicalObject::MP_CYLINDER_Y;
         }
         else
         {
             scale = 1.0f;
-            tire_model = PhysicalObject::MP_CYLINDER_X;
+            body_shape = PhysicalObject::MP_CYLINDER_X;
             radius = m_tire_radius[m_insert_tire-2];
             tire_offset = m_tire_offsets[m_insert_tire-2];
             if     (m_insert_tire == 2)
@@ -271,25 +329,38 @@ void ThreeStrikesBattle::update(float dt)
                 tire = m_tire_dir+"/wheel-rear-right.b3d";
         }
 
-        TrackObjectManager* tom = m_track->getTrackObjectManager();        
-        PhysicalObject* obj = 
-            tom->insertObject(tire,
-                              tire_model,
-                              15 /* mass */,
-                              radius /* radius */,
-                              core::vector3df(800.0f,0,m_tire_rotation 
-                                                      / M_PI * 180 + 180) ,
-                              m_tire_position + tire_offset,
-                              core::vector3df(scale,scale,scale) /* scale */);
-        
+
+        core::vector3df tire_xyz = m_tire_position + tire_offset;
+        core::vector3df tire_hpr = core::vector3df(800.0f,0,
+                                                   m_tire_rotation / M_PI * 180 + 180);
+        core::vector3df tire_scale(scale,scale,scale);
+
+        PhysicalObject::Settings physicsSettings;
+        physicsSettings.body_type = PhysicalObject::MP_CYLINDER_Y;
+        physicsSettings.crash_reset = false;
+        physicsSettings.knock_kart = false;
+        physicsSettings.mass = 15.0f;
+        physicsSettings.radius = radius;
+        physicsSettings.reset_when_too_low = false;
+        physicsSettings.flatten_kart = false;
+
+        TrackObjectPresentationMesh* tire_presentation =
+            new TrackObjectPresentationMesh(tire, tire_xyz, tire_hpr, tire_scale);
+
+        TrackObject* tire = new TrackObject(tire_xyz, tire_hpr, tire_scale,
+                                            "movable", tire_presentation,
+                                            true /* is_dynamic */,
+                                            &physicsSettings);
+        getTrack()->getTrackObjectManager()->insertObject(tire);
+
         // FIXME: orient the force relative to kart orientation
-        obj->getBody()->applyCentralForce(btVector3(60.0f, 0.0f, 0.0f));
+        tire->getPhysics()->getBody()->applyCentralForce(btVector3(60.0f, 0.0f, 0.0f));
 
         m_insert_tire--;
         if(m_insert_tire == 1)
             m_insert_tire = 0;
-        
-        m_tires.push_back(obj);
+
+        m_tires.push_back(tire);
     }
 }   // update
 
@@ -301,30 +372,30 @@ void ThreeStrikesBattle::updateKartRanks()
     beginSetKartPositions();
     // sort karts by their times then give each one its position.
     // in battle-mode, long time = good (meaning he survived longer)
-    
+
     const unsigned int NUM_KARTS = getNumKarts();
-    
+
     int *karts_list = new int[NUM_KARTS];
     for( unsigned int n = 0; n < NUM_KARTS; ++n ) karts_list[n] = n;
-    
+
     bool sorted=false;
     do
     {
         sorted = true;
         for( unsigned int n = 0; n < NUM_KARTS-1; ++n )
         {
-            const int this_karts_time = 
-                  m_karts[karts_list[n]]->hasFinishedRace() 
+            const int this_karts_time =
+                  m_karts[karts_list[n]]->hasFinishedRace()
                 ? (int)m_karts[karts_list[n]]->getFinishTime()
                 : (int)WorldStatus::getTime();
-            const int next_karts_time = 
+            const int next_karts_time =
                    m_karts[karts_list[n+1]]->hasFinishedRace()
                 ? (int)m_karts[karts_list[n+1]]->getFinishTime()
                 : (int)WorldStatus::getTime();
-            
+
             // Swap if next kart survived longer or has more lives
             bool swap = next_karts_time > this_karts_time ||
-                        m_kart_info[karts_list[n+1]].m_lives 
+                        m_kart_info[karts_list[n+1]].m_lives
                         > m_kart_info[karts_list[n]].m_lives;
 
             if(swap)
@@ -334,10 +405,10 @@ void ThreeStrikesBattle::updateKartRanks()
                 karts_list[n] = tmp;
                 sorted = false;
                 break;
-            } 
+            }
         }   // for n = 0; n < NUM_KARTS-1
     } while(!sorted);
-    
+
     for( unsigned int n = 0; n < NUM_KARTS; ++n )
     {
         setKartPosition(karts_list[n], n+1);
@@ -356,7 +427,7 @@ bool ThreeStrikesBattle::isRaceOver()
     {
         return false;
     }
-    
+
     return getCurrentNumKarts()==1 || getCurrentNumPlayers()==0;
 }   // isRaceOver
 
@@ -371,183 +442,90 @@ void ThreeStrikesBattle::terminateRace()
     WorldWithRank::terminateRace();
 }   // terminateRace
 
-//-----------------------------------------------------------------------------
-/** Called then a battle is restarted.
- */
-void ThreeStrikesBattle::restartRace()
-{
-    WorldWithRank::restartRace();
-    
-    const unsigned int kart_amount = m_karts.size();
-    
-    for(unsigned int n=0; n<kart_amount; n++)
-    {
-        m_kart_info[n].m_lives         = 3;
-        
-        // no positions in this mode
-        m_karts[n]->setPosition(-1);
-        
-        scene::ISceneNode* kart_node = m_karts[n]->getNode();
-        
-        // FIXME: sorry for this ugly const_cast, irrlicht doesn't seem to allow getting a writable list of children, wtf??
-        core::list<scene::ISceneNode*>& children = const_cast<core::list<scene::ISceneNode*>&>(kart_node->getChildren());
-        for (core::list<scene::ISceneNode*>::Iterator it = children.begin(); it != children.end(); it++)
-        {
-            scene::ISceneNode* curr = *it;
-            
-            if (core::stringc(curr->getName()) == "tire1")
-            {
-                curr->setVisible(true);
-            }
-            else if (core::stringc(curr->getName()) == "tire2")
-            {
-                curr->setVisible(true);
-            }
-        }
-        
-    }// next kart
-    
-    // remove old battle events
-    m_battle_events.clear();
-
-    // add initial battle event
-    BattleEvent evt;
-    evt.m_time = 0.0f;
-    evt.m_kart_info = m_kart_info;
-    m_battle_events.push_back(evt);
-
-    PhysicalObject *obj;
-    for_in(obj, m_tires)
-    {
-        m_track->getTrackObjectManager()->removeObject(obj);
-    }
-    m_tires.clearWithoutDeleting();
-}   // restartRace
-
 //-----------------------------------------------------------------------------
 /** Returns the data to display in the race gui.
  */
-RaceGUIBase::KartIconDisplayInfo* ThreeStrikesBattle::getKartsDisplayInfo()
+void ThreeStrikesBattle::getKartsDisplayInfo(
+                           std::vector<RaceGUIBase::KartIconDisplayInfo> *info)
 {
     const unsigned int kart_amount = getNumKarts();
     for(unsigned int i = 0; i < kart_amount ; i++)
     {
-        RaceGUIBase::KartIconDisplayInfo& rank_info = m_kart_display_info[i];
-        
+        RaceGUIBase::KartIconDisplayInfo& rank_info = (*info)[i];
+
         // reset color
         rank_info.lap = -1;
-        
+
         switch(m_kart_info[i].m_lives)
         {
             case 3:
-                rank_info.r = 0.0;
-                rank_info.g = 1.0;
-                rank_info.b = 0.0;
+                rank_info.m_color = video::SColor(255, 0, 255, 0);
                 break;
             case 2:
-                rank_info.r = 1.0;
-                rank_info.g = 0.9f;
-                rank_info.b = 0.0;
+                rank_info.m_color = video::SColor(255, 255, 229, 0);
                 break;
             case 1:
-                rank_info.r = 1.0;
-                rank_info.g = 0.0;
-                rank_info.b = 0.0;
+                rank_info.m_color = video::SColor(255, 255, 0, 0);
                 break;
             case 0:
-                rank_info.r = 0.5;
-                rank_info.g = 0.5;
-                rank_info.b = 0.5;
+                rank_info.m_color = video::SColor(128, 128, 128, 0);
                 break;
         }
-        
+
         char lives[4];
         sprintf(lives, "%i", m_kart_info[i].m_lives);
-        
+
         rank_info.m_text = lives;
     }
-    
-    return m_kart_display_info;
-}   // getKartDisplayInfo
+}   // getKartsDisplayInfo
 
 //-----------------------------------------------------------------------------
-/** Moves a kart to its rescue position.
- *  \param kart The kart that was rescued.
+/** Determines the rescue position for a kart. The rescue position is the
+ *  start position which is has the biggest accumulated distance to all other
+ *  karts, and which has no other kart very close. The latter avoids dropping
+ *  a kart on top of another kart.
+ *  \param kart The kart that is going to be rescued.
+ *  \returns The index of the start position to which the rescued kart
+ *           should be moved to.
  */
-void ThreeStrikesBattle::moveKartAfterRescue(AbstractKart* kart)
+
+unsigned int ThreeStrikesBattle::getRescuePositionIndex(AbstractKart *kart)
 {
-    // find closest point to drop kart on
-    World *world = World::getWorld();
-    const int start_spots_amount = world->getTrack()->getNumberOfStartPositions();
+    const int start_spots_amount = getTrack()->getNumberOfStartPositions();
     assert(start_spots_amount > 0);
-    
+
     float largest_accumulated_distance_found = -1;
-    int furthest_id_found = -1;
-    
-    const float kart_x = kart->getXYZ().getX();
-    const float kart_z = kart->getXYZ().getZ();
+    int   furthest_id_found                  = -1;
 
     for(int n=0; n<start_spots_amount; n++)
     {
-        // no need for the overhead to compute exact distance with sqrt(), 
-        // so using the 'manhattan' heuristic which will do fine enough.
-        const btTransform &s = world->getTrack()->getStartTransform(n);
+        const btTransform &s = getTrack()->getStartTransform(n);
         const Vec3 &v=s.getOrigin();
-        float accumulatedDistance = .0f;
-        bool spawnPointClear = true;
-                
-        for(unsigned int k=0; k<getCurrentNumKarts(); k++) 
-        {
-            const AbstractKart *currentKart = World::getWorld()->getKart(k);
-            const float currentKart_x = currentKart->getXYZ().getX();
-            const float currentKartk_z = currentKart->getXYZ().getZ();
+        float accumulated_distance = .0f;
+        bool spawn_point_clear = true;
 
-            if(kart_x!=currentKart_x && kart_z !=currentKartk_z)
+        for(unsigned int k=0; k<getCurrentNumKarts(); k++)
+        {
+            if(kart->getWorldKartId()==k) continue;
+            float abs_distance2 = (getKart(k)->getXYZ()-v).length2_2d();
+            const float CLEAR_SPAWN_RANGE2 = 5*5;
+            if( abs_distance2 < CLEAR_SPAWN_RANGE2)
             {
-                float absDistance = fabs(currentKart_x - v.getX()) +
-                    fabs(currentKartk_z - v.getZ());
-                if(absDistance < CLEAR_SPAWN_RANGE)
-                {
-                    spawnPointClear = false;
-                    break;
-                }
-                accumulatedDistance += absDistance;
+                spawn_point_clear = false;
+                break;
             }
+            accumulated_distance += sqrt(abs_distance2);
         }
 
-        if(largest_accumulated_distance_found < accumulatedDistance && spawnPointClear)
+        if(accumulated_distance > largest_accumulated_distance_found && 
+            spawn_point_clear)
         {
             furthest_id_found = n;
-            largest_accumulated_distance_found = accumulatedDistance;
+            largest_accumulated_distance_found = accumulated_distance;
         }
     }
-    
+
     assert(furthest_id_found != -1);
-    const btTransform &s = world->getTrack()->getStartTransform(furthest_id_found);
-    const Vec3 &xyz = s.getOrigin();
-    kart->setXYZ(xyz);
-    kart->setRotation(s.getRotation());
+    return furthest_id_found;
+}   // getRescuePositionIndex
 
-    //position kart from same height as in World::resetAllKarts
-    btTransform pos;
-    pos.setOrigin(kart->getXYZ()+btVector3(0, 0.5f*kart->getKartHeight(), 0.0f));
-    pos.setRotation( btQuaternion(btVector3(0.0f, 1.0f, 0.0f), 0 /* angle */) );
-
-    kart->getBody()->setCenterOfMassTransform(pos);
-
-    //project kart to surface of track
-    bool kart_over_ground = m_physics->projectKartDownwards(kart);
-
-    if (kart_over_ground)
-    {
-        //add vertical offset so that the kart starts off above the track
-        float vertical_offset = kart->getKartProperties()->getVertRescueOffset() *
-                                kart->getKartHeight();
-        kart->getBody()->translate(btVector3(0, vertical_offset, 0));
-    }
-    else
-    {
-        fprintf(stderr, "WARNING: invalid position after rescue for kart %s on track %s.\n",
-                (kart->getIdent().c_str()), m_track->getIdent().c_str());
-    }
-}   // moveKartAfterRescue
