@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -30,14 +30,17 @@
 #include "challenges/unlock_manager.hpp"
 #include "config/user_config.hpp"
 #include "graphics/camera.hpp"
+#include "graphics/irr_driver.hpp"
 #include "graphics/hardware_skinning.hpp"
 #include "io/file_manager.hpp"
+#include "input/device_manager.hpp"
 #include "items/projectile_manager.hpp"
 #include "karts/controller/player_controller.hpp"
 #include "karts/controller/end_controller.hpp"
 #include "karts/controller/skidding_ai.hpp"
 #include "karts/kart.hpp"
 #include "karts/kart_properties_manager.hpp"
+#include "modes/overworld.hpp"
 #include "modes/profile_world.hpp"
 #include "network/network_manager.hpp"
 #include "network/race_state.hpp"
@@ -51,10 +54,10 @@
 #include "replay/replay_recorder.hpp"
 #include "states_screens/dialogs/race_paused_dialog.hpp"
 #include "states_screens/race_gui_base.hpp"
+#include "states_screens/main_menu_screen.hpp"
 #include "states_screens/race_gui.hpp"
-#include "states_screens/state_manager.hpp"
-#include "states_screens/minimal_race_gui.hpp"
 #include "states_screens/race_result_gui.hpp"
+#include "states_screens/state_manager.hpp"
 #include "tracks/track.hpp"
 #include "tracks/track_manager.hpp"
 #include "utils/constants.hpp"
@@ -67,7 +70,7 @@ World* World::m_world = NULL;
 /** The main world class is used to handle the track and the karts.
  *  The end of the race is detected in two phases: first the (abstract)
  *  function isRaceOver, which must be implemented by all game modes,
- *  must return true. In which case enterRaceOverState is called. At 
+ *  must return true. In which case enterRaceOverState is called. At
  *  this time a winning (or losing) animation can be played. The WorldStatus
  *  class will in its enterRaceOverState switch to DELAY_FINISH_PHASE,
  *  but the remaining AI kart will keep on racing during that time.
@@ -76,35 +79,57 @@ World* World::m_world = NULL;
  *  of all karts is set (i.e. in a normal race the arrival time for karts
  *  will be estimated), highscore is updated, and the race result gui
  *  is being displayed.
+ *  Rescuing is handled via the three functions: 
+ *  getNumberOfRescuePositions() - which returns the number of rescue 
+ *           positions defined.
+ *  getRescuePositionIndex(AbstractKart *kart) - which determines the
+ *           index of the rescue position to be used for the given kart.
+ *  getRescueTransform(unsigned int index) - which returns the transform
+ *           (i.e. position and rotation) for the specified rescue
+ *           position.
+ *  This allows the world class to do some tests to make sure all rescue
+ *  positions are valid (when started with --track-debug). It tries to
+ *  place all karts on all rescue positions. If there are any problems
+ *  (e.g. a rescue position not over terrain (perhaps because it is too
+ *  low); or the rescue position is on a texture which will immediately
+ *  trigger another rescue), a warning message will be printed.
  */
+
 //-----------------------------------------------------------------------------
 /** Constructor. Note that in the constructor it is not possible to call any
  *  functions that use World::getWorld(), since this is only defined
- *  after the constructor. Those functions must be called in the init() 
+ *  after the constructor. Those functions must be called in the init()
  *  function, which is called immediately after the constructor.
  */
 World::World() : WorldStatus(), m_clear_color(255,100,101,140)
 {
+
+#ifdef DEBUG
+    m_magic_number = 0xB01D6543;
+#endif
+
     m_physics            = NULL;
     m_race_gui           = NULL;
     m_saved_race_gui     = NULL;
     m_use_highscores     = true;
     m_track              = NULL;
-    m_clear_back_buffer  = false;
     m_schedule_pause     = false;
     m_schedule_unpause   = false;
+    m_schedule_exit_race = false;
     m_self_destruct      = false;
-    
+    m_schedule_tutorial  = false;
+
     m_stop_music_when_dialog_open = true;
-    
+
     WorldStatus::setClockMode(CLOCK_CHRONO);
+
 }   // World
 
 // ----------------------------------------------------------------------------
-/** This function is called after instanciating. This can't be moved to the 
- *  contructor as child classes must be instanciated, otherwise polymorphism 
- *  will fail and the results will be incorrect . Also in init() functions
- *  can be called that use World::getWorld(). 
+/** This function is called after instanciating. The code here can't be moved
+ *  to the contructor as child classes must be instanciated, otherwise
+ *  polymorphism will fail and the results will be incorrect . Also in init()
+ *  functions can be called that use World::getWorld().
  */
 void World::init()
 {
@@ -114,10 +139,13 @@ void World::init()
     m_eliminated_karts    = 0;
     m_eliminated_players  = 0;
     m_num_players         = 0;
-    
+
     // Create the race gui before anything else is attached to the scene node
     // (which happens when the track is loaded). This allows the race gui to
-    // do any rendering on texture.
+    // do any rendering on texture. Note that this function can NOT be called
+    // in the World constuctor, since it might be overwritten by a the game
+    // mode class, which would not have been constructed at the time that this
+    // constructor is called, so the wrong race gui would be created.
     createRaceGUI();
 
     // Grab the track file
@@ -125,7 +153,7 @@ void World::init()
     if(!m_track)
     {
         std::ostringstream msg;
-        msg << "Track '" << race_manager->getTrackName() 
+        msg << "Track '" << race_manager->getTrackName()
             << "' not found.\n";
         throw std::runtime_error(msg.str());
     }
@@ -138,54 +166,106 @@ void World::init()
 
     // Load the track models - this must be done before the karts so that the
     // karts can be positioned properly on (and not in) the tracks.
-    m_track->loadTrackModel(this, race_manager->getReverseTrack());
+    m_track->loadTrackModel(race_manager->getReverseTrack());
 
     for(unsigned int i=0; i<num_karts; i++)
     {
-        std::string kart_ident = history->replayHistory() 
+        std::string kart_ident = history->replayHistory()
                                ? history->getKartIdent(i)
                                : race_manager->getKartIdent(i);
         int local_player_id  = race_manager->getKartLocalPlayerId(i);
         int global_player_id = race_manager->getKartGlobalPlayerId(i);
-        AbstractKart* newkart = createKart(kart_ident, i, local_player_id,  
-                                   global_player_id, 
+        AbstractKart* newkart = createKart(kart_ident, i, local_player_id,
+                                   global_player_id,
                                    race_manager->getKartType(i));
         m_karts.push_back(newkart);
         m_track->adjustForFog(newkart->getNode());
-        
+
     }  // for i
-    
+
+    // Must be called after all karts are created
+    m_race_gui->init();
+
     if(ReplayPlay::get())
         ReplayPlay::get()->Load();
 
+    network_manager->worldLoaded();
+
+    powerup_manager->updateWeightsForRace(num_karts);
+}   // init
+
+//-----------------------------------------------------------------------------
+/** This function is called before a race is started (i.e. either after
+ *  calling init() when starting a race for the first time, or after
+ *  restarting a race, in which case no init() is called.
+ */
+void World::reset()
+{
+    // If m_saved_race_gui is set, it means that the restart was done
+    // when the race result gui was being shown. In this case restore the
+    // race gui (note that the race result gui is cached and so never really
+    // destroyed).
+    if(m_saved_race_gui)
+    {
+        m_race_gui       = m_saved_race_gui;
+        m_saved_race_gui = NULL;
+    }
+
+    m_schedule_pause = false;
+    m_schedule_unpause = false;
+
+    WorldStatus::reset();
+    m_faster_music_active = false;
+    m_eliminated_karts    = 0;
+    m_eliminated_players  = 0;
+
+    for ( KartList::iterator i = m_karts.begin(); i != m_karts.end() ; ++i )
+    {
+        (*i)->reset();
+    }
+
+    for(unsigned int i=0; i<Camera::getNumCameras(); i++)
+    {
+        Camera::getCamera(i)->reset();
+    }
+
+    if(ReplayPlay::get())
+        ReplayPlay::get()->reset();
+
     resetAllKarts();
     // Note: track reset must be called after all karts exist, since check
     // objects need to allocate data structures depending on the number
     // of karts.
     m_track->reset();
 
+    // Reset the race gui.
+    m_race_gui->reset();
+
+    // Start music from beginning
+    music_manager->stopMusic();
+
+    // Enable SFX again
+    sfx_manager->resumeAll();
+
+    projectile_manager->cleanup();
+    race_manager->reset();
+    // Make sure to overwrite the data from the previous race.
     if(!history->replayHistory()) history->initRecording();
     if(ReplayRecorder::get()) ReplayRecorder::get()->init();
-    network_manager->worldLoaded();
-    
-    powerup_manager->updateWeightsForRace(num_karts);
-    // erase messages left over
-    RaceGUIBase* rg = getRaceGUI();
-    if (rg)
-    {
-        rg->init();
-        rg->clearAllMessages();
-    }
-}   // init
+
+    // Reset all data structures that depend on number of karts.
+    irr_driver->reset();
+
+    //Reset the Rubber Ball Collect Time to some negative value.
+    powerup_manager->setBallCollectTime(-100);
+
+}   // reset
 
 //-----------------------------------------------------------------------------
 
 void World::createRaceGUI()
 {
-    if(UserConfigParams::m_minimal_race_gui)
-		m_race_gui = new MinimalRaceGUI();
-	else
-		m_race_gui = new RaceGUI();
+    m_race_gui = new RaceGUI();
 }
 
 //-----------------------------------------------------------------------------
@@ -205,18 +285,18 @@ AbstractKart *World::createKart(const std::string &kart_ident, int index,
     int position           = index+1;
     btTransform init_pos   = m_track->getStartTransform(index);
     AbstractKart *new_kart = new Kart(kart_ident, index, position, init_pos);
-    new_kart->init(race_manager->getKartType(index), (local_player_id == 0));
+    new_kart->init(race_manager->getKartType(index));
     Controller *controller = NULL;
     switch(kart_type)
     {
     case RaceManager::KT_PLAYER:
-        controller = new PlayerController(new_kart, 
+        controller = new PlayerController(new_kart,
                          StateManager::get()->getActivePlayer(local_player_id),
                                           local_player_id);
         m_num_players ++;
         break;
     case RaceManager::KT_NETWORK_PLAYER:
-		break;  // Avoid compiler warning about enum not handled.
+        break;  // Avoid compiler warning about enum not handled.
         //controller = new NetworkController(kart_ident, position, init_pos,
         //                          global_player_id);
         //m_num_players++;
@@ -229,9 +309,9 @@ AbstractKart *World::createKart(const std::string &kart_ident, int index,
     case RaceManager::KT_LEADER:
         break;
     }
-    
+
     new_kart->setController(controller);
-    
+
     return new_kart;
 }   // createKart
 
@@ -251,7 +331,7 @@ Controller* World::loadAIController(AbstractKart *kart)
             controller = new SkiddingAI(kart);
             break;
         default:
-            fprintf(stderr, "Warning: Unknown robot, using default.\n");
+            Log::warn("World", "Unknown AI, using default.");
             controller = new SkiddingAI(kart);
             break;
     }
@@ -297,6 +377,8 @@ World::~World()
         delete m_karts[i];
 
     m_karts.clear();
+    Camera::removeAllCameras();
+
     projectile_manager->cleanup();
     // In case that the track is not found, m_physics is still undefined.
     if(m_physics)
@@ -304,6 +386,11 @@ World::~World()
 
     music_manager->stopMusic();
     m_world = NULL;
+
+#ifdef DEBUG
+    m_magic_number = 0xDEADBEEF;
+#endif
+
 }   // ~World
 
 //-----------------------------------------------------------------------------
@@ -312,10 +399,10 @@ World::~World()
  */
 void World::onGo()
 {
-    // Reset the brakes now that the prestart 
-    // phase is over (braking prevents the karts 
+    // Reset the brakes now that the prestart
+    // phase is over (braking prevents the karts
     // from sliding downhill)
-    for(unsigned int i=0; i<m_karts.size(); i++) 
+    for(unsigned int i=0; i<m_karts.size(); i++)
     {
         m_karts[i]->getVehicle()->setAllBrakes(0);
     }
@@ -331,7 +418,7 @@ void World::terminateRace()
 {
     m_schedule_pause = false;
     m_schedule_unpause = false;
-    
+
     // Update the estimated finishing time for all karts that haven't
     // finished yet.
     const unsigned int kart_amount = getNumKarts();
@@ -343,30 +430,30 @@ void World::terminateRace()
 
         }
     }   // i<kart_amount
-    
-    // Update highscores, and retrieve the best highscore if relevant 
+
+    // Update highscores, and retrieve the best highscore if relevant
     // to show it in the GUI
     int best_highscore_rank = -1;
     int best_finish_time = -1;
     std::string highscore_who = "";
     StateManager::ActivePlayer* best_player = NULL;
-    updateHighscores(&best_highscore_rank, &best_finish_time, &highscore_who, 
+    updateHighscores(&best_highscore_rank, &best_finish_time, &highscore_who,
                      &best_player);
-    
+
     unlock_manager->getCurrentSlot()->raceFinished();
-    
+
     if (m_race_gui) m_race_gui->clearAllMessages();
-    // we can't delete the race gui here, since it is needed in case of 
-    // a restart: the constructor of it creates some textures which assume 
-    // that no scene nodes exist. In case of a restart there are scene nodes, 
-    // so we can't create the race gui again, so we keep it around 
+    // we can't delete the race gui here, since it is needed in case of
+    // a restart: the constructor of it creates some textures which assume
+    // that no scene nodes exist. In case of a restart there are scene nodes,
+    // so we can't create the race gui again, so we keep it around
     // and save the pointer.
     assert(m_saved_race_gui==NULL);
     m_saved_race_gui = m_race_gui;
-    
+
     RaceResultGUI* results = RaceResultGUI::getInstance();
     m_race_gui       = results;
-    
+
     if (best_highscore_rank > 0)
     {
         results->setHighscore(highscore_who, best_player, best_highscore_rank,
@@ -376,7 +463,7 @@ void World::terminateRace()
     {
         results->clearHighscores();
     }
-    
+
     StateManager::get()->pushScreen(results);
     WorldStatus::terminateRace();
 }   // terminateRace
@@ -395,86 +482,53 @@ void World::resetAllKarts()
 
     // If track checking is requested, check all rescue positions if
     // they are heigh enough.
-    if(race_manager->getMinorMode()!=RaceManager::MINOR_MODE_3_STRIKES &&
-        UserConfigParams::m_track_debug)
+    if(UserConfigParams::m_track_debug)
     {
-        Vec3 eps = Vec3(0,1.5f*m_karts[0]->getKartHeight(),0);
-        for(unsigned int quad=0; quad<QuadGraph::get()->getNumNodes(); quad++)
-        {
-            const Quad &q   = QuadGraph::get()->getQuadOfNode(quad);
-            const Vec3 center = q.getCenter();
-            // We have to test for all karts, since the karts have different
-            // heights and so things might change from kart to kart.
-            for(unsigned int kart_id=0; kart_id<m_karts.size(); kart_id++)
-            {
-                AbstractKart *kart = m_karts[kart_id];
-                kart->setXYZ(center);
-    
-                btQuaternion heading(btVector3(0.0f, 1.0f, 0.0f),
-                                     m_track->getAngle(quad) );
-                kart->setRotation(heading);
-
-                btTransform pos;
-                pos.setOrigin(center+eps);
-                pos.setRotation(btQuaternion(btVector3(0.0f, 1.0f, 0.0f),
-                                m_track->getAngle(quad))                 );
-                kart->getBody()->setCenterOfMassTransform(pos);
-                bool kart_over_ground = m_physics->projectKartDownwards(kart);
-                if(kart_over_ground)
-                {
-                    const Vec3 &xyz = kart->getTrans().getOrigin()
-                                    + Vec3(0,0.3f,0);
-                    if(dynamic_cast<Kart*>(kart))
-                        dynamic_cast<Kart*>(kart)->getTerrainInfo()
-                                                 ->update(xyz);
-                    const Material *material = kart->getMaterial();
-                    if(!material || material->isDriveReset())
-                        kart_over_ground = false;
-                }
-                if(!kart_over_ground)
-                {
-                    printf("Kart '%s' not over quad '%d'\n",
-                        kart->getIdent().c_str(), quad);
-                    printf("Center point: %f %f %f\n",
-                        center.getX(), center.getY(), center.getZ());
-
-                }
-            }   // for kart_id<m_karts.size()
-        }   // for quad < quad_graph.getNumNodes
-
+        // Loop over all karts, in case that some karts are dfferent
         for(unsigned int kart_id=0; kart_id<m_karts.size(); kart_id++)
         {
+            for(unsigned int rescue_pos=0; 
+                rescue_pos<getNumberOfRescuePositions();
+                rescue_pos++)
+            {
+                btTransform t = getRescueTransform(rescue_pos);
+                // This will print out warnings if there is no terrain under
+                // the kart, or the kart is being dropped on a reset texture
+                moveKartTo(m_karts[kart_id], t);
+
+            }   // rescue_pos<getNumberOfRescuePositions
+
             // Reset the karts back to the original start position.
             // This call is a bit of an overkill, but setting the correct
             // transforms, positions, motion state is a bit of a hassle.
             m_karts[kart_id]->reset();
-        }
+        }   // for kart_id<m_karts.size()
+
 
     }   // if m_track_debug
 
-
     m_schedule_pause = false;
     m_schedule_unpause = false;
-    
+
     //Project karts onto track from above. This will lower each kart so
     //that at least one of its wheel will be on the surface of the track
     for ( KartList::iterator i=m_karts.begin(); i!=m_karts.end(); i++)
     {
-        ///start projection from top of kart
-        btVector3 up_offset(0, 0.5f * ((*i)->getKartHeight()), 0);
-        (*i)->getVehicle()->getRigidBody()->translate (up_offset);
+        Vec3 xyz = (*i)->getXYZ();
+        //start projection from top of kart
+        Vec3 up_offset(0, 0.5f * ((*i)->getKartHeight()), 0);
+        (*i)->setXYZ(xyz+up_offset);
 
-        bool kart_over_ground = m_physics->projectKartDownwards(*i);
+        bool kart_over_ground = m_track->findGround(*i);
 
         if (!kart_over_ground)
         {
-            fprintf(stderr, 
-                    "ERROR: no valid starting position for kart %d "
-                    "on track %s.\n",
-                    (int)(i-m_karts.begin()), m_track->getIdent().c_str());
+            Log::error("World",
+                       "No valid starting position for kart %d on track %s.",
+                        (int)(i-m_karts.begin()), m_track->getIdent().c_str());
             if (UserConfigParams::m_artist_debug_mode)
             {
-                fprintf(stderr, "Activating fly mode.\n");
+                Log::warn("World", "Activating fly mode.");
                 (*i)->flyUp();
                 continue;
             }
@@ -515,19 +569,19 @@ void World::resetAllKarts()
                 // This test can not be done only once before the loop, since
                 // it can happen that the kart falls through the track later!
                 Vec3 to = t.getOrigin()+Vec3(0, -10000, 0);
-                m_track->getTriangleMesh().castRay(t.getOrigin(), to, 
+                m_track->getTriangleMesh().castRay(t.getOrigin(), to,
                                                    &hit_point, &material,
                                                    &normal);
                 if(!material)
                 {
-                    fprintf(stderr, 
-                            "ERROR: no valid starting position for "
-                            "kart %d on track %s.\n",
-                            (int)(i-m_karts.begin()), 
-                            m_track->getIdent().c_str());
+                    Log::error("World",
+                               "No valid starting position for kart %d "
+                               "on track %s.",
+                               (int)(i-m_karts.begin()),
+                               m_track->getIdent().c_str());
                     if (UserConfigParams::m_artist_debug_mode)
                     {
-                        fprintf(stderr, "Activating fly mode.\n");
+                        Log::warn("World", "Activating fly mode.");
                         (*i)->flyUp();
                         continue;
                     }
@@ -544,19 +598,56 @@ void World::resetAllKarts()
 
     for ( KartList::iterator i=m_karts.begin(); i!=m_karts.end(); i++)
     {
-        // Now store the current (i.e. in rest) suspension length for each
-        // kart, so that the karts can visualise the suspension. 
-        (*i)->setSuspensionLength();
-        // Initialise the camera (if available), now that the correct
-        // kart position is set
-        if((*i)->getCamera())
-            (*i)->getCamera()->setInitialTransform();
         // Update the kart transforms with the newly computed position
         // after all karts are reset
         (*i)->setTrans((*i)->getBody()->getWorldTransform());
     }
+
+    // Initialise the cameras, now that the correct kart positions are set
+    for(unsigned int i=0; i<Camera::getNumCameras(); i++)
+    {
+        Camera::getCamera(i)->setInitialTransform();
+    }
 }   // resetAllKarts
 
+// ----------------------------------------------------------------------------
+/** Places a kart that is rescued. It calls getRescuePositionIndex to find
+ *  to which rescue position the kart should be moved, then getRescueTransform
+ *  to get the position and rotation of this rescue position, and then moves
+ *  the kart.
+ *  \param kart The kart that is rescued.
+ */
+void World::moveKartAfterRescue(AbstractKart* kart)
+{
+    unsigned int index = getRescuePositionIndex(kart);
+    btTransform t      = getRescueTransform(index);
+    moveKartTo(kart, t);
+}  // moveKartAfterRescue
+
+// ----------------------------------------------------------------------------
+/** Places the kart at a given position and rotation.
+ *  \param kart The kart to be moved.
+ *  \param transform
+ */
+void World::moveKartTo(AbstractKart* kart, const btTransform &transform)
+{
+    btTransform pos(transform);
+
+    // Move the kart
+    Vec3 xyz = pos.getOrigin() + btVector3(0, 0.5f*kart->getKartHeight(),0.0f);
+
+    pos.setOrigin(xyz);
+    kart->setXYZ(xyz);
+    kart->setRotation(pos.getRotation());
+
+    kart->getBody()->setCenterOfMassTransform(pos);
+
+    // Project kart to surface of track
+    // This will set the physics transform 
+    m_track->findGround(kart);
+
+}   // moveKartTo
+
 // ----------------------------------------------------------------------------
 void World::schedulePause(Phase phase)
 {
@@ -586,7 +677,7 @@ void World::scheduleUnpause()
 
 //-----------------------------------------------------------------------------
 /** This is the main interface to update the world. This function calls
- *  update(), and checks then for the end of the race. Note that race over 
+ *  update(), and checks then for the end of the race. Note that race over
  *  handling can not necessarily be done in update(), since not all
  *  data structures might have been updated (e.g.LinearWorld must
  *  call World::update() first, to get updated kart positions. If race
@@ -596,6 +687,11 @@ void World::scheduleUnpause()
  */
 void World::updateWorld(float dt)
 {
+#ifdef DEBUG
+    assert(m_magic_number == 0xB01D6543);
+#endif
+
+
     if (m_schedule_pause)
     {
         pause(m_scheduled_pause_phase);
@@ -606,16 +702,16 @@ void World::updateWorld(float dt)
         unpause();
         m_schedule_unpause = false;
     }
-    
+
     if (m_self_destruct)
     {
         delete this;
         return;
     }
-    
+
     // Don't update world if a menu is shown or the race is over.
     if( getPhase() == FINISH_PHASE         ||
-        getPhase() == IN_GAME_MENU_PHASE      )  
+        getPhase() == IN_GAME_MENU_PHASE      )
         return;
 
     update(dt);
@@ -623,18 +719,87 @@ void World::updateWorld(float dt)
     {
         enterRaceOverState();
     }
+    else
+    {
+        if (m_schedule_exit_race)
+        {
+            race_manager->exitRace();
+            race_manager->setAIKartOverride("");
+
+            StateManager::get()->resetAndGoToScreen(MainMenuScreen::getInstance());
+
+            if (m_schedule_tutorial)
+            {
+                race_manager->setNumLocalPlayers(1);
+                race_manager->setMajorMode (RaceManager::MAJOR_MODE_SINGLE);
+                race_manager->setMinorMode (RaceManager::MINOR_MODE_TUTORIAL);
+                race_manager->setNumKarts( 1 );
+                race_manager->setTrack( "tutorial" );
+                race_manager->setDifficulty(RaceManager::DIFFICULTY_EASY);
+                race_manager->setReverseTrack(false);
+
+                // Use keyboard 0 by default (FIXME: let player choose?)
+                InputDevice* device = input_manager->getDeviceList()->getKeyboard(0);
+
+                // Create player and associate player with keyboard
+                StateManager::get()
+                    ->createActivePlayer(unlock_manager->getCurrentPlayer(),
+                                         device);
+
+                if (!kart_properties_manager->getKart(UserConfigParams::m_default_kart))
+                {
+                    Log::warn("World", 
+                              "Cannot find kart '%s', will revert to default.",
+                              UserConfigParams::m_default_kart.c_str());
+                    UserConfigParams::m_default_kart.revertToDefaults();
+                }
+                race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart);
+
+                // ASSIGN should make sure that only input from assigned devices
+                // is read.
+                input_manager->getDeviceList()->setAssignMode(ASSIGN);
+                input_manager->getDeviceList()
+                    ->setSinglePlayer( StateManager::get()->getActivePlayer(0) );
+
+                StateManager::get()->enterGameState();
+                network_manager->setupPlayerKartInfo();
+                race_manager->startNew(true);
+            }
+            else
+            {
+                if (race_manager->raceWasStartedFromOverworld())
+                {
+                    OverWorld::enterOverWorld();
+                }
+
+            }
+        }
+    }
 }   // updateWorld
 
 #define MEASURE_FPS 0
 
+//-----------------------------------------------------------------------------
+
+void World::scheduleTutorial()
+{
+    m_schedule_exit_race = true;
+    m_schedule_tutorial = true;
+}
+
 //-----------------------------------------------------------------------------
 /** Updates the physics, all karts, the track, and projectile manager.
  *  \param dt Time step size.
  */
 void World::update(float dt)
 {
+#ifdef DEBUG
+    assert(m_magic_number == 0xB01D6543);
+#endif
+
+
     PROFILER_PUSH_CPU_MARKER("World::update()", 0x00, 0x7F, 0x00);
-    
+
 #if MEASURE_FPS
     static float time = 0.0f;
     time += dt;
@@ -666,19 +831,28 @@ void World::update(float dt)
         if(!m_karts[i]->isEliminated()) m_karts[i]->update(dt) ;
     }
 
+    for(unsigned int i=0; i<Camera::getNumCameras(); i++)
+    {
+        Camera::getCamera(i)->update(dt);
+    }
+
     projectile_manager->update(dt);
-    
+
     PROFILER_POP_CPU_MARKER();
+
+#ifdef DEBUG
+    assert(m_magic_number == 0xB01D6543);
+#endif
 }   // update
 
 // ----------------------------------------------------------------------------
-/** Only updates the track. The order in which the various parts of STK are 
+/** Only updates the track. The order in which the various parts of STK are
  *  updated is quite important (i.e. the track can't be updated as part of
  *  the standard update call):
  *  the track must be updated after updating the karts (otherwise the
  *  checklines would be using the previous kart positions to determine
  *  new laps, but linear world which determines distance along track would
- *  be using the new kart positions --> the lap counting line will be 
+ *  be using the new kart positions --> the lap counting line will be
  *  triggered one frame too late, potentially causing strange behaviour of
  *  the icons.
  *  Similarly linear world must update the position of all karts after all
@@ -713,18 +887,18 @@ Highscores* World::getHighscores() const
 
 // ----------------------------------------------------------------------------
 /** Called at the end of a race. Checks if the current times are worth a new
- *  score, if so it notifies the HighscoreManager so the new score is added 
+ *  score, if so it notifies the HighscoreManager so the new score is added
  *  and saved.
  */
-void World::updateHighscores(int* best_highscore_rank, int* best_finish_time, 
+void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
                              std::string* highscore_who,
                              StateManager::ActivePlayer** best_player)
 {
     *best_highscore_rank = -1;
     *best_player = NULL;
-    
+
     if(!m_use_highscores) return;
-    
+
     // Add times to highscore list. First compute the order of karts,
     // so that the timing of the fastest kart is added first (otherwise
     // someone might get into the highscore list, only to be kicked out
@@ -752,19 +926,19 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
             // the kart location data is wrong
 
 #ifdef DEBUG
-            fprintf(stderr, "Error, incorrect kart positions:\n");
+            Log::error("World", "Incorrect kart positions:");
             for (unsigned int i=0; i<m_karts.size(); i++ )
             {
-                fprintf(stderr, "i=%d position %d\n",i, 
-                        m_karts[i]->getPosition());
+                Log::error("World", "i=%d position %d.",i,
+                           m_karts[i]->getPosition());
             }
 #endif
             continue;
         }
 
-        // Only record times for player karts and only if 
+        // Only record times for player karts and only if
         // they finished the race
-        if(!m_karts[index[pos]]->getController()->isPlayerController()) 
+        if(!m_karts[index[pos]]->getController()->isPlayerController())
             continue;
         if (!m_karts[index[pos]]->hasFinishedRace()) continue;
 
@@ -775,14 +949,14 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
         Highscores* highscores = getHighscores();
 
         PlayerController *controller = (PlayerController*)(k->getController());
-        
+
         int highscore_rank = highscores->addData(k->getIdent(),
                               controller->getPlayer()->getProfile()->getName(),
                                                  k->getFinishTime());
-        
+
         if (highscore_rank > 0)
         {
-            if (*best_highscore_rank == -1 || 
+            if (*best_highscore_rank == -1 ||
                 highscore_rank < *best_highscore_rank)
             {
                 *best_highscore_rank = highscore_rank;
@@ -790,7 +964,7 @@ void World::updateHighscores(int* best_highscore_rank, int* best_finish_time,
                 *best_player = controller->getPlayer();
                 *highscore_who = k->getIdent();
             }
-            
+
             highscore_manager->saveHighscores();
         }
     } // next position
@@ -824,53 +998,43 @@ AbstractKart *World::getPlayerKart(unsigned int n) const
  */
 AbstractKart *World::getLocalPlayerKart(unsigned int n) const
 {
-    int count=-1;
-    const int kart_count = m_karts.size();
-    for(int i=0; i<kart_count; i++)
-    {
-        if(m_karts[i]->getCamera() && 
-            (m_karts[i]->getController()->isPlayerController() ||
-            ProfileWorld::isProfileMode()                          ) )
-        {
-            count++;
-            if(count == (int)n) return m_karts[i];
-        }
-    }
-    return NULL;
+    if(n>=Camera::getNumCameras()) return NULL;
+    return Camera::getCamera(n)->getKart();
 }   // getLocalPlayerKart
 
 //-----------------------------------------------------------------------------
 /** Remove (eliminate) a kart from the race */
-void World::eliminateKart(int kart_number, bool notify_of_elimination)
+void World::eliminateKart(int kart_id, bool notify_of_elimination)
 {
-    AbstractKart *kart = m_karts[kart_number];
-    
-    // Display a message about the eliminated kart in the race gui
+    AbstractKart *kart = m_karts[kart_id];
+
+    // Display a message about the eliminated kart in the race guia
     if (notify_of_elimination)
     {
-        for (KartList::iterator i = m_karts.begin(); i != m_karts.end();  i++ )
+        for(unsigned int i=0; i<Camera::getNumCameras(); i++)
         {
-            if(!(*i)->getCamera()) continue;
-            if(*i==kart)
-            {
-                m_race_gui->addMessage(_("You have been eliminated!"), *i, 
+            Camera *camera = Camera::getCamera(i);
+            if(camera->getKart()==kart)
+                m_race_gui->addMessage(_("You have been eliminated!"), kart,
                                        2.0f);
-            }
             else
-            {
                 m_race_gui->addMessage(_("'%s' has been eliminated.",
-                                       core::stringw(kart->getName())), *i, 
+                                       core::stringw(kart->getName())),
+                                       camera->getKart(),
                                        2.0f);
-            }
-        }   // for i in kart
-    }
-    
+        }  // for i < number of cameras
+    }   // if notify_of_elimination
+
     if(kart->getController()->isPlayerController())
     {
-        // Change the camera so that it will be attached to the leader
-        // and facing backwards.
-        Camera* camera=kart->getCamera();
-        camera->setMode(Camera::CM_LEADER_MODE);
+        for(unsigned int i=0; i<Camera::getNumCameras(); i++)
+        {
+            // Change the camera so that it will be attached to the leader
+            // and facing backwards.
+            Camera *camera = Camera::getCamera(i);
+            if(camera->getKart()==kart)
+                camera->setMode(Camera::CM_LEADER_MODE);
+        }
         m_eliminated_players++;
     }
 
@@ -885,8 +1049,8 @@ void World::eliminateKart(int kart_number, bool notify_of_elimination)
 }   // eliminateKart
 
 //-----------------------------------------------------------------------------
-/** Called to determine the default collectibles to give each player at the 
- *  start for this kind of race. Both parameters are of 'out' type. 
+/** Called to determine the default collectibles to give each player at the
+ *  start for this kind of race. Both parameters are of 'out' type.
  *  \param collectible_type The type of collectible each kart.
  *  \param amount The number of collectibles.
  */
@@ -896,53 +1060,6 @@ void World::getDefaultCollectibles(int *collectible_type, int *amount )
     *amount = 0;
 }   // getDefaultCollectibles
 
-//-----------------------------------------------------------------------------
-void World::restartRace()
-{
-    // If m_saved_race_gui is set, it means that the restart was done
-    // when the race result gui was being shown. In this case restore the 
-    // race gui (note that the race result gui is cached and so never really
-    // destroyed).
-    if(m_saved_race_gui)
-    {
-        m_race_gui       = m_saved_race_gui;
-        m_saved_race_gui = NULL;
-    }
-
-    m_race_gui->restartRace();
-    m_race_gui->clearAllMessages();
-
-    m_schedule_pause = false;
-    m_schedule_unpause = false;
-    
-    WorldStatus::reset();
-    m_faster_music_active = false;
-    m_eliminated_karts    = 0;
-    m_eliminated_players  = 0;
-    
-    for ( KartList::iterator i = m_karts.begin(); i != m_karts.end() ; ++i )
-    {
-        (*i)->reset();
-    }
-    if(ReplayPlay::get())
-        ReplayPlay::get()->reset();
-    resetAllKarts();
-
-    // Start music from beginning
-    music_manager->stopMusic();
-    m_track->reset();
-
-    // Enable SFX again
-    sfx_manager->resumeAll();
-
-    projectile_manager->cleanup();
-    race_manager->reset();
-    // Make sure to overwrite the data from the previous race.
-    if(!history->replayHistory()) history->initRecording();
-    if(ReplayRecorder::get()) ReplayRecorder::get()->init();
-
-}   // restartRace
-
 //-----------------------------------------------------------------------------
 /** Pauses the music (and then pauses WorldStatus).
  */
@@ -951,7 +1068,7 @@ void World::pause(Phase phase)
     if (m_stop_music_when_dialog_open)
         music_manager->pauseMusic();
     sfx_manager->pauseAll();
-    
+
     WorldStatus::pause(phase);
 }   // pause
 
@@ -961,15 +1078,15 @@ void World::unpause()
     if (m_stop_music_when_dialog_open)
         music_manager->resumeMusic();
     sfx_manager->resumeAll();
-    
+
     WorldStatus::unpause();
-    
+
     for(unsigned int i=0; i<m_karts.size(); i++)
     {
         // Note that we can not test for isPlayerController here, since
         // an EndController will also return 'isPlayerController' if the
         // kart belonged to a player.
-        PlayerController *pc = 
+        PlayerController *pc =
             dynamic_cast<PlayerController*>(m_karts[i]->getController());
         if(pc)
             pc->resetInputState();
