@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 Joerg Henrichs
+//  Copyright (C) 2006-2013 Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
 #include "physics/physics.hpp"
 
 #include "animations/three_d_animation.hpp"
-#include "config/user_config.hpp"
+#include "graphics/irr_driver.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/rescue_animation.hpp"
 #include "network/race_state.hpp"
@@ -32,6 +32,7 @@
 #include "physics/stk_dynamics_world.hpp"
 #include "physics/triangle_mesh.hpp"
 #include "tracks/track.hpp"
+#include "modes/soccer_world.hpp"
 
 // ----------------------------------------------------------------------------
 /** Initialise physics.
@@ -84,13 +85,13 @@ Physics::~Physics()
  */
 void Physics::addKart(const AbstractKart *kart)
 {
-	const btCollisionObjectArray &all_objs = 
-		m_dynamics_world->getCollisionObjectArray();
-	for(unsigned int i=0; i<(unsigned int)all_objs.size(); i++)
-	{
-		if(btRigidBody::upcast(all_objs[i])== kart->getBody())
-			return;
-	}
+    const btCollisionObjectArray &all_objs =
+        m_dynamics_world->getCollisionObjectArray();
+    for(unsigned int i=0; i<(unsigned int)all_objs.size(); i++)
+    {
+        if(btRigidBody::upcast(all_objs[i])== kart->getBody())
+            return;
+    }
     m_dynamics_world->addRigidBody(kart->getBody());
     m_dynamics_world->addVehicle(kart->getVehicle());
     m_dynamics_world->addConstraint(kart->getUprightConstraint());
@@ -159,9 +160,9 @@ void Physics::update(float dt)
             AbstractKart *b=p->getUserPointer(1)->getPointerKart();
             race_state->addCollision(a->getWorldKartId(),
                                      b->getWorldKartId());
-            KartKartCollision(p->getUserPointer(0)->getPointerKart(), 
+            KartKartCollision(p->getUserPointer(0)->getPointerKart(),
                               p->getContactPointCS(0),
-                              p->getUserPointer(1)->getPointerKart(), 
+                              p->getUserPointer(1)->getPointerKart(),
                               p->getContactPointCS(1)                );
             continue;
         }  // if kart-kart collision
@@ -182,6 +183,18 @@ void Physics::update(float dt)
                 AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
                 ExplosionAnimation::create(kart);
             }
+            else if (obj->isFlattenKartObject())
+            {
+                AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
+                const KartProperties* kp = kart->getKartProperties();
+                kart->setSquash(kp->getSquashDuration(), kp->getSquashSlowdown());
+            }
+            else if(obj->isSoccerBall())
+            {
+                int kartId = p->getUserPointer(1)->getPointerKart()->getWorldKartId();
+                SoccerWorld* soccerWorld = (SoccerWorld*)World::getWorld();
+                soccerWorld->setLastKartTohitBall(kartId);
+            }
             continue;
         }
 
@@ -199,6 +212,12 @@ void Physics::update(float dt)
                 AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
                 ExplosionAnimation::create(kart);
             }
+            else if (anim->isFlattenKartObject())
+            {
+                AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
+                const KartProperties* kp = kart->getKartProperties();
+                kart->setSquash(kp->getSquashDuration(), kp->getSquashSlowdown());
+            }
             continue;
 
         }
@@ -216,6 +235,13 @@ void Physics::update(float dt)
             // -------------------------------
             p->getUserPointer(0)->getPointerFlyable()
                 ->hit(NULL, p->getUserPointer(1)->getPointerPhysicalObject());
+            PhysicalObject* obj = p->getUserPointer(1)->getPointerPhysicalObject();
+            if(obj->isSoccerBall())
+            {
+                int kartId = p->getUserPointer(0)->getPointerFlyable()->getOwnerId();
+                SoccerWorld* soccerWorld = (SoccerWorld*)World::getWorld();
+                soccerWorld->setLastKartTohitBall(kartId);
+            }
 
         }
         else if(p->getUserPointer(1)->is(UserPointer::UP_KART))
@@ -224,11 +250,15 @@ void Physics::update(float dt)
             // --------------------
             // Only explode a bowling ball if the target is
             // not invulnerable
+            AbstractKart* target_kart = p->getUserPointer(1)->getPointerKart();
             if(p->getUserPointer(0)->getPointerFlyable()->getType()
-                !=PowerupManager::POWERUP_BOWLING                         ||
-                !p->getUserPointer(1)->getPointerKart()->isInvulnerable()   )
+                !=PowerupManager::POWERUP_BOWLING ||
+                !target_kart->isInvulnerable()      )
+            {
                     p->getUserPointer(0)->getPointerFlyable()
-                     ->hit(p->getUserPointer(1)->getPointerKart());
+                     ->hit(target_kart);
+            }
+
         }
         else
         {
@@ -249,33 +279,21 @@ void Physics::update(float dt)
 }   // update
 
 //-----------------------------------------------------------------------------
-/** Project all karts downwards onto the surface below.
- *  Used in setting the starting positions of all the karts.
- */
-
-bool Physics::projectKartDownwards(const AbstractKart *k)
-{
-    btVector3 hell(0, -10000, 0);
-    return k->getVehicle()->projectVehicleToSurface(hell, 
-                                                    /*allow translation*/true);
-} //projectKartsDownwards
-
-//-----------------------------------------------------------------------------
-/** Handles the special case of two karts colliding with each other, which 
- *  means that bombs must be passed on. If both karts have a bomb, they'll 
- *  explode immediately. This function is called from physics::update() on the 
- *  server and if no networking is used, and from race_state on the client to 
+/** Handles the special case of two karts colliding with each other, which
+ *  means that bombs must be passed on. If both karts have a bomb, they'll
+ *  explode immediately. This function is called from physics::update() on the
+ *  server and if no networking is used, and from race_state on the client to
  *  replay what happened on the server.
  *  \param kart_a First kart involved in the collision.
- *  \param contact_point_a Location of collision at first kart (in kart 
+ *  \param contact_point_a Location of collision at first kart (in kart
  *         coordinates).
  *  \param kart_b Second kart involved in the collision.
  *  \param contact_point_b Location of collision at second kart (in kart
  *         coordinates).
  */
-void Physics::KartKartCollision(AbstractKart *kart_a, 
+void Physics::KartKartCollision(AbstractKart *kart_a,
                                 const Vec3 &contact_point_a,
-                                AbstractKart *kart_b, 
+                                AbstractKart *kart_b,
                                 const Vec3 &contact_point_b)
 {
     // Only one kart needs to handle the attachments, it will
@@ -286,12 +304,12 @@ void Physics::KartKartCollision(AbstractKart *kart_a,
     AbstractKart *left_kart, *right_kart;
 
     // Determine which kart is pushed to the left, and which one to the
-    // right. Ideally the sign of the X coordinate of the local conact point 
+    // right. Ideally the sign of the X coordinate of the local conact point
     // could decide the direction (negative X --> was hit on left side, gets
     // push to right), but that can lead to both karts being pushed in the
     // same direction (front left of kart hits rear left).
     // So we just use a simple test (which does the right thing in ideal
-    // crashes, but avoids pushing both karts in corner cases 
+    // crashes, but avoids pushing both karts in corner cases
     // - pun intended ;) ).
     if(contact_point_a.getX() < contact_point_b.getX())
     {
@@ -307,12 +325,12 @@ void Physics::KartKartCollision(AbstractKart *kart_a,
     // Add a scaling factor depending on the mass (avoid div by zero).
     // The value of f_right is applied to the right kart, and f_left
     // to the left kart. f_left = 1 / f_right
-    float f_right =  right_kart->getKartProperties()->getMass() > 0 
-                     ? left_kart->getKartProperties()->getMass() 
+    float f_right =  right_kart->getKartProperties()->getMass() > 0
+                     ? left_kart->getKartProperties()->getMass()
                        / right_kart->getKartProperties()->getMass()
                      : 1.5f;
     // Add a scaling factor depending on speed (avoid div by 0)
-    f_right *= right_kart->getSpeed() > 0 
+    f_right *= right_kart->getSpeed() > 0
                ? left_kart->getSpeed()
                   / right_kart->getSpeed()
                : 1.5f;
@@ -329,7 +347,7 @@ void Physics::KartKartCollision(AbstractKart *kart_a,
     float vel_left  = left_kart->getVelocityLC().getX();
     float vel_right = right_kart->getVelocityLC().getX();
 
-    // Use the difference in speed to determine which kart gets a 
+    // Use the difference in speed to determine which kart gets a
     // ramming bonus. Normally vel_right and vel_left will have
     // a different sign: right kart will be driving to the left,
     // and left kart to the right (both pushing at each other).
@@ -402,7 +420,7 @@ void Physics::KartKartCollision(AbstractKart *kart_a,
  *  actual physics timestep. This list only stores a collision if it's not
  *  already in the list, so a collisions which is reported more than once is
  *  nevertheless only handled once.
- *  The list of collision 
+ *  The list of collision
  *  Parameters: see bullet documentation for details.
  */
 btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
@@ -410,16 +428,16 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                              btTypedConstraint** constraints,
                              int numConstraints,
                              const btContactSolverInfo& info,
-                             btIDebugDraw* debugDrawer, 
-                             btStackAlloc* stackAlloc, 
+                             btIDebugDraw* debugDrawer,
+                             btStackAlloc* stackAlloc,
                              btDispatcher* dispatcher)
 {
     btScalar returnValue=
-        btSequentialImpulseConstraintSolver::solveGroup(bodies, numBodies, 
+        btSequentialImpulseConstraintSolver::solveGroup(bodies, numBodies,
                                                         manifold, numManifolds,
                                                         constraints,
                                                         numConstraints, info,
-                                                        debugDrawer, 
+                                                        debugDrawer,
                                                         stackAlloc,
                                                         dispatcher);
     int currentNumManifolds = m_dispatcher->getNumManifolds();
@@ -427,15 +445,14 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
     // more than one object, and/or more than once with each object (if there
     // is more than one collision point). So keep a list of rockets that will
     // be exploded after the collisions
-    std::vector<Moveable*> rocketsToExplode;
     for(int i=0; i<currentNumManifolds; i++)
     {
-        btPersistentManifold* contact_manifold = 
+        btPersistentManifold* contact_manifold =
             m_dynamics_world->getDispatcher()->getManifoldByIndexInternal(i);
 
-        btCollisionObject* objA = 
+        btCollisionObject* objA =
             static_cast<btCollisionObject*>(contact_manifold->getBody0());
-        btCollisionObject* objB = 
+        btCollisionObject* objB =
             static_cast<btCollisionObject*>(contact_manifold->getBody1());
 
         unsigned int num_contacts = contact_manifold->getNumContacts();
@@ -459,7 +476,7 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                 AbstractKart *kart=upB->getPointerKart();
                 race_state->addCollision(kart->getWorldKartId());
                 int n = contact_manifold->getContactPoint(0).m_index0;
-                const Material *m 
+                const Material *m
                     = n>=0 ? upA->getPointerTriangleMesh()->getMaterial(n)
                            : NULL;
                 // I assume that the normal needs to be flipped in this case,
@@ -469,6 +486,16 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                                                             .m_normalWorldOnB;
                 kart->crashed(m, normal);
             }
+            else if(upB->is(UserPointer::UP_PHYSICAL_OBJECT))
+            {
+                int n = contact_manifold->getContactPoint(0).m_index1;
+                const Material *m
+                    = n>=0 ? upA->getPointerTriangleMesh()->getMaterial(n)
+                           : NULL;
+                const btVector3 &normal = contact_manifold->getContactPoint(0)
+                                                           .m_normalWorldOnB;
+                upB->getPointerPhysicalObject()->hit(m, normal);
+            }
         }
         // 2) object a is a kart
         // =====================
@@ -479,7 +506,7 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                 AbstractKart *kart = upA->getPointerKart();
                 race_state->addCollision(kart->getWorldKartId());
                 int n = contact_manifold->getContactPoint(0).m_index1;
-                const Material *m 
+                const Material *m
                     = n>=0 ? upB->getPointerTriangleMesh()->getMaterial(n)
                            : NULL;
                 const btVector3 &normal = contact_manifold->getContactPoint(0)
@@ -536,6 +563,16 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                 m_all_collisions.push_back(
                     upA, contact_manifold->getContactPoint(0).m_localPointA,
                     upB, contact_manifold->getContactPoint(0).m_localPointB);
+            else if(upB->is(UserPointer::UP_TRACK))
+            {
+                int n = contact_manifold->getContactPoint(0).m_index1;
+                const Material *m
+                    = n>=0 ? upB->getPointerTriangleMesh()->getMaterial(n)
+                           : NULL;
+                const btVector3 &normal = contact_manifold->getContactPoint(0)
+                                                           .m_normalWorldOnB;
+                upA->getPointerPhysicalObject()->hit(m, normal);
+            }
         }
         else if (upA->is(UserPointer::UP_ANIMATION))
         {
@@ -544,7 +581,7 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                     upA, contact_manifold->getContactPoint(0).m_localPointA,
                     upB, contact_manifold->getContactPoint(0).m_localPointB);
         }
-        else 
+        else
             assert("Unknown user pointer");           // 4) Should never happen
     }   // for i<numManifolds
 
@@ -568,7 +605,7 @@ void Physics::draw()
     material.BackfaceCulling = false;
     material.setFlag(video::EMF_LIGHTING, false);
     irr_driver->getVideoDriver()->setMaterial(material);
-    irr_driver->getVideoDriver()->setTransform(video::ETS_WORLD, 
+    irr_driver->getVideoDriver()->setTransform(video::ETS_WORLD,
                                                core::IdentityMatrix);
     m_dynamics_world->debugDrawWorld();
     return;
