@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -22,8 +22,9 @@
 #include <algorithm>
 
 #include "challenges/unlock_manager.hpp"
-#include "config/user_config.hpp"
+#include "config/saved_grand_prix.hpp"
 #include "config/stk_config.hpp"
+#include "config/user_config.hpp"
 #include "graphics/irr_driver.hpp"
 #include "input/device_manager.hpp"
 #include "input/input_manager.hpp"
@@ -32,12 +33,15 @@
 #include "karts/kart_properties_manager.hpp"
 #include "modes/cutscene_world.hpp"
 #include "modes/demo_world.hpp"
+#include "modes/easter_egg_hunt.hpp"
 #include "modes/follow_the_leader.hpp"
 #include "modes/overworld.hpp"
 #include "modes/profile_world.hpp"
 #include "modes/standard_race.hpp"
+#include "modes/tutorial_world.hpp"
 #include "modes/world.hpp"
 #include "modes/three_strikes_battle.hpp"
+#include "modes/soccer_world.hpp"
 #include "network/network_manager.hpp"
 #include "states_screens/grand_prix_lose.hpp"
 #include "states_screens/grand_prix_win.hpp"
@@ -53,6 +57,8 @@ RaceManager* race_manager= NULL;
  */
 RaceManager::RaceManager()
 {
+    // Several code depends on this, e.g. kart_properties
+    assert(DIFFICULTY_FIRST == 0);
     m_num_karts          = UserConfigParams::m_num_karts;
     m_difficulty         = DIFFICULTY_HARD;
     m_major_mode         = MAJOR_MODE_SINGLE;
@@ -69,14 +75,14 @@ RaceManager::RaceManager()
 }   // RaceManager
 
 //-----------------------------------------------------------------------------
-/** Destructor for the race manager. 
+/** Destructor for the race manager.
  */
 RaceManager::~RaceManager()
 {
 }   // ~RaceManager
 
 //-----------------------------------------------------------------------------
-/** Resets the race manager in preparation for a new race. It sets the 
+/** Resets the race manager in preparation for a new race. It sets the
  *  counter of finished karts to zero.
  */
 void RaceManager::reset()
@@ -107,14 +113,14 @@ void RaceManager::setDefaultAIKartList(const std::vector<std::string>& ai_list)
         const KartProperties *kp = kart_properties_manager->getKart(name);
         if(!kp)
         {
-            printf("Kart '%s' is unknown and therefore ignored.\n",
-                   name.c_str());
+            Log::warn("RaceManager", "Kart '%s' is unknown and therefore ignored.",
+                      name.c_str());
             continue;
         }
         // This doesn't work anymore, since this is called when
         // handling the command line options, at which time the
         // player (and therefore the current slot) is not defined yet.
-        //if(unlock_manager->getCurrentSlot()->isLocked(name)) 
+        //if(unlock_manager->getCurrentSlot()->isLocked(name))
         //{
         //   printf("Kart '%s' is locked and therefore ignored.\n",
         //           name.c_str());
@@ -125,12 +131,12 @@ void RaceManager::setDefaultAIKartList(const std::vector<std::string>& ai_list)
 }   // setDefaultAIKartList
 
 // ----------------------------------------------------------------------------
-/** Sets information about a kart used by a local player (i.e. on this 
+/** Sets information about a kart used by a local player (i.e. on this
  *  computer).
  *  \param player_id  Id of this player.
  *  \param kart The kart this player uses.
  */
-void RaceManager::setLocalKartInfo(unsigned int player_id, 
+void RaceManager::setLocalKartInfo(unsigned int player_id,
                                    const std::string& kart)
 {
     assert(kart.size() > 0);
@@ -142,6 +148,16 @@ void RaceManager::setLocalKartInfo(unsigned int player_id,
                                                   network_manager->getMyHostId());
 }   // setLocalKartInfo
 
+//-----------------------------------------------------------------------------
+/** Sets additional information for a player to indicate which soccer team it belong to
+*/
+void RaceManager::setLocalKartSoccerTeam(unsigned int player_id, SoccerTeam team)
+{
+    assert(0<=player_id && player_id <m_local_player_karts.size());
+
+    m_local_player_karts[player_id].setSoccerTeam(team);
+}
+
 //-----------------------------------------------------------------------------
 /** Returns a pointer to the kart which has a given GP rank.
  *  \param n The rank (1 to number of karts) to look for.
@@ -207,7 +223,7 @@ void RaceManager::setTrack(const std::string& track)
 {
     m_tracks.clear();
     m_tracks.push_back(track);
-    
+
     m_coin_target = 0;
 }   // setTrack
 
@@ -219,8 +235,8 @@ void RaceManager::computeRandomKartList()
 {
     int n = m_num_karts - m_player_karts.size();
     if(UserConfigParams::logMisc())
-        std::cout << "AI karts count = " << n << " for m_num_karts=" 
-                  << m_num_karts << " and m_player_karts.size()=" 
+        std::cout << "AI karts count = " << n << " for m_num_karts="
+                  << m_num_karts << " and m_player_karts.size()="
                   << m_player_karts.size() << std::endl;
 
     // If less kart selected than there are player karts, adjust the number of
@@ -241,7 +257,7 @@ void RaceManager::computeRandomKartList()
     }
 
     if(n>0)
-        kart_properties_manager->getRandomKartList(n, m_player_karts, 
+        kart_properties_manager->getRandomKartList(n, m_player_karts,
                                                    &m_ai_kart_list   );
 
     if (m_ai_kart_override != "")
@@ -259,7 +275,7 @@ void RaceManager::computeRandomKartList()
 void RaceManager::startNew(bool from_overworld)
 {
     m_started_from_overworld = from_overworld;
-    
+
     if(m_major_mode==MAJOR_MODE_GRAND_PRIX)
     {
         // GP: get tracks, laps and reverse info from grand prix
@@ -271,36 +287,36 @@ void RaceManager::startNew(bool from_overworld)
 
     // command line parameters: negative numbers=all karts
     if(m_num_karts < 0 ) m_num_karts = stk_config->m_max_karts;
-    if((size_t)m_num_karts < m_player_karts.size()) 
+    if((size_t)m_num_karts < m_player_karts.size())
         m_num_karts = (int)m_player_karts.size();
-    
+
     // Create the kart status data structure to keep track of scores, times, ...
     // ==========================================================================
     m_kart_status.clear();
 
-    // Before thi
+
     assert((unsigned int)m_num_karts == m_ai_kart_list.size()+m_player_karts.size());
     // First add the AI karts (randomly chosen)
     // ----------------------------------------
 
     // GP ranks start with -1 for the leader.
-    int init_gp_rank = 
-        race_manager->getMinorMode()==RaceManager::MINOR_MODE_FOLLOW_LEADER 
-        ? -1 
+    int init_gp_rank =
+        race_manager->getMinorMode()==RaceManager::MINOR_MODE_FOLLOW_LEADER
+        ? -1
         : 0;
     const unsigned int ai_kart_count = m_ai_kart_list.size();
     for(unsigned int i=0; i<ai_kart_count; i++)
     {
-        m_kart_status.push_back(KartStatus(m_ai_kart_list[i], i, -1, -1, 
+        m_kart_status.push_back(KartStatus(m_ai_kart_list[i], i, -1, -1,
                                            init_gp_rank, KT_AI));
         init_gp_rank ++;
         if(UserConfigParams::m_ftl_debug)
         {
-            printf("[ftl] rank %d ai-kart %s\n", init_gp_rank, 
+            Log::debug("RaceManager", "[ftl] rank %d ai-kart %s", init_gp_rank,
                    m_ai_kart_list[i].c_str());
         }
     }
-    
+
     // Then the players, which start behind the AI karts
     // -------------------------------------------------
     for(int i=m_player_karts.size()-1; i>=0; i--)
@@ -314,15 +330,40 @@ void RaceManager::startNew(bool from_overworld)
                                            ) );
         if(UserConfigParams::m_ftl_debug)
         {
-            printf("[ftl] rank %d kart %s\n", init_gp_rank, 
+            Log::debug("RaceManager", "[ftl] rank %d kart %s", init_gp_rank,
                 m_player_karts[i].getKartName().c_str());
         }
         init_gp_rank ++;
     }
 
-    // Then start the race with the first track
-    // ========================================
     m_track_number = 0;
+    if(m_major_mode==MAJOR_MODE_GRAND_PRIX)
+    {
+        //We look if Player 1 has a saved version of this GP.
+        // =================================================
+        SavedGrandPrix* gp = SavedGrandPrix::getSavedGP( StateManager::get()
+                                                         ->getActivePlayerProfile(0)
+                                                         ->getUniqueID(),
+                                                         m_grand_prix.getId(),
+                                                         m_difficulty,
+                                                         m_num_karts,
+                                                         m_player_karts.size());
+
+        // Start the race with the appropriate track
+        // =========================================
+        if(gp != NULL)
+        {
+            if (m_continue_saved_gp)
+            {
+                m_track_number = gp->getNextTrack();
+                gp->loadKarts(m_kart_status);
+            }
+            else
+            {
+                gp->remove();
+            }
+        }
+    }
     startNextRace();
 }   // startNew
 
@@ -335,22 +376,22 @@ void RaceManager::startNextRace()
 {
     // Uncomment to debug audio leaks
     // sfx_manager->dump();
-    
+
     stk_config->getAllScores(&m_score_for_position, m_num_karts);
     IrrlichtDevice* device = irr_driver->getDevice();
     GUIEngine::renderLoading();
     device->getVideoDriver()->endScene();
     device->getVideoDriver()->beginScene(true, true, video::SColor(255,100,101,140));
 
-    
+
     m_num_finished_karts   = 0;
     m_num_finished_players = 0;
 
     // if subsequent race, sort kart status structure
     // ==============================================
     if (m_track_number > 0)
-    {  
-        // In follow the leader mode do not change the first kart, 
+    {
+        // In follow the leader mode do not change the first kart,
         // since it's always the leader.
         int offset = (m_minor_mode==MINOR_MODE_FOLLOW_LEADER) ? 1 : 0;
 
@@ -372,35 +413,51 @@ void RaceManager::startNextRace()
         {
             std::reverse(m_kart_status.begin()+offset,
                          m_kart_status.end() - player_last_offset);
-        } 
+        }
     }   // not first race
-    
+
     // the constructor assigns this object to the global
     // variable world. Admittedly a bit ugly, but simplifies
     // handling of objects which get created in the constructor
     // and need world to be defined.
     if(DemoWorld::isDemoMode())
         World::setWorld(new DemoWorld());
-    else if(ProfileWorld::isProfileMode())            
+    else if(ProfileWorld::isProfileMode())
         World::setWorld(new ProfileWorld());
-    else if(m_minor_mode==MINOR_MODE_FOLLOW_LEADER) 
+    else if(m_minor_mode==MINOR_MODE_FOLLOW_LEADER)
         World::setWorld(new FollowTheLeaderRace());
-    else if(m_minor_mode==MINOR_MODE_NORMAL_RACE || 
+    else if(m_minor_mode==MINOR_MODE_NORMAL_RACE ||
             m_minor_mode==MINOR_MODE_TIME_TRIAL)
         World::setWorld(new StandardRace());
-    else if(m_minor_mode==MINOR_MODE_3_STRIKES)     
+    else if(m_minor_mode==MINOR_MODE_TUTORIAL)
+        World::setWorld(new TutorialWorld());
+    else if(m_minor_mode==MINOR_MODE_3_STRIKES)
         World::setWorld(new ThreeStrikesBattle());
-    else if(m_minor_mode==MINOR_MODE_OVERWORLD)     
+    else if(m_minor_mode==MINOR_MODE_SOCCER)
+        World::setWorld(new SoccerWorld());
+    else if(m_minor_mode==MINOR_MODE_OVERWORLD)
         World::setWorld(new OverWorld());
     else if(m_minor_mode==MINOR_MODE_CUTSCENE)
         World::setWorld(new CutsceneWorld());
+    else if(m_minor_mode==MINOR_MODE_EASTER_EGG)
+        World::setWorld(new EasterEggHunt());
     else
-    { 
-        fprintf(stderr,"Could not create given race mode\n"); 
-        assert(0); 
+    {
+        Log::error("RaceManager", "Could not create given race mode.");
+        assert(0);
     }
-    
+
+    // A second constructor phase is necessary in order to be able to
+    // call functions which are overwritten (otherwise polymorphism
+    // will fail and the results will be incorrect . Also in init() functions
+    // can be called that use World::getWorld().
     World::getWorld()->init();
+
+    // Now initialise all values that need to be reset from race to race
+    // Calling this here reduces code duplication in init and restartRace()
+    // functions.
+    World::getWorld()->reset();
+
     // Save the current score and set last time to zero. This is necessary
     // if someone presses esc after finishing a gp, and selects restart:
     // The race is rerun, and the points and scores get reset ... but if
@@ -417,13 +474,49 @@ void RaceManager::startNextRace()
 //-----------------------------------------------------------------------------
 
 void RaceManager::next()
-{    
+{
     World::deleteWorld();
     m_num_finished_karts   = 0;
     m_num_finished_players = 0;
     m_track_number++;
     if(m_track_number<(int)m_tracks.size())
     {
+        if(m_major_mode==MAJOR_MODE_GRAND_PRIX)
+        {
+            //Saving GP state
+            //We look if Player 1 has already saved this GP.
+            SavedGrandPrix* gp =
+                SavedGrandPrix::getSavedGP(StateManager::get()
+                                           ->getActivePlayerProfile(0)
+                                           ->getUniqueID(),
+                                           m_grand_prix.getId(),
+                                           m_difficulty,
+                                           m_num_karts,
+                                           m_player_karts.size());
+            if(gp != NULL)
+            {
+                //if so addept it
+                gp->setKarts(m_kart_status);
+                gp->setNextTrack(m_track_number);
+            }
+            else
+            {
+                //create a new entry
+                UserConfigParams::m_saved_grand_prix_list.push_back(
+                    new SavedGrandPrix(
+                        StateManager::get()->getActivePlayerProfile(0)
+                                           ->getUniqueID(),
+                        m_grand_prix.getId(),
+                        m_difficulty,
+                        m_player_karts.size(),
+                        m_track_number,
+                        m_kart_status
+                    )
+                );
+            }
+            user_config->saveConfig();
+        }
+
         if(network_manager->getMode()==NetworkManager::NW_SERVER)
             network_manager->beginReadySetGoBarrier();
         else
@@ -459,16 +552,23 @@ namespace computeGPRanksData
         float m_race_time;
         bool operator<(const SortData &a)
         {
-            return ( (m_score > a.m_score) || 
+            return ( (m_score > a.m_score) ||
                 (m_score == a.m_score && m_race_time < a.m_race_time) );
         }
 
+        // --------------------------------------------------------------------
+        bool operator>(const SortData &a)
+        {
+            return ( (m_score < a.m_score) ||
+                (m_score == a.m_score && m_race_time > a.m_race_time) );
+        }
+
     };   // SortData
 }   // namespace
 
-// ----------------------------------------------------------------------------    
+// ----------------------------------------------------------------------------
 /** Sort karts and update the m_gp_rank KartStatus member, in preparation
- *  for future calls to RaceManager::getKartGPRank or 
+ *  for future calls to RaceManager::getKartGPRank or
  *  RaceManager::getKartWithGPRank
  */
 void RaceManager::computeGPRanks()
@@ -491,9 +591,9 @@ void RaceManager::computeGPRanks()
         m_kart_status[0].m_gp_rank = -1;
         if(UserConfigParams::m_ftl_debug)
         {
-            printf("[ftl] kart '%s' has position %d.\n",
-                World::getWorld()->getKart(0)->getIdent().c_str(),
-                sd->m_position);
+            Log::debug("Race Manager","[ftl] kart '%s' has position %d.",
+                       World::getWorld()->getKart(0)->getIdent().c_str(),
+                       sd->m_position);
         }
     }
     for (unsigned int kart_id = start; kart_id < NUM_KARTS; ++kart_id)
@@ -505,49 +605,57 @@ void RaceManager::computeGPRanks()
         sort_data.push_back(sd);
         if(UserConfigParams::m_ftl_debug)
         {
-            printf("[ftl] kart '%s' has position %d score %d.\n",
+            Log::debug("Race Manager","[ftl] kart '%s' has position %d score %d.",
                 World::getWorld()->getKart(kart_id)->getIdent().c_str(),
                 sd->m_position, sd->m_score);
         }
     }
-    
+
     sort_data.insertionSort(start);
     for (unsigned int i=start; i < NUM_KARTS; ++i)
     {
-        //printf("setting kart %s to rank %i\n", 
-        //    m_kart_status[position[i]].m_ident.c_str(), i-start);
         if(UserConfigParams::m_ftl_debug)
         {
             const AbstractKart *kart =
                 World::getWorld()->getKart(sort_data[i].m_position);
-            printf("[ftl] kart '%s' has now position %d.\n",
-                kart->getIdent().c_str(), 
+            Log::debug("Race Manager","[ftl] kart '%s' has now position %d.",
+                kart->getIdent().c_str(),
                 i-start);
         }
 
         m_kart_status[sort_data[i].m_position].m_gp_rank = i - start;
     }
-    // printf("kart %s has rank %i\n", 0, m_kart_status[0].m_gp_rank);
 }   // computeGPRanks
 
 //-----------------------------------------------------------------------------
 
 void RaceManager::exitRace(bool delete_world)
 {
-    // Only display the grand prix result screen if all tracks 
+    // Only display the grand prix result screen if all tracks
     // were finished, and not when a race is aborted.
-    if (m_major_mode==MAJOR_MODE_GRAND_PRIX && m_track_number==(int)m_tracks.size()) 
+    if (m_major_mode==MAJOR_MODE_GRAND_PRIX && m_track_number==(int)m_tracks.size())
     {
         unlock_manager->getCurrentSlot()->grandPrixFinished();
-        
+        if(m_major_mode==MAJOR_MODE_GRAND_PRIX)
+        {
+            //Delete saved GP
+            SavedGrandPrix* gp =
+                SavedGrandPrix::getSavedGP(StateManager::get()
+                                           ->getActivePlayerProfile(0)
+                                           ->getUniqueID(),
+                                           m_grand_prix.getId(),
+                                           m_difficulty,
+                                           m_num_karts,
+                                           m_player_karts.size());
+            if(gp != NULL) gp->remove();
+        }
         StateManager::get()->resetAndGoToScreen( MainMenuScreen::getInstance() );
-        
 
         bool someHumanPlayerWon = false;
         const unsigned int kartStatusCount = m_kart_status.size();
-        
+
         const int loserThreshold = 3;
-        
+
         std::string winners[3];
         std::vector<std::string> humanLosers; // because we don't care about AIs that lost
         for (unsigned int i=0; i < kartStatusCount; ++i)
@@ -577,7 +685,7 @@ void RaceManager::exitRace(bool delete_world)
                 }
             }
         }
-        
+
         if (someHumanPlayerWon)
         {
             StateManager::get()->pushScreen( GrandPrixWin::getInstance()  );
@@ -586,7 +694,7 @@ void RaceManager::exitRace(bool delete_world)
         else
         {
             StateManager::get()->pushScreen( GrandPrixLose::getInstance()  );
-            
+
             if (humanLosers.size() >= 1)
             {
                 GrandPrixLose::getInstance()->setKarts( humanLosers );
@@ -602,13 +710,13 @@ void RaceManager::exitRace(bool delete_world)
     }
 
     if (delete_world) World::deleteWorld();
-    
+
     m_track_number = 0;
 }   // exitRace
 
 //-----------------------------------------------------------------------------
-/** A kart has finished the race at the specified time (which can be 
- *  different from World::getWorld()->getClock() in case of setting 
+/** A kart has finished the race at the specified time (which can be
+ *  different from World::getWorld()->getClock() in case of setting
  *  extrapolated arrival times). This function is only called from
  *  kart::finishedRace()
  *  \param kart The kart that finished the race.
@@ -640,12 +748,13 @@ void RaceManager::rerunRace()
         m_kart_status[i].m_score         = m_kart_status[i].m_last_score;
         m_kart_status[i].m_overall_time -= m_kart_status[i].m_last_time;
     }
-    World::getWorld()->restartRace();
+    World::getWorld()->reset();
 }   // rerunRace
 
 //-----------------------------------------------------------------------------
 
-void RaceManager::startGP(const GrandPrixData* gp, bool from_overworld)
+void RaceManager::startGP(const GrandPrixData* gp, bool from_overworld, 
+                          bool continue_saved_gp)
 {
     assert(gp != NULL);
 
@@ -653,19 +762,20 @@ void RaceManager::startGP(const GrandPrixData* gp, bool from_overworld)
     setGrandPrix(*gp);
     setCoinTarget( 0 ); // Might still be set from a previous challenge
     network_manager->setupPlayerKartInfo();
-    
+    m_continue_saved_gp = continue_saved_gp;
+
     setMajorMode(RaceManager::MAJOR_MODE_GRAND_PRIX);
     startNew(from_overworld);
 }
 
 //-----------------------------------------------------------------------------
-/** \brief Higher-level method to start a GP without having to care about 
+/** \brief Higher-level method to start a GP without having to care about
  *  the exact startup sequence.
  * \param trackIdent Internal name of the track to race on
- * \param num_laps   Number of laps to race, or -1 if number of laps is 
+ * \param num_laps   Number of laps to race, or -1 if number of laps is
  *        not relevant in current mode
  */
-void RaceManager::startSingleRace(const std::string &track_ident, 
+void RaceManager::startSingleRace(const std::string &track_ident,
                                   const int num_laps,
                                   bool from_overworld)
 {
@@ -673,12 +783,12 @@ void RaceManager::startSingleRace(const std::string &track_ident,
     setTrack(track_ident.c_str());
 
     if (num_laps != -1) setNumLaps( num_laps );
-    
+
     setMajorMode(RaceManager::MAJOR_MODE_SINGLE);
-    
+
     setCoinTarget( 0 ); // Might still be set from a previous challenge
     network_manager->setupPlayerKartInfo();
-    
+
     startNew(from_overworld);
 }
 
