@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2006 SuperTuxKart-Team
+//  Copyright (C) 2006-2013 SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -42,10 +42,12 @@ static const std::string IDENT_STD      ("STANDARD"        );
 static const std::string IDENT_TTRIAL   ("STD_TIMETRIAL"   );
 static const std::string IDENT_FTL      ("FOLLOW_LEADER"   );
 static const std::string IDENT_STRIKES  ("BATTLE_3_STRIKES");
+static const std::string IDENT_EASTER   ("EASTER_EGG_HUNT");
+static const std::string IDENT_SOCCER   ("SOCCER"          );
 static const std::string IDENT_OVERWORLD("OVERWORLD"   );
 static const std::string IDENT_CUSTSCENE("CUTSCENE"   );
 
-/** 
+/**
  * The race manager has two functions:
  *  1) it stores information about the race the user selected (e.g. number
  *     of karts, track, race mode etc.). Most of the values are just stored
@@ -55,8 +57,8 @@ static const std::string IDENT_CUSTSCENE("CUTSCENE"   );
  *     the GrandPrix object), and local player information (number of local
  *     players, and selected karts). The local player information is read
  *     from the NetworkManager, gathered on the server (from all clients and
- *     the server, see NetworkManager::setupPlayerKartInfo), and then the 
- *     combined information distributed to all RaceManagers in all clients 
+ *     the server, see NetworkManager::setupPlayerKartInfo), and then the
+ *     combined information distributed to all RaceManagers in all clients
  *     and server. Even in no networking mode, the data flow is the same:
  *     information about local players is stored here, then processed by
  *     NetworkManager::setupPlayerKartInfo and the 'global' information about
@@ -88,12 +90,13 @@ public:
         MAJOR_MODE_GRAND_PRIX,
         MAJOR_MODE_SINGLE
     };
-    
+
     // quick method to tell the difference between battle modes and race modes
-    // think of it like a bitmask, but done in decimal to avoid endianness 
+    // think of it like a bitmask, but done in decimal to avoid endianness
     // issues
 #define LINEAR_RACE(ID, COUNT_LAPSES) (1000+ID+100*COUNT_LAPSES)
 #define BATTLE_ARENA(ID) (2000+ID)
+#define EASTER_EGG(ID)   (3000+ID)
 
     // ------------------------------------------------------------------------
     /** Minor variants to the major types of race.
@@ -101,14 +104,17 @@ public:
     enum MinorRaceModeType
     {
         MINOR_MODE_NONE          = -1,
-        
+
         MINOR_MODE_NORMAL_RACE   = LINEAR_RACE(0, true),
         MINOR_MODE_TIME_TRIAL    = LINEAR_RACE(1, true),
         MINOR_MODE_FOLLOW_LEADER = LINEAR_RACE(2, false),
         MINOR_MODE_OVERWORLD     = LINEAR_RACE(3, false),
-        
+        MINOR_MODE_TUTORIAL      = LINEAR_RACE(4, false),
+
         MINOR_MODE_3_STRIKES     = BATTLE_ARENA(0),
-        MINOR_MODE_CUTSCENE      = BATTLE_ARENA(1)
+        MINOR_MODE_SOCCER        = BATTLE_ARENA(1),
+        MINOR_MODE_CUTSCENE      = BATTLE_ARENA(2),
+        MINOR_MODE_EASTER_EGG    = EASTER_EGG(0)
     };
 
     // ------------------------------------------------------------------------
@@ -119,7 +125,7 @@ public:
         SUPERPOWER_NONE       = 0,
         SUPERPOWER_NOLOK_BOSS = 1
     };
-    
+
     // ------------------------------------------------------------------------
     /** Returns a string identifier for each minor race mode.
      *  \param mode Minor race mode.
@@ -132,14 +138,16 @@ public:
             case MINOR_MODE_TIME_TRIAL:     return IDENT_TTRIAL;
             case MINOR_MODE_FOLLOW_LEADER:  return IDENT_FTL;
             case MINOR_MODE_3_STRIKES:      return IDENT_STRIKES;
-            default: assert(false); 
+            case MINOR_MODE_EASTER_EGG:     return IDENT_EASTER;
+            case MINOR_MODE_SOCCER:         return IDENT_SOCCER;
+            default: assert(false);
                      return IDENT_STD;  // stop compiler warning
         }
     }   // getIdentOf
-    
+
     // ------------------------------------------------------------------------
-    /** Returns the icon for a minor race mode. 
-     *  \param mode Minor race mode. 
+    /** Returns the icon for a minor race mode.
+     *  \param mode Minor race mode.
      */
     static const char* getIconOf(const MinorRaceModeType mode)
     {
@@ -149,6 +157,8 @@ public:
             case MINOR_MODE_TIME_TRIAL:     return "/gui/mode_tt.png";
             case MINOR_MODE_FOLLOW_LEADER:  return "/gui/mode_ftl.png";
             case MINOR_MODE_3_STRIKES:      return "/gui/mode_3strikes.png";
+            case MINOR_MODE_EASTER_EGG:     return "/gui/mode_easter.png";
+            case MINOR_MODE_SOCCER:         return "/gui/mode_soccer.png";
             default: assert(false); return NULL;
         }
     }   // getIconOf
@@ -169,10 +179,14 @@ public:
             case MINOR_MODE_FOLLOW_LEADER:  return _("Follow the Leader");
             //I18N: Game mode
             case MINOR_MODE_3_STRIKES:      return _("3 Strikes Battle");
+            //I18N: Game mode
+            case MINOR_MODE_EASTER_EGG:     return _("Egg Hunt");
+            //I18N: Game mode
+            case MINOR_MODE_SOCCER:         return _("Soccer");
             default: assert(false); return NULL;
         }
     }
-    
+
     // ------------------------------------------------------------------------
     static bool hasAI(const MinorRaceModeType mode)
     {
@@ -182,11 +196,13 @@ public:
             case MINOR_MODE_TIME_TRIAL:     return true;
             case MINOR_MODE_FOLLOW_LEADER:  return true;
             case MINOR_MODE_3_STRIKES:      return false;
+            case MINOR_MODE_EASTER_EGG:     return false;
+            case MINOR_MODE_SOCCER:         return false;
             default: assert(false); return NULL;
         }
     }
-                          
-    
+
+
     // ------------------------------------------------------------------------
     /** Returns the minor mode id from a string identifier. This function is
      *  used from challenge_data, which reads the mode from a challenge file.
@@ -199,28 +215,35 @@ public:
         else if (name==IDENT_TTRIAL ) return MINOR_MODE_TIME_TRIAL;
         else if (name==IDENT_FTL    ) return MINOR_MODE_FOLLOW_LEADER;
         else if (name==IDENT_STRIKES) return MINOR_MODE_3_STRIKES;
+        else if (name==IDENT_EASTER ) return MINOR_MODE_EASTER_EGG;
+        else if (name==IDENT_SOCCER)  return MINOR_MODE_SOCCER;
 
         assert(0);
         return MINOR_MODE_NONE;
     }
-    
+
 #undef LINEAR_RACE
 #undef BATTLE_ARENA
-    
+
     /** Game difficulty. */
     enum Difficulty     { DIFFICULTY_EASY,
-                          DIFFICULTY_MEDIUM, 
+                          DIFFICULTY_FIRST = DIFFICULTY_EASY,
+                          DIFFICULTY_MEDIUM,
                           DIFFICULTY_HARD,
+                          DIFFICULTY_BEST,
+                          DIFFICULTY_LAST = DIFFICULTY_BEST,
                           DIFFICULTY_COUNT};
-    
+
     /** Different kart types: A local player, a player connected via network,
      *  an AI kart, the leader kart (currently not used), a ghost kart. */
-    enum KartType       { KT_PLAYER, KT_NETWORK_PLAYER, KT_AI, KT_LEADER, 
+    enum KartType       { KT_PLAYER, KT_NETWORK_PLAYER, KT_AI, KT_LEADER,
                           KT_GHOST };
 private:
 
     bool m_started_from_overworld;
 
+public:
+
     /** This data structure accumulates kart data and race result data from
      *  each race. */
     struct KartStatus
@@ -237,30 +260,29 @@ private:
         float       m_overall_time;
         /** Needed for restart. */
         float       m_last_time;
-        /** Previous finished position. */
-        int         m_prev_finish_pos;
         /** Kart type: AI, player, network player etc. */
         KartType    m_kart_type;
         /** Player controling the kart, for AI: -1 */
         int         m_local_player_id;
         /** Global ID of player. */
         int         m_global_player_id;
-        /** In GPs, at the end, will hold the overall rank of this kart 
+        /** In GPs, at the end, will hold the overall rank of this kart
          *  (0<=m_gp_rank < num_karts-1). */
-        int         m_gp_rank; 
-        
-        KartStatus(const std::string& ident, const int& prev_finish_pos, 
-                   int local_player_id, int global_player_id, 
+        int         m_gp_rank;
+
+        KartStatus(const std::string& ident, const int& prev_finish_pos,
+                   int local_player_id, int global_player_id,
                    int init_gp_rank, KartType kt) :
-                   m_ident(ident), m_score(0), m_last_score(0), 
+                   m_ident(ident), m_score(0), m_last_score(0),
                    m_overall_time(0.0f), m_last_time(0.0f),
-                   m_prev_finish_pos(prev_finish_pos), m_kart_type(kt),
+                   m_kart_type(kt),
                    m_local_player_id(local_player_id),
                    m_global_player_id(global_player_id),
                    m_gp_rank(init_gp_rank)
                 {}
-        
+
     };   // KartStatus
+private:
 
     /** The kart status data for each kart. */
     std::vector<KartStatus>          m_kart_status;
@@ -282,7 +304,7 @@ private:
     /** The number of laps for each track of a GP (only one element
      *  is used if only a single track is used. */
     std::vector<int>                 m_num_laps;
-    
+
     /** Whether a track should be reversed */
     std::vector<bool>                m_reverse_track;
 
@@ -309,6 +331,7 @@ private:
     int                              m_coin_target;
     bool                             m_has_time_target;
     float                            m_time_target;
+    int                                 m_goal_target;
 
     void startNextRace();    // start a next race
 
@@ -321,11 +344,14 @@ private:
 
     bool m_have_kart_last_position_on_overworld;
     Vec3 m_kart_last_position_on_overworld;
+    
+    /** Determines if saved GP should be continued or not*/
+    bool m_continue_saved_gp;
 
 public:
          RaceManager();
         ~RaceManager();
-    
+
     /** Resets the race manager. It is called by world when restarting a race.
      */
     void reset();
@@ -334,47 +360,55 @@ public:
       * \{
       * \name Setting race parameters
       */
-    
+
     /** \brief Stores the information which local players uses which karts.
       * \param player_id  Id of the local player for which the kart is set.
       * \param kart       Kart name this player is using.
       */
     void setLocalKartInfo(unsigned int player_id, const std::string& kart);
-    
+
+    /** Sets additional information for a player to indicate which soccer team it belong to
+    */
+    void setLocalKartSoccerTeam(unsigned int player_id, SoccerTeam team);
+
     /** Sets the number of local players playing on this computer (including
       * split screen).
       * \param n Number of local players.
       */
     void setNumLocalPlayers(unsigned int n);
-    
+
     /** In case of non GP mode set the track to use.
      *  \param track Pointer to the track to use.
      */
     void setTrack(const std::string& track);
-    
-    /** \brief Returns the kart with a given GP rank (or NULL if no such 
+
+    /** \brief Returns the kart with a given GP rank (or NULL if no such
      *  kart exists).
      *  \param n Rank (0<=n<num_karts) to look for.
      */
     const AbstractKart* getKartWithGPRank(unsigned int n);
-    
-    /** \return the GP rank of a local player, or -1 if the given player ID 
+
+    /** \return the GP rank of a local player, or -1 if the given player ID
      *  doesn't exist */
     int getLocalPlayerGPRank(const int playerID) const;
-    
+
     bool hasTimeTarget() const { return m_has_time_target; }
-    
+
+    void setMaxGoal(int maxGoal){ m_goal_target = maxGoal; }
+
+    int getMaxGoal(){ return m_goal_target; }
+
     /** Sort karts and update the m_gp_rank KartStatus member, in preparation
-      * for future calls to RaceManager::getKartGPRank or 
+      * for future calls to RaceManager::getKartGPRank or
       *  RaceManager::getKartWithGPRank
       */
     void computeGPRanks();
-    
+
     /** \brief Sets the difficulty.
       * \param diff Difficulty.
       */
     void setDifficulty(Difficulty diff);
-    
+
     // ------------------------------------------------------------------------
     void setGrandPrix(const GrandPrixData &gp)
     {
@@ -385,17 +419,17 @@ public:
     void setAIKartOverride(const std::string& kart)
     {
         m_ai_kart_override = kart;
-    }    
+    }
     // ------------------------------------------------------------------------
-    void setAISuperPower(AISuperPower superpower) 
+    void setAISuperPower(AISuperPower superpower)
     {
         m_ai_superpower = superpower;
     }
     // ------------------------------------------------------------------------
     AISuperPower getAISuperPower() const { return m_ai_superpower; }
-    
+
     // ------------------------------------------------------------------------
-    void setNumLaps(int num) { 
+    void setNumLaps(int num) {
         m_num_laps.clear();
         m_num_laps.push_back(num);
     }
@@ -411,7 +445,7 @@ public:
     void setMinorMode(MinorRaceModeType mode) { m_minor_mode = mode;
                                                 m_has_time_target = false; }
     // ------------------------------------------------------------------------
-    void setNumKarts(int num) 
+    void setNumKarts(int num)
     {
         m_num_karts = num;
         m_ai_kart_override = "";
@@ -423,26 +457,26 @@ public:
     void setTimeTarget(float num) { m_has_time_target = true;
                                     m_time_target = num;        }
     /** \} */
-    
+
     // ------------------------------------------------------------------------
     /** \{
       * \name Getters
       * Get current race manager state and settings
       */
-    const RemoteKartInfo& getLocalKartInfo(unsigned int n) const 
-    { 
+    const RemoteKartInfo& getLocalKartInfo(unsigned int n) const
+    {
         return m_local_player_karts[n];
     }
     // ------------------------------------------------------------------------
     unsigned int getNumLocalPlayers() const
     {
-        return m_local_player_karts.size(); 
-    }    
+        return m_local_player_karts.size();
+    }
     // ------------------------------------------------------------------------
-    /** Returns the selected number of karts (selected number of players and 
+    /** Returns the selected number of karts (selected number of players and
      *  AI karts. */
     unsigned int getNumberOfKarts() const {return m_num_karts; }
-    
+
     // ------------------------------------------------------------------------
     MajorRaceModeType getMajorMode() const { return m_major_mode; }
     // ------------------------------------------------------------------------
@@ -451,14 +485,14 @@ public:
     unsigned int getNumPlayers() const { return m_player_karts.size(); }
     // ------------------------------------------------------------------------
     /** \brief Returns the number lf laps.
-     *  In case of FTL or battle mode always return 9999, since they don't 
+     *  In case of FTL or battle mode always return 9999, since they don't
      *  have laps. This avoids problems in FTL GP, since in this case no laps
-     *  would be set (otherwise we would need many more tests in calls to 
+     *  would be set (otherwise we would need many more tests in calls to
      *  getNumLaps).
      */
-    int          getNumLaps()             const 
+    int          getNumLaps()             const
     {
-        if(m_minor_mode==MINOR_MODE_3_STRIKES     || 
+        if(m_minor_mode==MINOR_MODE_3_STRIKES     ||
            m_minor_mode==MINOR_MODE_FOLLOW_LEADER    )
             return 9999;
         return m_num_laps[m_track_number];
@@ -467,7 +501,22 @@ public:
     /** \return whether the track should be reversed */
     bool getReverseTrack() const { return m_reverse_track[m_track_number]; }
     // ------------------------------------------------------------------------
-    Difficulty   getDifficulty() const { return m_difficulty; }
+    /** Returns the difficulty. */
+    Difficulty getDifficulty() const { return m_difficulty; }
+    // ------------------------------------------------------------------------
+    /** Returns the specified difficulty as a string. */
+    std::string getDifficultyAsString(Difficulty diff) const
+    {
+        switch(diff)
+        {
+        case RaceManager::DIFFICULTY_EASY:   return "easy";   break;
+        case RaceManager::DIFFICULTY_MEDIUM: return "medium"; break;
+        case RaceManager::DIFFICULTY_HARD:   return "hard";   break;
+        case RaceManager::DIFFICULTY_BEST:   return "best";   break;
+        default:  assert(false);
+        }
+        return "";
+    }   // getDifficultyAsString
     // ------------------------------------------------------------------------
     const std::string& getTrackName() const { return m_tracks[m_track_number];}
     // ------------------------------------------------------------------------
@@ -477,9 +526,9 @@ public:
     // ------------------------------------------------------------------------
     unsigned int getFinishedPlayers() const { return m_num_finished_players; }
     // ------------------------------------------------------------------------
-    int getKartGPRank(const int kart_id)const 
+    int getKartGPRank(const int kart_id)const
     {
-        return m_kart_status[kart_id].m_gp_rank; 
+        return m_kart_status[kart_id].m_gp_rank;
     }
     // ------------------------------------------------------------------------
     const std::string& getKartIdent(int kart) const
@@ -496,20 +545,20 @@ public:
     // ------------------------------------------------------------------------
     int getKartLocalPlayerId(int k) const
     {
-        return m_kart_status[k].m_local_player_id; 
+        return m_kart_status[k].m_local_player_id;
     }
     // ------------------------------------------------------------------------
-    int getKartGlobalPlayerId(int k) const 
-    { 
-        return m_kart_status[k].m_global_player_id; 
+    int getKartGlobalPlayerId(int k) const
+    {
+        return m_kart_status[k].m_global_player_id;
     }
     // ------------------------------------------------------------------------
-    float getOverallTime(int kart) const 
-    { 
+    float getOverallTime(int kart) const
+    {
         return m_kart_status[kart].m_overall_time;
     }
     // ------------------------------------------------------------------------
-    KartType getKartType(int kart) const 
+    KartType getKartType(int kart) const
     {
         return m_kart_status[kart].m_kart_type;
     }
@@ -520,11 +569,11 @@ public:
     // ------------------------------------------------------------------------
     int getPositionScore(int p) const { return m_score_for_position[p-1]; }
     // ------------------------------------------------------------------------
-    int getTrackNumber() const { return m_track_number; } 
+    int getTrackNumber() const { return m_track_number; }
     // ------------------------------------------------------------------------
     /** Returns the list of AI karts to use. Used for networking, and for
      *  the --ai= command line option. */
-    const std::vector<std::string>& getAIKartList() const 
+    const std::vector<std::string>& getAIKartList() const
     {
         return m_ai_kart_list;
     }
@@ -539,18 +588,25 @@ public:
         if(id > 999 && id < 2000) return true;
         else return false;
     }   // isLinearRaceMode
-    
+
     // ------------------------------------------------------------------------
     /** \brief Returns true if the current mode is a battle mode. */
     bool isBattleMode()
     {
         const int id = (int)m_minor_mode;
-        // This uses the  numerical id of the mode, see the macros 
+        // This uses the  numerical id of the mode, see the macros
         // LINEAR_RACE and BATTLE_ARENA above for exact meaning.
         if (id >= 2000) return true;
         else            return false;
     }   // isBattleMode
-    
+
+    // ------------------------------------------------------------------------
+
+    bool isTutorialMode()
+    {
+        return m_minor_mode == MINOR_MODE_TUTORIAL;
+    }
+
     // ------------------------------------------------------------------------
     /** \brief Returns true if the current mode has laps. */
     bool modeHasLaps()
@@ -565,71 +621,73 @@ public:
     /** Returns true if the currently selected minor mode has highscores. */
     bool modeHasHighscores()
     {
-        //FIXME: this information is duplicated. RaceManager knows about it, 
-        //       and each World may set m_use_highscores to true or false. 
+        //FIXME: this information is duplicated. RaceManager knows about it,
+        //       and each World may set m_use_highscores to true or false.
         //       The reason for this duplication is that we might want to know
         //       whether to display highscores without creating a World.
-        return m_minor_mode != MINOR_MODE_3_STRIKES && 
+        return m_minor_mode != MINOR_MODE_3_STRIKES &&
+               m_minor_mode != MINOR_MODE_SOCCER &&
                m_minor_mode != MINOR_MODE_FOLLOW_LEADER;
     }   // modeHasHighscore
 
     /** \} */
-    
+
     // ------------------------------------------------------------------------
     /**
       * \{
       * \name Controlling race
       * Start, stop, continue, restart races
       */
-    
+
     /**
      *  \brief Starts a new race or GP (or other mode).
-     *  It sets up the list of player karts, AI karts, GP tracks if relevant 
+     *  It sets up the list of player karts, AI karts, GP tracks if relevant
      *  etc.
-     *  \pre The list of AI karts to use must be set up first. This is 
-     *       usually being done by a call to computeRandomKartList() from 
-     *       NetworkManager::setupPlayerKartInfo, but could be done differently 
+     *  \pre The list of AI karts to use must be set up first. This is
+     *       usually being done by a call to computeRandomKartList() from
+     *       NetworkManager::setupPlayerKartInfo, but could be done differently
      *       (e.g. depending on user command line options to test certain AIs)
      */
     void startNew(bool from_overworld);
-    
+
     /** \brief Start the next race or go back to the start screen
       * If there are more races to do, starts the next race, otherwise
       * calls exitRace to finish the race.
       */
     void next();
-    
+
     /** \brief Rerun the same race again
       * This is called after a race is finished, and it will adjust
       * the number of points and the overall time before restarting the race.
       */
     void rerunRace();
-    
+
     /** \brief Exit a race (and don't start the next one)
       * \note In GP, displays the GP result screen first
       * \note Deletes the world.
       */
     void exitRace(bool delete_world=true);
-    
+
     /**
       * \brief Higher-level method to start a GP without having to care about
       *  the exact startup sequence
       */
-    void  startGP(const GrandPrixData* gp, bool from_overworld);
+    void  startGP(const GrandPrixData* gp, bool from_overworld, 
+                  bool continue_saved_gp);
 
     /**
-      * \brief Higher-level method to start a GP without having to care about 
+      * \brief Higher-level method to start a GP without having to care about
       *  the exact startup sequence.
       * \param trackIdent Internal name of the track to race on
-      * \param num_laps   Number of laps to race, or -1 if number of laps is 
+      * \param num_laps   Number of laps to race, or -1 if number of laps is
       *        not relevant in current mode
       */
     void  startSingleRace(const std::string &track_ident, const int num_laps,
                           bool from_overworld);
 
-    bool raceWasStartedFromOverworld() const 
+    bool raceWasStartedFromOverworld() const
     {
-        return m_started_from_overworld; 
+        return m_started_from_overworld;
     }
 
     /** \} */
@@ -637,10 +695,10 @@ public:
     /**
       * \{
       * \name Callbacks from the race classes
-      * These methods are to be used by the classes that manage the various 
+      * These methods are to be used by the classes that manage the various
       *  races, to let the race manager know about current status
       */
-    bool allPlayerFinished() const 
+    bool allPlayerFinished() const
     {
         return m_num_finished_players==m_player_karts.size();
     }
@@ -648,7 +706,7 @@ public:
     void kartFinishedRace(const AbstractKart* kart, float time);
 
     /** \} */
-    
+
     /**
      * \{
      * \name For internal use
@@ -656,8 +714,8 @@ public:
      * You shouldn't need to call any of those from higher-level code.
      */
     void setNumPlayers(int num);
-    
-    void setPlayerKart(unsigned int player_id, 
+
+    void setPlayerKart(unsigned int player_id,
                        const RemoteKartInfo& ki);
     void setDefaultAIKartList(const std::vector<std::string> &ai_list);
     void computeRandomKartList();
@@ -666,14 +724,14 @@ public:
      *  over the random selection. */
     void setAIKartList(const std::vector<std::string>& rkl)
     {
-        m_ai_kart_list = rkl; 
+        m_ai_kart_list = rkl;
     }
 
     /** \} */
     // ------------------------------------------------------------------------
     bool haveKartLastPositionOnOverworld()
-    { 
-        return m_have_kart_last_position_on_overworld; 
+    {
+        return m_have_kart_last_position_on_overworld;
     }
     // ------------------------------------------------------------------------
     void setKartLastPositionOnOverworld(Vec3 pos)
@@ -688,10 +746,10 @@ public:
     }
     // ------------------------------------------------------------------------
     Vec3 getKartLastPositionOnOverworld()
-    { 
+    {
         return m_kart_last_position_on_overworld;
     }
-    
+
 };   // RaceManager
 
 extern RaceManager *race_manager;
