@@ -1,5 +1,5 @@
 //  tinygettext - A gettext replacement that works directly on .po files
-//  Copyright (C) 2006 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2006-2013 Ingo Ruhnke <grumbel@gmx.de>
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -20,9 +20,10 @@
 #include <map>
 #include <assert.h>
 #include <vector>
+#include <algorithm>
 
 namespace tinygettext {
-
+
 struct LanguageSpec {
   /** Language code: "de", "en", ... */
   const char* language;
@@ -36,10 +37,10 @@ struct LanguageSpec {
   /** Language name: "German", "English", "French", ... */
   const char* name;
 };
-
+
 /** Language Definitions */
 //*{
-LanguageSpec languages[] = {
+static const LanguageSpec languages[] = {
   { "aa", 0,    0, "Afar"                        },
   { "af", 0,    0, "Afrikaans"                   },
   { "af", "ZA", 0, "Afrikaans (South Africa)"    },
@@ -282,7 +283,7 @@ LanguageSpec languages[] = {
   { NULL, 0,    0, NULL                          }
 };
 //*}
-
+
 std::string
 resolve_language_alias(const std::string& name)
 {
@@ -351,7 +352,7 @@ resolve_language_alias(const std::string& name)
     name_lowercase[i] = static_cast<char>(tolower(name[i]));
 
   Aliases::iterator i = language_aliases.find(name_lowercase);
-  if (i != language_aliases.end()) 
+  if (i != language_aliases.end())
   {
     return i->second;
   }
@@ -360,32 +361,32 @@ resolve_language_alias(const std::string& name)
     return name;
   }
 }
-
+
 Language
 Language::from_spec(const std::string& language, const std::string& country, const std::string& modifier)
 {
-  static std::map<std::string, std::vector<LanguageSpec*> > language_map;
+  static std::map<std::string, std::vector<const LanguageSpec*> > language_map;
 
   if (language_map.empty())
   { // Init language_map
     for(int i = 0; languages[i].language != NULL; ++i)
       language_map[languages[i].language].push_back(&languages[i]);
   }
-  
-  std::map<std::string, std::vector<LanguageSpec*> >::iterator i = language_map.find(language);
+
+  std::map<std::string, std::vector<const LanguageSpec*> >::iterator i = language_map.find(language);
   if (i != language_map.end())
   {
-    std::vector<LanguageSpec*>& lst = i->second;
+    std::vector<const LanguageSpec*>& lst = i->second;
 
     LanguageSpec tmpspec;
     tmpspec.language = language.c_str();
     tmpspec.country  = country.c_str();
     tmpspec.modifier = modifier.c_str();
     Language tmplang(&tmpspec);
-      
-    LanguageSpec* best_match = 0;
+
+    const LanguageSpec* best_match = 0;
     int best_match_score = 0;
-    for(std::vector<LanguageSpec*>::iterator j = lst.begin(); j != lst.end(); ++j)
+    for(std::vector<const LanguageSpec*>::iterator j = lst.begin(); j != lst.end(); ++j)
     { // Search for the language that best matches the given spec, value country more then modifier
       int score = Language::match(Language(*j), tmplang);
 
@@ -444,8 +445,8 @@ Language::from_env(const std::string& env)
 
   return from_spec(language, country, modifier);
 }
-
-Language::Language(LanguageSpec* language_spec_)
+
+Language::Language(const LanguageSpec* language_spec_)
   : language_spec(language_spec_)
 {
 }
@@ -470,7 +471,7 @@ Language::match(const Language& lhs, const Language& rhs)
       { 7, 6, 3 }, // country wildcard
       { 4, 2, 1 }, // country miss
     };
-  
+
     int c;
     if (lhs.get_country() == rhs.get_country())
       c = 0;
@@ -478,7 +479,7 @@ Language::match(const Language& lhs, const Language& rhs)
       c = 1;
     else
       c = 2;
-  
+
     int m;
     if (lhs.get_modifier() == rhs.get_modifier())
       m = 0;
@@ -564,7 +565,7 @@ Language::operator!=(const Language& rhs)
 {
   return language_spec != rhs.language_spec;
 }
-
+
 } // namespace tinygettext
 
 /* EOF */
