@@ -1,6 +1,6 @@
 //
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2009 Joerg Henrichs
+//  Copyright (C) 2009-2013 Joerg Henrichs
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -42,7 +42,7 @@ QuadGraph *QuadGraph::m_quad_graph = NULL;
  *  \param quad_file_name Name of the file of all quads
  *  \param graph_file_name Name of the file describing the actual graph
  */
-QuadGraph::QuadGraph(const std::string &quad_file_name, 
+QuadGraph::QuadGraph(const std::string &quad_file_name,
                      const std::string graph_file_name,
                      const bool reverse) : m_reverse(reverse)
 {
@@ -86,7 +86,7 @@ void QuadGraph::load(const std::string &filename)
 {
     const XMLNode *xml = file_manager->createXMLTree(filename);
 
-    if(!xml) 
+    if(!xml)
     {
         // No graph file exist, assume a default loop X -> X+1
         // i.e. each quad is part of the graph exactly once.
@@ -104,10 +104,10 @@ void QuadGraph::load(const std::string &filename)
         }
         else
         {
-            fprintf(stderr, "[QuadGraph] No node in driveline graph\n");
+            Log::error("Quad Graph", "No node in driveline graph.");
             m_lap_length = 10.0f;
         }
-        
+
         return;
     }
 
@@ -176,14 +176,14 @@ void QuadGraph::load(const std::string &filename)
         }   // edge
         else
         {
-            fprintf(stderr, "Incorrect specification in '%s': '%s' ignored\n",
+            Log::error("Quad Graph", "Incorrect specification in '%s': '%s' ignored.",
                     filename.c_str(), xml_node->getName().c_str());
             continue;
         }   // incorrect specification
     }
     delete xml;
 
-    setDefaultSuccessors();    
+    setDefaultSuccessors();
     computeDistanceFromStart(getStartNode(), 0.0f);
     computeDirectionData();
 
@@ -201,14 +201,14 @@ void QuadGraph::load(const std::string &filename)
 
 // ----------------------------------------------------------------------------
 /** Returns the index of the first graph node (i.e. the graph node which
- *  will trigger a new lap when a kart first enters it). This is always 
+ *  will trigger a new lap when a kart first enters it). This is always
  *  0 for normal direction (this is guaranteed by the track exporter),
  *  but in reverse mode (where node 0 is actually the end of the track)
  *  this is 0's successor.
  */
 unsigned int QuadGraph::getStartNode() const
 {
-    return m_reverse ? m_all_nodes[0]->getSuccessor(0) 
+    return m_reverse ? m_all_nodes[0]->getSuccessor(0)
                      : 0;
 }   // getStartNode
 
@@ -217,7 +217,7 @@ unsigned int QuadGraph::getStartNode() const
  */
 void QuadGraph::computeChecklineRequirements()
 {
-    computeChecklineRequirements(m_all_nodes[0], 
+    computeChecklineRequirements(m_all_nodes[0],
                                  CheckManager::get()->getLapLineIndex());
 }   // computeChecklineRequirements
 
@@ -225,19 +225,19 @@ void QuadGraph::computeChecklineRequirements()
 /** Finds which checklines must be visited before driving on this quad
  *  (useful for rescue)
  */
-void QuadGraph::computeChecklineRequirements(GraphNode* node, 
+void QuadGraph::computeChecklineRequirements(GraphNode* node,
                                              int latest_checkline)
-{        
+{
     for (unsigned int n=0; n<node->getNumberOfSuccessors(); n++)
     {
         const int succ_id = node->getSuccessor(n);
-        
+
         // warp-around
         if (succ_id == 0) break;
 
-        GraphNode* succ = m_all_nodes[succ_id];        
-        int new_latest_checkline = 
-            CheckManager::get()->getChecklineTriggering(node->getCenter(), 
+        GraphNode* succ = m_all_nodes[succ_id];
+        int new_latest_checkline =
+            CheckManager::get()->getChecklineTriggering(node->getCenter(),
                                                         succ->getCenter() );
         if(new_latest_checkline==-1)
             new_latest_checkline = latest_checkline;
@@ -251,26 +251,26 @@ void QuadGraph::computeChecklineRequirements(GraphNode* node,
             printf("    Depends on checkline %i\n", *it);
         }
         */
-        
+
         if (new_latest_checkline != -1)
             succ->setChecklineRequirements(new_latest_checkline);
-        
+
         computeChecklineRequirements(succ, new_latest_checkline);
     }
 }   // computeChecklineRequirements
 
 // ----------------------------------------------------------------------------
-/** This function defines the "path-to-nodes" for each graph node that has 
+/** This function defines the "path-to-nodes" for each graph node that has
  *  more than one successor. The path-to-nodes indicates which successor to
- *  use to reach a certain node. This is e.g. used for the rubber ball to 
+ *  use to reach a certain node. This is e.g. used for the rubber ball to
  *  determine which path it is going to use to reach its target (this allows
  *  the ball to hit a target that is on a shortcut). The algorithm for the
  *  path computation favours the use of successor 0, i.e. it will if possible
- *  only use main driveline paths, not a shortcut (even though a shortcut 
- *  could result in a faster way to the target) - but since shotcuts can 
+ *  only use main driveline paths, not a shortcut (even though a shortcut
+ *  could result in a faster way to the target) - but since shotcuts can
  *  potentially be hidden they should not be used (unless necessary).
  *  Only graph nodes with more than one successor have this data structure
- *  (since on other graph nodes only one path can be used anyway, this 
+ *  (since on other graph nodes only one path can be used anyway, this
  *  saves some memory).
  */
 void QuadGraph::setupPaths()
@@ -296,27 +296,27 @@ void QuadGraph::setDefaultSuccessors()
 }   // setDefaultSuccessors
 
 // -----------------------------------------------------------------------------
-/** Sets all start positions depending on the quad graph. The number of 
+/** Sets all start positions depending on the quad graph. The number of
  *  entries needed is defined by the size of the start_transform (though all
  *  entries will be overwritten).
  *  E.g. the karts will be placed as:
  *   1           \
  *     2          +--  row with three karts, each kart is 'sidewards_distance'
- *       3       /     to the right of the previous kart, and 
+ *       3       /     to the right of the previous kart, and
  *   4                 'forwards_distance' behind the previous kart.
  *     5               The next row starts again with the kart being
  *       6             'forwards_distance' behind the end of the previous row.
  *  etc.
- *  \param start_transforms A vector sized to the needed number of start 
- *               positions. The values will all be overwritten with the 
+ *  \param start_transforms A vector sized to the needed number of start
+ *               positions. The values will all be overwritten with the
  *               default start positions.
  *  \param karts_per_row How many karts to place in each row.
- *  \param forwards_distance Distance in forward (Z) direction between 
+ *  \param forwards_distance Distance in forward (Z) direction between
  *               each kart.
- *  \param sidewards_distance Distance in sidewards (X) direction between 
+ *  \param sidewards_distance Distance in sidewards (X) direction between
  *               karts.
  */
-void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform> 
+void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform>
                                                        *start_transforms,
                                          unsigned int karts_per_row,
                                          float forwards_distance,
@@ -327,7 +327,7 @@ void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform>
     // counting when reached). The first predecessor is the one on
     // the main driveline.
     int current_node = m_all_nodes[getStartNode()]->getPredecessor(0);
-    
+
     float distance_from_start = 0.1f+forwards_distance;
 
     // Maximum distance to left (or right) of centre line
@@ -350,7 +350,7 @@ void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform>
             while(distance_from_start > getNode(current_node).getNodeLength())
             {
                 distance_from_start -= getNode(current_node).getNodeLength();
-                // Only follow the main driveline, i.e. first predecessor 
+                // Only follow the main driveline, i.e. first predecessor
                 current_node = getNode(current_node).getPredecessor(0);
             }
             const GraphNode &gn   = getNode(current_node);
@@ -359,20 +359,20 @@ void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform>
 
             Vec3 horizontal_line = gn[2] - gn[3];
             horizontal_line.normalize();
-           
+
             Vec3 start = gn.getUpperCenter()
-                       + center_line     * distance_from_start 
+                       + center_line     * distance_from_start
                        + horizontal_line * x_pos;
             // Add a certain epsilon to the height in case that the
             // drivelines are beneath the track.
             (*start_transforms)[i].setOrigin(start+Vec3(0,upwards_distance,0));
             (*start_transforms)[i].setRotation(
-                btQuaternion(btVector3(0, 1, 0), 
+                btQuaternion(btVector3(0, 1, 0),
                              gn.getAngleToSuccessor(0)));
             if(x_pos >= max_x_dist-sidewards_distance*0.5f)
             {
                 x_pos  = -max_x_dist;
-                // Every 2nd row will be pushed sideways by half the distance 
+                // Every 2nd row will be pushed sideways by half the distance
                 // between karts, so that a kart can drive between the karts in
                 // the row ahead of it.
                 row_number ++;
@@ -387,7 +387,7 @@ void QuadGraph::setDefaultStartPositions(AlignedArray<btTransform>
 }   // setStartPositions
 
 // -----------------------------------------------------------------------------
-/** Creates a mesh for this graph. The mesh is not added to a scene node and 
+/** Creates a mesh for this graph. The mesh is not added to a scene node and
  *  is stored in m_mesh.
  *  \param show_invisble  If true, also create a mesh for parts of the
  *         driveline that are invisible.
@@ -421,10 +421,10 @@ void QuadGraph::createMesh(bool show_invisible,
 
     // Four vertices for each of the n-1 remaining quads
     video::S3DVertex *new_v = new video::S3DVertex[4*n];
-    // Each quad consists of 2 triangles with 3 elements, so 
+    // Each quad consists of 2 triangles with 3 elements, so
     // we need 2*3 indices for each quad.
     irr::u16         *ind   = new irr::u16[6*n];
-    video::SColor     c(255, 255, 0, 0);    
+    video::SColor     c(255, 255, 0, 0);
 
     if(track_color)
         c = *track_color;
@@ -439,14 +439,14 @@ void QuadGraph::createMesh(bool show_invisible,
         // Swap the colours from red to blue and back
         if(!track_color)
         {
-            c.setRed (i%2 ? 255 : 0); 
-            c.setBlue(i%2 ? 0 : 255);
+            c.setRed ((i%2) ? 255 : 0);
+            c.setBlue((i%2) ? 0 : 255);
         }
         // Transfer the 4 points of the current quad to the list of vertices
         m_all_nodes[count]->getQuad().getVertices(new_v+4*i, c);
 
         // Set up the indices for the triangles
-        // (note, afaik with opengl we could use quads directly, but the code 
+        // (note, afaik with opengl we could use quads directly, but the code
         // would not be portable to directx anymore).
         ind[6*i  ] = 4*i;  // First triangle: vertex 0, 1, 2
         ind[6*i+1] = 4*i+1;
@@ -456,7 +456,7 @@ void QuadGraph::createMesh(bool show_invisible,
         ind[6*i+5] = 4*i+3;
         i++;
     }   // for i=1; i<QuadSet::get()
-    
+
     m_mesh_buffer->append(new_v, n*4, ind, n*6);
 
     if(lap_color)
@@ -516,14 +516,14 @@ void QuadGraph::createMesh(bool show_invisible,
     //m_node->setAutomaticCulling(scene::EAC_OFF);
     m_mesh_buffer->recalculateBoundingBox();
     m_mesh->setBoundingBox(m_mesh_buffer->getBoundingBox());
-    
+
     delete[] ind;
     delete[] new_v;
 }   // createMesh
 
 // -----------------------------------------------------------------------------
 
-/** Creates the debug mesh to display the quad graph on top of the track 
+/** Creates the debug mesh to display the quad graph on top of the track
  *  model. */
 void QuadGraph::createDebugMesh()
 {
@@ -533,13 +533,13 @@ void QuadGraph::createDebugMesh()
                /*enable_transparency*/true);
 
     // Now colour the quads red/blue/red ...
-    video::SColor     c( 128, 255, 0, 0);    
+    video::SColor     c( 128, 255, 0, 0);
     video::S3DVertex *v = (video::S3DVertex*)m_mesh_buffer->getVertices();
     for(unsigned int i=0; i<m_mesh_buffer->getVertexCount(); i++)
     {
         // Swap the colours from red to blue and back
-        c.setRed (i%2 ? 255 : 0); 
-        c.setBlue(i%2 ? 0 : 255);
+        c.setRed ((i%2) ? 255 : 0);
+        c.setBlue((i%2) ? 0 : 255);
         v[i].Color = c;
     }
     m_node = irr_driver->addMesh(m_mesh);
@@ -568,12 +568,12 @@ void QuadGraph::cleanupDebugMesh()
  *  \param succ A vector of ints to which the successors are added.
  *  \param for_ai true if only quads accessible by the AI should be returned.
  */
-void QuadGraph::getSuccessors(int node_number, 
+void QuadGraph::getSuccessors(int node_number,
                               std::vector<unsigned int>& succ,
                               bool for_ai) const
 {
     const GraphNode *gn=m_all_nodes[node_number];
-    for(unsigned int i=0; i<gn->getNumberOfSuccessors(); i++) 
+    for(unsigned int i=0; i<gn->getNumberOfSuccessors(); i++)
     {
         // If getSuccessor is called for the AI, only add
         // quads that are meant for the AI to be used.
@@ -594,10 +594,10 @@ void QuadGraph::computeDistanceFromStart(unsigned int node, float new_distance)
     float current_distance = gn->getDistanceFromStart();
 
     // If this node already has a distance defined, check if the new distance
-    // is longer, and if so adjust all following nodes. Without this the 
-    // length of the track (as taken by the distance from start of the last 
-    // node) could be smaller than some of the paths. This can result in 
-    // incorrect results for the arrival time estimation of the AI karts. 
+    // is longer, and if so adjust all following nodes. Without this the
+    // length of the track (as taken by the distance from start of the last
+    // node) could be smaller than some of the paths. This can result in
+    // incorrect results for the arrival time estimation of the AI karts.
     // See trac #354 for details.
     // Then there is no need to test/adjust any more nodes.
     if(current_distance>=0)
@@ -605,12 +605,12 @@ void QuadGraph::computeDistanceFromStart(unsigned int node, float new_distance)
         if(current_distance<new_distance)
         {
             float delta = new_distance - current_distance;
-            updateDistancesForAllSuccessors(gn->getQuadIndex(), delta);
+            updateDistancesForAllSuccessors(gn->getQuadIndex(), delta, 0);
         }
         return;
     }
 
-    // Otherwise this node has no distance defined yet. Set the new 
+    // Otherwise this node has no distance defined yet. Set the new
     // distance, and recursively update all following nodes.
     gn->setDistanceFromStart(new_distance);
 
@@ -622,7 +622,7 @@ void QuadGraph::computeDistanceFromStart(unsigned int node, float new_distance)
         if(gn_next->getDistanceFromStart()==0)
             continue;
 
-        computeDistanceFromStart(gn_next->getQuadIndex(), 
+        computeDistanceFromStart(gn_next->getQuadIndex(),
                                  new_distance + gn->getDistanceToSuccessor(i));
     }   // for i
 }   // computeDistanceFromStart
@@ -634,39 +634,52 @@ void QuadGraph::computeDistanceFromStart(unsigned int node, float new_distance)
  *  distance from start.
  *  \param indx Index of the node for which to increase the distance.
  *  \param delta Amount by which to increase the distance.
+ *  \param recursive_count Counts how often this function was called 
+ *         recursively in order to catch incorrect graphs that contain loops.
  */
-void QuadGraph::updateDistancesForAllSuccessors(unsigned int indx, float delta)
+void QuadGraph::updateDistancesForAllSuccessors(unsigned int indx, float delta,
+                                                 unsigned int recursive_count)
 {
+    if(recursive_count>getNumNodes())
+    {
+        Log::error("QuadGraph",
+                   "Quad graph contains a loop (without start node).");
+        Log::fatal("QuadGraph",
+                   "Fix graph, check for directions of all shortcuts etc.");
+    }
+    recursive_count++;
+
     GraphNode &g=getNode(indx);
     g.setDistanceFromStart(g.getDistanceFromStart()+delta);
     for(unsigned int i=0; i<g.getNumberOfSuccessors(); i++)
     {
         GraphNode &g_next = getNode(g.getSuccessor(i));
-        // Stop when we reach the start node, i.e. the only node with a 
+        // Stop when we reach the start node, i.e. the only node with a
         // distance of 0
         if(g_next.getDistanceFromStart()==0)
             continue;
 
-        // Only increase the distance from start of a successor node, if 
-        // this successor has a distance from start that is smaller then 
+        // Only increase the distance from start of a successor node, if
+        // this successor has a distance from start that is smaller then
         // the increased amount.
         if(g.getDistanceFromStart()+g.getDistanceToSuccessor(i) >
             g_next.getDistanceFromStart())
         {
-            updateDistancesForAllSuccessors(g.getSuccessor(i), delta);
+            updateDistancesForAllSuccessors(g.getSuccessor(i), delta, 
+                                            recursive_count);
         }
     }
 }   // updateDistancesForAllSuccessors
 
 //-----------------------------------------------------------------------------
 /** Computes the direction (straight, left, right) of all graph nodes and the
- *  lastest graph node that is still turning in the given direction. For 
+ *  lastest graph node that is still turning in the given direction. For
  *  example, if a successor to this graph node is turning left, it will compute
  *  the last graph node that is still turning left. This data is used by the
  *  AI to estimate the turn radius.
  *  At this stage there is one restriction: if a node with more than one
  *  successor is ahead, only successor 0 is used. That might lead to somewhat
- *  incorrect results (i.e. the last successor is determined assuming that 
+ *  incorrect results (i.e. the last successor is determined assuming that
  *  the kart is always using successor 0, while in reality it might follow
  *  a different successor, resulting in a different turn radius. It is not
  *  expected that this makes much difference for the AI (since it will update
@@ -677,7 +690,7 @@ void QuadGraph::computeDirectionData()
 {
     for(unsigned int i=0; i<m_all_nodes.size(); i++)
     {
-        for(unsigned int succ_index=0; 
+        for(unsigned int succ_index=0;
             succ_index<m_all_nodes[i]->getNumberOfSuccessors();
             succ_index++)
         {
@@ -705,7 +718,7 @@ float QuadGraph::normalizeAngle(float f)
  *  the lines connecting n+1 and n+2 (where n is the current node, and +1 etc.
  *  specifying the successor). Then it keeps on testing the line from n+2 to
  *  n+3, n+3 to n+4, ... as long as the turn direction is the same. The last
- *  value that still has the same direction is then set as the last node 
+ *  value that still has the same direction is then set as the last node
  *  with the same direction in the specified graph node.
  *  \param current Index of the graph node with which to start ('n' in the
  *                 description above).
@@ -713,7 +726,7 @@ float QuadGraph::normalizeAngle(float f)
  *                    If there should be any other branches later, successor
  *                    0 will always be tetsed.
  */
-void QuadGraph::determineDirection(unsigned int current, 
+void QuadGraph::determineDirection(unsigned int current,
                                    unsigned int succ_index)
 {
     // The maximum angle which is still considered to be straight
@@ -728,7 +741,6 @@ void QuadGraph::determineDirection(unsigned int current,
     if(fabsf(rel_angle)<max_straight_angle)
         rel_angle = 0;
 
-    int prev = next;                           // is now n+1
     next     = getNode(next).getSuccessor(0);  // next is now n+2
 
     // If the direction is still the same during a lap the last node
@@ -752,13 +764,12 @@ void QuadGraph::determineDirection(unsigned int current,
             break;
         rel_angle = new_rel_angle;
 
-        prev = next;
         next = getNode(next).getSuccessor(0);
     }    // while(1)
 
-    GraphNode::DirectionType dir = 
+    GraphNode::DirectionType dir =
         rel_angle==0 ? GraphNode::DIR_STRAIGHT
-                     : (rel_angle>0) ? GraphNode::DIR_RIGHT 
+                     : (rel_angle>0) ? GraphNode::DIR_RIGHT
                                      : GraphNode::DIR_LEFT;
     m_all_nodes[current]->setDirectionData(succ_index, dir, next);
 }   // determineDirection
@@ -774,12 +785,12 @@ void QuadGraph::determineDirection(unsigned int current,
  *  \param xyz The position of the kart.
  *  \param sector The graph node the position is on.
  */
-void QuadGraph::spatialToTrack(Vec3 *dst, const Vec3& xyz, 
+void QuadGraph::spatialToTrack(Vec3 *dst, const Vec3& xyz,
                               const int sector) const
 {
     if(sector == UNKNOWN_SECTOR )
     {
-        fprintf(stderr, "WARNING: UNKNOWN_SECTOR in spatialToTrack().\n");
+        Log::warn("Quad Graph", "UNKNOWN_SECTOR in spatialToTrack().");
         return;
     }
 
@@ -793,7 +804,7 @@ void QuadGraph::spatialToTrack(Vec3 *dst, const Vec3& xyz,
  *  \param xyz Position for which the segment should be determined.
  *  \param sector Contains the previous sector (as a shortcut, since usually
  *         the sector is the same as the last one), and on return the result
- *  \param all_sectors If this is not NULL, it is a list of all sectors to 
+ *  \param all_sectors If this is not NULL, it is a list of all sectors to
  *         test. This is used by the AI to make sure that it ends up on the
  *         selected way in case of a branch, and also to make sure that it
  *         doesn't skip e.g. a loop (see explanation below for details).
@@ -805,7 +816,7 @@ void QuadGraph::findRoadSector(const Vec3& xyz, int *sector,
     // so this simple case is tested first.
     if(*sector!=UNKNOWN_SECTOR && getQuadOfNode(*sector).pointInQuad(xyz) )
     {
-        return; 
+        return;
     }   // if still on same quad
 
     // Now we search through all graph nodes, starting with
@@ -822,7 +833,7 @@ void QuadGraph::findRoadSector(const Vec3& xyz, int *sector,
     // and the track is supposed to be driven: ABCDEBF, the AI might find
     // the node on F, and then keep on going straight ahead instead of
     // using the loop at all.
-    unsigned int max_count  = (*sector!=UNKNOWN_SECTOR && all_sectors!=NULL) 
+    unsigned int max_count  = (*sector!=UNKNOWN_SECTOR && all_sectors!=NULL)
                             ? all_sectors->size()
                             : m_all_nodes.size();
     *sector = UNKNOWN_SECTOR;
@@ -842,7 +853,7 @@ void QuadGraph::findRoadSector(const Vec3& xyz, int *sector,
             *sector  = indx;
         }
     }   // for i<m_all_nodes.size()
-    
+
     return;
 }   // findRoadSector
 
@@ -874,7 +885,7 @@ void QuadGraph::findRoadSector(const Vec3& xyz, int *sector,
     until the next higher overlapping line segment, and find the closest
     one to XYZ.
  */
-int QuadGraph::findOutOfRoadSector(const Vec3& xyz, 
+int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
                                    const int curr_sector,
                                    std::vector<int> *all_sectors) const
 {
@@ -887,7 +898,7 @@ int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
         // the quads of the shortcuts. So after quad n-1 (the last one
         // before the lap counting line) quad n will not be 0 (the first
         // quad after the lap counting line), but one of the quads on a
-        // shortcut. If we only tested a limited number of quads to 
+        // shortcut. If we only tested a limited number of quads to
         // improve the performance the crossing of a lap might not be
         // detected (because quad 0 is not tested, only quads on the
         // shortcuts are tested). If this should become a performance
@@ -905,7 +916,7 @@ int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
     int   min_sector = UNKNOWN_SECTOR;
     float min_dist_2 = 999999.0f*999999.0f;
 
-    // If a kart is falling and in between (or too far below) 
+    // If a kart is falling and in between (or too far below)
     // a driveline point it might not fulfill
     // the height condition. So we run the test twice: first with height
     // condition, then again without the height condition - just to make sure
@@ -915,11 +926,11 @@ int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
         for(int j=0; j<count; j++)
         {
             int next_sector;
-            if(all_sectors) 
+            if(all_sectors)
                 next_sector = (*all_sectors)[j];
             else
-                next_sector  = current_sector+1 == (int)getNumNodes() 
-                ? 0 
+                next_sector  = current_sector+1 == (int)getNumNodes()
+                ? 0
                 : current_sector+1;
 
             // A first simple test uses the 2d distance to the center of the quad.
@@ -929,7 +940,7 @@ int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
                 const Quad &q = getQuadOfNode(next_sector);
                 float dist    = xyz.getY() - q.getMinHeight();
                 // While negative distances are unlikely, we allow some small
-                // negative numbers in case that the kart is partly in the 
+                // negative numbers in case that the kart is partly in the
                 // track. Only do the height test in phase==0, in phase==1
                 // accept any point, independent of height.
                 if(phase==1 || (dist < 5.0f && dist>-1.0f) )
@@ -947,7 +958,7 @@ int QuadGraph::findOutOfRoadSector(const Vec3& xyz,
 
     if(min_sector==UNKNOWN_SECTOR )
     {
-        printf("unknown sector found.\n");
+        Log::info("Quad Grap", "unknown sector found.");
     }
     return min_sector;
 }   // findOutOfRoadSector
@@ -965,7 +976,7 @@ video::ITexture *QuadGraph::makeMiniMap(const core::dimension2du &dimension,
                /*enable_transparency*/ false,
                /*track_color*/    &fill_color,
                /*lap line color*/  &red                       );
-    
+
     m_node = irr_driver->addMesh(m_mesh);   // add Debug Mesh
 #ifdef DEBUG
     m_node->setName("minimap-mesh");
@@ -983,17 +994,17 @@ video::ITexture *QuadGraph::makeMiniMap(const core::dimension2du &dimension,
     float dx = bb_max.getX()-bb_min.getX();
     float dz = bb_max.getZ()-bb_min.getZ();
 
-    // Set the scaling correctly. Also the center point (which is used 
+    // Set the scaling correctly. Also the center point (which is used
     // as the camera position) needs to be adjusted: the track must
     // be aligned to the left/top of the texture which is used (otherwise
     // mapPoint2MiniMap doesn't work), so adjust the camera position
     // that the track is properly aligned (view from the side):
     //          c        camera
-    //         / \       . 
+    //         / \       .
     //        /   \     <--- camera angle
-    //       /     \     . 
+    //       /     \     .
     //      {  [-]  }   <--- track flat (viewed from the side)
-    // If [-] is the shorter side of the track, then the camera will 
+    // If [-] is the shorter side of the track, then the camera will
     // actually render the area in { } - which is the length of the
     // longer side of the track.
     // To align the [-] side to the left, the camera must be moved
@@ -1005,7 +1016,7 @@ video::ITexture *QuadGraph::makeMiniMap(const core::dimension2du &dimension,
         center.setX(center.getX() + (dz-dx)*0.5f);
         m_scaling = dimension.Width / dz;
     }
-    else 
+    else
     {
         center.setZ(center.getZ() + (dx-dz)*0.5f);
         m_scaling = dimension.Width / dx;
@@ -1014,7 +1025,7 @@ video::ITexture *QuadGraph::makeMiniMap(const core::dimension2du &dimension,
     float range = (dx>dz) ? dx : dz;
 
     core::matrix4 projection;
-    projection.buildProjectionMatrixOrthoLH(range, 
+    projection.buildProjectionMatrixOrthoLH(range,
                                             range,
                                             -1, bb_max.getY()-bb_min.getY()+1);
     camera->setProjectionMatrix(projection, true);
@@ -1024,25 +1035,25 @@ video::ITexture *QuadGraph::makeMiniMap(const core::dimension2du &dimension,
     camera->setPosition(core::vector3df(center.getX(), bb_max.getY()+1, center.getZ()));
     camera->setUpVector(core::vector3df(0, 0, 1));
     camera->setTarget(core::vector3df(center.getX(),bb_min.getY()-1,center.getZ()));
-    
+
     video::ITexture *texture = rttProvider.renderToTexture();
-    
+
     cleanupDebugMesh();
     irr_driver->removeCameraSceneNode(camera);
     m_min_coord = bb_min;
 
-    
+
     if (texture == NULL)
     {
-        fprintf(stderr, "[QuadGraph::makeMiniMap] WARNING: RTT does not appear to work,"
-                        "mini-map will not be available\n");
+        Log::error("Quad Graph", "[makeMiniMap] WARNING: RTT does not appear to work,"
+                        "mini-map will not be available.");
     }
-    
+
     return texture;
 }   // makeMiniMap
 
 //-----------------------------------------------------------------------------
-    /** Returns the 2d coordinates of a point when drawn on the mini map 
+    /** Returns the 2d coordinates of a point when drawn on the mini map
      *  texture.
      *  \param xyz Coordinates of the point to map.
      *  \param draw_at The coordinates in pixel on the mini map of the point,
