@@ -1,5 +1,7 @@
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
+//
+//  Copyright (C) 2004-2013 Steve Baker <sjbaker1@airmail.net>
+//  Copyright (C) 2009-2013  Joerg Henrichs, Steve Baker
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -70,11 +72,11 @@ struct OverworldForceField
     core::vector3df m_position;
     bool m_is_locked;
     int m_required_points;
-    
+
     OverworldForceField()
     {
     }
-    
+
     OverworldForceField(core::vector3df position, bool is_locked, int required_points)
     {
         m_position = position;
@@ -88,30 +90,31 @@ private:
     OverworldForceField m_force_field;
     bool m_force_field_set;
 public:
-    
+
     core::vector3df m_position;
     std::string m_challenge_id;
-    
+
     OverworldChallenge(core::vector3df position, std::string challenge_id)
     {
         m_position = position;
         m_challenge_id = challenge_id;
         m_force_field_set = false;
     }
-    
+
     void setForceField(OverworldForceField f)
     {
         m_force_field = f;
         m_force_field_set = true;
     }
-    
+
     OverworldForceField& getForceField()
     {
         assert(m_force_field_set);
         return m_force_field;
     }
-    
-    
+
+    bool isForceFieldSet() const { return m_force_field_set; }
+
     const OverworldForceField& getForceField() const
     {
         assert(m_force_field_set);
@@ -124,7 +127,7 @@ struct Subtitle
 {
     int m_from, m_to;
     core::stringw m_text;
-    
+
     Subtitle(int from, int to, core::stringw text)
     {
         m_from = from;
@@ -142,11 +145,11 @@ struct Subtitle
 class Track
 {
 private:
-    
+
 #ifdef DEBUG
     unsigned int             m_magic_number;
 #endif
-    
+
     float                    m_gravity;
     std::string              m_ident;
     std::string              m_screenshot;
@@ -154,11 +157,11 @@ private:
 
     /** Will only be used on overworld */
     std::vector<OverworldChallenge> m_challenges;
-    
+
     std::vector<OverworldForceField> m_force_fields;
-    
+
     std::vector<Subtitle> m_subtitles;
-    
+
     /** Start transforms of karts (either the default, or the ones taken
      *  from the scene file). */
     AlignedArray<btTransform> m_start_transforms;
@@ -166,15 +169,21 @@ private:
     std::string              m_item_style;
     std::string              m_description;
     core::stringw            m_designer;
-    
+
     /** The full filename of the config (xml) file. */
     std::string              m_filename;
 
     /** The base dir of all files of this track. */
     std::string              m_root;
     std::vector<std::string> m_groups;
+
+    /** The list of all nodes. */
     std::vector<scene::ISceneNode*> m_all_nodes;
 
+    /** The list of all nodes that are to be converted into physics,
+     *  but not to be drawn (e.g. invisible walls). */
+    std::vector<scene::ISceneNode*> m_all_physics_only_nodes;
+
     /** The list of all meshes that are loaded from disk, which means
      *  that those meshes are being cached by irrlicht, and need to be freed. */
     std::vector<scene::IMesh*>      m_all_cached_meshes;
@@ -184,7 +193,7 @@ private:
       * This one assumes the mesh is NOT connected to any node.
       */
     std::vector<scene::IMesh*>      m_detached_cached_meshes;
-    
+
     /** A list of all textures loaded by the track, so that they can
      *  be removed from the cache at cleanup time. */
     std::vector<video::ITexture*>   m_all_cached_textures;
@@ -198,7 +207,7 @@ private:
     bool m_cache_track;
 
 #ifdef DEBUG
-    /** A list of textures that were cached before the track is loaded. 
+    /** A list of textures that were cached before the track is loaded.
      *  After cleanup of ta track it can be tested which new textures
      *  are still in the cache, and print a report of leaked textures
      *  (in debug mode only). */
@@ -217,7 +226,7 @@ private:
     TriangleMesh*            m_track_mesh;
     /** Used to collect the triangles which do not have a physical
      *  representation, but are needed for some raycast effects. An
-     *  example is a water surface: the karts ignore this (i.e. 
+     *  example is a water surface: the karts ignore this (i.e.
      *  allowing the kart to drive in/partly under water), but the
      *  actual surface position is needed for the water splash effect. */
     TriangleMesh*            m_gfx_effect_mesh;
@@ -227,9 +236,13 @@ private:
     Vec3                     m_aabb_max;
     /** True if this track is an arena. */
     bool                     m_is_arena;
-    
+    /** True if this track has easter eggs. */
+    bool                     m_has_easter_eggs;
+    /** True if this track is a soccer arena. */
+    bool                     m_is_soccer;
+
     bool                     m_is_cutscene;
-    
+
     /** The version of this track. A certain STK version will only support
      *  certain track versions. */
     int                      m_version;
@@ -241,7 +254,7 @@ private:
       * in the track seelction screen
       */
     bool                     m_internal;
-    
+
     /** Whether this track should be available in reverse version */
     bool                     m_reverse_available;
 
@@ -256,30 +269,30 @@ private:
     bool                     m_enable_push_back;
 
     /** The type of sky to be used for the track. */
-    enum {SKY_NONE, SKY_BOX, 
+    enum {SKY_NONE, SKY_BOX,
           SKY_DOME, SKY_COLOR}          m_sky_type;
 
     /** sky rotation speed */
     float m_sky_dx, m_sky_dy;
-    
+
     /** A list of the textures for the sky to use. It contains one texture
      *  in case of a dome, and 6 textures for a box. */
     std::vector<video::ITexture*> m_sky_textures;
 
     /** Used if m_sky_type is SKY_COLOR only */
     irr::video::SColor m_sky_color;
-    
+
     /** The list of all animated textures. */
     std::vector<MovingTexture*> m_animated_textures;
 
     /** Manager for all track objects. */
     TrackObjectManager *m_track_object_manager;
 
-    /** If a sky dome is used, the number of horizontal segments 
+    /** If a sky dome is used, the number of horizontal segments
      *  the sphere should be divided in. */
     int                      m_sky_hori_segments;
 
-    /** If a sky dome is used, the number of vertical segments 
+    /** If a sky dome is used, the number of vertical segments
      *  the sphere should be divided in. */
     int                      m_sky_vert_segments;
 
@@ -291,10 +304,10 @@ private:
 
     /** Particles emitted from the sky (wheather) */
     ParticleKind*            m_sky_particles;
-    
+
     /** Use a special built-in wheather */
     WeatherType              m_weather_type;
-        
+
     /** A simple class to keep information about a track mode. */
     class TrackMode
     {
@@ -303,20 +316,20 @@ private:
         std::string m_quad_name;   /**< Name of the quad file to use.    */
         std::string m_graph_name;  /**< Name of the graph file to use.   */
         std::string m_scene;       /**< Name of the scene file to use.   */
-        
+
 #ifdef DEBUG
         unsigned int m_magic_number;
 #endif
-        
+
         /** Default constructor, sets default names for all fields. */
         TrackMode() : m_name("default"),         m_quad_name("quads.xml"),
-                      m_graph_name("graph.xml"), m_scene("scene.xml") 
+                      m_graph_name("graph.xml"), m_scene("scene.xml")
         {
 #ifdef DEBUG
             m_magic_number = 0x46825179;
 #endif
         }
-        
+
         ~TrackMode()
         {
 #ifdef DEBUG
@@ -324,7 +337,7 @@ private:
             m_magic_number = 0xDEADBEEF;
 #endif
         }
-        
+
     };   // TrackMode
 
     /** List of all modes for a track. */
@@ -332,7 +345,7 @@ private:
 
     /** Name of the track to display. */
     std::string         m_name;
-    
+
     bool                m_use_fog;
     /** True if this track supports using smoothed normals. */
     bool                m_smooth_normals;
@@ -353,18 +366,16 @@ private:
     core::dimension2du      m_mini_map_size;
     float                   m_minimap_x_scale;
     float                   m_minimap_y_scale;
-    
+
     /** List of all bezier curves in the track - for e.g. camera, ... */
     std::vector<BezierCurve*> m_all_curves;
 
     void loadTrackInfo();
-    void itemCommand(const Vec3 &xyz, Item::ItemType item_type, 
-                     bool drop);
     void loadQuadGraph(unsigned int mode_id, const bool reverse);
     void convertTrackToBullet(scene::ISceneNode *node);
     bool loadMainTrack(const XMLNode &node);
     void createWater(const XMLNode &node);
-    void getMusicInformation(std::vector<std::string>&  filenames, 
+    void getMusicInformation(std::vector<std::string>&  filenames,
                              std::vector<MusicInformation*>& m_music   );
     void loadCurves(const XMLNode &node);
     void handleSky(const XMLNode &root, const std::string &filename);
@@ -373,38 +384,49 @@ public:
 
     bool reverseAvailable() { return m_reverse_available; }
     void handleAnimatedTextures(scene::ISceneNode *node, const XMLNode &xml);
-    
+
     static const float NOHIT;
 
                        Track             (const std::string &filename);
                       ~Track             ();
     void               cleanup           ();
+    void               removeCachedData  ();
     void               startMusic        () const;
-    
+
     bool               setTerrainHeight(Vec3 *pos) const;
     void               createPhysicsModel(unsigned int main_track_count);
     void               update(float dt);
     void               reset();
     void               adjustForFog(scene::ISceneNode *node);
-    void               adjustForFog(scene::IMesh* mesh, scene::ISceneNode* parent_scene_node);
+    void               adjustForFog(scene::IMesh* mesh,
+                                    scene::ISceneNode* parent_scene_node);
+    void               itemCommand(const XMLNode *node);
     const core::vector3df& getSunRotation();
     /** Sets the current ambient color for a kart with index k. */
     void               setAmbientColor(const video::SColor &color,
                                        unsigned int k);
-    void               handleExplosion(const Vec3 &pos, 
+    void               handleExplosion(const Vec3 &pos,
                                        const PhysicalObject *mp,
                                        bool secondary_hits=true) const;
-    std::vector< std::vector<float> >     
-                       buildHeightMap();
+    void               loadTrackModel  (bool reverse_track = false,
+                                        unsigned int mode_id=0);
+    bool findGround(AbstractKart *kart);
+
+    std::vector< std::vector<float> > buildHeightMap();
     // ------------------------------------------------------------------------
     /** Returns the texture with the mini map for this track. */
     const video::ITexture*    getMiniMap    () const { return m_mini_map; }
     // ------------------------------------------------------------------------
     const core::dimension2du& getMiniMapSize() const { return m_mini_map_size; }
     // ------------------------------------------------------------------------
-    bool               isArena              () const { return m_is_arena; }
+    /** Returns true if this track has an arena mode. */
+    bool isArena() const { return m_is_arena; }
     // ------------------------------------------------------------------------
-    void               loadTrackModel  (World* parent, 
+    /** Returns true if this track has easter eggs. */
+    bool hasEasterEggs() const { return m_has_easter_eggs; }
+    bool               isSoccer             () const { return m_is_soccer; }
+    // ------------------------------------------------------------------------
+    void               loadTrackModel  (World* parent,
                                         bool reverse_track = false,
                                         unsigned int mode_id=0);
     // ------------------------------------------------------------------------
@@ -417,7 +439,7 @@ public:
     int                getVersion        () const {return m_version;          }
     // ------------------------------------------------------------------------
     /** Returns the length of the main driveline. */
-    float              getTrackLength    () const 
+    float              getTrackLength    () const
                                      {return QuadGraph::get()->getLapLength();}
     // ------------------------------------------------------------------------
     /** Returns a unique identifier for this track (the directory name). */
@@ -425,7 +447,7 @@ public:
     // ------------------------------------------------------------------------
     /** Returns the name of the track, which is e.g. displayed on the screen.
         \note this is the LTR name, invoke fribidi as needed. */
-    const wchar_t* getName               () const 
+    const wchar_t* getName               () const
                              {return translations->w_gettext(m_name.c_str()); }
     // ------------------------------------------------------------------------
     /** Returns all groups this track belongs to. */
@@ -443,7 +465,7 @@ public:
     // ------------------------------------------------------------------------
     /** Returns the start coordinates for a kart with a given index.
      *  \param index Index of kart ranging from 0 to kart_num-1. */
-    btTransform        getStartTransform (unsigned int index) const     
+    btTransform        getStartTransform (unsigned int index) const
     {
         if (index >= m_start_transforms.size())
         {
@@ -461,12 +483,12 @@ public:
      *  after a rescue, and to detect wrong directions. This function will
      *  always return the angle towards the first successor, i.e. the angle
      *  in the direction of the default way on the track.
-     *  \param n Number of the quad for which the angle is asked. 
+     *  \param n Number of the quad for which the angle is asked.
      */
-    float              getAngle(int n) const 
+    float              getAngle(int n) const
                             { return QuadGraph::get()->getAngleToNext(n, 0);  }
     // ------------------------------------------------------------------------
-    /** Returns the 2d coordinates of a point when drawn on the mini map 
+    /** Returns the 2d coordinates of a point when drawn on the mini map
      *  texture.
      *  \param xyz Coordinates of the point to map.
      *  \param draw_at The coordinates in pixel on the mini map of the point,
@@ -475,14 +497,14 @@ public:
     void               mapPoint2MiniMap(const Vec3 &xyz, Vec3 *draw_at) const;
     // ------------------------------------------------------------------------
     /** Returns the full path of a given file inside this track directory. */
-    std::string        getTrackFile(const std::string &s) const 
+    std::string        getTrackFile(const std::string &s) const
                                 { return m_root+"/"+s; }
     // ------------------------------------------------------------------------
     /** Returns the number of modes available for this track. */
     unsigned int       getNumberOfModes() const { return m_all_modes.size();  }
     // ------------------------------------------------------------------------
     /** Returns the name of the i-th. mode. */
-    const std::string &getModeName(unsigned int i) const 
+    const std::string &getModeName(unsigned int i) const
                                               { return m_all_modes[i].m_name; }
     // ------------------------------------------------------------------------
     /** Returns the default ambient color. */
@@ -499,8 +521,8 @@ public:
     const TriangleMesh& getGFXEffectMesh() const {return *m_gfx_effect_mesh;}
     // ------------------------------------------------------------------------
     /** Get the number of start positions defined in the scene file. */
-    unsigned int getNumberOfStartPositions() const 
-                                          { return m_start_transforms.size(); }    
+    unsigned int getNumberOfStartPositions() const
+                                          { return m_start_transforms.size(); }
     // ------------------------------------------------------------------------
     WeatherType   getWeatherType          () const { return m_weather_type; }
     // ------------------------------------------------------------------------
@@ -525,16 +547,16 @@ public:
     bool isPushBackEnabled() const { return m_enable_push_back; }
     // ------------------------------------------------------------------------
     /** Returns true if the normals of this track can be smoothed. */
-    bool smoothNormals() const { return m_smooth_normals; }        
+    bool smoothNormals() const { return m_smooth_normals; }
     // ------------------------------------------------------------------------
-    TrackObjectManager* getTrackObjectManager() {return m_track_object_manager;}
-    
+    TrackObjectManager* getTrackObjectManager() const {return m_track_object_manager;}
+
     /** Get list of challenges placed on that world. Works only for overworld. */
     const std::vector<OverworldChallenge>& getChallengeList() const
         { return m_challenges; }
 
     const std::vector<Subtitle>& getSubtitles() const { return m_subtitles; }
-    
+
 };   // class Track
 
 #endif
