@@ -1,6 +1,7 @@
-//
 //  SuperTuxKart - a fun racing game with go-kart
 //
+//  Copyright (C) 2012-2013 SuperTuxKart-Team
+//
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
 //  as published by the Free Software Foundation; either version 3
@@ -15,6 +16,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#include "utils/constants.hpp"
+
 // for code that needs to know about endianness so do a simple test.
 // 0 : little endian
 // 1 : big endian
@@ -24,6 +27,6 @@ static const int endianness_test = 0x01000000;
 static const char* endianness_test_ptr = (const char*)&endianness_test;
 
 // in little-endian, byte 0 will be 0. in big endian, byte 0 will be 1
-bool IS_LITTLE_ENDIAN = (endianness_test_ptr[0] == 0);
+const bool IS_LITTLE_ENDIAN = (endianness_test_ptr[0] == 0);
 
-const char* STK_VERSION = "0.8";
+const char STK_VERSION[] = "0.8.1";
