@@ -1,7 +1,8 @@
-//
 //  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>,
-//                     Ingo Ruhnke <grumbel@gmx.de>
+//
+//  Copyright (C) 2004-2013  Steve Baker <sjbaker1@airmail.net>,
+//  Copyright (C) 2004-2013  Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2006-2013  SuperTuxKart-Team
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -19,9 +20,11 @@
 
 #include "utils/string_utils.hpp"
 
+#include "utils/log.hpp"
+
 #include "coreutil.h"
 
-#include "math.h"
+#include <math.h>
 #include <algorithm>
 #include <cstring>
 #include <stdio.h>
@@ -38,10 +41,10 @@ namespace StringUtils
             // While this is basically correct, it fails with older
             // g++ versions (at least 2.95.3), which have a wrong template. To
             // avoid this issue, a more C-traditional way is used.
-            return strcmp(lhs.c_str()+(lhs.length()-rhs.length()), 
+            return strcmp(lhs.c_str()+(lhs.length()-rhs.length()),
                           rhs.c_str()                             )==0;
     }   // hasSuffix
-    
+
     bool startsWith(const std::string& str, const std::string& prefix)
     {
         if (str.length() < prefix.length())
@@ -49,7 +52,7 @@ namespace StringUtils
         else
             return strncmp(str.c_str(), prefix.c_str(), prefix.size())==0;
     }
-    
+
     //-------------------------------------------------------------------------
     /** Returns the path of a filename, i.e. everything till the last '/'.
      */
@@ -64,7 +67,7 @@ namespace StringUtils
         }
         return "";
     }   // getPath
-    
+
     //-------------------------------------------------------------------------
     /** Returns the basename of a filename, i.e. everything after the last "/".
      */
@@ -79,7 +82,7 @@ namespace StringUtils
         }
         return filename;
     }   // getBasename
-    
+
     //-------------------------------------------------------------------------
     /** Removes the extension, i.e. everything after the last ".".
      */
@@ -94,7 +97,7 @@ namespace StringUtils
         }
         return filename;
     }   // removeExtension
-    
+
     //-------------------------------------------------------------------------
     /** Returns the extension, i.e. everything after the last "."
      */
@@ -109,7 +112,7 @@ namespace StringUtils
         }
         return filename;
     }   // getExtension
-    
+
     //-------------------------------------------------------------------------
     /** Returns a string converted to upper case.
      */
@@ -119,7 +122,7 @@ namespace StringUtils
         std::transform(name.begin(), name.end(), name.begin(), ::toupper);
         return name;
     }   // toUpperCase
-    
+
     //-------------------------------------------------------------------------
     /** Returns a string converted to lower case.
      */
@@ -131,17 +134,17 @@ namespace StringUtils
     }   // toLowerCase
 
     //-------------------------------------------------------------------------
-    /** Splits a string into substrings separated by a certain character, and 
+    /** Splits a string into substrings separated by a certain character, and
      *  returns a std::vector of all those substring. E.g.:
      *  split("a b=c d=e",' ')  --> ["a", "b=c", "d=e"]
      *  \param s The string to split.
      *  \param c The character  by which the string is split.
      */
-    std::vector<std::string> split(const std::string& s, char c, 
+    std::vector<std::string> split(const std::string& s, char c,
                                    bool keepSplitChar)
     {
         std::vector<std::string> result;
-        
+
         try
         {
             std::string::size_type start=0;
@@ -158,14 +161,14 @@ namespace StringUtils
                         result.push_back(std::string(s, from, i-from));
                     }
                     else result.push_back(std::string(s,start, i-start));
-                    
+
                     start=i+1;
-                } 
+                }
                 else   // end of string reached
                 {
-                    if (keepSplitChar) 
+                    if (keepSplitChar)
                         result.push_back(std::string(s,start-1));
-                    else 
+                    else
                         result.push_back(std::string(s,start));
                     start = i;
                 }
@@ -174,36 +177,36 @@ namespace StringUtils
         }
         catch (std::exception& e)
         {
-            fprintf(stderr, 
-                    "Fatal error in split(std::string) : %s @ line %i : %s\n",
+            Log::error("StringUtils", 
+                       "Error in split(std::string) : %s @ line %i : %s.",
                      __FILE__, __LINE__, e.what());
-            printf("Splitting %s\n", s.c_str());
-            
+            Log::error("StringUtils", "Splitting '%s'.", s.c_str());
+
             for (int n=0; n<(int)result.size(); n++)
             {
-                printf("Split : %s\n", result[n].c_str());
+                Log::error("StringUtils", "Split : %s", result[n].c_str());
             }
-            
+
             assert(false); // in debug mode, trigger debugger
             exit(1);
         }
     }   // split
-    
+
     //-------------------------------------------------------------------------
-    /** Splits a string into substrings separated by a certain character, and 
+    /** Splits a string into substrings separated by a certain character, and
      *  returns a std::vector of all those substring. E.g.:
      *  split("a b=c d=e",' ')  --> ["a", "b=c", "d=e"]
      *  This is the version for wide strings.
      *  \param s The string to split.
      *  \param c The character  by which the string is split.
      */
-    std::vector<irr::core::stringw> split(const irr::core::stringw& s, char c, 
+    std::vector<irr::core::stringw> split(const irr::core::stringw& s, char c,
                                            bool keepSplitChar)
     {
         try
         {
             std::vector<irr::core::stringw> result;
-            
+
             irr::s32 start = 0;
             while (start < (irr::s32)s.size())
             {
@@ -218,15 +221,15 @@ namespace StringUtils
                     }
                     else result.push_back( s.subString(start, i-start) );
                     start = i+1;
-                } 
+                }
                 else
                 {
-                    if (keepSplitChar) 
-                        result.push_back( s.subString(start - 1, 
+                    if (keepSplitChar)
+                        result.push_back( s.subString(start - 1,
                                                       s.size()-start + 1) );
-                    else 
+                    else
                         result.push_back( s.subString(start, s.size()-start) );
-                    
+
                     return result;
                     //start = i+1;
                 }
@@ -236,15 +239,14 @@ namespace StringUtils
         catch (std::exception& e)
         {
             (void)e;  // avoid warning about unused variable
-            fprintf(stderr, 
-                    "Fatal error in split(stringw) : %s @ line %i : %s\n", 
-                     __FILE__, __LINE__, e.what());
-            assert(false); // in dbug mode, trigger debugger
+            Log::fatal("StringUtils",
+                       "Fatal error in split(stringw) : %s @ line %i : '%s'.",
+                       __FILE__, __LINE__, e.what());
             exit(1);
         }
     }   // split
-    
-    
+
+
     // ------------------------------------------------------------------------
     /** Splits a : separated string (like PATH) into its individual components.
      *  It especially handles Windows-style paths (c:/mydir1:d:/mydir2)
@@ -293,23 +295,24 @@ namespace StringUtils
         catch (std::exception& e)
         {
             (void)e;  // avoid warning about unused variable
-            fprintf(stderr, "Fatal error in splitPath : %s @ line %i\n", 
-                    __FILE__, __LINE__);
+            Log::fatal("StringUtils", 
+                "Fatal error in splitPath : %s @ line %i: '%s'.",
+                        __FILE__, __LINE__, path.c_str());
             exit(1);
         }
     }   // splitPath
-    
+
     // ------------------------------------------------------------------------
-    std::string insertValues(const std::string &s, 
+    std::string insertValues(const std::string &s,
                              std::vector<std::string>& all_vals)
     {
         try
         {
             std::vector<std::string> sv = StringUtils::split(s, '%', true);
             std::string new_string="";
-            
+
             unsigned int insertValID = 0;
-            
+
             const unsigned int item_count = sv.size();
             for (unsigned int i=0; i<item_count; i++)
             {
@@ -323,15 +326,15 @@ namespace StringUtils
                     {
                         if (insertValID >= all_vals.size())
                         {
-                            fprintf(stderr, 
-                                    "[StringUtils::insertValues] ERROR: "
-                                    "Invalid number of arguments in '%s'\n", 
-                                    s.c_str());
+                            Log::warn("StringUtils", 
+                                      "insertValues: "
+                                      "Invalid number of arguments in '%s'.",
+                                      s.c_str());
                             new_string += "??" + sv[i].substr(2);
                         }
                         else
                         {
-                            new_string += all_vals[insertValID] 
+                            new_string += all_vals[insertValID]
                                         + sv[i].substr(2);
                         }
                         insertValID++;
@@ -341,9 +344,9 @@ namespace StringUtils
                         const unsigned int index = sv[i][1] - '0';
                         if (index >= all_vals.size())
                         {
-                            fprintf(stderr,"[StringUtils::insertValues] ERROR:"
-                                           " Invalid argument index in '%s' "
-                                           "for %i\n", s.c_str(), index);
+                            Log::warn("StringUtils", "insertValues: "
+                                      " Invalid argument index in '%s' "
+                                      "for %i.", s.c_str(), index);
                             new_string += "??" + sv[i].substr(2);
                         }
                         else
@@ -362,25 +365,26 @@ namespace StringUtils
         catch (std::exception& e)
         {
             (void)e;  // avoid warning about unused variable
-            fprintf(stderr, "Fatal error in insertValues(std::string) : %s @ "
-                            "line %i\n", __FILE__, __LINE__);
+            Log::fatal("StringUtils", 
+                       "Fatal error in insertValues(std::string) : %s @ "
+                       "line %i: '%s'", __FILE__, __LINE__, s.c_str());
             exit(1);
         }
     }
-    
+
     // ------------------------------------------------------------------------
-    irr::core::stringw insertValues(const irr::core::stringw &s, 
+    irr::core::stringw insertValues(const irr::core::stringw &s,
                                     std::vector<irr::core::stringw>& all_vals)
     {
         try
         {
             unsigned int insertValID = 0;
 
-            const std::vector<irr::core::stringw> sv = 
+            const std::vector<irr::core::stringw> sv =
                                               StringUtils::split(s, '%', true);
-            
+
             irr::core::stringw new_string="";
-            
+
             const unsigned int size = sv.size();
             for (unsigned int i=0; i<size; i++)
             {
@@ -394,10 +398,9 @@ namespace StringUtils
                     {
                         if (insertValID >= all_vals.size())
                         {
-                            fprintf(stderr,
-                                    "[StringUtils::insertValues] ERROR: "
-                                    "Invalid number of arguments in '%s'\n",
-                                    irr::core::stringc(s.c_str()).c_str());
+                            Log::warn("StringUtils", "insertValues: "
+                                      "Invalid number of arguments in '%s'\n",
+                                      irr::core::stringc(s.c_str()).c_str());
                             new_string += "??";
                             new_string += sv[i].subString(2, sv[i].size()-2);
                         }
@@ -410,26 +413,25 @@ namespace StringUtils
                     }
                     else if(irr::core::isdigit(sv[i][1]))
                     {
-                        irr::core::stringw rest = 
+                        irr::core::stringw rest =
                                             sv[i].subString(2, sv[i].size()-2);
                         int delta = 0;
-                        
+
                         if (sv[i].size() >= 4 && sv[i][2]=='$')
                         {
                             rest = sv[i].subString(4, sv[i].size()-4);
                             delta = -1;
                         }
-                        
-                        const unsigned int index = 
-                                 irr::core::stringc(sv[i].c_str()).c_str()[1] 
+
+                        const unsigned int index =
+                                 irr::core::stringc(sv[i].c_str()).c_str()[1]
                                  - '0' + delta;
                         if (index >= all_vals.size())
                         {
-                            fprintf(stderr, 
-                                    "[StringUtils::insertValues] ERROR: "
-                                    "Invalid argument ID in '%s' : %i\n",
-                                    irr::core::stringc(s.c_str()).c_str(), 
-                                    index);
+                            Log::warn("StringUtils", "insertValues: "
+                                      "Invalid argument ID in '%s' : %i\n",
+                                      irr::core::stringc(s.c_str()).c_str(),
+                                      index);
                             new_string += "??";
                             new_string += rest;
                         }
@@ -449,14 +451,14 @@ namespace StringUtils
         catch (std::exception& e)
         {
             (void)e;  // avoid warning about unused variable
-            fprintf(stderr,
-                    "Fatal error in insertValues(stringw) : %s @ line %i\n",
-                     __FILE__, __LINE__);
+            Log::fatal("StringUtils",
+                       "Fatal error in insertValues(stringw) : %s @ line %i.",
+                       __FILE__, __LINE__);
             exit(1);
         }
     }
-    
-    
+
+
     // ------------------------------------------------------------------------
     /** Converts a time in seconds into a string of the form mm:ss:hh (minutes,
      *  seconds, 1/100 seconds.
@@ -466,8 +468,8 @@ namespace StringUtils
     {
         int int_time   = (int)(time*100.0f+0.5f);
 
-        // Avoid problems if time is negative or way too large (which 
-        // should only happen if something is broken in a track elsewhere, 
+        // Avoid problems if time is negative or way too large (which
+        // should only happen if something is broken in a track elsewhere,
         // and an incorrect finishing time is estimated.
         if(int_time<0)
             return std::string("00:00:00");
@@ -484,21 +486,21 @@ namespace StringUtils
         //     <<std::setw(2)<<std::setfill(' ')<<hundredths;
         // return s.str();
         // but that appears to be awfully complicated and slow, compared to
-        // which admittedly only works for min < 100000 - which is about 68 
+        // which admittedly only works for min < 100000 - which is about 68
         // days - good enough.
         char s[12];
         sprintf ( s, "%02d:%02d:%02d", min,  sec,  hundredths) ;
         return std::string(s);
     }   // timeToString
-    
+
     // ------------------------------------------------------------------------
 
-    std::string replace(const std::string& other, const std::string& from, 
+    std::string replace(const std::string& other, const std::string& from,
                         const std::string& to)
     {
         std::string wip = other;
-        
-        
+
+
         while (true)
         {
             const int pos = wip.find(from);
@@ -508,9 +510,9 @@ namespace StringUtils
             }
             wip.replace(pos, from.size(), to.c_str(), to.size());
         }
-        
+
     }
-    
+
     // ------------------------------------------------------------------------
     /** Converts ASCII text with HTML entities (e.g. &xE9;) to unicode strings
      *  \param input The input string which should be decoded.
@@ -521,14 +523,14 @@ namespace StringUtils
         irr::core::stringw output;
         std::string entity;
         bool isHex = false;
-        
+
         enum
         {
             NORMAL,
             ENTITY_PREAMBLE,
             ENTITY_BODY
         } state = NORMAL;
-        
+
         for (unsigned int n=0; n<input.size(); n++)
         {
             switch (state)
@@ -545,7 +547,7 @@ namespace StringUtils
                         output += wchar_t(input[n]);
                     }
                     break;
-                
+
                 case ENTITY_PREAMBLE:
                     if (input[n] != '#')
                     {
@@ -565,7 +567,7 @@ namespace StringUtils
                         state = ENTITY_BODY;
                     }
                     break;
-                
+
                 case ENTITY_BODY:
                     if (input[n] == 'x' && entity.size() == 0)
                     {
@@ -574,7 +576,7 @@ namespace StringUtils
                     else if (input[n] == ';')
                     {
                         int c;
-                        
+
                         const char* format = (isHex ? "%x" : "%i");
                         if (sscanf(entity.c_str(), format, &c) == 1)
                         {
@@ -582,10 +584,9 @@ namespace StringUtils
                         }
                         else
                         {
-                            fprintf(stderr, 
-                                    "[StringUtils] WARNING: non-numeric HTML "
-                                    "entity not supported in '%s'\n", 
-                                    input.c_str());
+                            Log::warn("StringUtils", "non-numeric HTML "
+                                      "entity not supported in '%s'.",
+                                      input.c_str());
                         }
                         state = NORMAL;
                     }
@@ -596,7 +597,7 @@ namespace StringUtils
                     break;
             }
         }
-        
+
         return output;
     }   // decodeFromHtmlEntities
 
@@ -629,7 +630,7 @@ namespace StringUtils
     }   // encodeToHtmlEntities
 
     // ------------------------------------------------------------------------
-    
+
     unsigned int simpleHash(const char* input)
     {
          int hash = 0;
@@ -637,60 +638,62 @@ namespace StringUtils
          {
             hash += (hash << (hash & 0xF)) ^ input[n];
          }
-         
+
          return hash;
     }
 
     // ------------------------------------------------------------------------
     /** Converts a version string (in the form of 'X.Y.Za-rcU' into an
-	    *  integer number.
-	    *  \param s The version string to convert.
-	    */
+     *  integer number.
+     *  \param s The version string to convert.
+     */
     int versionToInt(const std::string &version_string)
     {
-	    // Special case: SVN
-	    if(version_string=="SVN" || version_string=="svn")
-		    // SVN version will be version 99.99.99i-rcJ
-		    return 1000000*99     
-				    +  10000*99
-				    +    100*99
-				    +     10* 9
-				    +         9;
+        // Special case: SVN
+        if(version_string=="SVN" || version_string=="svn")
+        {
+            // SVN version will be version 99.99.99i-rcJ
+            return 1000000*99
+                    +  10000*99
+                    +    100*99
+                    +     10* 9
+                    +         9;
+        }
 
-	    std::string s=version_string;
-	    // To guarantee that a release gets a higher version number than 
-	    // a release candidate, we assign a 'release_candidate' number
-	    // of 9 to versions which are not a RC. We assert that any RC
-	    // is less than 9 to guarantee the ordering.
-	    int release_candidate=9;
-	    if(s.length()>4 && sscanf(s.substr(s.length()-4, 4).c_str(), "-rc%d", 
-			    &release_candidate)==1)
-	    {
-		    s = s.substr(0, s.length()-4);
-		    // Otherwise a RC can get a higher version number than
-		    // the corresponding release! If this should ever get
-		    // triggered, multiply all scaling factors above and
-		    // below by 10, to get two digits for RC numbers.
-		    assert(release_candidate<9);
-	    }
-	    int very_minor=0;
-	    if(s.length()>0 && s[s.size()-1]>='a' && s[s.size()-1]<='z')
-	    {
-		    very_minor = s[s.size()-1]-'a'+1;
-		    s = s.substr(0, s.size()-1);
-	    }
-	    std::vector<std::string> l = StringUtils::split(s, '.');
-	    while(l.size()<3)
-		    l.push_back("0");
-	    int version = 1000000*atoi(l[0].c_str())
-				    +   10000*atoi(l[1].c_str())
-				    +     100*atoi(l[2].c_str())
-				    +      10*very_minor
-				    +         release_candidate;
+        std::string s=version_string;
+        // To guarantee that a release gets a higher version number than
+        // a release candidate, we assign a 'release_candidate' number
+        // of 9 to versions which are not a RC. We assert that any RC
+        // is less than 9 to guarantee the ordering.
+        int release_candidate=9;
+        if(s.length()>4 && sscanf(s.substr(s.length()-4, 4).c_str(), "-rc%d",
+                &release_candidate)==1)
+        {
+            s = s.substr(0, s.length()-4);
+            // Otherwise a RC can get a higher version number than
+            // the corresponding release! If this should ever get
+            // triggered, multiply all scaling factors above and
+            // below by 10, to get two digits for RC numbers.
+            assert(release_candidate<9);
+        }
+        int very_minor=0;
+        if(s.length()>0 && s[s.size()-1]>='a' && s[s.size()-1]<='z')
+        {
+            very_minor = s[s.size()-1]-'a'+1;
+            s = s.substr(0, s.size()-1);
+        }
+        std::vector<std::string> l = StringUtils::split(s, '.');
+        while(l.size()<3)
+            l.push_back("0");
+        int version = 1000000*atoi(l[0].c_str())
+                    +   10000*atoi(l[1].c_str())
+                    +     100*atoi(l[2].c_str())
+                    +      10*very_minor
+                    +         release_candidate;
 
-	    if(version<=0)
-		    printf("Invalid version string '%s'.\n", s.c_str());
-	    return version;
+        if(version<=0)
+            printf("Invalid version string '%s'.\n", s.c_str());
+        return version;
     }   // versionToInt
 } // namespace StringUtils
 
